% ============================================= %
% Description
% ============================================= %

% Produces Panel B of Tables A14-A17 in the Online Appendix.

% ============================================= %
% Clear
% ============================================= %
clc;
close all;
clear all;

% ============================================= %
% Options
% ============================================= %

date_begin    = datenum('3/31/1975');    % use for results on portfolios (3-month HPR) and individual country 3-month predictability regressions
date_begin_FX = datenum('3/31/1975');    % use for results on portfolios (3-month HPR) and individual country 3-month predictability regressions

% date_begin    = datenum('4/30/1985');   % first date with 3 FX ER in the sample  
% date_begin_FX = datenum('4/30/1985');   % first date with 3 FX ER in the sample  

date_end    = datenum('12/31/2015');    % Spliced Wright/Bloomberg sample 
date_end_FX = datenum('12/31/2015');    % Spliced Wright/Bloomberg sample 

% date_end    = datenum('12/31/2007');    % Exclude crisis
% date_end_FX = datenum('12/31/2007');    % Exclude crisis


source      = 1; % Source of the raw ZC
            % 1 = Spliced Wright/Bloomberg (small cross-section of countries)
            % 2 = Spliced Wright/Bloomberg (large cross-section of countries)
                    
horizon     = 1; % Length of holding period for HPRs
            % 1 = 3 months
            
maturity    = 40;  % Maturity in quarters (40 = 10 years [benchmark], max = 60 quarters = 15 years); maturity grid for graphs: 1, 4, 8, 12, 16, 20, 40, 60 quarters; maturity grid for tables: 4, 20, 40, 60 quarters


sorting_variable    = 1; % 1 : Using the portfolio sorting variable at each date
                         % 2 : Using unconditional averages of the portfolio sorting variable
                         % 3 : Using deviations from the unconditional averages of the portfolio sorting variable
                         % 4 : Using deviations from the benchmark of the portfolio sorting variable
                         
                         
NW_lag_choice       = 4; % 1 : lags = 6 
                         % 2 : lags = ceil(4*(T/100)^(2/9))-1
                         % 3 : lags = ceil(0.75*(T)^(1/3))-1
                         % 4 : lags = max(ceil(4*(T/100)^(2/9))-1,ceil(0.75*(T)^(1/3))-1)    
                         % 5 : lags = ceil(1.3*(T)^(1/2))-1
                                                                           
nboot=10000;

horizon_bootstrap = 3;
    
if source == 1
    
    Number_portfolios   = 3;

elseif source == 2
    
    Number_portfolios   = 5;
            
end

rng(1234);

% ============================================= %
% Import ZC yields and HPRS & HPERs 
% ============================================= %

str = computer;
if or(strcmp(str,'PCWIN')==1, strcmp(str,'PCWIN64'))==1;
    newpath   =  strcat(pwd,'\');
    
else
    newpath   =  strcat(pwd,'/');
    
end;

switch source
                
        case 1 % Spliced Wright/Bloomberg dataset (small cross-section)
        
        Countries_Sample = {'Australia';'Canada';'Germany'; 'Japan';'New Zealand';'Norway';'Sweden';'Switzerland';'United Kingdom';'United States'}; 

        Nc = size(Countries_Sample,1);
        
        % ZC_Spliced is a 3-dimensional (T x Maturities x Country) matrix
        
                load(strcat(newpath,'ZC_Spliced_M_2016.mat'),'ZC_Spliced_M');
                ZC_Sample = ZC_Spliced_M;
                load(strcat(newpath,'HPR_Spliced_Hor_3M_Freq_M_2016.mat'),'HPR_Spliced_Hor_3M_Freq_M');
                HPR_Sample  = HPR_Spliced_Hor_3M_Freq_M;
                load(strcat(newpath,'HPER_Spliced_Hor_3M_Freq_M_2016.mat'),'HPER_Spliced_Hor_3M_Freq_M');
                HPER_Sample = HPER_Spliced_Hor_3M_Freq_M;
                    
        
        case 2 % Spliced Wright/Bloomberg dataset (large cross-section)
        
        Countries_Sample = {'United Kingdom'; 'Switzerland'; 'Sweden'; 'Spain'; 'South Africa'; 'Singapore'; 'Portugal'; 'Poland'; ...
                        'Norway'; 'New Zealand'; 'Netherlands'; 'Mexico'; 'Malaysia'; 'Japan'; 'Italy'; 'Ireland'; 'Indonesia'; 'Hungary';
                        'Germany'; 'France'; 'Finland'; 'Denmark'; 'Czech Republic'; 'Canada'; 'Belgium'; 'Austria'; 'Australia'; 'United States'}; 
                    
        Nc = size(Countries_Sample,1);
        
        % ZC_Spliced_Large is a 3-dimensional (T x Maturities x Country) matrix
        
          
                load(strcat(newpath,'ZC_Spliced_Large_M_2016.mat'),'ZC_Spliced_Large_M');
                ZC_Sample = ZC_Spliced_Large_M;
                load(strcat(newpath,'HPR_Spliced_Large_Hor_3M_Freq_M_2016.mat'),'HPR_Spliced_Large_Hor_3M_Freq_M');
                HPR_Sample  = HPR_Spliced_Large_Hor_3M_Freq_M;
                load(strcat(newpath,'HPER_Spliced_Large_Hor_3M_Freq_M_2016.mat'),'HPER_Spliced_Large_Hor_3M_Freq_M');
                HPER_Sample = HPER_Spliced_Large_Hor_3M_Freq_M;
                        
end
        


% ============================================= %
% Import Spot FX Reuters rates
% ============================================= %
% Source: Datastream
% See program Merge_Barclays_Reuters_D.m
% FX Forward and spot data are in units of foreign currency per USD (Ex: 1 USD = 5.19 FF), 

% load(strcat(newpath,'BR_Spot_dM.mat')); %FX_Spot
% FX_Spot  = BR_Spot_M;
% load(strcat(newpath,'List_names_BR.mat')); %List_names

load(strcat(newpath,'FXSpotRates_USD_M.mat'),'FXSpotRates_USD_M');
FX_Spot  = FXSpotRates_USD_M;


% RE-INTRODUCE THE EXCHANGE RATES FOR EURO COUNTRIES         
% Fixed euro conversion rates
% 1 = 
% BEF 40.3399 (Belgian francs) 
% DEM 1.95583 (Deutsche Mark) 
% IEP 0.787564 (Irish pound) 
% GRD 340.750 (Greek drachmas) 
% ESP 166.386 (Spanish pesetas) 
% FRF 6.55957 (French francs) 
% ITL 1936.27 (Italian lire) 
% CYP 0.585274 (Cyprus pound) 
% LUF 40.3399 (Luxembourg francs) 
% MTL 0.429300 (Maltese lira) 
% NLG 2.20371 (Dutch guilders) 
% ATS 13.7603 (Austrian schillings) 
% PTE 200.482 (Portuguese escudos) 
% SIT 239.640 (Slovenian tolars) 
% FIM 5.94573 (Finnish markkas) 
euro_begin = datenum('01/29/1999');
euro_line  = find(FX_Spot(:,1)==euro_begin);
col_euro   = find(FX_Spot(1,:)==163);
    % Euro area countries: Belgium (124), Germany (134), Greece (174), Spain (184), France (132), Ireland (178),
    % Italy (136), Luxembourg (137), Netherlands (138), Austria (122), Portugal (182), Finland (172)
    % Euro area (163)
FX_Spot(euro_line:end,FX_Spot(1,:)==124) = FX_Spot(euro_line:end,col_euro)*40.3399;
FX_Spot(euro_line:end,FX_Spot(1,:)==134) = FX_Spot(euro_line:end,col_euro)*1.95583;
FX_Spot(euro_line:end,FX_Spot(1,:)==174) = FX_Spot(euro_line:end,col_euro)*340.750;
FX_Spot(euro_line:end,FX_Spot(1,:)==184) = FX_Spot(euro_line:end,col_euro)*166.386;
FX_Spot(euro_line:end,FX_Spot(1,:)==132) = FX_Spot(euro_line:end,col_euro)*6.55957;
FX_Spot(euro_line:end,FX_Spot(1,:)==178) = FX_Spot(euro_line:end,col_euro)*0.787564;
FX_Spot(euro_line:end,FX_Spot(1,:)==136) = FX_Spot(euro_line:end,col_euro)*1936.27;
FX_Spot(euro_line:end,FX_Spot(1,:)==138) = FX_Spot(euro_line:end,col_euro)*2.20371;
FX_Spot(euro_line:end,FX_Spot(1,:)==122) = FX_Spot(euro_line:end,col_euro)*13.7603;
FX_Spot(euro_line:end,FX_Spot(1,:)==182) = FX_Spot(euro_line:end,col_euro)*200.482;
FX_Spot(euro_line:end,FX_Spot(1,:)==172) = FX_Spot(euro_line:end,col_euro)*5.94573;


[IMF_codes IMF_names]          = xlsread('IMF_codes.xls');
ListHPRIMFCodes                = NaN*zeros(size(Countries_Sample,1),1);

for i=1:size(Countries_Sample,1);
    for j=1:size(IMF_names,1)
        if strcmpi(Countries_Sample(i,1),IMF_names(j,1))
            ListHPRIMFCodes(i,1)=IMF_codes(j,1);
        end;
    end;
end;

% ---------        
% Keep only subsample (and re-order the countries)
% ---------
FX_Spot_Subsample     = NaN*zeros(size(FX_Spot,1),1+size(Countries_Sample,1));

FX_Spot_Subsample(:,1) = FX_Spot(:,1); % Dates
for i=1:size(ListHPRIMFCodes,1)-1
    col=find(FX_Spot(1,:)==ListHPRIMFCodes(i,1));
    if isempty(col)==0 % NB: no data for the US exchange rate
        FX_Spot_Subsample(:,1+i)       = FX_Spot(:,col);
    end;
end;
% For the US: exchange rate = 1
FX_Spot_Subsample(1,end) = 111;
FX_Spot_Subsample(2:end,end) = 1;

% Keep only subsample
FX_Spot     = FX_Spot_Subsample;


% ============================================= %
% Import CPI and Bond Ratings Data
% ============================================= %

load(strcat(newpath,'CPI_M.mat'),'CPI_M_GFD');                                               % CPI data start in January 1950
load(strcat(newpath,'Ratings_Backfilled_M.mat'),'Ratings_Backfilled_M');                     % Backfilled ratings data start in January 1974   
load(strcat(newpath,'Ratings_wOutlook_Backfilled_M.mat'),'Ratings_wOutlook_Backfilled_M');   % Backfilled ratings (with outlook) data start in January 1974  
Ratings_M          = Ratings_Backfilled_M;
Ratings_wOutlook_M = Ratings_wOutlook_Backfilled_M;

Ratings_M(1,105)          = 528;  % add country code for Taiwan
Ratings_wOutlook_M(1,105) = 528;  % add country code for Taiwan

CPI              = CPI_M_GFD;
Ratings          = Ratings_M;
RatingsOut       = Ratings_wOutlook_M;

[IMF_codes IMF_names]          = xlsread('IMF_codes.xls');
ListHPRIMFCodes                = NaN*zeros(size(Countries_Sample,1),1);

for i=1:size(Countries_Sample,1);
    for j=1:size(IMF_names,1)
        if strcmpi(Countries_Sample(i,1),IMF_names(j,1))
            ListHPRIMFCodes(i,1)=IMF_codes(j,1);
        end;
    end;
end;

% ---------        
% Keep only subsample (and re-order the countries)
% ---------
CPI_Subsample                  = NaN*zeros(size(CPI,1),1+size(Countries_Sample,1));
Ratings_Subsample              = NaN*zeros(size(Ratings,1),1+size(Countries_Sample,1));
RatingsOut_Subsample           = NaN*zeros(size(RatingsOut,1),1+size(Countries_Sample,1));

CPI_Subsample(:,1)          = CPI(:,1);         % Dates
for i=1:size(ListHPRIMFCodes,1)
    col=find(CPI(1,:)==ListHPRIMFCodes(i,1));
    if isempty(col)==0 
        CPI_Subsample(:,1+i)        = CPI(:,col);
    end;
end;

Ratings_Subsample(:,1)      = Ratings(:,1);     % Dates
for i=1:size(ListHPRIMFCodes,1)
    col=find(Ratings(1,:)==ListHPRIMFCodes(i,1));
    if isempty(col)==0 
        Ratings_Subsample(:,1+i)    = Ratings(:,col);
    end;
end;

RatingsOut_Subsample(:,1)   = RatingsOut(:,1);  % Dates
for i=1:size(ListHPRIMFCodes,1)
    col=find(RatingsOut(1,:)==ListHPRIMFCodes(i,1));
    if isempty(col)==0 
        RatingsOut_Subsample(:,i+1) = RatingsOut(:,col);
    end;
end;


% Keep only subsample
CPI            = CPI_Subsample;
Ratings        = Ratings_Subsample;
RatingsOut     = RatingsOut_Subsample;


% ============================================= %
% Time-window
% ============================================= %

% ----------------
% Exchange rates
% ----------------
line_begin_spot       = find(FX_Spot(:,1)==date_begin_FX);
line_end_spot         = find(FX_Spot(:,1)==date_end_FX);

FX_spotchge     = [FX_Spot(line_begin_spot:line_end_spot,1) ...
                                            log(FX_Spot(line_begin_spot:line_end_spot,2:end)./FX_Spot(line_begin_spot-3:line_end_spot-3,2:end))];

% ----------------
% Inflation rates
% ----------------
line_begin_infl       = find(CPI(:,1)==date_begin);
line_end_infl         = find(CPI(:,1)==date_end);


CPI_chge          = [CPI(line_begin_infl:line_end_infl,1) ...
                                            log(CPI(line_begin_infl:line_end_infl,2:end)./CPI(line_begin_infl-3:line_end_infl-3,2:end))];                     
   
% ----------------
% Credit ratings
% ----------------
line_begin_rat        = find(Ratings(:,1)==date_begin);
line_end_rat          = find(Ratings(:,1)==date_end);
        
Ratings_lag           = Ratings(line_begin_rat-3:line_end_rat-3,:,:);       % one-period before == 3 months before
RatingsOut_lag        = RatingsOut(line_begin_rat-3:line_end_rat-3,:,:);    % one-period before == 3 months before

Ratings               = Ratings(line_begin_rat:line_end_rat,:);
RatingsOut            = RatingsOut(line_begin_rat:line_end_rat,:); 


% ----------------
% Bond HPR
% ----------------
line_begin_hpr        = find(HPR_Sample(:,1,1)==date_begin);
line_end_hpr          = find(HPR_Sample(:,1,1)==date_end);
HPR                   = HPR_Sample(line_begin_hpr:line_end_hpr,:,:);

line_begin_hpr        = find(HPER_Sample(:,1,1)==date_begin);
line_end_hpr          = find(HPER_Sample(:,1,1)==date_end);
HPER                  = HPER_Sample(line_begin_hpr:line_end_hpr,:,:);


% ----------------
% Interest rates
% ----------------
line_begin_zc        = find(ZC_Sample(:,1,1)==date_begin);
line_end_zc          = find(ZC_Sample(:,1,1)==date_end);
ZC                   = ZC_Sample(line_begin_zc:line_end_zc,:,:);

ZC_lag         = ZC_Sample(line_begin_zc-3:line_end_zc-3,:,:);    % one-period before == 3 months before
ZC_lag_Long    = NaN(size(ZC_lag,1)+120,size(ZC_lag,2),size(ZC_lag,3));
ZC_lag_Long(max(-(line_begin_zc-123)+2,1):end,:,:)        = ZC_Sample(max(line_begin_zc-123,1):line_end_zc-3,:,:);  % one-period before == 3 months before, go back 120 months (10 years)
CPI_chge_Long        = NaN(size(CPI_chge,1)+120,size(CPI_chge,2));
CPI_chge_Long(max(-(line_begin_infl-120)+2,1):end,1)     = CPI(max(line_begin_infl-120,1):line_end_infl,1);  % go back 120 months (10 years)
CPI_chge_Long(max(-(line_begin_infl-120)+2,1):end,2:end) = log(CPI(max(line_begin_infl-120,1):line_end_infl,2:end)./CPI(max(line_begin_infl-123,1):line_end_infl-3,2:end));  % go back 120 months (10 years)
CPI_lag               = CPI(line_begin_infl-3:line_end_infl-3,:);    % one-period before == 3 months before 
CPI_chge_lag          = [CPI(line_begin_infl-3:line_end_infl-3,1) log(CPI(line_begin_infl-3:line_end_infl-3,2:end)./CPI(line_begin_infl-6:line_end_infl-6,2:end))];
CPI_chge_lag_Long     = NaN(size(CPI_lag,1)+120,size(CPI_lag,2));
CPI_chge_lag_Long(max(-(line_begin_infl-123)+2,1):end,1)     = CPI(max(line_begin_infl-123,1):line_end_infl-3,1);  % one-period before == 3 months before, go back 120 months (10 years)
CPI_chge_lag_Long(max(-(line_begin_infl-123)+2,1):end,2:end) = log(CPI(max(line_begin_infl-123,1):line_end_infl-3,2:end)./CPI(max(line_begin_infl-126,1):line_end_infl-6,2:end));  % one-period before == 3 months before, go back 120 months (10 years)
                            

% Select short-term interest rates (3-month)
ZC_3M      = NaN*zeros(size(ZC,1),1+size(ZC,3));
ZC_3M(:,1) = ZC(:,1); % Dates
for n=1:size(ZC,3)
    ZC_3M(:,1+n) = ZC(:,1+1,n);
end

ZC_3M_lag           = NaN*zeros(size(ZC_lag,1),1+size(ZC_lag,3));
ZC_3M_lag(:,1)      = ZC_lag(:,1); % Dates
ZC_3M_lag_Long      = NaN*zeros(size(ZC_lag_Long,1),1+size(ZC_lag_Long,3));
ZC_3M_lag_Long(:,1) = ZC_lag_Long(:,1); % Dates
for n=1:size(ZC,3)
    ZC_3M_lag(:,1+n) = ZC_lag(:,1+1,n);
    ZC_3M_lag_Long(:,1+n) = ZC_lag_Long(:,1+1,n);
end

% 10-year yield

col_10yr = 40;
        
ZC_10Y_lag           = NaN*zeros(size(ZC_lag,1),1+size(ZC_lag,3));
ZC_10Y_lag(:,1)      = ZC_lag(:,1); % Dates
ZC_10Y_lag_Long      = NaN*zeros(size(ZC_lag_Long,1),1+size(ZC_lag_Long,3));
ZC_10Y_lag_Long(:,1) = ZC_lag_Long(:,1); % Dates
for n=1:size(ZC,3)
    ZC_10Y_lag(:,1+n)      = ZC_lag(:,1+col_10yr,n);
    ZC_10Y_lag_Long(:,1+n) = ZC_lag_Long(:,1+col_10yr,n);
end

% Slope of the yield curve (10-year minus 3-month)

ZC_10Yminus3M_lag           = NaN*zeros(size(ZC_lag,1),1+size(ZC_lag,3));
ZC_10Yminus3M_lag(:,1)      = ZC_lag(:,1); % Dates
ZC_10Yminus3M_lag_Long      = NaN*zeros(size(ZC_lag_Long,1),1+size(ZC_lag_Long,3));
ZC_10Yminus3M_lag_Long(:,1) = ZC_lag_Long(:,1); % Dates
for n=1:size(ZC,3)
    ZC_10Yminus3M_lag(:,1+n)      = ZC_lag(:,1+col_10yr,n)- ZC_lag(:,1+1,n);
    ZC_10Yminus3M_lag_Long(:,1+n) = ZC_lag_Long(:,1+col_10yr,n)- ZC_lag_Long(:,1+1,n);
end

ZC_3M_lag_Rolling_Mean            = NaN(size(ZC_3M_lag));
ZC_3M_lag_Rolling_Mean_dev        = NaN(size(ZC_3M_lag));
ZC_10Y_lag_Rolling_Mean           = NaN(size(ZC_3M_lag));
ZC_10Yminus3M_lag_Rolling_Mean    = NaN(size(ZC_3M_lag));
CPI_chge_Rolling_Mean             = NaN(size(ZC_3M_lag));
CPI_chge_lag_Rolling_Mean         = NaN(size(ZC_3M_lag));


ZC_3M_lag_Rolling_Mean(:,1)         = ZC_3M_lag_Long(120:size(ZC_3M_lag,1)+119,1);
ZC_3M_lag_Rolling_Mean_dev(:,1)     = ZC_3M_lag_Long(120:size(ZC_3M_lag,1)+119,1);
ZC_10Y_lag_Rolling_Mean(:,1)        = ZC_10Y_lag_Long(120:size(ZC_3M_lag,1)+119,1);
ZC_10Yminus3M_lag_Rolling_Mean(:,1) = ZC_10Yminus3M_lag_Long(120:size(ZC_3M_lag,1)+119,1);
CPI_chge_Rolling_Mean(:,1)          = CPI_chge_Long(120:size(ZC_3M_lag,1)+119,1);
CPI_chge_lag_Rolling_Mean(:,1)      = CPI_chge_lag_Long(120:size(ZC_3M_lag,1)+119,1);

for t = 1:size(ZC_3M_lag,1) 
    
            ZC_3M_lag_Rolling_Mean(t,2:end)            = mean(ZC_3M_lag_Long(t:t+119,2:end)); 
            ZC_3M_lag_Rolling_Mean_dev(t,2:end)        = ZC_3M_lag(t,2:end)-ZC_3M_lag_Rolling_Mean(t,2:end);
            ZC_10Y_lag_Rolling_Mean(t,2:end)           = mean(ZC_10Y_lag_Long(t:t+119,2:end)); 
            ZC_10Yminus3M_lag_Rolling_Mean(t,2:end)    = mean(ZC_10Yminus3M_lag_Long(t:t+119,2:end)); 
            CPI_chge_Rolling_Mean(t,2:end)             = mean(CPI_chge_Long(t:t+119,2:end)); 
            CPI_chge_lag_Rolling_Mean(t,2:end)         = mean(CPI_chge_lag_Long(t:t+119,2:end)); 
               
end


% ----------------
% Import TB data from GFD
% ----------------

load(strcat(newpath,'TB_M.mat'),'GFD_TB_M'); % TB data start in January 1900
TBILL_M_GFD        = GFD_TB_M(:,[1:13,15:52,54:56,58:end]);  % eliminate duplicate (14,53,57)
TBILL_M_GFD_Subsample       = NaN*zeros(size(TBILL_M_GFD,1),1+size(Countries_Sample,1));

TBILL_M_GFD_Subsample(:,1)  = TBILL_M_GFD(:,1);              % Dates

for i=1:size(ListHPRIMFCodes,1)
   
   col_TB=find(TBILL_M_GFD(1,:)==ListHPRIMFCodes(i,1));
    if isempty(col_TB)==0 
        TBILL_M_GFD_Subsample(:,1+i) = TBILL_M_GFD(:,col_TB);
    end
    
end

TBILL_M_GFD      = TBILL_M_GFD_Subsample;   % Keep only subsample

line_begin       = find(TBILL_M_GFD(:,1)==date_begin);
line_end         = find(TBILL_M_GFD(:,1)==date_end);
TB               = NaN(line_end-line_begin+1,size(TBILL_M_GFD,2));
TB_month         = NaN(line_end-line_begin+1,size(TBILL_M_GFD,2));

TB(:,1)                   = ZC_lag(:,1); % dates
TB_month(:,1)             = ZC_lag(:,1); % dates

TB(:,2:end)               = TBILL_M_GFD(line_begin:line_end,2:end)./TBILL_M_GFD(line_begin-3:line_end-3,2:end)-1;             % period TB rate (first obs. is t0; theoretically known at t0-horizon)
TB_month(:,2:end)         = TBILL_M_GFD(line_begin-2:line_end-2,2:end)./TBILL_M_GFD(line_begin-3:line_end-3,2:end)-1;         % 1-month TB rate (first obs. is t0-horizon+1; theoretically known at t0-horizon)
TB_month_Long             = TBILL_M_GFD(line_begin-122:line_end-2,2:end)./TBILL_M_GFD(line_begin-123:line_end-3,2:end)-1;     % TB_month starting 120 obs. early

TB_month_Rolling_Mean          = NaN(size(TB_month));
TB_month_Rolling_Mean_Adj      = NaN(size(TB_month));

TB_month_Rolling_Mean(:,1)         = ZC_lag(:,1); % dates
TB_month_Rolling_Mean_Adj(:,1)     = ZC_lag(:,1); % dates

for t = 1:size(TB_month,1)
    
TB_month_Rolling_Mean(t,2:end)     = nanmean(TB_month_Long(t:t+119,:)); 
TB_month_Rolling_Mean_Adj(t,2:end) = nanmean((1+TB_month_Long(t:t+119,:)).^12-1); 

end


% ============================================= %
% Currency Excess Returns (US investor)
% ============================================= %
FXER      = NaN*zeros(size(FX_spotchge));
FXER(:,1) = FX_spotchge(:,1); % Dates
FX        = NaN*zeros(size(FX_spotchge));
FX(:,1)   = FX_spotchge(:,1); % DatesFXER      = NaN*zeros(size(FX_spotchge));
FWD       = NaN*zeros(size(FX_spotchge));
FWD(:,1)  = FX_spotchge(:,1); % Daswitch frequency
Infl      = NaN*zeros(size(CPI_chge));
Infl(:,1) = CPI_chge(:,1); % Dates

   

for n = 2:size(FX_spotchge,2)
     FXER(:,n)    = ZC_3M_lag(:,n)/4 - ZC_3M_lag(:,end)/4 - FX_spotchge(:,n);
     FX(:,n)      =  - FX_spotchge(:,n);
     FWD(:,n)     = ZC_3M_lag(:,n)/4 - ZC_3M_lag(:,end)/4;
     Infl(:,n)    = CPI_chge(:,n);
end
                


% ============================================= %
% Holding-Period Returns and Holding-Period Excess Returns in USD
% ============================================= %
% hpr(t,N)      = p(t,N-1)-p(t-1,N) = -(N-1) y(t,N-1) + N y(t-1,N)

% hprUSD(t+1,N) = hpr(t+1,N)-[s(t+1)-s(t)]
HPRUSD = NaN *zeros(size(HPR));
for c= 1:size(HPR,3)-1         % For each country, except the US (last one)
    HPRUSD(:,1,c)= HPR(:,1,c); % Copy dates
    for i=1:size(HPR,2)-1;   % For each maturity
        HPRUSD(:,1+i,c) = HPR(:,1+i,c)-FX_spotchge(:,1+c);
    end;
end;
for i=1:size(HPR,2)-1;
    HPRUSD(:,1+i,end) = HPR(:,1+i,end); % for the US
end;


% hperUSD(t+1,N) = hper*(t+1,N)+ rf* - rf -[s(t+1)-s(t)] = hper*(t+1,N) +  currer(t+1)
HPERUSD = NaN *zeros(size(HPR));
for c= 1:size(HPR,3)-1    % For each country, except the US (last one)
    HPERUSD(:,1,c)= HPR(:,1,c); % Copy dates
    for i=1:size(HPR,2)-1;   % For each maturity
        HPERUSD(:,1+i,c) = HPER(:,1+i,c) + FXER(:,1+c) ;
    end;
end;
for i=1:size(HPER,2)-1;
    HPERUSD(:,1+i,end) = HPER(:,1+i,end); % for the US
end;


% ============================================= %
% Select One Maturity
% ============================================= %
% HPR_smple and ZC_Wright is a 3-dimensional (T x Maturities x Country) matrix

HPR_LongBond      = NaN*zeros(size(HPR,1),1+size(HPR,3));
HPR_LongBond(:,1) = HPR(:,1); % Dates
for n=1:size(HPR,3)
    HPR_LongBond(:,1+n) = HPR(:,1+maturity,n);
end;

HPER_LongBond      = NaN*zeros(size(HPER,1),1+size(HPER,3));
HPER_LongBond(:,1) = HPER(:,1); % Dates
for n=1:size(HPER,3)
    HPER_LongBond(:,1+n) = HPER(:,1+maturity,n);
end;

HPRUSD_LongBond      = NaN*zeros(size(HPRUSD,1),1+size(HPRUSD,3));
HPRUSD_LongBond(:,1) = HPRUSD(:,1); % Dates
for n=1:size(HPR,3)
    HPRUSD_LongBond(:,1+n) = HPRUSD(:,1+maturity,n);
end;

HPERUSD_LongBond      = NaN*zeros(size(HPERUSD,1),1+size(HPERUSD,3));
HPERUSD_LongBond(:,1) = HPERUSD(:,1); % Dates
for n=1:size(HPER,3)
    HPERUSD_LongBond(:,1+n) = HPERUSD(:,1+maturity,n);
end

HPRUSDminusUSHPR_LongBond      = NaN*zeros(size(HPRUSD_LongBond,1),size(HPRUSD_LongBond,2)-1);
HPRUSDminusUSHPR_LongBond(:,1) = HPRUSD_LongBond(:,1); % Dates
for n=2:size(HPRUSDminusUSHPR_LongBond,2)
    HPRUSDminusUSHPR_LongBond(:,n) = HPRUSD_LongBond(:,n)- HPRUSD_LongBond(:,end); % without US
end

% ============================================= %
% Build Portfolios -- Sorted By Short-Term Interest Rates
% ============================================= %
                         
% Use variables without their first column of dates
FX_temp                        = FX(:,2:end-1);   % without US
FWD_temp                       = FWD(:,2:end-1);  % without US
Infl_temp                      = Infl(:,2:end-1); % without US
Ratings_lag_temp               = Ratings_lag(:,2:end-1); % without US
RatingsOut_lag_temp            = RatingsOut_lag(:,2:end-1); % without US

FXER_temp                      = FXER(:,2:end-1);
HPR_LongBond_temp              = HPR_LongBond(:,2:end-1); % without US
HPER_LongBond_temp             = HPER_LongBond(:,2:end-1); % without US
HPRUSD_LongBond_temp           = HPRUSD_LongBond(:,2:end-1); % without US
HPERUSD_LongBond_temp          = HPERUSD_LongBond(:,2:end-1); % without US
HPRUSDminusUSHPR_LongBond_temp = HPRUSDminusUSHPR_LongBond(:,2:end); 

ZC_3M_lag_temp                                = ZC_3M_lag(:,2:end-1);
ZC_3M_lag_Rolling_Mean_temp                   = ZC_3M_lag_Rolling_Mean(:,2:end-1);


% Get rid of interest rates if no data on currency or bond excess returns and vice-versa
for t=1:size(ZC_3M_lag_temp,1)                                               % For each date
    for c=1:size(ZC_3M_lag_temp,2)                                           % For each country
        if  isnan(ZC_3M_lag_temp(t,c)*FXER_temp(t,c)*HPER_LongBond_temp(t,c)*HPRUSD_LongBond_temp(t,c)*HPERUSD_LongBond_temp(t,c)*HPERUSD(t,end,c))==1
           ZC_3M_lag_temp(t,c)        = NaN;
           FXER_temp(t,c)             = NaN;
           FX_temp(t,c)               = NaN;
           FWD_temp(t,c)              = NaN;
           HPER_LongBond_temp(t,c)    = NaN;
           HPRUSD_LongBond_temp(t,c)  = NaN;
           HPERUSD_LongBond_temp(t,c) = NaN;
       end         
    end
end

ZC_3M_lag_Rolling_Mean_selected_temp = ZC_3M_lag_Rolling_Mean_temp;



[Portfolios_FXER,Portfolios_HPER,Portfolios_HPRUSDminusUSHPR_LongBond,Portfolios_HPERUSD,Portfolios_VarSort_IR,Track_portfolios_IR1,Portfolios_VarSort_Benchmark_IR,Portfolios_Y_IR] = ...
    Build_portfolios_for4variables(FXER_temp,HPER_LongBond_temp,HPRUSDminusUSHPR_LongBond_temp,HPERUSD_LongBond_temp,ZC_3M_lag_temp,sorting_variable,Number_portfolios,ZC_3M_lag_Rolling_Mean_selected_temp);
[Portfolios_FXER,Portfolios_HPR,Portfolios_Infl,Portfolios_HPRUSD,Portfolios_VarSort_IR,Track_portfolios_IR2] = ...
    Build_portfolios_for4variables(FXER_temp,HPR_LongBond_temp,Infl_temp,HPRUSD_LongBond_temp,ZC_3M_lag_temp,sorting_variable,Number_portfolios,ZC_3M_lag_Rolling_Mean_selected_temp);
[Portfolios_FX,Portfolios_FWD,Portfolios_Ratings,Portfolios_RatingsOut, Portfolios_VarSort_IR,Track_portfolios_IR3] = ...
    Build_portfolios_for4variables(FX_temp,FWD_temp,Ratings_lag_temp,RatingsOut_lag_temp,ZC_3M_lag_temp,sorting_variable,Number_portfolios,ZC_3M_lag_Rolling_Mean_selected_temp);

Portfolios_FXER_IR     = Portfolios_FXER;
Portfolios_HPR_IR      = Portfolios_HPR;
Portfolios_HPER_IR     = Portfolios_HPER;
Portfolios_HPRUSD_IR   = Portfolios_HPRUSD;
Portfolios_HPERUSD_IR  = Portfolios_HPERUSD;
Portfolios_FX_IR       = Portfolios_FX;
Portfolios_Infl_IR     = Portfolios_Infl;
Portfolios_Ratings_IR        = Portfolios_Ratings;
Portfolios_RatingsOut_IR     = Portfolios_RatingsOut;

ScalingFactor = 4;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%bootstrapping
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Portfolios_HPER_logdiff               =(Portfolios_HPER(:,Number_portfolios))-(Portfolios_HPER(:,1));
Portfolios_HPERUSD_logdiff            =(Portfolios_HPERUSD(:,Number_portfolios))-(Portfolios_HPERUSD(:,1));
Portfolios_HPRUSDminusUSHPR_logdiff   =(Portfolios_HPRUSDminusUSHPR_LongBond(:,Number_portfolios))-(Portfolios_HPRUSDminusUSHPR_LongBond(:,1));
Portfolios_FXER_logdiff               =(Portfolios_FXER(:,Number_portfolios))-(Portfolios_FXER(:,1));
Portfolios_FX_logdiff                 =(Portfolios_FX(:,Number_portfolios))-(Portfolios_FX(:,1));
Portfolios_FWD_logdiff                =(Portfolios_FWD(:,Number_portfolios))-(Portfolios_FWD(:,1));
Portfolios_Infl_logdiff               =Portfolios_Infl(:,Number_portfolios) - Portfolios_Infl(:,1);
Portfolios_Ratings_logdiff            =Portfolios_Ratings(:,Number_portfolios) - Portfolios_Ratings(:,1);
Portfolios_RatingsOut_logdiff         =Portfolios_RatingsOut(:,Number_portfolios) - Portfolios_RatingsOut(:,1);

portfolio_data                                    = [(Portfolios_FXER) Portfolios_FXER_logdiff (Portfolios_HPER) Portfolios_HPER_logdiff (Portfolios_HPERUSD) Portfolios_HPERUSD_logdiff  Portfolios_HPRUSDminusUSHPR_LongBond Portfolios_HPRUSDminusUSHPR_logdiff Portfolios_FX Portfolios_FX_logdiff Portfolios_FWD Portfolios_FWD_logdiff Portfolios_Infl Portfolios_Infl_logdiff Portfolios_Ratings Portfolios_Ratings_logdiff Portfolios_RatingsOut Portfolios_RatingsOut_logdiff];

[standard_error_mean,standard_error_SR]                               = bootstrap_portfolio_returns_growth(portfolio_data,nboot,horizon_bootstrap);

Total_portfolios=Number_portfolios+1;

Portfolios_FXER_se(1:Total_portfolios)                        = standard_error_mean(1:Total_portfolios);
Portfolios_HPER_se(1:Total_portfolios)                        = standard_error_mean(1+Total_portfolios:2*Total_portfolios);
Portfolios_HPERUSD_se(1:Total_portfolios)                     = standard_error_mean(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_HPRUSDminusUSHPR_se(1:Total_portfolios)            = standard_error_mean(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_se(1:Total_portfolios)                          = standard_error_mean(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_FWD_se(1:Total_portfolios)                         = standard_error_mean(1+5*Total_portfolios:6*Total_portfolios);
Portfolios_Infl_se(1:Total_portfolios)                        = standard_error_mean(1+6*Total_portfolios:7*Total_portfolios);
Portfolios_Ratings_se(1:Total_portfolios)                     = standard_error_mean(1+7*Total_portfolios:8*Total_portfolios);
Portfolios_RatingsOut_se(1:Total_portfolios)                  = standard_error_mean(1+8*Total_portfolios:9*Total_portfolios);

Portfolios_FXER_SR_se(1:Total_portfolios)                     = standard_error_SR(1:Total_portfolios);
Portfolios_HPER_SR_se(1:Total_portfolios)                     = standard_error_SR(1+Total_portfolios:2*Total_portfolios);
Portfolios_HPERUSD_SR_se(1:Total_portfolios)                  = standard_error_SR(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_HPRUSDminusUSHPR_SR_se(1:Total_portfolios)         = standard_error_SR(1+3*Total_portfolios:4*Total_portfolios);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Portfolios_HPRUSDminusUSHPR                                  = Portfolios_HPRUSDminusUSHPR_LongBond;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


Table = [100*nanmean((Portfolios_FXER))*ScalingFactor                100*nanmean((Portfolios_FXER_logdiff))*ScalingFactor; ...
         100*Portfolios_FXER_se*ScalingFactor; ...
         100*nanstd((Portfolios_FXER))*sqrt(ScalingFactor)           100*nanstd((Portfolios_FXER_logdiff))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_FXER))*sqrt(ScalingFactor)./nanstd((Portfolios_FXER))        nanmean((Portfolios_FXER_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_FXER_logdiff)); ...
         Portfolios_FXER_SR_se*sqrt(ScalingFactor); ...
         100*nanmean((Portfolios_HPER))*ScalingFactor                100*nanmean((Portfolios_HPER_logdiff))*ScalingFactor; ...                     
         100*Portfolios_HPER_se*ScalingFactor; ...
         100*nanstd((Portfolios_HPER))*sqrt(ScalingFactor)           100*nanstd((Portfolios_HPER_logdiff))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_HPER))*sqrt(ScalingFactor)./nanstd((Portfolios_HPER))        nanmean((Portfolios_HPER_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_HPER_logdiff)); ...
         Portfolios_HPER_SR_se*sqrt(ScalingFactor); ...
         100*nanmean((Portfolios_HPERUSD))*ScalingFactor             100*nanmean((Portfolios_HPERUSD_logdiff))*ScalingFactor;  ...
         100*Portfolios_HPERUSD_se*ScalingFactor; ...             
         100*nanstd((Portfolios_HPERUSD))*sqrt(ScalingFactor)        100*nanstd((Portfolios_HPERUSD_logdiff))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_HPERUSD))*sqrt(ScalingFactor)./nanstd((Portfolios_HPERUSD))  nanmean((Portfolios_HPERUSD_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_HPERUSD_logdiff)); ...
         Portfolios_HPERUSD_SR_se*sqrt(ScalingFactor); ...
         100*nanmean((Portfolios_HPRUSDminusUSHPR))*ScalingFactor    100*nanmean((Portfolios_HPRUSDminusUSHPR_logdiff))*ScalingFactor; ...         
         100*Portfolios_HPRUSDminusUSHPR_se*ScalingFactor; ...
         100*nanstd((Portfolios_HPRUSDminusUSHPR))*sqrt(ScalingFactor)                    100*nanstd((Portfolios_HPRUSDminusUSHPR_logdiff))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_HPRUSDminusUSHPR))*sqrt(ScalingFactor)./nanstd((Portfolios_HPRUSDminusUSHPR))       nanmean((Portfolios_HPRUSDminusUSHPR_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_HPRUSDminusUSHPR_logdiff));
         Portfolios_HPRUSDminusUSHPR_SR_se*sqrt(ScalingFactor); ...   
         100*nanmean((Portfolios_FX))*ScalingFactor                  100*nanmean((Portfolios_FX_logdiff))*ScalingFactor; ...
         100*nanmean((Portfolios_FWD))*ScalingFactor                 100*nanmean((Portfolios_FWD_logdiff))*ScalingFactor; ...
         100*nanmean((Portfolios_Infl))*ScalingFactor                100*nanmean((Portfolios_Infl_logdiff))*ScalingFactor; ...
         100*Portfolios_Infl_se*ScalingFactor; ...
         nanmean(Portfolios_Ratings)                                 nanmean(Portfolios_Ratings_logdiff); ...
         Portfolios_Ratings_se; ...
         nanmean(Portfolios_RatingsOut)                              nanmean(Portfolios_RatingsOut_logdiff); ...
         Portfolios_RatingsOut_se];

rows = {'Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','Mean', 'Mean','Mean','s.e.', 'Mean', 's.e.', 'Mean', 's.e.'}; 
Opts.Style = 'jf'; 
disp(' ');
disp('----------------------------------------------');
disp('Countries Sorted By Interest Rates');
disp('----------------------------------------------'); 
disp('Currency Risk Premium / Local Term Premium / Dollar Term Premium / Local (in USD) minus US HPR');
mat2tex(Table,'%6.2f',rows,[], [], Opts)


xlswrite(strcat(newpath,'portfolio_composition_BB_sample_',int2str(source),'_horizon_',int2str(horizon),'_maturity_',int2str(maturity),'_2016.xls'),Table, 'Interest Table', 'C3');
xlswrite(strcat(newpath,'portfolio_composition_BB_sample_',int2str(source),'_horizon_',int2str(horizon),'_maturity_',int2str(maturity),'_2016.xls'),rows', 'Interest Table', 'B3');


% ============================================= %
% Build Portfolios -- Sorted By Interest Rate Slopes
% ============================================= %

% Use variables without their first column of dates
FXER_temp                      = FXER(:,2:end-1); % without US
FX_temp                        = FX(:,2:end-1);  % without US
FWD_temp                       = FWD(:,2:end-1); % without US
Infl_temp                      = Infl(:,2:end-1); % without US
Ratings_lag_temp               = Ratings_lag(:,2:end-1); % without US
RatingsOut_lag_temp            = RatingsOut_lag(:,2:end-1); % without US

HPER_LongBond_temp             = HPER_LongBond(:,2:end-1); % without US
HPRUSD_LongBond_temp           = HPRUSD_LongBond(:,2:end-1); % without US
HPERUSD_LongBond_temp          = HPERUSD_LongBond(:,2:end-1); % without US
HPRUSDminusUSHPR_LongBond_temp = HPRUSDminusUSHPR_LongBond(:,2:end); 
ZC_10Yminus3M_lag_temp         = ZC_10Yminus3M_lag(:,2:end-1);
ZC_10Yminus3M_lag_Rolling_Mean_temp         = ZC_10Yminus3M_lag_Rolling_Mean(:,2:end-1);


% Get rid of interest rates if no data on currency or bond excess returns and vice-versa
for t=1:size(ZC_10Yminus3M_lag_temp,1)                                               % For each date
    for c=1:size(ZC_10Yminus3M_lag_temp,2)                                           % For each country
        if  isnan(ZC_3M_lag_temp(t,c)*FXER_temp(t,c)*HPER_LongBond_temp(t,c)*HPRUSD_LongBond_temp(t,c)*HPERUSD_LongBond_temp(t,c)*HPERUSD(t,end,c))==1;
           ZC_10Yminus3M_lag_temp(t,c)= NaN;
           FXER_temp(t,c)             = NaN;
           FX_temp(t,c)               = NaN;
           FWD_temp(t,c)              = NaN;
           HPER_LongBond_temp(t,c)    = NaN;
           HPRUSD_LongBond_temp(t,c)  = NaN;
           HPERUSD_LongBond_temp(t,c) = NaN;
       end         
    end;
end;

[Portfolios_FXER,Portfolios_HPER,Portfolios_HPRUSDminusUSHPR_LongBond,Portfolios_HPERUSD,Portfolios_VarSort_Slope,Track_portfolios_Slope1] = ...
    Build_portfolios_for4variables(FXER_temp,HPER_LongBond_temp,HPRUSDminusUSHPR_LongBond_temp,HPERUSD_LongBond_temp,ZC_10Yminus3M_lag_temp,sorting_variable,Number_portfolios,ZC_10Yminus3M_lag_Rolling_Mean_temp);
[Portfolios_FXER,Portfolios_HPR,Portfolios_Infl,Portfolios_HPRUSD,Portfolios_VarSort_Slope,Track_portfolios_Slope2] = ...
    Build_portfolios_for4variables(FXER_temp,HPR_LongBond_temp,Infl_temp,HPRUSD_LongBond_temp,ZC_10Yminus3M_lag_temp,sorting_variable,Number_portfolios,ZC_10Yminus3M_lag_Rolling_Mean_temp);
[Portfolios_FX,Portfolios_FWD,Portfolios_Ratings,Portfolios_RatingsOut,Portfolios_VarSort_Slope,Track_portfolios_Slope3] = ...
    Build_portfolios_for4variables(FX_temp,FWD_temp,Ratings_lag_temp,RatingsOut_lag_temp,ZC_10Yminus3M_lag_temp,sorting_variable,Number_portfolios,ZC_10Yminus3M_lag_Rolling_Mean_temp);

Portfolios_FXER_Slope     = Portfolios_FXER;
Portfolios_HPR_Slope      = Portfolios_HPR;
Portfolios_HPER_Slope     = Portfolios_HPER;
Portfolios_HPRUSD_Slope   = Portfolios_HPRUSD;
Portfolios_HPERUSD_Slope  = Portfolios_HPERUSD;
Portfolios_FX_Slope       = Portfolios_FX;
Portfolios_Infl_Slope     = Portfolios_Infl;
Portfolios_Ratings_Slope        = Portfolios_Ratings;
Portfolios_RatingsOut_Slope     = Portfolios_RatingsOut;

    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%bootstrapping
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Portfolios_HPER_logdiff               =(Portfolios_HPER(:,Number_portfolios))-(Portfolios_HPER(:,1));
Portfolios_HPERUSD_logdiff            =(Portfolios_HPERUSD(:,Number_portfolios))-(Portfolios_HPERUSD(:,1));
Portfolios_HPRUSDminusUSHPR_logdiff   =(Portfolios_HPRUSDminusUSHPR_LongBond(:,Number_portfolios))-(Portfolios_HPRUSDminusUSHPR_LongBond(:,1));
Portfolios_FXER_logdiff               =(Portfolios_FXER(:,Number_portfolios))-(Portfolios_FXER(:,1));
Portfolios_FX_logdiff                 =(Portfolios_FX(:,Number_portfolios))-(Portfolios_FX(:,1));
Portfolios_FWD_logdiff                =(Portfolios_FWD(:,Number_portfolios))-(Portfolios_FWD(:,1));
Portfolios_Infl_logdiff               =Portfolios_Infl(:,Number_portfolios) - Portfolios_Infl(:,1);
Portfolios_Ratings_logdiff            =Portfolios_Ratings(:,Number_portfolios) - Portfolios_Ratings(:,1);
Portfolios_RatingsOut_logdiff         =Portfolios_RatingsOut(:,Number_portfolios) - Portfolios_RatingsOut(:,1);

portfolio_data                                    = [(Portfolios_FXER) Portfolios_FXER_logdiff (Portfolios_HPER) Portfolios_HPER_logdiff (Portfolios_HPERUSD) Portfolios_HPERUSD_logdiff  Portfolios_HPRUSDminusUSHPR_LongBond Portfolios_HPRUSDminusUSHPR_logdiff Portfolios_FX Portfolios_FX_logdiff Portfolios_FWD Portfolios_FWD_logdiff Portfolios_Infl Portfolios_Infl_logdiff Portfolios_Ratings Portfolios_Ratings_logdiff Portfolios_RatingsOut Portfolios_RatingsOut_logdiff];

[standard_error_mean,standard_error_SR]                               = bootstrap_portfolio_returns_growth(portfolio_data,nboot,horizon_bootstrap);

Total_portfolios=Number_portfolios+1;

Portfolios_FXER_se(1:Total_portfolios)                        = standard_error_mean(1:Total_portfolios);
Portfolios_HPER_se(1:Total_portfolios)                        = standard_error_mean(1+Total_portfolios:2*Total_portfolios);
Portfolios_HPERUSD_se(1:Total_portfolios)                     = standard_error_mean(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_HPRUSDminusUSHPR_se(1:Total_portfolios)            = standard_error_mean(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_se(1:Total_portfolios)                          = standard_error_mean(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_FWD_se(1:Total_portfolios)                         = standard_error_mean(1+5*Total_portfolios:6*Total_portfolios);
Portfolios_Infl_se(1:Total_portfolios)                        = standard_error_mean(1+6*Total_portfolios:7*Total_portfolios);
Portfolios_Ratings_se(1:Total_portfolios)                     = standard_error_mean(1+7*Total_portfolios:8*Total_portfolios);
Portfolios_RatingsOut_se(1:Total_portfolios)                  = standard_error_mean(1+8*Total_portfolios:9*Total_portfolios);

Portfolios_FXER_SR_se(1:Total_portfolios)                     = standard_error_SR(1:Total_portfolios);
Portfolios_HPER_SR_se(1:Total_portfolios)                     = standard_error_SR(1+Total_portfolios:2*Total_portfolios);
Portfolios_HPERUSD_SR_se(1:Total_portfolios)                  = standard_error_SR(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_HPRUSDminusUSHPR_SR_se(1:Total_portfolios)         = standard_error_SR(1+3*Total_portfolios:4*Total_portfolios);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Portfolios_HPRUSDminusUSHPR                                  = Portfolios_HPRUSDminusUSHPR_LongBond;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


Table = [100*nanmean((Portfolios_FXER))*ScalingFactor                100*nanmean((Portfolios_FXER_logdiff))*ScalingFactor; ...
         100*Portfolios_FXER_se*ScalingFactor; ...
         100*nanstd((Portfolios_FXER))*sqrt(ScalingFactor)           100*nanstd((Portfolios_FXER_logdiff))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_FXER))*sqrt(ScalingFactor)./nanstd((Portfolios_FXER))        nanmean((Portfolios_FXER_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_FXER_logdiff)); ...
         Portfolios_FXER_SR_se*sqrt(ScalingFactor); ...
         100*nanmean((Portfolios_HPER))*ScalingFactor                100*nanmean((Portfolios_HPER_logdiff))*ScalingFactor; ...                     
         100*Portfolios_HPER_se*ScalingFactor; ...
         100*nanstd((Portfolios_HPER))*sqrt(ScalingFactor)           100*nanstd((Portfolios_HPER_logdiff))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_HPER))*sqrt(ScalingFactor)./nanstd((Portfolios_HPER))        nanmean((Portfolios_HPER_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_HPER_logdiff)); ...
         Portfolios_HPER_SR_se*sqrt(ScalingFactor); ...
         100*nanmean((Portfolios_HPERUSD))*ScalingFactor             100*nanmean((Portfolios_HPERUSD_logdiff))*ScalingFactor;  ...
         100*Portfolios_HPERUSD_se*ScalingFactor; ...             
         100*nanstd((Portfolios_HPERUSD))*sqrt(ScalingFactor)        100*nanstd((Portfolios_HPERUSD_logdiff))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_HPERUSD))*sqrt(ScalingFactor)./nanstd((Portfolios_HPERUSD))  nanmean((Portfolios_HPERUSD_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_HPERUSD_logdiff)); ...
         Portfolios_HPERUSD_SR_se*sqrt(ScalingFactor); ...
         100*nanmean((Portfolios_HPRUSDminusUSHPR))*ScalingFactor    100*nanmean((Portfolios_HPRUSDminusUSHPR_logdiff))*ScalingFactor; ...         
         100*Portfolios_HPRUSDminusUSHPR_se*ScalingFactor; ...
         100*nanstd((Portfolios_HPRUSDminusUSHPR))*sqrt(ScalingFactor)                    100*nanstd((Portfolios_HPRUSDminusUSHPR_logdiff))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_HPRUSDminusUSHPR))*sqrt(ScalingFactor)./nanstd((Portfolios_HPRUSDminusUSHPR))       nanmean((Portfolios_HPRUSDminusUSHPR_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_HPRUSDminusUSHPR_logdiff));
         Portfolios_HPRUSDminusUSHPR_SR_se*sqrt(ScalingFactor); ...   
         100*nanmean((Portfolios_FX))*ScalingFactor                  100*nanmean((Portfolios_FX_logdiff))*ScalingFactor; ...
         100*nanmean((Portfolios_FWD))*ScalingFactor                 100*nanmean((Portfolios_FWD_logdiff))*ScalingFactor; ...
         100*nanmean((Portfolios_Infl))*ScalingFactor                100*nanmean((Portfolios_Infl_logdiff))*ScalingFactor; ...
         100*Portfolios_Infl_se*ScalingFactor; ...
         nanmean(Portfolios_Ratings)                                 nanmean(Portfolios_Ratings_logdiff); ...
         Portfolios_Ratings_se; ...
         nanmean(Portfolios_RatingsOut)                              nanmean(Portfolios_RatingsOut_logdiff); ...
         Portfolios_RatingsOut_se];

rows = {'Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','Mean', 'Mean','Mean','s.e.', 'Mean', 's.e.', 'Mean', 's.e.'}; 
Opts.Style = 'jf'; 
disp(' ');
disp('----------------------------------------------');
disp('Countries Sorted By Slopes');
disp('----------------------------------------------'); 
disp('Currency Risk Premium / Local Term Premium / Dollar Term Premium / Local (in USD) minus US HPR');
mat2tex(Table,'%6.2f',rows,[], [], Opts)

xlswrite(strcat(newpath,'portfolio_composition_BB_sample_',int2str(source),'_horizon_',int2str(horizon),'_maturity_',int2str(maturity),'_2016.xls'),Table, 'Slope Table', 'C3');
xlswrite(strcat(newpath,'portfolio_composition_BB_sample_',int2str(source),'_horizon_',int2str(horizon),'_maturity_',int2str(maturity),'_2016.xls'),rows', 'Slope Table', 'B3');


% ============================================= %
% Time-Series Predictability Using Interest Rate Levels
% ============================================= %
TSPred_BetasDiff3_con   = NaN*zeros(Nc,1);
TSPred_BetasDiff3_con_se= NaN*zeros(Nc,1);
TSPred_BetasDiff3       = NaN*zeros(Nc,1);
TSPred_BetasDiff3_se    = NaN*zeros(Nc,1);
TSPred_BetasDiff4_con   = NaN*zeros(Nc,1);
TSPred_BetasDiff4_con_se= NaN*zeros(Nc,1);
TSPred_BetasDiff4       = NaN*zeros(Nc,1);
TSPred_BetasDiff4_se    = NaN*zeros(Nc,1);
TSPred_BetasDiff5_con   = NaN*zeros(Nc,1);
TSPred_BetasDiff5_con_se= NaN*zeros(Nc,1);
TSPred_BetasDiff5       = NaN*zeros(Nc,1);
TSPred_BetasDiff5_se    = NaN*zeros(Nc,1);
TSPred_BetasDiff3_R2adj = NaN*zeros(Nc,1);
TSPred_BetasDiff3_Nobs  = NaN*zeros(Nc,1);
TSPred_BetasDiff4_R2adj = NaN*zeros(Nc,1);
TSPred_BetasDiff4_Nobs  = NaN*zeros(Nc,1);
TSPred_BetasDiff5_R2adj = NaN*zeros(Nc,1);
TSPred_BetasDiff5_Nobs  = NaN*zeros(Nc,1);

for n=1:Nc
    
    
  if NW_lag_choice == 1
        lags = 6;  
  elseif NW_lag_choice == 2
        lags = ceil(4*(size(HPERUSD_LongBond,1)/100)^(2/9))-1;
  elseif NW_lag_choice == 3
        lags = ceil(0.75*(size(HPERUSD_LongBond,1))^(1/3))-1;
  elseif NW_lag_choice == 4
        lags = max(ceil(4*(size(HPERUSD_LongBond,1)/100)^(2/9))-1,ceil(0.75*(size(HPERUSD_LongBond,1))^(1/3))-1);
  elseif NW_lag_choice == 5
        lags = ceil(1.3*(size(HPERUSD_LongBond,1))^(1/2))-1;
  end
    
    
    
    lhv = (HPERUSD_LongBond(:,1+n)-HPERUSD_LongBond(:,end))*ScalingFactor;
    rhv = [ones(size(lhv,1),1) ZC_3M_lag(:,1+n)-ZC_3M_lag(:,end)];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        TSPred_BetasDiff3_con(n,1)     = bv(1);
        TSPred_BetasDiff3_con_se(n,1)  = sebv(1);
        TSPred_BetasDiff3(n,1)         = bv(2);
        TSPred_BetasDiff3_se(n,1)      = sebv(2);
        TSPred_BetasDiff3_R2adj(n,1)   = R2vadj;
        TSPred_BetasDiff3_Nobs(n,1)    = size(lhv_s,1);
        TSPred_BetasDiff3_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasDiff3_con(n,1)/TSPred_BetasDiff3_con_se(n,1),(lags+1)/TSPred_BetasDiff3_Nobs(n,1));                 
        TSPred_BetasDiff3_pval_b(n,1)     = tstat_pval_b(TSPred_BetasDiff3(n,1)/TSPred_BetasDiff3_se(n,1),(lags+1)/TSPred_BetasDiff3_Nobs(n,1));   
    end;
      
    
    lhv = FXER(:,1+n)*ScalingFactor;
    rhv = [ones(size(lhv,1),1) ZC_3M_lag(:,1+n)-ZC_3M_lag(:,end)];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        TSPred_BetasDiff4_con(n,1)     = bv(1);
        TSPred_BetasDiff4_con_se(n,1)  = sebv(1);
        TSPred_BetasDiff4(n,1)         = bv(2);
        TSPred_BetasDiff4_se(n,1)      = sebv(2);
        TSPred_BetasDiff4_R2adj(n,1)   = R2vadj;
        TSPred_BetasDiff4_Nobs(n,1)    = size(lhv_s,1);
        TSPred_BetasDiff4_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasDiff4_con(n,1)/TSPred_BetasDiff4_con_se(n,1),(lags+1)/TSPred_BetasDiff4_Nobs(n,1));                 
        TSPred_BetasDiff4_pval_b(n,1)     = tstat_pval_b(TSPred_BetasDiff4(n,1)/TSPred_BetasDiff4_se(n,1),(lags+1)/TSPred_BetasDiff4_Nobs(n,1));  
    end;
    
    
    lhv = (HPER_LongBond(:,1+n)-HPER_LongBond(:,end))*ScalingFactor;
    rhv = [ones(size(lhv,1),1) ZC_3M_lag(:,1+n)-ZC_3M_lag(:,end)];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        TSPred_BetasDiff5_con(n,1)     = bv(1);
        TSPred_BetasDiff5_con_se(n,1)  = sebv(1);
        TSPred_BetasDiff5(n,1)         = bv(2);
        TSPred_BetasDiff5_se(n,1)      = sebv(2);
        TSPred_BetasDiff5_R2adj(n,1)   = R2vadj;
        TSPred_BetasDiff5_Nobs(n,1)    = size(lhv_s,1);
        TSPred_BetasDiff5_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasDiff5_con(n,1)/TSPred_BetasDiff5_con_se(n,1),(lags+1)/TSPred_BetasDiff5_Nobs(n,1));                 
        TSPred_BetasDiff5_pval_b(n,1)     = tstat_pval_b(TSPred_BetasDiff5(n,1)/TSPred_BetasDiff5_se(n,1),(lags+1)/TSPred_BetasDiff5_Nobs(n,1));  
    end;
    
    
        
end;

for n = 1:Nc-1
    
    HPERUSD_LongBond_Diff(:,n)       = HPERUSD_LongBond(:,1+n)-HPERUSD_LongBond(:,end);
    HPER_LongBond_Diff(:,n)          = HPER_LongBond(:,1+n)-HPER_LongBond(:,end);
    FXER_Panel(:,n)                  = FXER(:,1+n);
    ZC_3M_lag_Diff(:,n)              = ZC_3M_lag(:,1+n)-ZC_3M_lag(:,end);
    ZC_10Yminus3M_lag_Diff(:,n)      = ZC_10Yminus3M_lag(:,1+n)-ZC_10Yminus3M_lag(:,end);
    Panel_Unit(:,n)                  = n*ones(size(HPRUSD_LongBond,1),1);
    Panel_Time(:,n)                  = [1:size(HPRUSD_LongBond,1)]';
    
end

% Panel regressions with Driscoll-Kraay s.e.

Panel_Var = [HPERUSD_LongBond_Diff*ScalingFactor, ZC_3M_lag_Diff];
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se]   = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasDiff3_con(Nc+2,1)           = NaN;
TSPred_BetasDiff3_con_se(Nc+2,1)        = NaN; 
TSPred_BetasDiff3(Nc+2,1)               = theta(1,1);
TSPred_BetasDiff3_se(Nc+2,1)            = theta_se(1,1);
TSPred_BetasDiff3_Nobs(Nc+2,1)          = sum(sum(~isnan(yhat)));
TSPred_BetasDiff3_R2adj(Nc+2,1)         = 1-(1-Rsq)*(TSPred_BetasDiff3_Nobs(Nc+2,1)-1)/(TSPred_BetasDiff3_Nobs(Nc+2,1)-size(X_DK,3));
TSPred_BetasDiff3_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasDiff3_con(Nc+2,1)/TSPred_BetasDiff3_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasDiff3_pval_b(Nc+2,1)        = tstat_pval_b(TSPred_BetasDiff3(Nc+2,1)/TSPred_BetasDiff3_se(Nc+2,1), (lags+1)/size(y_DK,1));                 



Panel_Var = [FXER_Panel*ScalingFactor, ZC_3M_lag_Diff];
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasDiff4_con(Nc+2,1)         = NaN;
TSPred_BetasDiff4_con_se(Nc+2,1)      = NaN; 
TSPred_BetasDiff4(Nc+2,1)             = theta(1,1);
TSPred_BetasDiff4_se(Nc+2,1)          = theta_se(1,1);
TSPred_BetasDiff4_Nobs(Nc+2,1)        = sum(sum(~isnan(yhat)));
TSPred_BetasDiff4_R2adj(Nc+2,1)       = 1-(1-Rsq)*(TSPred_BetasDiff4_Nobs(Nc+2,1)-1)/(TSPred_BetasDiff4_Nobs(Nc+2,1)-size(X_DK,3));
TSPred_BetasDiff4_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasDiff4_con(Nc+2,1)/TSPred_BetasDiff4_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasDiff4_pval_b(Nc+2,1)        = tstat_pval_b(TSPred_BetasDiff4(Nc+2,1)/TSPred_BetasDiff4_se(Nc+2,1), (lags+1)/size(y_DK,1));                 



Panel_Var = [HPER_LongBond_Diff*ScalingFactor, ZC_3M_lag_Diff];
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasDiff5_con(Nc+2,1)         = NaN;
TSPred_BetasDiff5_con_se(Nc+2,1)      = NaN;
TSPred_BetasDiff5(Nc+2,1)             = theta(1,1);
TSPred_BetasDiff5_se(Nc+2,1)          = theta_se(1,1);
TSPred_BetasDiff5_Nobs(Nc+2,1)        = sum(sum(~isnan(yhat)));
TSPred_BetasDiff5_R2adj(Nc+2,1)       = 1-(1-Rsq)*(TSPred_BetasDiff5_Nobs(Nc+2,1)-1)/(TSPred_BetasDiff5_Nobs(Nc+2,1)-size(X_DK,3));
TSPred_BetasDiff5_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasDiff5_con(Nc+2,1)/TSPred_BetasDiff5_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasDiff5_pval_b(Nc+2,1)        = tstat_pval_b(TSPred_BetasDiff5(Nc+2,1)/TSPred_BetasDiff5_se(Nc+2,1), (lags+1)/size(y_DK,1));                 


% Joint estimation

T          = size(HPERUSD_LongBond,1);
R1_con     = zeros(Nc-1,6*(Nc-1));
R1_slope   = zeros(Nc-1,6*(Nc-1));
R1_all     = zeros(2*(Nc-1),6*(Nc-1));
R2_con     = zeros(Nc-1,6*(Nc-1));
R2_slope   = zeros(Nc-1,6*(Nc-1));
R2_all     = zeros(2*(Nc-1),6*(Nc-1));
R3_con     = zeros(Nc-1,6*(Nc-1));
R3_slope   = zeros(Nc-1,6*(Nc-1));
R3_all     = zeros(2*(Nc-1),6*(Nc-1));
R_diff     = zeros(Nc-1,6*(Nc-1));
r          = zeros(Nc-1,1);
r_all      = zeros(2*(Nc-1),1);
rhv        = zeros(3*T*(Nc-1),6*(Nc-1));
for n=1:Nc-1
lhv((n-1)*T+1:n*T,1)                                                     = (HPERUSD_LongBond(:,1+n)-HPERUSD_LongBond(:,end))*ScalingFactor;
rhv((n-1)*T+1:n*T,(n-1)*2+1:n*2)                                         = [ones(T,1) ZC_3M_lag(:,1+n)-ZC_3M_lag(:,end)];
lhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,1)                                   = FXER(:,1+n)*ScalingFactor;
rhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,2*(Nc-1)+(n-1)*2+1:2*(Nc-1)+n*2)     = [ones(T,1) ZC_3M_lag(:,1+n)-ZC_3M_lag(:,end)];
lhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,1)                               = (HPER_LongBond(:,1+n)-HPER_LongBond(:,end))*ScalingFactor;
rhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,4*(Nc-1)+(n-1)*2+1:4*(Nc-1)+n*2) = [ones(T,1) ZC_3M_lag(:,1+n)-ZC_3M_lag(:,end)];
R1_con(n,2*n-1)          =  1;
R1_slope(n,2*n)          =  1;
R1_all(n,2*n-1)          =  1;
R1_all(Nc-1+n,2*n)       =  1;
R2_con(n,2*(Nc-1)+2*n-1) =  1;
R2_slope(n,2*(Nc-1)+2*n) =  1;
R2_all(n,2*(Nc-1)+2*n-1) =  1;
R2_all(Nc-1+n,2*(Nc-1)+2*n) =  1;
R3_con(n,4*(Nc-1)+2*n-1) =  1;
R3_slope(n,4*(Nc-1)+2*n) =  1;
R3_all(n,4*(Nc-1)+2*n-1) =  1;
R3_all(Nc-1+n,4*(Nc-1)+2*n) =  1;
R_diff(n,2*n)            =  1;
R_diff(n,2*(Nc-1)+2*n)   = -1;
end
[lhv_s,rhv_s]            = FindCommonSample(lhv,rhv);
[bv,sebv,R2v,R2vadj,v]   = olsgmm(lhv_s,rhv_s,lags,1);
W_Diff3_con(1,1)         = (R1_con*bv - r)'*inv(R1_con*v*R1_con')*(R1_con*bv - r);          % test if all constants are zero in the HPERDollar regressions
Wp_Diff3_con(1,1)        = 1-chi2cdf(W_Diff3_con(1,1),Nc-1);
W_Diff3_slope(1,1)       = (R1_slope*bv - r)'*inv(R1_slope*v*R1_slope')*(R1_slope*bv - r);  % test if all slopes are zero in the HPERDollar regressions
Wp_Diff3_slope(1,1)      = 1-chi2cdf(W_Diff3_slope(1,1),Nc-1);
W_Diff3_all(1,1)         = (R1_all*bv - r_all)'*inv(R1_all*v*R1_all')*(R1_all*bv - r_all);  % test if all constants and slopes are zero in the HPERDollar regressions
Wp_Diff3_all(1,1)        = 1-chi2cdf(W_Diff3_all(1,1),2*(Nc-1));
W_Diff4_con(1,1)         = (R2_con*bv - r)'*inv(R2_con*v*R2_con')*(R2_con*bv - r);          % test if all constants are zero in the FXEXR regressions
Wp_Diff4_con(1,1)        = 1-chi2cdf(W_Diff4_con(1,1),Nc-1);
W_Diff4_slope(1,1)       = (R2_slope*bv - r)'*inv(R2_slope*v*R2_slope')*(R2_slope*bv - r);  % test if all slopes are zero in the FXEXR regressions
Wp_Diff4_slope(1,1)      = 1-chi2cdf(W_Diff4_slope(1,1),Nc-1);
W_Diff4_all(1,1)         = (R2_all*bv - r_all)'*inv(R2_all*v*R2_all')*(R2_all*bv - r_all);  % test if all constants and slopes are zero in the FXEXR regressions
Wp_Diff4_all(1,1)        = 1-chi2cdf(W_Diff4_all(1,1),2*(Nc-1));
W_Diff5_con(1,1)         = (R3_con*bv - r)'*inv(R3_con*v*R3_con')*(R3_con*bv - r);          % test if all constants are zero in the HPER regressions
Wp_Diff5_con(1,1)        = 1-chi2cdf(W_Diff5_con(1,1),Nc-1);
W_Diff5_slope(1,1)       = (R3_slope*bv - r)'*inv(R3_slope*v*R3_slope')*(R3_slope*bv - r);  % test if all slopes are zero in the HPER regressions
Wp_Diff5_slope(1,1)      = 1-chi2cdf(W_Diff5_slope(1,1),Nc-1);
W_Diff5_all(1,1)         = (R3_all*bv - r_all)'*inv(R3_all*v*R3_all')*(R3_all*bv - r_all);  % test if all constants and slopes are zero in the HPER regressions
Wp_Diff5_all(1,1)        = 1-chi2cdf(W_Diff5_all(1,1),2*(Nc-1));
W_Diff_TermStr(Nc+2,1)   = (R_diff*bv - r)'*inv(R_diff*v*R_diff')*(R_diff*bv - r);          % test if all HPERDollar/FXEXR slope differentials are zero
Wp_Diff_TermStr(Nc+2,1)  = 1-chi2cdf(W_Diff_TermStr(Nc+2,1),Nc-1);

Wp_Diff3_con_b(1,1)        = chisq_pval_b(W_Diff3_con(1,1),Nc-1,(lags+1)/T);
Wp_Diff3_slope_b(1,1)      = chisq_pval_b(W_Diff3_slope(1,1),Nc-1,(lags+1)/T);
Wp_Diff3_all_b(1,1)        = chisq_pval_b(W_Diff3_all(1,1),2*(Nc-1),(lags+1)/T);
Wp_Diff4_con_b(1,1)        = chisq_pval_b(W_Diff4_con(1,1),Nc-1,(lags+1)/T);
Wp_Diff4_slope_b(1,1)      = chisq_pval_b(W_Diff4_slope(1,1),Nc-1,(lags+1)/T);
Wp_Diff4_all_b(1,1)        = chisq_pval_b(W_Diff4_all(1,1),2*(Nc-1),(lags+1)/T);
Wp_Diff5_con_b(1,1)        = chisq_pval_b(W_Diff5_con(1,1),Nc-1,(lags+1)/T);
Wp_Diff5_slope_b(1,1)      = chisq_pval_b(W_Diff5_slope(1,1),Nc-1,(lags+1)/T);
Wp_Diff5_all_b(1,1)        = chisq_pval_b(W_Diff5_all(1,1),2*(Nc-1),(lags+1)/T); 
Wp_Diff_TermStr_b(Nc+2,1)  = chisq_pval_b(W_Diff_TermStr(Nc+2,1),Nc-1,(lags+1)/T);


% Panel regression

lhv_panel = [];
rhv_panel = zeros(2*T*(Nc-1),2*Nc);
R_slope = [zeros(1,Nc-1) 1 zeros(1,Nc-1) -1];
R_con_1 = zeros(Nc-1,2*Nc);
R_con_2 = zeros(Nc-1,2*Nc);
r = 0;
r_con = zeros(Nc-1,1);
for n = 1:Nc-1
T = size(HPERUSD_LongBond,1);
R = [0 1 0 -1];
r = 0;
lhv  = [(HPERUSD_LongBond(:,1+n)-HPERUSD_LongBond(:,end))*ScalingFactor;  FXER(:,1+n)*ScalingFactor];
rhv  = zeros(2*T,4);
rhv(1:T,1:2)     = [ones(T,1) ZC_3M_lag(:,1+n)-ZC_3M_lag(:,end)];
rhv(T+1:2*T,3:4) = [ones(T,1) ZC_3M_lag(:,1+n)-ZC_3M_lag(:,end)];
[lhv_s,rhv_s]                        = FindCommonSample(lhv,rhv);
[bv,sebv,R2v,R2vadj,v]               = olsgmm(lhv_s,rhv_s,lags,1);
W_Diff_TermStr(n,1)  = (R*bv - r)'*inv(R*v*R')*(R*bv - r);  % test if long-term and short-term bond slopes are equal
Wp_Diff_TermStr(n,1) = 1-chi2cdf(W_Diff_TermStr(n,1),1);
Wp_Diff_TermStr_b(n,1)  = chisq_pval_b(W_Diff_TermStr(n,1),1,(lags+1)/T);


lhv_panel = [lhv_panel; lhv];
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,n) = ones(T,1);
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,Nc)= ZC_3M_lag(:,1+n)-ZC_3M_lag(:,end);
rhv_panel((n-1)*2*T+T+1:n*2*T,Nc+n)  = ones(T,1);
rhv_panel((n-1)*2*T+T+1:n*2*T,2*Nc)  = ZC_3M_lag(:,1+n)-ZC_3M_lag(:,end);

lhv_panel_DK(:,n)        = lhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:);
lhv_panel_DK(:,Nc-1+n)   = lhv_panel((n-1)*2*T+T+1:n*2*T,:);
rhv_panel_DK(:,n,:)      = reshape(rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:),T,[]);
rhv_panel_DK(:,Nc-1+n,:) = reshape(rhv_panel((n-1)*2*T+T+1:n*2*T,:),T,[]);

R_con_1(n,n)    = 1;
R_con_2(n,Nc+n) = 1;
end

[bv, ~, ~,~,~,~,~,v,bv_se]    = HszDk5dwPs(lhv_panel_DK,ones(T,1),rhv_panel_DK,1,lags);
W_Diff_TermStr(Nc+1,1)  = (R_slope*bv - r)'*inv(R_slope*v*R_slope')*(R_slope*bv - r);            % test if long-term and short-term bond slopes are equal in the panel regression
Wp_Diff_TermStr(Nc+1,1) = 1-chi2cdf(W_Diff_TermStr(Nc+1,1),1);
W_Diff3_con(2,1)        = (R_con_1*bv - r_con)'*inv(R_con_1*v*R_con_1')*(R_con_1*bv - r_con);    % test if all USD bond return constants are zero in the panel regression
Wp_Diff3_con(2,1)       = 1-chi2cdf(W_Diff3_con(2,1),Nc-1);
W_Diff4_con(2,1)        = (R_con_2*bv - r_con)'*inv(R_con_2*v*R_con_2')*(R_con_2*bv - r_con);    % test if all FXEXR constants are zero in the panel regression
Wp_Diff4_con(2,1)       = 1-chi2cdf(W_Diff4_con(2,1),Nc-1);

Wp_Diff_TermStr_b(Nc+1,1) = chisq_pval_b(W_Diff_TermStr(Nc+1,1),1,(lags+1)/T); 
Wp_Diff3_con_b(2,1)       = chisq_pval_b(W_Diff3_con(2,1),Nc-1,(lags+1)/T); 
Wp_Diff4_con_b(2,1)       = chisq_pval_b(W_Diff4_con(2,1),Nc-1,(lags+1)/T); 


% Tables

rows  = Countries_Sample';
rows{1,Nc+1} = '--'; 
rows{1,Nc+2} = 'Panel (DK)';
rows{1,Nc+3} = 'Joint zero (p-value)';

Table_IR_Collected_Simple = [TSPred_BetasDiff3_con TSPred_BetasDiff3_con_se ...
                         TSPred_BetasDiff3 TSPred_BetasDiff3_se 100*TSPred_BetasDiff3_R2adj...
                         TSPred_BetasDiff4_con TSPred_BetasDiff4_con_se ...
                         TSPred_BetasDiff4 TSPred_BetasDiff4_se 100*TSPred_BetasDiff4_R2adj...
                         TSPred_BetasDiff5_con TSPred_BetasDiff5_con_se...
                         TSPred_BetasDiff5 TSPred_BetasDiff5_se 100*TSPred_BetasDiff5_R2adj...
                         [Wp_Diff_TermStr(1:Nc+1,1); Wp_Diff_TermStr(Nc+1,1)] TSPred_BetasDiff3_Nobs];       
Table_IR_Collected_Simple = [Table_IR_Collected_Simple; Wp_Diff3_con(1,1) NaN Wp_Diff3_slope(1,1) NaN NaN Wp_Diff4_con(1,1) NaN Wp_Diff4_slope(1,1) NaN NaN Wp_Diff5_con(1,1) NaN Wp_Diff5_slope(1,1) NaN NaN Wp_Diff_TermStr(Nc+2,1) NaN];
cols  = {'coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','p-value','Obs.'}; 
disp(' ');
disp('-----------------------------------------');
disp('Collected Summary Results of Time-Series Regressions on Interest Rate Differentials');
disp('-----------------------------------------');
mat2tex(Table_IR_Collected_Simple,'%6.2f',rows,cols, [], Opts)

% Build Portfolios -- Sorted By Short-Term Interest Rates
                         
% Use variables without their first column of dates
FX_temp                        = FX(:,2:end-1);  
FWD_temp                       = FWD(:,2:end-1);  
Infl_temp                      = Infl(:,2:end-1);
Ratings_lag_temp               = Ratings_lag(:,2:end-1); 
RatingsOut_lag_temp            = RatingsOut_lag(:,2:end-1);

FXER_temp                      = FXER(:,2:end-1);
HPR_LongBond_temp              = HPR_LongBond(:,2:end-1);
HPER_LongBond_temp             = HPER_LongBond(:,2:end-1);
HPRUSD_LongBond_temp           = HPRUSD_LongBond(:,2:end-1);
HPERUSD_LongBond_temp          = HPERUSD_LongBond(:,2:end-1); 
HPRUSDminusUSHPR_LongBond_temp = HPRUSDminusUSHPR_LongBond(:,2:end); 
ZC_3M_lag_temp                 = ZC_3M_lag(:,2:end-1);
ZC_3M_lag_Rolling_Mean_temp    = ZC_3M_lag_Rolling_Mean(:,2:end-1);


% Get rid of interest rates if no data on currency or bond excess returns and vice-versa
for t=1:size(ZC_3M_lag_temp,1)                                               % For each date
    for c=1:size(ZC_3M_lag_temp,2)                                           % For each country
       if  isnan(ZC_3M_lag_temp(t,c)*FXER_temp(t,c)*HPER_LongBond_temp(t,c)*HPRUSD_LongBond_temp(t,c)*HPERUSD_LongBond_temp(t,c))==1;
           ZC_3M_lag_temp(t,c)        = NaN;
           FXER_temp(t,c)             = NaN;
           FX_temp(t,c)               = NaN;
           FWD_temp(t,c)              = NaN;
           HPER_LongBond_temp(t,c)    = NaN;
           HPRUSD_LongBond_temp(t,c)  = NaN;
           HPERUSD_LongBond_temp(t,c) = NaN;
       end         
    end;
end;

% Sign

sign_mat_IR = sign(ZC_3M_lag_temp-repmat(ZC_3M_lag(:,end),1,Nc-1));

FXER_temp_signed                         = (FXER_temp-repmat(FXER(:,end),1,Nc-1)).*sign_mat_IR; 
HPR_LongBond_temp_signed                 = (HPR_LongBond_temp-repmat(HPR_LongBond(:,end),1,Nc-1)).*sign_mat_IR; 
HPER_LongBond_temp_signed                = (HPER_LongBond_temp-repmat(HPER_LongBond(:,end),1,Nc-1)).*sign_mat_IR; 
HPRUSD_LongBond_temp_signed              = (HPRUSD_LongBond_temp-repmat(HPRUSD_LongBond(:,end),1,Nc-1)).*sign_mat_IR; 
HPERUSD_LongBond_temp_signed             = (HPERUSD_LongBond_temp-repmat(HPERUSD_LongBond(:,end),1,Nc-1)).*sign_mat_IR; 
HPRUSDminusUSHPR_LongBond_temp_signed    = (HPRUSDminusUSHPR_LongBond_temp).*sign_mat_IR; 
FX_temp_signed                           = (FX_temp-repmat(FX(:,end),1,Nc-1)).*sign_mat_IR; 
FWD_temp_signed                          = (FWD_temp-repmat(FWD(:,end),1,Nc-1)).*sign_mat_IR; 
Infl_temp_signed                         = (Infl_temp-repmat(Infl(:,end),1,Nc-1)).*sign_mat_IR; 
Ratings_lag_temp_signed                  = (Ratings_lag_temp-repmat(Ratings_lag(:,end),1,Nc-1)).*sign_mat_IR; 
RatingsOut_lag_temp_signed               = (RatingsOut_lag_temp-repmat(RatingsOut_lag(:,end),1,Nc-1)).*sign_mat_IR; 

for n=1:Nc-1


[Portfolios_FXER(:,n),Portfolios_HPER(:,n),Portfolios_HPRUSDminusUSHPR_LongBond(:,n),Portfolios_HPERUSD(:,n),Portfolios_VarSort_IR_signed(:,n),Track_portfolios_IR1_signed(:,n)] = ...
    Build_portfolios_for4variables(FXER_temp_signed(:,n),HPER_LongBond_temp_signed(:,n),HPRUSDminusUSHPR_LongBond_temp_signed(:,n),HPERUSD_LongBond_temp_signed(:,n),ZC_3M_lag_temp(:,n),1,1,ZC_3M_lag_Rolling_Mean_temp(:,n));
[Portfolios_FXER(:,n),Portfolios_HPR(:,n),Portfolios_Infl(:,n),Portfolios_HPRUSD(:,n),~,Track_portfolios_IR2_signed(:,n)] = ...
    Build_portfolios_for4variables(FXER_temp_signed(:,n),HPR_LongBond_temp_signed(:,n),Infl_temp_signed(:,n),HPRUSD_LongBond_temp_signed(:,n),ZC_3M_lag_temp(:,n),1,1,ZC_3M_lag_Rolling_Mean_temp(:,n));
[Portfolios_FX(:,n),Portfolios_FWD(:,n),Portfolios_Ratings(:,n),Portfolios_RatingsOut(:,n), ~,Track_portfolios_IR3_signed(:,n)] = ...
    Build_portfolios_for4variables(FX_temp_signed(:,n),FWD_temp_signed(:,n),Ratings_lag_temp_signed(:,n),RatingsOut_lag_temp_signed(:,n),ZC_3M_lag_temp(:,n),1,1,ZC_3M_lag_Rolling_Mean_temp(:,n));

end

[Portfolios_FXER(:,Nc+1),Portfolios_HPER(:,Nc+1),Portfolios_HPRUSDminusUSHPR_LongBond(:,Nc+1),Portfolios_HPERUSD(:,Nc+1),Portfolios_VarSort_IR_signed(:,Nc+1),Track_portfolios_IR1_signed] = ...
    Build_portfolios_for4variables(FXER_temp_signed,HPER_LongBond_temp_signed,HPRUSDminusUSHPR_LongBond_temp_signed,HPERUSD_LongBond_temp_signed,ZC_3M_lag_temp,1,1,ZC_3M_lag_Rolling_Mean_temp);
[Portfolios_FXER(:,Nc+1),Portfolios_HPR(:,Nc+1),Portfolios_Infl(:,Nc+1),Portfolios_HPRUSD(:,Nc+1),~,Track_portfolios_IR2_signed] = ...
    Build_portfolios_for4variables(FXER_temp_signed,HPR_LongBond_temp_signed,Infl_temp_signed,HPRUSD_LongBond_temp_signed,ZC_3M_lag_temp,1,1,ZC_3M_lag_Rolling_Mean_temp);
[Portfolios_FX(:,Nc+1),Portfolios_FWD(:,Nc+1),Portfolios_Ratings(:,Nc+1),Portfolios_RatingsOut(:,Nc+1), ~,Track_portfolios_IR3_signed] = ...
    Build_portfolios_for4variables(FX_temp_signed,FWD_temp_signed,Ratings_lag_temp_signed,RatingsOut_lag_temp_signed,ZC_3M_lag_temp,1,1,ZC_3M_lag_Rolling_Mean_temp);

Portfolios_FXER_IR_signed     = Portfolios_FXER;
Portfolios_HPR_IR_signed      = Portfolios_HPR;
Portfolios_HPER_IR_signed     = Portfolios_HPER;
Portfolios_HPRUSD_IR_signed   = Portfolios_HPRUSD;
Portfolios_HPERUSD_IR_signed  = Portfolios_HPERUSD;
Portfolios_FX_IR_signed       = Portfolios_FX;
Portfolios_Infl_IR_signed     = Portfolios_Infl;
Portfolios_Ratings_IR_signed        = Portfolios_Ratings;
Portfolios_RatingsOut_IR_signed     = Portfolios_RatingsOut;

ScalingFactor = 4;  
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%bootstrapping
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

portfolio_data                                    = [(Portfolios_FXER) (Portfolios_HPER) (Portfolios_HPERUSD) Portfolios_HPRUSDminusUSHPR_LongBond Portfolios_FX Portfolios_FWD Portfolios_Infl Portfolios_Ratings Portfolios_RatingsOut];

[standard_error_mean,standard_error_SR]           = bootstrap_portfolio_returns_growth(portfolio_data,nboot,horizon_bootstrap);

Total_portfolios=Nc+1;

Portfolios_FXER_se(1:Total_portfolios)                        = standard_error_mean(1:Total_portfolios);
Portfolios_HPER_se(1:Total_portfolios)                        = standard_error_mean(1+Total_portfolios:2*Total_portfolios);
Portfolios_HPERUSD_se(1:Total_portfolios)                     = standard_error_mean(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_HPRUSDminusUSHPR_se(1:Total_portfolios)            = standard_error_mean(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_se(1:Total_portfolios)                          = standard_error_mean(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_FWD_se(1:Total_portfolios)                         = standard_error_mean(1+5*Total_portfolios:6*Total_portfolios);
Portfolios_Infl_se(1:Total_portfolios)                        = standard_error_mean(1+6*Total_portfolios:7*Total_portfolios);
Portfolios_Ratings_se(1:Total_portfolios)                     = standard_error_mean(1+7*Total_portfolios:8*Total_portfolios);
Portfolios_RatingsOut_se(1:Total_portfolios)                  = standard_error_mean(1+8*Total_portfolios:9*Total_portfolios);

Portfolios_FXER_SR_se(1:Total_portfolios)                     = standard_error_SR(1:Total_portfolios);
Portfolios_HPER_SR_se(1:Total_portfolios)                     = standard_error_SR(1+Total_portfolios:2*Total_portfolios);
Portfolios_HPERUSD_SR_se(1:Total_portfolios)                  = standard_error_SR(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_HPRUSDminusUSHPR_SR_se(1:Total_portfolios)         = standard_error_SR(1+3*Total_portfolios:4*Total_portfolios);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Portfolios_HPRUSDminusUSHPR                                  = Portfolios_HPRUSDminusUSHPR_LongBond;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


Table_Ptf_IR_signed_Simple = [100*nanmean((Portfolios_HPERUSD))*ScalingFactor;  ...
         100*Portfolios_HPERUSD_se*ScalingFactor; ...             
         100*nanstd((Portfolios_HPERUSD))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_HPERUSD))*sqrt(ScalingFactor)./nanstd((Portfolios_HPERUSD)); ...
         Portfolios_HPERUSD_SR_se*sqrt(ScalingFactor); ...
         NaN(1,Nc+1); ...
         100*nanmean((Portfolios_FXER))*ScalingFactor; ...
         100*Portfolios_FXER_se*ScalingFactor; ...
         100*nanstd((Portfolios_FXER))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_FXER))*sqrt(ScalingFactor)./nanstd((Portfolios_FXER)); ...
         Portfolios_FXER_SR_se*sqrt(ScalingFactor); ...
         NaN(1,Nc+1); ...
         100*nanmean((Portfolios_HPER))*ScalingFactor; ...                     
         100*Portfolios_HPER_se*ScalingFactor; ...
         100*nanstd((Portfolios_HPER))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_HPER))*sqrt(ScalingFactor)./nanstd((Portfolios_HPER)); ...
         Portfolios_HPER_SR_se*sqrt(ScalingFactor)]';
rows = Countries_Sample';      
rows{1,Nc+1} = 'Equally-weighted';   
Opts.Style = 'jf'; 
disp(' ');
disp('----------------------------------------------');
disp('Long-Short Portfolio Sorted By Interest Rates');
disp('----------------------------------------------'); 
mat2tex(Table_Ptf_IR_signed_Simple,'%6.2f',rows,[], [], Opts)


% ============================================= %
% Time-Series Predictability Using Yield Curve Slopes
% ============================================= %
TSPred_BetasDiff3_con       = NaN*zeros(Nc,1);
TSPred_BetasDiff3_con_se    = NaN*zeros(Nc,1);
TSPred_BetasDiff3       = NaN*zeros(Nc,1);
TSPred_BetasDiff3_se    = NaN*zeros(Nc,1);
TSPred_BetasDiff4_con       = NaN*zeros(Nc,1);
TSPred_BetasDiff4_con_se    = NaN*zeros(Nc,1);
TSPred_BetasDiff4       = NaN*zeros(Nc,1);
TSPred_BetasDiff4_se    = NaN*zeros(Nc,1);
TSPred_BetasDiff5_con       = NaN*zeros(Nc,1);
TSPred_BetasDiff5_con_se    = NaN*zeros(Nc,1);
TSPred_BetasDiff5       = NaN*zeros(Nc,1);
TSPred_BetasDiff5_se    = NaN*zeros(Nc,1);
TSPred_BetasDiff3_R2adj = NaN*zeros(Nc,1);
TSPred_BetasDiff3_Nobs  = NaN*zeros(Nc,1);
TSPred_BetasDiff4_R2adj = NaN*zeros(Nc,1);
TSPred_BetasDiff4_Nobs  = NaN*zeros(Nc,1);
TSPred_BetasDiff5_R2adj = NaN*zeros(Nc,1);
TSPred_BetasDiff5_Nobs  = NaN*zeros(Nc,1);


for n=1:Nc
    
    
  if NW_lag_choice == 1
        lags = 6;  
  elseif NW_lag_choice == 2
        lags = ceil(4*(size(HPERUSD_LongBond,1)/100)^(2/9))-1;
  elseif NW_lag_choice == 3
        lags = ceil(0.75*(size(HPERUSD_LongBond,1))^(1/3))-1;
  elseif NW_lag_choice == 4
        lags = max(ceil(4*(size(HPERUSD_LongBond,1)/100)^(2/9))-1,ceil(0.75*(size(HPERUSD_LongBond,1))^(1/3))-1);
  elseif NW_lag_choice == 5
        lags = ceil(1.3*(size(HPERUSD_LongBond,1))^(1/2))-1;
  end
    
    
    
    lhv = (HPERUSD_LongBond(:,1+n)-HPERUSD_LongBond(:,end))*ScalingFactor;
    rhv = [ones(size(lhv,1),1) ZC_10Yminus3M_lag(:,1+n)-ZC_10Yminus3M_lag(:,end)];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        TSPred_BetasDiff3_con(n,1)     = bv(1);
        TSPred_BetasDiff3_con_se(n,1)  = sebv(1);
        TSPred_BetasDiff3(n,1)         = bv(2);
        TSPred_BetasDiff3_se(n,1)      = sebv(2);
        TSPred_BetasDiff3_R2adj(n,1)   = R2vadj;
        TSPred_BetasDiff3_Nobs(n,1)    = size(lhv_s,1);
        TSPred_BetasDiff3_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasDiff3_con(n,1)/TSPred_BetasDiff3_con_se(n,1),(lags+1)/TSPred_BetasDiff3_Nobs(n,1));                 
        TSPred_BetasDiff3_pval_b(n,1)     = tstat_pval_b(TSPred_BetasDiff3(n,1)/TSPred_BetasDiff3_se(n,1),(lags+1)/TSPred_BetasDiff3_Nobs(n,1));   
    end;
    
    lhv = FXER(:,1+n)*ScalingFactor;
    rhv = [ones(size(lhv,1),1) ZC_10Yminus3M_lag(:,1+n)-ZC_10Yminus3M_lag(:,end)];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        TSPred_BetasDiff4_con(n,1)     = bv(1);
        TSPred_BetasDiff4_con_se(n,1)  = sebv(1);
        TSPred_BetasDiff4(n,1)         = bv(2);
        TSPred_BetasDiff4_se(n,1)      = sebv(2);
        TSPred_BetasDiff4_R2adj(n,1)   = R2vadj;
        TSPred_BetasDiff4_Nobs(n,1)    = size(lhv_s,1);
        TSPred_BetasDiff4_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasDiff4_con(n,1)/TSPred_BetasDiff4_con_se(n,1),(lags+1)/TSPred_BetasDiff4_Nobs(n,1));                 
        TSPred_BetasDiff4_pval_b(n,1)     = tstat_pval_b(TSPred_BetasDiff4(n,1)/TSPred_BetasDiff4_se(n,1),(lags+1)/TSPred_BetasDiff4_Nobs(n,1));   
    end;
    
    lhv = (HPER_LongBond(:,1+n)-HPER_LongBond(:,end))*ScalingFactor;
    rhv = [ones(size(lhv,1),1) ZC_10Yminus3M_lag(:,1+n)-ZC_10Yminus3M_lag(:,end)];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        TSPred_BetasDiff5_con(n,1)     = bv(1);
        TSPred_BetasDiff5_con_se(n,1)  = sebv(1);
        TSPred_BetasDiff5(n,1)         = bv(2);
        TSPred_BetasDiff5_se(n,1)      = sebv(2);
        TSPred_BetasDiff5_R2adj(n,1)   = R2vadj;
        TSPred_BetasDiff5_Nobs(n,1)    = size(lhv_s,1);
        TSPred_BetasDiff5_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasDiff5_con(n,1)/TSPred_BetasDiff5_con_se(n,1),(lags+1)/TSPred_BetasDiff5_Nobs(n,1));                 
        TSPred_BetasDiff5_pval_b(n,1)     = tstat_pval_b(TSPred_BetasDiff5(n,1)/TSPred_BetasDiff5_se(n,1),(lags+1)/TSPred_BetasDiff5_Nobs(n,1));   
    end;
    
end;

% Panel regressions with Driscoll-Kraay s.e.

Panel_Var = [HPERUSD_LongBond_Diff*ScalingFactor, ZC_10Yminus3M_lag_Diff];
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se]   = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasDiff3_con(Nc+2,1)           = NaN;
TSPred_BetasDiff3_con_se(Nc+2,1)        = NaN; 
TSPred_BetasDiff3(Nc+2,1)               = theta(1,1);
TSPred_BetasDiff3_se(Nc+2,1)            = theta_se(1,1);
TSPred_BetasDiff3_Nobs(Nc+2,1)          = sum(sum(~isnan(yhat)));
TSPred_BetasDiff3_R2adj(Nc+2,1)         = 1-(1-Rsq)*(TSPred_BetasDiff3_Nobs(Nc+2,1)-1)/(TSPred_BetasDiff3_Nobs(Nc+2,1)-size(X_DK,3));
TSPred_BetasDiff3_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasDiff3_con(Nc+2,1)/TSPred_BetasDiff3_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasDiff3_pval_b(Nc+2,1)        = tstat_pval_b(TSPred_BetasDiff3(Nc+2,1)/TSPred_BetasDiff3_se(Nc+2,1), (lags+1)/size(y_DK,1));                 


Panel_Var = [FXER_Panel*ScalingFactor, ZC_10Yminus3M_lag_Diff];
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasDiff4_con(Nc+2,1)         = NaN;
TSPred_BetasDiff4_con_se(Nc+2,1)      = NaN; 
TSPred_BetasDiff4(Nc+2,1)             = theta(1,1);
TSPred_BetasDiff4_se(Nc+2,1)          = theta_se(1,1);
TSPred_BetasDiff4_Nobs(Nc+2,1)        = sum(sum(~isnan(yhat)));
TSPred_BetasDiff4_R2adj(Nc+2,1)       = 1-(1-Rsq)*(TSPred_BetasDiff4_Nobs(Nc+2,1)-1)/(TSPred_BetasDiff4_Nobs(Nc+2,1)-size(X_DK,3));
TSPred_BetasDiff4_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasDiff4_con(Nc+2,1)/TSPred_BetasDiff4_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasDiff4_pval_b(Nc+2,1)        = tstat_pval_b(TSPred_BetasDiff4(Nc+2,1)/TSPred_BetasDiff4_se(Nc+2,1), (lags+1)/size(y_DK,1));                 


Panel_Var = [HPER_LongBond_Diff*ScalingFactor, ZC_10Yminus3M_lag_Diff];
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasDiff5_con(Nc+2,1)         = NaN;
TSPred_BetasDiff5_con_se(Nc+2,1)      = NaN;
TSPred_BetasDiff5(Nc+2,1)             = theta(1,1);
TSPred_BetasDiff5_se(Nc+2,1)          = theta_se(1,1);
TSPred_BetasDiff5_Nobs(Nc+2,1)        = sum(sum(~isnan(yhat)));
TSPred_BetasDiff5_R2adj(Nc+2,1)       = 1-(1-Rsq)*(TSPred_BetasDiff5_Nobs(Nc+2,1)-1)/(TSPred_BetasDiff5_Nobs(Nc+2,1)-size(X_DK,3));
TSPred_BetasDiff5_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasDiff5_con(Nc+2,1)/TSPred_BetasDiff5_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasDiff5_pval_b(Nc+2,1)        = tstat_pval_b(TSPred_BetasDiff5(Nc+2,1)/TSPred_BetasDiff5_se(Nc+2,1), (lags+1)/size(y_DK,1));                 


% Joint estimation

T          = size(HPERUSD_LongBond,1);
R1_con     = zeros(Nc-1,6*(Nc-1));
R1_slope   = zeros(Nc-1,6*(Nc-1));
R1_all     = zeros(2*(Nc-1),6*(Nc-1));
R2_con     = zeros(Nc-1,6*(Nc-1));
R2_slope   = zeros(Nc-1,6*(Nc-1));
R2_all     = zeros(2*(Nc-1),6*(Nc-1));
R3_con     = zeros(Nc-1,6*(Nc-1));
R3_slope   = zeros(Nc-1,6*(Nc-1));
R3_all     = zeros(2*(Nc-1),6*(Nc-1));
R_diff     = zeros(Nc-1,6*(Nc-1));
r          = zeros(Nc-1,1);
rhv        = zeros(3*T*(Nc-1),6*(Nc-1));
for n=1:Nc-1
lhv((n-1)*T+1:n*T,1)                                                     = (HPERUSD_LongBond(:,1+n)-HPERUSD_LongBond(:,end))*ScalingFactor;
rhv((n-1)*T+1:n*T,(n-1)*2+1:n*2)                                         = [ones(T,1) ZC_10Yminus3M_lag(:,1+n)-ZC_10Yminus3M_lag(:,end)];
lhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,1)                                   = FXER(:,1+n)*ScalingFactor;
rhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,2*(Nc-1)+(n-1)*2+1:2*(Nc-1)+n*2)     = [ones(T,1) ZC_10Yminus3M_lag(:,1+n)-ZC_10Yminus3M_lag(:,end)];
lhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,1)                               = (HPER_LongBond(:,1+n)-HPER_LongBond(:,end))*ScalingFactor;
rhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,4*(Nc-1)+(n-1)*2+1:4*(Nc-1)+n*2) = [ones(T,1) ZC_10Yminus3M_lag(:,1+n)-ZC_10Yminus3M_lag(:,end)];
R1_con(n,2*n-1)          =  1;
R1_slope(n,2*n)          =  1;
R1_all(n,2*n-1)          =  1;
R1_all(Nc-1+n,2*n)       =  1;
R2_con(n,2*(Nc-1)+2*n-1) =  1;
R2_slope(n,2*(Nc-1)+2*n) =  1;
R2_all(n,2*(Nc-1)+2*n-1) =  1;
R2_all(Nc-1+n,2*(Nc-1)+2*n) =  1;
R3_con(n,4*(Nc-1)+2*n-1) =  1;
R3_slope(n,4*(Nc-1)+2*n) =  1;
R3_all(n,4*(Nc-1)+2*n-1) =  1;
R3_all(Nc-1+n,4*(Nc-1)+2*n) =  1;
R_diff(n,2*n)            =  1;
R_diff(n,2*(Nc-1)+2*n)   = -1;
end
[lhv_s,rhv_s]            = FindCommonSample(lhv,rhv);
[bv,sebv,R2v,R2vadj,v]   = olsgmm(lhv_s,rhv_s,lags,1);
W_Diff3_con(1,1)         = (R1_con*bv - r)'*inv(R1_con*v*R1_con')*(R1_con*bv - r);          % test if all constants are zero in the HPERDollar regressions
Wp_Diff3_con(1,1)        = 1-chi2cdf(W_Diff3_con(1,1),Nc-1);
W_Diff3_slope(1,1)       = (R1_slope*bv - r)'*inv(R1_slope*v*R1_slope')*(R1_slope*bv - r);  % test if all slopes are zero in the HPERDollar regressions
Wp_Diff3_slope(1,1)      = 1-chi2cdf(W_Diff3_slope(1,1),Nc-1);
W_Diff3_all(1,1)         = (R1_all*bv - r_all)'*inv(R1_all*v*R1_all')*(R1_all*bv - r_all);  % test if all constants and slopes are zero in the HPERDollar regressions
Wp_Diff3_all(1,1)        = 1-chi2cdf(W_Diff3_all(1,1),2*(Nc-1));
W_Diff4_con(1,1)         = (R2_con*bv - r)'*inv(R2_con*v*R2_con')*(R2_con*bv - r);          % test if all constants are zero in the FXEXR regressions
Wp_Diff4_con(1,1)        = 1-chi2cdf(W_Diff4_con(1,1),Nc-1);
W_Diff4_slope(1,1)       = (R2_slope*bv - r)'*inv(R2_slope*v*R2_slope')*(R2_slope*bv - r);  % test if all slopes are zero in the FXEXR regressions
Wp_Diff4_slope(1,1)      = 1-chi2cdf(W_Diff4_slope(1,1),Nc-1);
W_Diff4_all(1,1)         = (R2_all*bv - r_all)'*inv(R2_all*v*R2_all')*(R2_all*bv - r_all);  % test if all constants and slopes are zero in the FXEXR regressions
Wp_Diff4_all(1,1)        = 1-chi2cdf(W_Diff4_all(1,1),2*(Nc-1));
W_Diff5_con(1,1)         = (R3_con*bv - r)'*inv(R3_con*v*R3_con')*(R3_con*bv - r);          % test if all constants are zero in the HPER regressions
Wp_Diff5_con(1,1)        = 1-chi2cdf(W_Diff5_con(1,1),Nc-1);
W_Diff5_slope(1,1)       = (R3_slope*bv - r)'*inv(R3_slope*v*R3_slope')*(R3_slope*bv - r);  % test if all slopes are zero in the HPER regressions
Wp_Diff5_slope(1,1)      = 1-chi2cdf(W_Diff5_slope(1,1),Nc-1);
W_Diff5_all(1,1)         = (R3_all*bv - r_all)'*inv(R3_all*v*R3_all')*(R3_all*bv - r_all);  % test if all constants and slopes are zero in the HPER regressions
Wp_Diff5_all(1,1)        = 1-chi2cdf(W_Diff5_all(1,1),2*(Nc-1));
W_Diff_TermStr(Nc+2,1)   = (R_diff*bv - r)'*inv(R_diff*v*R_diff')*(R_diff*bv - r);          % test if all HPERDollar/FXEXR slope differentials are zero
Wp_Diff_TermStr(Nc+2,1)  = 1-chi2cdf(W_Diff_TermStr(Nc+2,1),Nc-1);

Wp_Diff3_con_b(1,1)        = chisq_pval_b(W_Diff3_con(1,1),Nc-1,(lags+1)/T);
Wp_Diff3_slope_b(1,1)      = chisq_pval_b(W_Diff3_slope(1,1),Nc-1,(lags+1)/T);
Wp_Diff3_all_b(1,1)        = chisq_pval_b(W_Diff3_all(1,1),2*(Nc-1),(lags+1)/T);
Wp_Diff4_con_b(1,1)        = chisq_pval_b(W_Diff4_con(1,1),Nc-1,(lags+1)/T);
Wp_Diff4_slope_b(1,1)      = chisq_pval_b(W_Diff4_slope(1,1),Nc-1,(lags+1)/T);
Wp_Diff4_all_b(1,1)        = chisq_pval_b(W_Diff4_all(1,1),2*(Nc-1),(lags+1)/T);
Wp_Diff5_con_b(1,1)        = chisq_pval_b(W_Diff5_con(1,1),Nc-1,(lags+1)/T);
Wp_Diff5_slope_b(1,1)      = chisq_pval_b(W_Diff5_slope(1,1),Nc-1,(lags+1)/T);
Wp_Diff5_all_b(1,1)        = chisq_pval_b(W_Diff5_all(1,1),2*(Nc-1),(lags+1)/T); 
Wp_Diff_TermStr_b(Nc+2,1)  = chisq_pval_b(W_Diff_TermStr(Nc+2,1),Nc-1,(lags+1)/T);


% Panel regression

clear lhv_panel_DK rhv_panel_DK;
lhv_panel = [];
rhv_panel = zeros(2*T*(Nc-1),2*Nc);
R_slope = [zeros(1,Nc-1) 1 zeros(1,Nc-1) -1];
R_con_1 = zeros(Nc-1,2*Nc);
R_con_2 = zeros(Nc-1,2*Nc);
r = 0;
r_con = zeros(Nc-1,1);
for n = 1:Nc-1
T = size(HPERUSD_LongBond,1);
R = [0 1 0 -1];
r = 0;
lhv  = [(HPERUSD_LongBond(:,1+n)-HPERUSD_LongBond(:,end))*ScalingFactor;  FXER(:,1+n)*ScalingFactor];
rhv  = zeros(2*T,4);
rhv(1:T,1:2)     = [ones(T,1) ZC_10Yminus3M_lag(:,1+n)-ZC_10Yminus3M_lag(:,end)];
rhv(T+1:2*T,3:4) = [ones(T,1) ZC_10Yminus3M_lag(:,1+n)-ZC_10Yminus3M_lag(:,end)];
[lhv_s,rhv_s]             = FindCommonSample(lhv,rhv);
[bv,sebv,R2v,R2vadj,v]    = olsgmm(lhv_s,rhv_s,lags,1);
W_Diff_TermStr(n,1)  = (R*bv - r)'*inv(R*v*R')*(R*bv - r);
Wp_Diff_TermStr(n,1) = 1-chi2cdf(W_Diff_TermStr(n,1),1);
Wp_Diff_TermStr_b(n,1)  = chisq_pval_b(W_Diff_TermStr(n,1),1,(lags+1)/T);

lhv_panel = [lhv_panel; lhv];
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,n) = ones(T,1);
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,Nc)= ZC_10Yminus3M_lag(:,1+n)-ZC_10Yminus3M_lag(:,end);
rhv_panel((n-1)*2*T+T+1:n*2*T,Nc+n)  = ones(T,1);
rhv_panel((n-1)*2*T+T+1:n*2*T,2*Nc)  = ZC_10Yminus3M_lag(:,1+n)-ZC_10Yminus3M_lag(:,end);

lhv_panel_DK(:,n)        = lhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:);
lhv_panel_DK(:,Nc-1+n)   = lhv_panel((n-1)*2*T+T+1:n*2*T,:);
rhv_panel_DK(:,n,:)      = reshape(rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:),T,[]);
rhv_panel_DK(:,Nc-1+n,:) = reshape(rhv_panel((n-1)*2*T+T+1:n*2*T,:),T,[]);

R_con_1(n,n)    = 1;
R_con_2(n,Nc+n) = 1;
end

[bv, ~, ~,~,~,~,~,v,~]       = HszDk5dwPs(lhv_panel_DK,ones(T,1),rhv_panel_DK,1,lags);
W_Diff_TermStr(Nc+1,1)       = (R_slope*bv - r)'*inv(R_slope*v*R_slope')*(R_slope*bv - r);             % test if long-term and short-term bond slopes are equal in the panel regression
Wp_Diff_TermStr(Nc+1,1)      = 1-chi2cdf(W_Diff_TermStr(Nc+1,1),1);
W_Diff3_con(2,1)             = (R_con_1*bv - r_con)'*inv(R_con_1*v*R_con_1')*(R_con_1*bv - r_con);    % test if all USD bond return constants are zero in the panel regression
Wp_Diff3_con(2,1)            = 1-chi2cdf(W_Diff3_con(2,1),Nc-1);
W_Diff4_con(2,1)             = (R_con_2*bv - r_con)'*inv(R_con_2*v*R_con_2')*(R_con_2*bv - r_con);      % test if all FXEXR constants are zero in the panel regression
Wp_Diff4_con(2,1)            = 1-chi2cdf(W_Diff4_con(2,1),Nc-1);

Wp_Diff_TermStr_b(Nc+1,1) = chisq_pval_b(W_Diff_TermStr(Nc+1,1),1,(lags+1)/T); 
Wp_Diff3_con_b(2,1)       = chisq_pval_b(W_Diff3_con(2,1),Nc-1,(lags+1)/T); 
Wp_Diff4_con_b(2,1)       = chisq_pval_b(W_Diff4_con(2,1),Nc-1,(lags+1)/T); 


% Tables

rows  = Countries_Sample';
rows{1,Nc+1} = '--';  
rows{1,Nc+2} = 'Panel (DK)';
rows{1,Nc+3} = 'Joint zero (p-value)';

Table_Slope_Collected_Simple = [TSPred_BetasDiff3_con TSPred_BetasDiff3_con_se ...
                         TSPred_BetasDiff3 TSPred_BetasDiff3_se 100*TSPred_BetasDiff3_R2adj...
                         TSPred_BetasDiff4_con TSPred_BetasDiff4_con_se ...
                         TSPred_BetasDiff4 TSPred_BetasDiff4_se 100*TSPred_BetasDiff4_R2adj...
                         TSPred_BetasDiff5_con TSPred_BetasDiff5_con_se...
                         TSPred_BetasDiff5 TSPred_BetasDiff5_se 100*TSPred_BetasDiff5_R2adj...
                         [Wp_Diff_TermStr(1:Nc+1,1); Wp_Diff_TermStr(Nc+1,1)] TSPred_BetasDiff3_Nobs];       
Table_Slope_Collected_Simple = [Table_Slope_Collected_Simple; Wp_Diff3_con(1,1) NaN Wp_Diff3_slope(1,1) NaN NaN Wp_Diff4_con(1,1) NaN Wp_Diff4_slope(1,1) NaN NaN Wp_Diff5_con(1,1) NaN Wp_Diff5_slope(1,1) NaN NaN Wp_Diff_TermStr(Nc+2,1) NaN];
cols  = {'coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','p-value','Obs.'}; 
disp(' ');
disp('-----------------------------------------');
disp('Collected Summary Results of Time-Series Regressions on Yield Curve Slope Differentials');
disp('-----------------------------------------');
mat2tex(Table_Slope_Collected_Simple,'%6.2f',rows,cols, [], Opts)

% Build Portfolios -- Sorted By Yield Curve Slopes
                         
% Use variables without their first column of dates
FXER_temp                      = FXER(:,2:end-1); 
FX_temp                        = FX(:,2:end-1); 
FWD_temp                       = FWD(:,2:end-1); 
Infl_temp                      = Infl(:,2:end-1); 
Ratings_lag_temp               = Ratings_lag(:,2:end-1); 
RatingsOut_lag_temp            = RatingsOut_lag(:,2:end-1); 

HPER_LongBond_temp             = HPER_LongBond(:,2:end-1);
HPRUSD_LongBond_temp           = HPRUSD_LongBond(:,2:end-1); 
HPERUSD_LongBond_temp          = HPERUSD_LongBond(:,2:end-1); 
HPRUSDminusUSHPR_LongBond_temp = HPRUSDminusUSHPR_LongBond(:,2:end); 
ZC_10Yminus3M_lag_temp         = ZC_10Yminus3M_lag(:,2:end-1);
ZC_10Yminus3M_lag_Rolling_Mean_temp         = ZC_10Yminus3M_lag_Rolling_Mean(:,2:end-1);



% Get rid of interest rates if no data on currency or bond excess returns
% and vice-versa
for t=1:size(ZC_3M_lag_temp,1)                                               % For each date
    for c=1:size(ZC_3M_lag_temp,2)                                           % For each country
       if  isnan(ZC_3M_lag_temp(t,c)*FXER_temp(t,c)*HPER_LongBond_temp(t,c)*HPRUSD_LongBond_temp(t,c)*HPERUSD_LongBond_temp(t,c))==1;
           ZC_3M_lag_temp(t,c)        = NaN;
           FXER_temp(t,c)             = NaN;
           FX_temp(t,c)               = NaN;
           FWD_temp(t,c)              = NaN;
           HPER_LongBond_temp(t,c)    = NaN;
           HPRUSD_LongBond_temp(t,c)  = NaN;
           HPERUSD_LongBond_temp(t,c) = NaN;
       end         
    end;
end;

% Sign


sign_mat_Slope = -sign(ZC_10Yminus3M_lag_temp-repmat(ZC_10Yminus3M_lag(:,end),1,Nc-1));

FXER_temp_signed                         = (FXER_temp-repmat(FXER(:,end),1,Nc-1)).*sign_mat_Slope; 
HPR_LongBond_temp_signed                 = (HPR_LongBond_temp-repmat(HPR_LongBond(:,end),1,Nc-1)).*sign_mat_Slope; 
HPER_LongBond_temp_signed                = (HPER_LongBond_temp-repmat(HPER_LongBond(:,end),1,Nc-1)).*sign_mat_Slope; 
HPRUSD_LongBond_temp_signed              = (HPRUSD_LongBond_temp-repmat(HPRUSD_LongBond(:,end),1,Nc-1)).*sign_mat_Slope; 
HPERUSD_LongBond_temp_signed             = (HPERUSD_LongBond_temp-repmat(HPERUSD_LongBond(:,end),1,Nc-1)).*sign_mat_Slope; 
HPRUSDminusUSHPR_LongBond_temp_signed    = (HPRUSDminusUSHPR_LongBond_temp).*sign_mat_Slope; 
FX_temp_signed                           = (FX_temp-repmat(FX(:,end),1,Nc-1)).*sign_mat_Slope; 
FWD_temp_signed                          = (FWD_temp-repmat(FWD(:,end),1,Nc-1)).*sign_mat_Slope; 
Infl_temp_signed                         = (Infl_temp-repmat(Infl(:,end),1,Nc-1)).*sign_mat_Slope; 
Ratings_lag_temp_signed                  = (Ratings_lag_temp-repmat(Ratings_lag(:,end),1,Nc-1)).*sign_mat_Slope; 
RatingsOut_lag_temp_signed               = (RatingsOut_lag_temp-repmat(RatingsOut_lag(:,end),1,Nc-1)).*sign_mat_Slope; 

for n=1:Nc-1


[Portfolios_FXER(:,n),Portfolios_HPER(:,n),Portfolios_HPRUSDminusUSHPR_LongBond(:,n),Portfolios_HPERUSD(:,n),Portfolios_VarSort_IR_signed(:,n),Track_portfolios_IR1_signed(:,n)] = ...
    Build_portfolios_for4variables(FXER_temp_signed(:,n),HPER_LongBond_temp_signed(:,n),HPRUSDminusUSHPR_LongBond_temp_signed(:,n),HPERUSD_LongBond_temp_signed(:,n),ZC_10Yminus3M_lag_temp(:,n),1,1,ZC_10Yminus3M_lag_Rolling_Mean_temp(:,n));
[Portfolios_FXER(:,n),Portfolios_HPR(:,n),Portfolios_Infl(:,n),Portfolios_HPRUSD(:,n),~,Track_portfolios_IR2_signed(:,n)] = ...
    Build_portfolios_for4variables(FXER_temp_signed(:,n),HPR_LongBond_temp_signed(:,n),Infl_temp_signed(:,n),HPRUSD_LongBond_temp_signed(:,n),ZC_10Yminus3M_lag_temp(:,n),1,1,ZC_10Yminus3M_lag_Rolling_Mean_temp(:,n));
[Portfolios_FX(:,n),Portfolios_FWD(:,n),Portfolios_Ratings(:,n),Portfolios_RatingsOut(:,n), ~,Track_portfolios_IR3_signed(:,n)] = ...
    Build_portfolios_for4variables(FX_temp_signed(:,n),FWD_temp_signed(:,n),Ratings_lag_temp_signed(:,n),RatingsOut_lag_temp_signed(:,n),ZC_10Yminus3M_lag_temp(:,n),1,1,ZC_10Yminus3M_lag_Rolling_Mean_temp(:,n));

end

[Portfolios_FXER(:,Nc+1),Portfolios_HPER(:,Nc+1),Portfolios_HPRUSDminusUSHPR_LongBond(:,Nc+1),Portfolios_HPERUSD(:,Nc+1),Portfolios_VarSort_IR_signed(:,Nc+1),Track_portfolios_IR1_signed] = ...
    Build_portfolios_for4variables(FXER_temp_signed,HPER_LongBond_temp_signed,HPRUSDminusUSHPR_LongBond_temp_signed,HPERUSD_LongBond_temp_signed,ZC_10Yminus3M_lag_temp,1,1,ZC_10Yminus3M_lag_Rolling_Mean_temp);
[Portfolios_FXER(:,Nc+1),Portfolios_HPR(:,Nc+1),Portfolios_Infl(:,Nc+1),Portfolios_HPRUSD(:,Nc+1),~,Track_portfolios_IR2_signed] = ...
    Build_portfolios_for4variables(FXER_temp_signed,HPR_LongBond_temp_signed,Infl_temp_signed,HPRUSD_LongBond_temp_signed,ZC_10Yminus3M_lag_temp,1,1,ZC_10Yminus3M_lag_Rolling_Mean_temp);
[Portfolios_FX(:,Nc+1),Portfolios_FWD(:,Nc+1),Portfolios_Ratings(:,Nc+1),Portfolios_RatingsOut(:,Nc+1), ~,Track_portfolios_IR3_signed] = ...
    Build_portfolios_for4variables(FX_temp_signed,FWD_temp_signed,Ratings_lag_temp_signed,RatingsOut_lag_temp_signed,ZC_10Yminus3M_lag_temp,1,1,ZC_10Yminus3M_lag_Rolling_Mean_temp);

Portfolios_FXER_Slope_signed     = Portfolios_FXER;
Portfolios_HPR_Slope_signed      = Portfolios_HPR;
Portfolios_HPER_Slope_signed     = Portfolios_HPER;
Portfolios_HPRUSD_Slope_signed   = Portfolios_HPRUSD;
Portfolios_HPERUSD_Slope_signed  = Portfolios_HPERUSD;
Portfolios_FX_Slope_signed       = Portfolios_FX;
Portfolios_Infl_Slope_signed     = Portfolios_Infl;
Portfolios_Ratings_Slope_signed        = Portfolios_Ratings;
Portfolios_RatingsOut_Slope_signed     = Portfolios_RatingsOut;

ScalingFactor = 4;  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%bootstrapping
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

portfolio_data                                    = [(Portfolios_FXER) (Portfolios_HPER) (Portfolios_HPERUSD) Portfolios_HPRUSDminusUSHPR_LongBond Portfolios_FX Portfolios_FWD Portfolios_Infl Portfolios_Ratings Portfolios_RatingsOut];

[standard_error_mean,standard_error_SR]           = bootstrap_portfolio_returns_growth(portfolio_data,nboot,horizon_bootstrap);

Total_portfolios=Nc+1;

Portfolios_FXER_se(1:Total_portfolios)                        = standard_error_mean(1:Total_portfolios);
Portfolios_HPER_se(1:Total_portfolios)                        = standard_error_mean(1+Total_portfolios:2*Total_portfolios);
Portfolios_HPERUSD_se(1:Total_portfolios)                     = standard_error_mean(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_HPRUSDminusUSHPR_se(1:Total_portfolios)            = standard_error_mean(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_se(1:Total_portfolios)                          = standard_error_mean(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_FWD_se(1:Total_portfolios)                         = standard_error_mean(1+5*Total_portfolios:6*Total_portfolios);
Portfolios_Infl_se(1:Total_portfolios)                        = standard_error_mean(1+6*Total_portfolios:7*Total_portfolios);
Portfolios_Ratings_se(1:Total_portfolios)                     = standard_error_mean(1+7*Total_portfolios:8*Total_portfolios);
Portfolios_RatingsOut_se(1:Total_portfolios)                  = standard_error_mean(1+8*Total_portfolios:9*Total_portfolios);

Portfolios_FXER_SR_se(1:Total_portfolios)                     = standard_error_SR(1:Total_portfolios);
Portfolios_HPER_SR_se(1:Total_portfolios)                     = standard_error_SR(1+Total_portfolios:2*Total_portfolios);
Portfolios_HPERUSD_SR_se(1:Total_portfolios)                  = standard_error_SR(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_HPRUSDminusUSHPR_SR_se(1:Total_portfolios)         = standard_error_SR(1+3*Total_portfolios:4*Total_portfolios);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Portfolios_HPRUSDminusUSHPR                                  = Portfolios_HPRUSDminusUSHPR_LongBond;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Table_Ptf_Slope_signed_Simple = [100*nanmean((Portfolios_HPERUSD))*ScalingFactor;  ...
         100*Portfolios_HPERUSD_se*ScalingFactor; ...             
         100*nanstd((Portfolios_HPERUSD))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_HPERUSD))*sqrt(ScalingFactor)./nanstd((Portfolios_HPERUSD)); ...
         Portfolios_HPERUSD_SR_se*sqrt(ScalingFactor); ...
         NaN(1,Nc+1); ...
         100*nanmean((Portfolios_FXER))*ScalingFactor; ...
         100*Portfolios_FXER_se*ScalingFactor; ...
         100*nanstd((Portfolios_FXER))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_FXER))*sqrt(ScalingFactor)./nanstd((Portfolios_FXER)); ...
         Portfolios_FXER_SR_se*sqrt(ScalingFactor); ...
         NaN(1,Nc+1); ...
         100*nanmean((Portfolios_HPER))*ScalingFactor; ...                     
         100*Portfolios_HPER_se*ScalingFactor; ...
         100*nanstd((Portfolios_HPER))*sqrt(ScalingFactor); ...
         nanmean((Portfolios_HPER))*sqrt(ScalingFactor)./nanstd((Portfolios_HPER)); ...
         Portfolios_HPER_SR_se*sqrt(ScalingFactor)]';
rows = Countries_Sample';      
rows{1,Nc+1} = 'Equally-weighted';   
Opts.Style = 'jf'; 
disp(' ');
disp('----------------------------------------------');
disp('Long-Short Portfolio Sorted By Slopes');
disp('----------------------------------------------'); 
mat2tex(Table_Ptf_Slope_signed_Simple,'%6.2f',rows,[], [], Opts)