function [Portfolios_ERba,Portfolios_ER,Portfolios_Q,Portfolios_TBdifnom] = Build_portfolios_generic(ER_nom,ER_nom_long,ER_nom_short,Q_ch_nom,TBdif_nom,sorting_variable,Number_portfolios)


[T,N] = size(ER_nom);


%==========================================
% Sort on interest rates (nominal or real)
%==========================================

switch sorting_variable
    case 1 
        [TBdif_sorted,Index_sorting]= sort(TBdif_nom,2);   %#ok<ASGLU> % Using nominal interest rates
    case 2
        [TBdif_sorted,Index_sorting]= sort(repmat(nanmean(TBdif_nom),T,1),2);   %#ok<ASGLU> % Using unconditional averages of nominal interest rates
end;


ER_sorted            =  zeros(T,N);
ERlong_sorted        =  zeros(T,N);
ERshort_sorted       =  zeros(T,N);
Q_ch_sorted          =  zeros(T,N);
TBdifnom_sorted      =  zeros(T,N);
Track_countries      =  zeros(T,N);

for t=1:T                        % For each date
    for s=1:N                    % For each country
       ER_sorted(t,s)        =  ER_nom(t,Index_sorting(t,s));
       ERlong_sorted(t,s)    =  ER_nom_long(t,Index_sorting(t,s));
       ERshort_sorted(t,s)   =  ER_nom_short(t,Index_sorting(t,s));
       Q_ch_sorted(t,s)      =  Q_ch_nom(t,Index_sorting(t,s));
       TBdifnom_sorted(t,s)  =  TBdif_nom(t,Index_sorting(t,s));
       Track_countries(t,s)  =  Index_sorting(t,s);
    end;
end;


%\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
% Count the numbers of countries available at every date
%\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
Count_Returns     = zeros(size(ER_sorted,1),1);
for t=1:size(ER_sorted,1)                                               % For each date
    for c=1:size(ER_sorted,2)                                           % For each country
        Count_Returns(t,1) =          Count_Returns(t,1)+(not(isnan(ER_sorted(t,c))));
    end;
end;

%==========================================
% Build portfolios
%==========================================
Portfolios_ER        =  zeros(T,Number_portfolios);
Portfolios_ERba      =  zeros(T,Number_portfolios);
Portfolios_TBdifnom  =  zeros(T,Number_portfolios);
Portfolios_Q         =  zeros(T,Number_portfolios);
Count_group          =  zeros(T,Number_portfolios);
trades               =  zeros(T,Number_portfolios);
Track_portfolios     =  zeros(T,Number_portfolios);

for t=1:T                   % For each date        
    for n=1:Number_portfolios;                  % For each portfolio
        if n==Number_portfolios;                % Special case: last portfolio
            for m=1+(n-1)*floor(Count_Returns(t,1)*(1/Number_portfolios)):n*floor(Count_Returns(t,1)*(1/Number_portfolios))+rem(Count_Returns(t,1),Number_portfolios);                 
                Portfolios_ER(t,n)                        =   Portfolios_ER(t,n)+ER_sorted(t,m);
                Portfolios_Q(t,n)                         =   Portfolios_Q(t,n)+Q_ch_sorted(t,m);
                Portfolios_ERba(t,n)                      =   Portfolios_ERba(t,n)+ERlong_sorted(t,m);
                Portfolios_TBdifnom(t,n)                  =   Portfolios_TBdifnom(t,n)+TBdifnom_sorted(t,m);
                Count_group(t,n)                          =   Count_group(t,n)+1;
                Track_portfolios(t,Track_countries(t,m))  =   n;

                % Test if the country changes portfolio
                if  t>1 && Track_portfolios(t-1,Track_countries(t,m)) ~=n
                    trades(t,n)=trades(t,n)+1;
                end;                 
            end;
        else                                    % All portfolios except the last one
            for m=1+(n-1)*floor(Count_Returns(t)*(1/Number_portfolios)):n*floor(Count_Returns(t)*(1/Number_portfolios));
                    Portfolios_ER(t,n)                        =   Portfolios_ER(t,n)+ER_sorted(t,m);
                    if n==1
                        Portfolios_ERba(t,n)                      =   Portfolios_ERba(t,n)+ERlong_sorted(t,m);
                    else
                        Portfolios_ERba(t,n)                      =   Portfolios_ERba(t,n)+ERshort_sorted(t,m);
                    end;
                    Portfolios_TBdifnom(t,n)                  =   Portfolios_TBdifnom(t,n)+TBdifnom_sorted(t,m);
                    Portfolios_Q(t,n)                         =   Portfolios_Q(t,n)+Q_ch_sorted(t,m);
                    Count_group(t,n)                          =   Count_group(t,n)+1;
                    Track_portfolios(t,Track_countries(t,m))  =   n;
                        
                    % Test if the country changes portfolio
                    if  t>1 && Track_portfolios(t-1,Track_countries(t,m)) ~=n
                        trades(t,n)=trades(t,n)+1;
                    end;
            end;
        end;
        if Count_group(t,n)>0;
           Portfolios_ER(t,n)                      =   Portfolios_ER(t,n)/Count_group(t,n);  
           Portfolios_ERba(t,n)                    =   Portfolios_ERba(t,n)/Count_group(t,n);  
           Portfolios_TBdifnom(t,n)                =   Portfolios_TBdifnom(t,n)/Count_group(t,n);  
           Portfolios_Q(t,n)                       =   Portfolios_Q(t,n)/Count_group(t,n);  
        else
           Portfolios_ER(t,n)                      =   NaN;  
           Portfolios_ERba(t,n)                    =   NaN;  
           Portfolios_TBdifnom(t,n)                =   NaN;  
           Portfolios_Q(t,n)                       =   NaN;  
        end;       
    end;
end;

