clear 
close all
restoredefaultpath
addpath(genpath('jsz_library'))
clc

% This gives a sample estimation of an N-factor "without error" GDTSM.
% See "A New Perspective on Gaussian Dynamic Term Structure Models" by Joslin, Singleton and Zhu

% Load some data for the US: mats (1*J) and yields (T*J) (to match time period)
load('sample_zeros.mat') 
% For the US, maturities are  0.5    1.0    2.0    3.0    5.0    7.0   10
dates_US = dates;

% load ZC data for G10 countries
load('ZC_Spliced_M_2016.mat','ZC_Spliced_M'); % Maturities are 3, 6, 9, ..., 120 or 180 months (ie up to 15 years max)
ZC_Sample = ZC_Spliced_M;
Countries_Sample = {'Australia';'Canada';'Germany'; 'Japan';'New Zealand';'Norway';'Sweden';'Switzerland';'United Kingdom';'United States'}; 
Nc = size(Countries_Sample,1);

T_all                     = size(ZC_Sample,1);
yields_foreign_all        = NaN(T_all,7,Nc);
yields_model_all          = NaN(T_all,200*12,Nc);
yields_model_selected_all = NaN(T_all,7,Nc);
ZC_Spliced_Model_M_Test   = NaN(T_all,801,Nc);
logSDF_all                = NaN(T_all,1,Nc);

for n=1:Nc

    mats = [0.5,1,2,3,5,7,10];
    
    ZC_foreign    = ZC_Sample(:,:,n);
    % Replace US yields with foreign yields
    ZC_foreign   = [ZC_foreign(:,1) ZC_foreign(:,1+4*mats)];
    
    % Same time window if possible
    line_start = find(ZC_foreign(:,1) == datenum(1985,4,30));
    line_end = find(ZC_foreign(:,1) == datenum(2015,12,31));
    yields_foreign  = ZC_foreign(line_start:line_end,2:end); % no dates
    dates_foreign   = ZC_foreign(line_start:line_end,1);

    % Delete NaNs
    
    existing_obs = isnan(yields_foreign(:,end))==0;
    
    yields_foreign  = yields_foreign(existing_obs,:);
    dates_foreign   = dates_foreign(existing_obs,:);

    % replace US yields by foreign yields
    yields = yields_foreign;
    dates  = dates_foreign;
    
    line_start_existing = find(ZC_foreign(:,1) == datenum(dates_foreign(1,1)));
    line_end_existing   = find(ZC_foreign(:,1) == datenum(dates_foreign(end,1)));

    % Setup format of model/data:
    M = 200*12;  % maximum maturity (in months) to consider; M+1 is infinite maturity
    N = 3;
    W_pca = pcacov(cov(yields));
    W = W_pca(:,1:N)';       % N*J
    PC1_obs = W_pca(:,1)';   % 1*J
    PC2_obs = W_pca(:,2)';   % 1*J
    PC3_obs = W_pca(:,3)';   % 1*J
    PC4_obs = W_pca(:,4)';   % 1*J
    PC5_obs = W_pca(:,5)';   % 1*J
    cP = yields*W';          % T*N
    dt = 1/12;
    T  = length(dates);

    % Estimate the model by ML. 
    help sample_estimation_fun
    VERBOSE = true;
    % [llks, AcP, BcP, AX, BX, kinfQ, K0P_cP, K1P_cP, sigma_e, K0Q_cP, K1Q_cP, rho0_cP, rho1_cP, cP, llkP, llkQ,  K0Q_X, K1Q_X, rho0_X, rho1_X] = ...
    %         sample_estimation_fun(W, yields, mats, dt, VERBOSE);


    [llks, AcP, BcP, AX, BX, kinfQ, K0P_cP, K1P_cP, sigma_e, K0Q_cP, K1Q_cP, rho0_cP, rho1_cP, cP, llkP, llkQ,  K0Q_X, K1Q_X, rho0_X, rho1_X, Sigma_cP, dlamQ] = ...
            sample_estimation_fun(W, yields, mats, dt, VERBOSE);

    check0 =  yields*W.' - cP;                          % should be all zeros   

    % Replicate Tables 2, 3 and 4
    lamQ   = sort(eig(K1Q_X),'descend');                % Table 2
    rinfQ  = -kinfQ/lamQ(1,1)*100;                      % Table 2

    K1P_cP_annual = K1P_cP*12;                          % Table 3
    theta  = -inv(K1P_cP)*(K0P_cP);                     % Table 3

    Sigma_cP_annual = Sigma_cP*12;
    Vol_cP          = sqrt(diag(Sigma_cP_annual))*100;  % expressed in percentage terms (Table 4)
    Corr_cP         = 10000*[Sigma_cP_annual(1,2)/(Vol_cP(1,1)*Vol_cP(2,1)), Sigma_cP_annual(1,3)/(Vol_cP(1,1)*Vol_cP(3,1)), Sigma_cP_annual(2,3)/(Vol_cP(2,1)*Vol_cP(3,1))]'; % Table 4

    yields_JSZ     = ones(T,1)*AcP + (yields*W.')*BcP;
    log_prices_JSZ = -yields_JSZ.*repmat(mats,T,1);
    prices_JSZ     = exp(log_prices_JSZ);

    A_price_JSZ    = - AcP.*(mats*12);
    B_price_JSZ    = - BcP.*repmat(mats*12,N,1);

    check1         = ones(T,1)*A_price_JSZ/12 + (yields*W.')*B_price_JSZ/12 - log_prices_JSZ;  % should be all zeros

    rf_model = rho0_cP + (yields*W.')*rho1_cP;


    % Maturities
    A_price =[];
    B_price =[];
    A_price(:,1)  = -rho0_cP/12;
    B_price(:,1)  = -rho1_cP/12;

    H0 = Sigma_cP;

    for m = 2:M

        A_price(1,m) = A_price(1,m-1) + (K0Q_cP)'*B_price(:,m-1) + 0.5*B_price(:,m-1)'*H0*B_price(:,m-1) - rho0_cP/12;
        B_price(:,m) = B_price(:,m-1) + (K1Q_cP)'*B_price(:,m-1) - rho1_cP/12;

    end

    A_price          = 12*A_price;
    B_price          = 12*B_price;

    A_yield = - A_price./[1:M]; 
    B_yield = - B_price./repmat([1:M],N,1);

    B_price(:,M+1)   = (K1Q_cP')\rho1_cP;

    yields_model     = ones(T,1)*A_yield + (yields*W.')*B_yield(:,1:M);
    log_prices_model = -yields_model.*repmat([1:M]/12,T,1);
    prices_model     = exp(log_prices_model);

    check2           = ones(T,1)*A_price/12 + (yields*W.')*B_price(:,1:M)/12 - log_prices_model;  % should be all zeros

    A_price_selected = A_price(:, mats*12);
    B_price_selected = B_price(:, mats*12);

    A_yield_selected = A_yield(:, mats*12);
    B_yield_selected = B_yield(:, mats*12);

    yields_model_selected = yields_model(:, mats*12); 
    prices_model_selected = prices_model(:, mats*12);

    check3 = A_price_JSZ - A_price(:, mats*12);  % should be all zeros
    check4 = B_price_JSZ - B_price(:, mats*12);  % should be all zeros
    check5 = AcP - A_yield(:, mats*12);          % should be all zeros
    check6 = BcP - B_yield(:, mats*12);          % should be all zeros
    check7 = yields_JSZ - yields_model_selected; % should be all zeros
    check8 = prices_JSZ - prices_model_selected; % should be all zeros

    T10_yields = [mean(rf_model) mean(yields_model(:,360))];   % Table 10

    W_pca_model = pcacov(cov(yields_model_selected));
    PC1_model = W_pca_model(:,1)';  
    PC2_model = W_pca_model(:,2)'; 
    PC3_model = W_pca_model(:,3)';  
    PC4_model = W_pca_model(:,4)';  
    PC5_model = W_pca_model(:,5)';   

    % Bond returns
    log_1m_return_model        = NaN*zeros(T,M+1);
    log_1m_return_model_expr   = NaN*zeros(T,M);

    log_3m_return_model        = NaN*zeros(T,M+1);
    log_3m_return_model_expr   = NaN*zeros(T,M);

    log_12m_return_model       = NaN*zeros(T,M+1);
    log_12m_return_model_expr  = NaN*zeros(T,M);

    % 1-month returns
    for t = 1:T-1
    
    for m = 1
        
    log_1m_return_model(t+1,m)      = - log_prices_model(t,m);
    log_1m_return_model_expr(t+1,m) = - A_price(1,m)/12 - cP(t,:)*B_price(:,m)/12;    
        
    end
    
    for m = 2:M
    
    log_1m_return_model(t+1,m)      = log_prices_model(t+1,m-1) - log_prices_model(t,m);
    log_1m_return_model_expr(t+1,m) = A_price(1,m-1)/12 - A_price(1,m)/12 + cP(t+1,:)*B_price(:,m-1)/12 - cP(t,:)*B_price(:,m)/12;
    
    end
    
    log_1m_return_model(t+1,M+1)   = -[(K0Q_cP)'*B_price(:,M+1)/12 + 0.5*(B_price(:,M+1)/12)'*H0*(B_price(:,M+1)/12) - rho0_cP/12] + cP(t+1,:)*B_price(:,M+1)/12 - cP(t,:)*B_price(:,M+1)/12;
    
    end

    % 3-month returns
    for t = 1:T-3
    
    for m = 3
        
    log_3m_return_model(t+3,m)      = - log_prices_model(t,m);
    log_3m_return_model_expr(t+3,m) = - A_price(1,m)/12 - cP(t,:)*B_price(:,m)/12;    
        
    end
    
    for m = 4:M
    
    log_3m_return_model(t+3,m)      = log_prices_model(t+3,m-3) - log_prices_model(t,m);
    log_3m_return_model_expr(t+3,m) = A_price(1,m-3)/12 - A_price(1,m)/12 + cP(t+3,:)*B_price(:,m-3)/12 - cP(t,:)*B_price(:,m)/12;
    
    end
    
    log_3m_return_model(t+3,M+1)   = -3*[(K0Q_cP)'*B_price(:,M+1)/12 + 0.5*(B_price(:,M+1)/12)'*H0*(B_price(:,M+1)/12) - rho0_cP/12] + cP(t+3,:)*B_price(:,M+1)/12 - cP(t,:)*B_price(:,M+1)/12;
    
    end
    
    % PC-mimicking returns
    
    cP_expected        = NaN*zeros(T,N);
    cP_error           = NaN*zeros(T,N);
    cP_expected(2:T,:) = cP(1:T-1,:) + repmat(K0P_cP',T-1,1) + cP(1:T-1,:)*K1P_cP';
    cP_error(2:T,:)    = cP(2:T,:) - cP_expected(2:T,:);

    xrP = repmat(K0P_cP' - K0Q_cP',T,1) + cP*(K1P_cP-K1Q_cP)';  % price of risk for each of the factors/PCs

    A_yield_PC1 = A_yield_selected*PC1_obs';
    A_yield_PC2 = A_yield_selected*PC2_obs';
    A_yield_PC3 = A_yield_selected*PC3_obs';
    A_yield_PC4 = A_yield_selected*PC4_obs';
    A_yield_PC5 = A_yield_selected*PC5_obs';

    B_yield_PC1 = B_yield_selected*PC1_obs';
    B_yield_PC2 = B_yield_selected*PC2_obs';
    B_yield_PC3 = B_yield_selected*PC3_obs';
    B_yield_PC4 = B_yield_selected*PC4_obs';
    B_yield_PC5 = B_yield_selected*PC5_obs';

   rep_PC1 = (ones(T,1)*A_yield_PC1 + cP*B_yield_PC1);

    for t = 1:T 
    
    xrPC1(t,:)   = xrP(t,:)*B_yield_PC1;
    xrPC2(t,:)   = xrP(t,:)*B_yield_PC2;
    xrPC3(t,:)   = xrP(t,:)*B_yield_PC3;
    xrPC4(t,:)   = xrP(t,:)*B_yield_PC4;
    xrPC5(t,:)   = xrP(t,:)*B_yield_PC5;

    end

    logSR_PC1   = xrPC1./sqrt(B_yield_PC1'*H0*B_yield_PC1);
    logSR_PC2   = xrPC2./sqrt(B_yield_PC2'*H0*B_yield_PC2);
    logSR_PC3   = xrPC3./sqrt(B_yield_PC3'*H0*B_yield_PC3);
    logSR_PC4   = xrPC4./sqrt(B_yield_PC4'*H0*B_yield_PC4);
    logSR_PC5   = xrPC5./sqrt(B_yield_PC5'*H0*B_yield_PC5);

    Table_10 = [mean(rf_model); mean(yields_model(:,360)); mean(logSR_PC4); std(logSR_PC4); mean(logSR_PC5); std(logSR_PC5)];   % Table 10


    % SDF
    V = chol(Sigma_cP)';         % The volatility matrix V is lower triangular and satisfies: V*V' = Sigma_cP
    cP_error_I   = NaN*zeros(T,N);
    Lambda       = NaN*zeros(T,N);
    logSDF_exp   = NaN*zeros(T,1);
    logSDF_error = NaN*zeros(T,1);
    logSDF       = NaN*zeros(T,1);
    logSDF_cvar  = NaN*zeros(T,1);

    for t = 1:T-1
    
    Lambda(t,:)         = (V\xrP(t,:)')';
    cP_error_I(t+1,:)   = (V\(cP_error(t+1,:)'))';
    logSDF_exp(t+1,1)   = - rf_model(t,1)/12 - 0.5*Lambda(t,:)*Lambda(t,:)';
    logSDF_error(t+1,1) = - Lambda(t,:)*cP_error_I(t+1,:)';
    logSDF(t+1,1)       = logSDF_exp(t+1,1) +logSDF_error(t+1,1) ;
    logSDF_cvar(t+1,1)  = Lambda(t,:)*Lambda(t,:)';
    
    end

    check9  = nanmean(cP_error_I);   % should be close to all zeros
    check10 = nancov(cP_error_I);    % should be close to identity matrix
    check11 = nanmean(logSDF_error); % should be close to all zeros

    logSDF_cvol = sqrt(logSDF_cvar);

    check12 = rf_model(1:T-1,1)/12 + logSDF_exp(2:T,1) + 0.5*logSDF_cvar(2:T,1);  % should be all zeros
    check13 = nanmean(logSDF) - nanmean(logSDF_exp);                              % should be close to zero
    check14 = nanvar(logSDF) - nanmean(logSDF_cvar) - nanvar(logSDF_exp);         % should be close to zero
    
    % Collect and save data
    
    save(strcat('log_3m_return_model_',int2str(n),'_2016.mat'),'log_3m_return_model');
    save(strcat('yields_model_selected_',int2str(n),'_2016.mat'),'yields_model_selected');
    save(strcat('dates_foreign_',int2str(n),'_2016.mat'),'dates_foreign');
    save(strcat('logSDF_',int2str(n),'_2016.mat'),'logSDF');
    
    yields_foreign_all(line_start_existing:line_end_existing,:,n)          = yields_foreign;    
    yields_model_all(line_start_existing:line_end_existing,:,n)            = yields_model;
    yields_model_selected_all(line_start_existing:line_end_existing,:,n)   = yields_model_selected;
    logSDF_all(line_start_existing:line_end_existing,:,n)                  = logSDF;
    
    ZC_Spliced_Model_M_Test(line_start_existing:line_end_existing,:,n)     = [dates_foreign yields_model_all(line_start_existing:line_end_existing,3:3:2400,n)];

end

% Save ZC data

ZC_Spliced_Model_M = ZC_Spliced_Model_M_Test(line_start:line_end,:,:);

save('ZC_Spliced_Model_M_2016.mat','ZC_Spliced_Model_M');

% Tables

for n = 1:Nc
    
Table(:,n) = [100*nanmean(yields_foreign_all(:,7,n)); 100*nanmean(yields_model_selected_all(:,7,n)); 100*nanstd(logSDF_all(:,1,n))*sqrt(12); nancorr(logSDF_all(:,1,n), logSDF_all(:,1,end)); 100*nanstd(logSDF_all(:,1,end)-logSDF_all(:,1,n))*sqrt(12)];

end

Table_Selected_2015 = Table(:,[10 1 2 3 4 6 8 9]);

disp('----------------------------------------------'); 
disp('Results for Selected G10 Countries (1985-2015 sample)');
disp('----------------------------------------------');     
latex(Table_Selected_2015,'%-4.2f');
