clear 
close all
restoredefaultpath
addpath(genpath('jsz_library'))
clc

% This gives a sample estimation of an N-factor "without error" GDTSM.
% See "A New Perspective on Gaussian Dynamic Term Structure Models" by Joslin, Singleton and Zhu

% Load some data: mats (1*J) and yields (T*J)
load('sample_zeros.mat') 
% For the US, maturities are  0.5    1.0    2.0    3.0    5.0    7.0   10

load('ZC_Spliced_M.mat','ZC_Spliced_M');% Maturities are 3, 6, 9, ..., 120 or 180 months (ie up to 15 years max)
ZC_Sample = ZC_Spliced_M;
Countries_Sample = {'Australia';'Canada';'Germany'; 'Japan';'New Zealand';'Norway';'Sweden';'Switzerland';'United Kingdom';'United States'}; 
Nc = size(Countries_Sample,1);

ZC_UK = ZC_Sample(:,:,end-1);
% Replace US yields with UK yields
ZC_UK_selection = [ZC_UK(:,2) ZC_UK(:,1+4) ZC_UK(:,1+2*4) ZC_UK(:,1+3*4) ZC_UK(:,1+5*4) ZC_UK(:,1+7*4) ZC_UK(:,1+10*4)];

% Same time window
line_start = find(ZC_UK(:,1) == dates(1,1));
line_end = find(ZC_UK(:,1) == dates(end,1));
yields_UK = ZC_UK_selection(line_start:line_end,:);

% replace US yields by UK yields
countrychoice = 1;

switch countrychoice
    case 1 % US
        
    case 2 % UK
        yields = yields_UK;
end;


% Setup format of model/data:
M = 200*12;  % maximum maturity (in months) to consider; M+1 is infinite maturity
N = 3;
W_pca = pcacov(cov(yields));
W = W_pca(:,1:N)';       % N*J
PC1_obs = W_pca(:,1)';   % 1*J
PC2_obs = W_pca(:,2)';   % 1*J
PC3_obs = W_pca(:,3)';   % 1*J
PC4_obs = W_pca(:,4)';   % 1*J
PC5_obs = W_pca(:,5)';   % 1*J
cP = yields*W';          % T*N
dt = 1/12;
T  = length(dates);

% Estimate the model by ML. 
help sample_estimation_fun
VERBOSE = true;
% [llks, AcP, BcP, AX, BX, kinfQ, K0P_cP, K1P_cP, sigma_e, K0Q_cP, K1Q_cP, rho0_cP, rho1_cP, cP, llkP, llkQ,  K0Q_X, K1Q_X, rho0_X, rho1_X] = ...
%         sample_estimation_fun(W, yields, mats, dt, VERBOSE);


[llks, AcP, BcP, AX, BX, kinfQ, K0P_cP, K1P_cP, sigma_e, K0Q_cP, K1Q_cP, rho0_cP, rho1_cP, cP, llkP, llkQ,  K0Q_X, K1Q_X, rho0_X, rho1_X, Sigma_cP, dlamQ] = ...
        sample_estimation_fun(W, yields, mats, dt, VERBOSE);
    
check0 =  yields*W.' - cP;                          % should be all zeros   

% Replicate Tables 2, 3 and 4
lamQ   = sort(eig(K1Q_X),'descend');                % Table 2
rinfQ  = -kinfQ/lamQ(1,1)*100;                      % Table 2

K1P_cP_annual = K1P_cP*12;                          % Table 3
theta  = -inv(K1P_cP)*(K0P_cP);                     % Table 3

Sigma_cP_annual = Sigma_cP*12;
Vol_cP          = sqrt(diag(Sigma_cP_annual))*100;  % expressed in percentage terms (Table 4)
Corr_cP         = 10000*[Sigma_cP_annual(1,2)/(Vol_cP(1,1)*Vol_cP(2,1)), Sigma_cP_annual(1,3)/(Vol_cP(1,1)*Vol_cP(3,1)), Sigma_cP_annual(2,3)/(Vol_cP(2,1)*Vol_cP(3,1))]'; % Table 4

yields_JSZ     = ones(T,1)*AcP + (yields*W.')*BcP;
log_prices_JSZ = -yields_JSZ.*repmat(mats,216,1);
prices_JSZ     = exp(log_prices_JSZ);

A_price_JSZ    = - AcP.*(mats*12);
B_price_JSZ    = - BcP.*repmat(mats*12,N,1);

check1         = ones(T,1)*A_price_JSZ/12 + (yields*W.')*B_price_JSZ/12 - log_prices_JSZ;  % should be all zeros

rf_model = rho0_cP + (yields*W.')*rho1_cP;

figure(1)
plot(year(dates) + month(dates)/12, yields_JSZ);
xlabel('date')
ylabel('yields')


% Maturities
A_price(:,1)  = -rho0_cP/12;
B_price(:,1)  = -rho1_cP/12;

H0 = Sigma_cP;

for m = 2:M
    
    A_price(1,m) = A_price(1,m-1) + (K0Q_cP)'*B_price(:,m-1) + 0.5*B_price(:,m-1)'*H0*B_price(:,m-1) - rho0_cP/12;
    B_price(:,m) = B_price(:,m-1) + (K1Q_cP)'*B_price(:,m-1) - rho1_cP/12;
       
end

A_price          = 12*A_price;
B_price          = 12*B_price;

A_yield = - A_price./[1:M]; 
B_yield = - B_price./repmat([1:M],N,1);

B_price(:,M+1)   = (K1Q_cP')\rho1_cP;

yields_model     = ones(T,1)*A_yield + (yields*W.')*B_yield(:,1:M);
log_prices_model = -yields_model.*repmat([1:M]/12,216,1);
prices_model     = exp(log_prices_model);

check2           = ones(T,1)*A_price/12 + (yields*W.')*B_price(:,1:M)/12 - log_prices_model;  % should be all zeros

A_price_selected = A_price(:, mats*12);
B_price_selected = B_price(:, mats*12);

A_yield_selected = A_yield(:, mats*12);
B_yield_selected = B_yield(:, mats*12);

yields_model_selected = yields_model(:, mats*12); 
prices_model_selected = prices_model(:, mats*12);

check3 = A_price_JSZ - A_price(:, mats*12);  % should be all zeros
check4 = B_price_JSZ - B_price(:, mats*12);  % should be all zeros
check5 = AcP - A_yield(:, mats*12);          % should be all zeros
check6 = BcP - B_yield(:, mats*12);          % should be all zeros
check7 = yields_JSZ - yields_model_selected; % should be all zeros
check8 = prices_JSZ - prices_model_selected; % should be all zeros

T10_yields = [mean(rf_model) mean(yields_model(:,360))];   % Table 10

W_pca_model = pcacov(cov(yields_model_selected));
PC1_model = W_pca_model(:,1)';  
PC2_model = W_pca_model(:,2)'; 
PC3_model = W_pca_model(:,3)';  
PC4_model = W_pca_model(:,4)';  
PC5_model = W_pca_model(:,5)';   

% Bond returns
log_1m_return_model        = NaN*zeros(T,M+1);
log_1m_return_model_expr   = NaN*zeros(T,M);

log_3m_return_model        = NaN*zeros(T,M+1);
log_3m_return_model_expr   = NaN*zeros(T,M);

log_12m_return_model       = NaN*zeros(T,M+1);
log_12m_return_model_expr  = NaN*zeros(T,M);

% 1-month returns
for t = 1:T-1
    
    for m = 1
        
    log_1m_return_model(t+1,m)      = - log_prices_model(t,m);
    log_1m_return_model_expr(t+1,m) = - A_price(1,m)/12 - cP(t,:)*B_price(:,m)/12;    
        
    end
    
    for m = 2:M
    
    log_1m_return_model(t+1,m)      = log_prices_model(t+1,m-1) - log_prices_model(t,m);
    log_1m_return_model_expr(t+1,m) = A_price(1,m-1)/12 - A_price(1,m)/12 + cP(t+1,:)*B_price(:,m-1)/12 - cP(t,:)*B_price(:,m)/12;
    
    end
    
    log_1m_return_model(t+1,M+1)   = -[(K0Q_cP)'*B_price(:,M+1)/12 + 0.5*(B_price(:,M+1)/12)'*H0*(B_price(:,M+1)/12) - rho0_cP/12] + cP(t+1,:)*B_price(:,M+1)/12 - cP(t,:)*B_price(:,M+1)/12;
    
end

%3-month returns
for t = 1:T-3
    
    for m = 3
        
    log_3m_return_model(t+3,m)      = - log_prices_model(t,m);
    log_3m_return_model_expr(t+3,m) = - A_price(1,m)/12 - cP(t,:)*B_price(:,m)/12;    
        
    end
    
    for m = 4:M
    
    log_3m_return_model(t+3,m)      = log_prices_model(t+3,m-3) - log_prices_model(t,m);
    log_3m_return_model_expr(t+3,m) = A_price(1,m-3)/12 - A_price(1,m)/12 + cP(t+3,:)*B_price(:,m-3)/12 - cP(t,:)*B_price(:,m)/12;
    
    end
    
    log_3m_return_model(t+3,M+1)   = -3*[(K0Q_cP)'*B_price(:,M+1)/12 + 0.5*(B_price(:,M+1)/12)'*H0*(B_price(:,M+1)/12) - rho0_cP/12] + cP(t+3,:)*B_price(:,M+1)/12 - cP(t,:)*B_price(:,M+1)/12;
    
end

save(strcat('log_3m_return_model_',int2str(countrychoice),'.mat'),'log_3m_return_model');
save(strcat('yields_model_selected_',int2str(countrychoice),'.mat'),'yields_model_selected');


% Compare HPR difference (infinity vs 10yr, UK vs US)
countrychoice = 1; %US
load(strcat('log_3m_return_model_',int2str(countrychoice),'.mat'),'log_3m_return_model');
log_3m_return_model_US = log_3m_return_model;
load(strcat('yields_model_selected_',int2str(countrychoice),'.mat'),'yields_model_selected');
yields_model_selected_US = yields_model_selected;

countrychoice = 2; %UK
load(strcat('log_3m_return_model_',int2str(countrychoice),'.mat'),'log_3m_return_model');
log_3m_return_model_UK = log_3m_return_model;
load(strcat('yields_model_selected_',int2str(countrychoice),'.mat'),'yields_model_selected');
yields_model_selected_UK = yields_model_selected;

disp('US')
disp(nanmean(log_3m_return_model_US(:,end))*400-nanmean(log_3m_return_model_US(:,3))*400);
disp(nanmean(log_3m_return_model_US(:,120))*400-nanmean(log_3m_return_model_US(:,3))*400);
disp(nanmean(log_3m_return_model_US(:,end))*400-nanmean(log_3m_return_model_US(:,120))*400);

disp('UK')
disp(nanmean(log_3m_return_model_UK(:,end))*400-nanmean(log_3m_return_model_UK(:,3))*400);
disp(nanmean(log_3m_return_model_UK(:,120))*400-nanmean(log_3m_return_model_UK(:,3))*400);
disp(nanmean(log_3m_return_model_UK(:,end))*400-nanmean(log_3m_return_model_UK(:,120))*400);

% Build conditional HPR
cond_HPR_US = log_3m_return_model_US;
for t=1:size(log_3m_return_model_US,1)
    if yields_model_selected_US(t,1)<yields_model_selected_US(t,end)
        cond_HPR_US(t,:) = log_3m_return_model_US(t,:);
    else
        cond_HPR_US(t,:) = -log_3m_return_model_US(t,:);
    end;
end;

cond_HPR_UK = log_3m_return_model_UK;
for t=1:size(log_3m_return_model_UK,1)
    if yields_model_selected_UK(t,1)<yields_model_selected_UK(t,end)
        cond_HPR_UK(t,:) = log_3m_return_model_UK(t,:);
    else
        cond_HPR_UK(t,:) = -log_3m_return_model_UK(t,:);
    end;
end;
 
cond_HPR_High = log_3m_return_model_UK;
cond_HPR_Low  = log_3m_return_model_UK;
for t=1:size(log_3m_return_model_UK,1)
    if yields_model_selected_UK(t,end)-yields_model_selected_UK(t,1) > yields_model_selected_US(t,end)-yields_model_selected_US(t,1)
        cond_HPR_High(t,:) = log_3m_return_model_UK(t,:);
        cond_HPR_Low(t,:)  = log_3m_return_model_US(t,:);
    else
        cond_HPR_High(t,:) = log_3m_return_model_US(t,:);
        cond_HPR_Low(t,:)  = log_3m_return_model_UK(t,:);
    end;
end;

disp('US')
disp(nanmean(cond_HPR_US(:,end))*400-nanmean(log_3m_return_model_US(:,3))*400);
disp(nanmean(cond_HPR_US(:,120))*400-nanmean(log_3m_return_model_US(:,3))*400);
disp(nanmean(cond_HPR_US(:,end))*400-nanmean(cond_HPR_US(:,120))*400);

disp('UK')
disp(nanmean(cond_HPR_UK(:,end))*400-nanmean(log_3m_return_model_UK(:,3))*400);
disp(nanmean(cond_HPR_UK(:,120))*400-nanmean(log_3m_return_model_UK(:,3))*400);
disp(nanmean(cond_HPR_UK(:,end))*400-nanmean(cond_HPR_UK(:,120))*400);

disp('Both')
disp(nanmean(cond_HPR_High(:,120))*400);
disp(nanmean(cond_HPR_Low(:,120))*400);
disp(nanmean(cond_HPR_High(:,end))*400-nanmean(cond_HPR_Low(:,end))*400);
disp(nanmean(cond_HPR_High(:,120))*400-nanmean(cond_HPR_Low(:,120))*400);

diffindiff= (cond_HPR_High(:,120) - cond_HPR_Low(:,120)) - (cond_HPR_High(:,end) - cond_HPR_Low(:,end));
disp(nanmean(diffindiff*400));
disp(nanstd(diffindiff*400)/sqrt(216));

break;







%12-month returns
for t = 1:T-12
    
    for m = 12
        
    log_12m_return_model(t+12,m)      = - log_prices_model(t,m);
    log_12m_return_model_expr(t+12,m) = - A_price(1,m)/12 - cP(t,:)*B_price(:,m)/12;    
        
    end
    
    for m = 13:M
    
    log_12m_return_model(t+12,m)      = log_prices_model(t+12,m-12) - log_prices_model(t,m);
    log_12m_return_model_expr(t+12,m) = A_price(1,m-12)/12 - A_price(1,m)/12 + cP(t+12,:)*B_price(:,m-12)/12 - cP(t,:)*B_price(:,m)/12;
    
    end
    
    log_12m_return_model(t+12,M+1)   = -[(K0Q_cP)'*B_price(:,M+1)/12 + 0.5*(B_price(:,M+1)/12)'*H0*(B_price(:,M+1)/12) - rho0_cP/12] + cP(t+12,:)*B_price(:,M+1)/12 - cP(t,:)*B_price(:,M+1)/12;
    
end

log_1m_return_model       = log_1m_return_model(2:T,:);
log_1m_return_model_expr  = log_1m_return_model_expr(2:T,:);

log_3m_return_model       = log_3m_return_model(2:T,:);
log_3m_return_model_expr  = log_3m_return_model_expr(2:T,:);

log_12m_return_model      = log_12m_return_model(2:T,:);
log_12m_return_model_expr = log_12m_return_model_expr(2:T,:);

check9  = log_1m_return_model(:,1:M) - log_1m_return_model_expr;     % should be all zeros
check10 = log_3m_return_model(:,1:M) - log_3m_return_model_expr;     % should be all zeros
check11 = log_12m_return_model(:,1:M) - log_12m_return_model_expr;   % should be all zeros

grs_1m_return_model   = exp(log_1m_return_model);    % gross returns (1 month)
net_1m_return_model   = grs_1m_return_model -1;      % net returns (1 month)

grs_3m_return_model   = exp(log_3m_return_model);    % gross returns (3 months)
net_3m_return_model   = grs_3m_return_model -1;      % net returns (3 months)

grs_12m_return_model  = exp(log_12m_return_model);   % gross returns (12 months)
net_12m_return_model  = grs_12m_return_model -1;     % net returns (12 months)

log_1m_xreturn_model  = log_1m_return_model - repmat(log_1m_return_model(:,1),1,M+1);  % log excess returns (1 month)
grs_1m_xreturn_model  = grs_1m_return_model - repmat(grs_1m_return_model(:,1),1,M+1);  % gross excess returns (1 month)
net_1m_xreturn_model  = net_1m_return_model - repmat(net_1m_return_model(:,1),1,M+1);  % net excess returns (1 month)

log_3m_xreturn_model  = log_3m_return_model - repmat(log_3m_return_model(:,3),1,M+1);  % log excess returns (3 month)
grs_3m_xreturn_model  = grs_3m_return_model - repmat(grs_3m_return_model(:,3),1,M+1);  % gross excess returns (3 months)
net_3m_xreturn_model  = net_3m_return_model - repmat(net_3m_return_model(:,3),1,M+1);  % net excess returns (3 months)

log_12m_xreturn_model = log_12m_return_model - repmat(log_12m_return_model(:,12),1,M+1);  % log excess returns (12 months)
grs_12m_xreturn_model = grs_12m_return_model - repmat(grs_12m_return_model(:,12),1,M+1);  % gross excess returns (12 months)
net_12m_xreturn_model = net_12m_return_model - repmat(net_12m_return_model(:,12),1,M+1);  % net excess returns (12 months)

% Conditional Sharpe ratios
cP_expected        = NaN*zeros(T,N);
cP_expected(2:T,:) = cP(1:T-1,:) + repmat(K0P_cP',T-1,1) + cP(1:T-1,:)*K1P_cP';
cP_error           = cP(2:T,:) - cP_expected(2:T,:);

cP_error_mean = mean(cP_error)';
cP_error_cov  = cov(cP_error);
cP_error_vol  = sqrt(diag(cP_error_cov)*12)*100;
cP_error_corr = 120000*[cP_error_cov(1,2)/(cP_error_vol(1,1)*cP_error_vol(2,1)), cP_error_cov(1,3)/(cP_error_vol(1,1)*cP_error_vol(3,1)), cP_error_cov(2,3)/(cP_error_vol(2,1)*cP_error_vol(3,1))]';

check12       = cP_error_mean;          % check if average error is close to zero
check13       = cP_error_cov./Sigma_cP; % should be close to all ones 
check14       = cP_error_vol./Vol_cP;   % should be close to all ones
check15       = cP_error_corr./Corr_cP; % should be close to all ones

log_1m_er      = zeros(T-1,M+1);     % log expected return
log_1m_er_NJ   = zeros(T-1,M+1);     % log expected return (without Jensen term)
log_1m_rp_expr = zeros(T-1,M+1);     % log risk premia (analytical)
log_1m_cvar    = zeros(T-1,M+1);     % log conditional variance
log_1m_ccov    = zeros(T-1,M+1,M+1); % log conditional covariance matrix

for m = 2:M

    log_1m_cvar(:,m) = repmat((B_price(:,m-1)/12)'*Sigma_cP*(B_price(:,m-1)/12), T-1, 1);

end

 log_1m_cvar(:,M+1)  = repmat((B_price(:,M+1)/12)'*Sigma_cP*(B_price(:,M+1)/12), T-1, 1);
 
 for i = 2:M
     
     for j = 2:M
         
         log_1m_ccov(:,i,j) = repmat((B_price(:,i-1)/12)'*Sigma_cP*(B_price(:,j-1)/12), T-1, 1);
         
     end     
     
 end
 
 for i = 2:M
     
     log_1m_ccov(:,i,M+1) = repmat((B_price(:,i-1)/12)'*Sigma_cP*(B_price(:,M+1)/12), T-1, 1);
     log_1m_ccov(:,M+1,i) = repmat((B_price(:,M+1)/12)'*Sigma_cP*(B_price(:,i-1)/12), T-1, 1);
     
 end
 
 log_1m_ccov(:,M+1,M+1) = log_1m_cvar(:,M+1);
 
 log_1m_cvol = sqrt(log_1m_cvar); % log conditional volatility

for t = 1:T-1
    
    for m = 1
    
    log_1m_er(t,m)    = - A_price(1,m)/12 - cP(t,:)*B_price(:,m)/12 + 0.5*log_1m_cvar(t,m);
    log_1m_er_NJ(t,m) = - A_price(1,m)/12 - cP(t,:)*B_price(:,m)/12;
        
    end
    
    for m = 2:M
    
    log_1m_er(t,m)      = [A_price(1,m-1)/12 - A_price(1,m)/12] + cP_expected(t+1,:)*B_price(:,m-1)/12 - cP(t,:)*B_price(:,m)/12 + 0.5*log_1m_cvar(t,m);
    log_1m_er_NJ(t,m)   = [A_price(1,m-1)/12 - A_price(1,m)/12] + cP_expected(t+1,:)*B_price(:,m-1)/12 - cP(t,:)*B_price(:,m)/12;
    log_1m_rp_expr(t,m) = ((K0P_cP-K0Q_cP)' + cP(t,:)*(K1P_cP-K1Q_cP)')*B_price(:,m-1)/12;
    
    end
    
    log_1m_er(t,M+1)      = -[(K0Q_cP)'*B_price(:,M+1)/12 + 0.5*(B_price(:,M+1)/12)'*H0*(B_price(:,M+1)/12) - rho0_cP/12] + cP_expected(t+1,:)*B_price(:,M+1)/12 - cP(t,:)*B_price(:,M+1)/12 + 0.5*log_1m_cvar(t,M+1);
    log_1m_er_NJ(t,M+1)   = -[(K0Q_cP)'*B_price(:,M+1)/12 + 0.5*(B_price(:,M+1)/12)'*H0*(B_price(:,M+1)/12) - rho0_cP/12] + cP_expected(t+1,:)*B_price(:,M+1)/12 - cP(t,:)*B_price(:,M+1)/12;
    log_1m_rp_expr(t,M+1) = ((K0P_cP-K0Q_cP)' + cP(t,:)*(K1P_cP-K1Q_cP)')*B_price(:,M+1)/12;

end

log_1m_rp       = log_1m_er - repmat(rf_model(1:T-1,1)/12,1,M+1);    % log risk premium
log_1m_rp_NJ    = log_1m_er_NJ - repmat(rf_model(1:T-1,1)/12,1,M+1); % log risk premium (without Jensen term)
log_1m_cSR      = log_1m_rp./log_1m_cvol;

check16         = log_1m_er(:,1) - rf_model(1:T-1,1)/12; % should be all zeros
check17         = log_1m_rp - log_1m_rp_expr;            % should be all zeros

log_1m_er_avg   = mean(log_1m_er)';
log_1m_rp_avg   = mean(log_1m_rp)';
log_1m_cvol_avg = mean(log_1m_cvol)';
log_1m_cSR_avg  = mean(log_1m_cSR)';
log_1m_cSR_vol  = std(log_1m_cSR)';
log_1m_cmom     = [log_1m_er_avg, log_1m_rp_avg, log_1m_cvol_avg, log_1m_cSR_avg, log_1m_cSR_vol];
log_1m_cmom_JSZ = [mats', log_1m_cmom(12*mats, :)];
log_1m_cmom_selected = [[5,10,15,20,25,30,100,M/12,inf]', log_1m_cmom([60, 120, 180, 240, 300, 360, 1200, M, M+1], :)];

grs_1m_er       = exp(log_1m_er);                                      % gross expected return
grs_1m_rp       = grs_1m_er - repmat(exp(rf_model(1:T-1,1)/12),1,M+1); % gross risk premium
grs_1m_cvar     = (exp(log_1m_cvar) -1).*((grs_1m_er).^2);             % gross conditional variance
grs_1m_cvol     = sqrt(grs_1m_cvar);                                   % gross conditional volatility
grs_1m_cSR      = grs_1m_rp./grs_1m_cvol;                              % gross Sharpe ratio
grs_1m_ccov     = NaN*zeros(T-1,M+1,M+1);                              % gross conditional covariance

grs_1m_er_avg   = mean(grs_1m_er)';
grs_1m_rp_avg   = mean(grs_1m_rp)';
grs_1m_cvol_avg = mean(grs_1m_cvol)';
grs_1m_cSR_avg  = mean(grs_1m_cSR)';
grs_1m_cSR_vol  = std(grs_1m_cSR)';
grs_1m_cmom     = [grs_1m_er_avg, grs_1m_rp_avg, grs_1m_cvol_avg, grs_1m_cSR_avg, grs_1m_cSR_vol];
grs_1m_cmom_JSZ = [mats', grs_1m_cmom(12*mats, :)];
grs_1m_cmom_selected = [[5,10,15,20,25,30,100,M/12,inf]', grs_1m_cmom([60, 120, 180, 240, 300, 360, 1200, M, M+1], :)];

for i = 1:M+1
    
    for j = 1:M+1
        
        grs_1m_ccov(:,i,j) = (exp(log_1m_ccov(:,i,j)) - 1).*(grs_1m_er(:,i)).*(grs_1m_er(:,j));
        
    end
   
end

for t = 1:T-1

    PC4_obs_log_1m_cSR(t,1)   = (PC4_obs*log_1m_rp(t,12*mats)')/sqrt(PC4_obs*reshape(log_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC4_obs');
    PC5_obs_log_1m_cSR(t,1)   = (PC5_obs*log_1m_rp(t,12*mats)')/sqrt(PC5_obs*reshape(log_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC5_obs');
    PC4_obs_grs_1m_cSR(t,1)   = (PC4_obs*grs_1m_rp(t,12*mats)')/sqrt(PC4_obs*reshape(grs_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC4_obs');
    PC5_obs_grs_1m_cSR(t,1)   = (PC5_obs*grs_1m_rp(t,12*mats)')/sqrt(PC5_obs*reshape(grs_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC5_obs');
    PC4_obs_log_1m_cSR_NJ(t,1)   = (PC4_obs*log_1m_rp_NJ(t,12*mats)')/sqrt(PC4_obs*reshape(log_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC4_obs');
    PC5_obs_log_1m_cSR_NJ(t,1)   = (PC5_obs*log_1m_rp_NJ(t,12*mats)')/sqrt(PC5_obs*reshape(log_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC5_obs');
    PC4_model_log_1m_cSR(t,1)    = (PC4_model*log_1m_rp(t,12*mats)')/sqrt(PC4_model*reshape(log_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC4_model');
    PC5_model_log_1m_cSR(t,1)    = (PC5_model*log_1m_rp(t,12*mats)')/sqrt(PC5_model*reshape(log_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC5_model');
    PC4_model_grs_1m_cSR(t,1)    = (PC4_model*grs_1m_rp(t,12*mats)')/sqrt(PC4_model*reshape(grs_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC4_model');
    PC5_model_grs_1m_cSR(t,1)    = (PC5_model*grs_1m_rp(t,12*mats)')/sqrt(PC5_model*reshape(grs_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC5_model');
    PC4_model_log_1m_cSR_NJ(t,1) = (PC4_model*log_1m_rp_NJ(t,12*mats)')/sqrt(PC4_model*reshape(log_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC4_model');
    PC5_model_log_1m_cSR_NJ(t,1) = (PC5_model*log_1m_rp_NJ(t,12*mats)')/sqrt(PC5_model*reshape(log_1m_ccov(t,12*mats,12*mats), length(mats), length(mats))*PC5_model');

end

T10_cSR = [nanmean(PC4_obs_log_1m_cSR), nanstd(PC4_obs_log_1m_cSR), nanmean(PC5_obs_log_1m_cSR), nanstd(PC5_obs_log_1m_cSR); ...
           nanmean(PC4_obs_grs_1m_cSR), nanstd(PC4_obs_grs_1m_cSR), nanmean(PC5_obs_grs_1m_cSR), nanstd(PC5_obs_grs_1m_cSR); ...
           nanmean(PC4_obs_log_1m_cSR_NJ), nanstd(PC4_obs_log_1m_cSR_NJ), nanmean(PC5_obs_log_1m_cSR_NJ), nanstd(PC5_obs_log_1m_cSR_NJ); ...
           nanmean(PC4_model_log_1m_cSR), nanstd(PC4_model_log_1m_cSR), nanmean(PC5_model_log_1m_cSR), nanstd(PC5_model_log_1m_cSR); ...
           nanmean(PC4_model_grs_1m_cSR), nanstd(PC4_model_grs_1m_cSR), nanmean(PC5_model_grs_1m_cSR), nanstd(PC5_model_grs_1m_cSR); ...
           nanmean(PC4_model_log_1m_cSR_NJ), nanstd(PC4_model_log_1m_cSR_NJ), nanmean(PC5_model_log_1m_cSR_NJ), nanstd(PC5_model_log_1m_cSR_NJ);]';

net_1m_er       = grs_1m_er - 1;                                         % net expected return
net_1m_rp       = net_1m_er - repmat(exp(rf_model(1:T-1,1)/12)-1,1,M+1); % net risk premium
net_1m_cvar     = grs_1m_cvar;                                           % net conditional variance
net_1m_cvol     = sqrt(net_1m_cvar);                                     % net conditional volatility
net_1m_cSR      = net_1m_rp./net_1m_cvol;                                % net Sharpe ratio

net_1m_er_avg   = mean(net_1m_er)';
net_1m_rp_avg   = mean(net_1m_rp)';
net_1m_cvol_avg = mean(net_1m_cvol)';
net_1m_cSR_avg  = mean(net_1m_cSR)';
net_1m_cSR_vol  = std(net_1m_cSR)';
net_1m_cmom     = [net_1m_er_avg, net_1m_rp_avg, net_1m_cvol_avg, net_1m_cSR_avg, net_1m_cSR_vol];
net_1m_cmom_JSZ = [mats', net_1m_cmom(12*mats, :)];
net_1m_cmom_selected = [[5,10,15,20,25,30,100,M/12,inf]', net_1m_cmom([60, 120, 180, 240, 300, 360, 1200, M, M+1], :)];

% Return properties
log_mom_selected         = NaN*zeros(10,8,3);
grs_mom_selected         = NaN*zeros(10,8,3);
net_mom_selected         = NaN*zeros(10,8,3);
log_mom_inf_selected     = NaN*zeros(10,8,3);
grs_mom_inf_selected     = NaN*zeros(10,8,3);
net_mom_inf_selected     = NaN*zeros(10,8,3);

log_return_model_all(:,:,1) = log_1m_return_model;
log_return_model_all(:,:,2) = log_3m_return_model;
log_return_model_all(:,:,3) = log_12m_return_model;

grs_return_model_all(:,:,1) = grs_1m_return_model;
grs_return_model_all(:,:,2) = grs_3m_return_model;
grs_return_model_all(:,:,3) = grs_12m_return_model;

net_return_model_all(:,:,1) = net_1m_return_model;
net_return_model_all(:,:,2) = net_3m_return_model;
net_return_model_all(:,:,3) = net_12m_return_model;

log_xreturn_model_all(:,:,1) = log_1m_xreturn_model;
log_xreturn_model_all(:,:,2) = log_3m_xreturn_model;
log_xreturn_model_all(:,:,3) = log_12m_xreturn_model;

grs_xreturn_model_all(:,:,1) = grs_1m_xreturn_model;
grs_xreturn_model_all(:,:,2) = grs_3m_xreturn_model;
grs_xreturn_model_all(:,:,3) = grs_12m_xreturn_model;

net_xreturn_model_all(:,:,1) = net_1m_xreturn_model;
net_xreturn_model_all(:,:,2) = net_3m_xreturn_model;
net_xreturn_model_all(:,:,3) = net_12m_xreturn_model;

for k = 1:3
    
log_return_model  = log_return_model_all(:,:,k);
grs_return_model  = grs_return_model_all(:,:,k);
net_return_model  = net_return_model_all(:,:,k);
log_xreturn_model = log_xreturn_model_all(:,:,k);
grs_xreturn_model = grs_xreturn_model_all(:,:,k);
net_xreturn_model = net_xreturn_model_all(:,:,k);
      
log_avg  = nanmean(log_return_model)';
log_std  = nanstd(log_return_model)';
log_var  = nanvar(log_return_model)';
log_SR   = (nanmean(log_xreturn_model) + 0.5*nanvar(log_xreturn_model))'./nanstd(log_xreturn_model)';
log_cor  = nancorr(log_return_model,log_return_model(:,M+1));
log_skw  = skewness(log_return_model)';
log_kur  = kurtosis(log_return_model)';
log_mom  = [log_avg, log_std, log_var, log_SR, log_cor, log_skw, log_kur];

grs_avg  = nanmean(grs_return_model)';
grs_std  = nanstd(grs_return_model)';
grs_var  = nanvar(grs_return_model)';
grs_SR   = nanmean(grs_xreturn_model)'./nanstd(grs_xreturn_model)';
grs_cor  = nancorr(grs_return_model,grs_return_model(:,M+1));
grs_skw  = skewness(grs_return_model)';
grs_kur  = kurtosis(grs_return_model)';
grs_mom  = [grs_avg, grs_std, grs_var, grs_SR, grs_cor, grs_skw, grs_kur];

net_avg  = nanmean(net_return_model)';
net_std  = nanstd(net_return_model)';
net_var  = nanvar(net_return_model)';
net_SR   = nanmean(net_xreturn_model)'./nanstd(net_xreturn_model)';
net_cor  = nancorr(net_return_model, net_return_model(:,M+1));
net_skw  = skewness(net_return_model)';
net_kur  = kurtosis(net_return_model)';
net_mom  = [net_avg, net_std, net_var, net_SR, net_cor, net_skw, net_kur];

log_avg_inf  = log_avg./repmat(log_avg(M+1,:),M+1,1);
log_std_inf  = log_std./repmat(log_std(M+1,:),M+1,1);
log_var_inf  = log_var./repmat(log_var(M+1,:),M+1,1);
log_SR_inf   = log_SR./repmat(log_SR(M+1,:),M+1,1);
log_cor_inf  = log_cor;
log_skw_inf  = log_skw./repmat(log_skw(M+1,:),M+1,1);
log_kur_inf  = log_kur./repmat(log_kur(M+1,:),M+1,1);
log_mom_inf  = [1./log_avg_inf, 1./log_std_inf, 1./log_var_inf, 1./log_SR_inf, log_cor_inf, 1./log_skw_inf, 1./log_kur_inf];

grs_avg_inf  = grs_avg./repmat(grs_avg(M+1,:),M+1,1);
grs_std_inf  = grs_std./repmat(grs_std(M+1,:),M+1,1);
grs_var_inf  = grs_var./repmat(grs_var(M+1,:),M+1,1);
grs_SR_inf   = grs_SR./repmat(grs_SR(M+1,:),M+1,1);
grs_cor_inf  = grs_cor;
grs_skw_inf  = grs_skw./repmat(grs_skw(M+1,:),M+1,1);
grs_kur_inf  = grs_kur./repmat(grs_kur(M+1,:),M+1,1);
grs_mom_inf  = [1./grs_avg_inf, 1./grs_std_inf, 1./grs_var_inf, 1./grs_SR_inf, grs_cor_inf, 1./grs_skw_inf, 1./grs_kur_inf];

net_avg_inf  = net_avg./repmat(net_avg(M+1,:),M+1,1);
net_std_inf  = net_std./repmat(net_std(M+1,:),M+1,1);
net_var_inf  = net_var./repmat(net_var(M+1,:),M+1,1);
net_SR_inf   = net_SR./repmat(net_SR(M+1,:),M+1,1);
net_cor_inf  = net_cor;
net_skw_inf  = net_skw./repmat(net_skw(M+1,:),M+1,1);
net_kur_inf  = net_kur./repmat(net_kur(M+1,:),M+1,1);
net_mom_inf  = [1./net_avg_inf, 1./net_std_inf, 1./net_var_inf, 1./net_SR_inf, net_cor_inf, 1./net_skw_inf, 1./net_kur_inf];

if k == 1 
    
    min_hor = 1;
    
elseif k == 2
    
    min_hor = 3;
    
elseif k == 3
    
    min_hor = 12;
    
end

log_mom_selected(:,:,k)     = [[min_hor/12,5,10,15,20,25,30,100,M/12,inf]', log_mom([min_hor, 60, 120, 180, 240, 300, 360, 1200, M, M+1], :)];
grs_mom_selected(:,:,k)     = [[min_hor/12,5,10,15,20,25,30,100,M/12,inf]', grs_mom([min_hor, 60, 120, 180, 240, 300, 360, 1200, M, M+1], :)];
net_mom_selected(:,:,k)     = [[min_hor/12,5,10,15,20,25,30,100,M/12,inf]', net_mom([min_hor, 60, 120, 180, 240, 300, 360, 1200, M, M+1], :)];
log_mom_inf_selected(:,:,k) = [[min_hor/12,5,10,15,20,25,30,100,M/12,inf]', log_mom_inf([min_hor, 60, 120, 180, 240, 300, 360, 1200, M, M+1], :)];
grs_mom_inf_selected(:,:,k) = [[min_hor/12,5,10,15,20,25,30,100,M/12,inf]', grs_mom_inf([min_hor, 60, 120, 180, 240, 300, 360, 1200, M, M+1], :)];
net_mom_inf_selected(:,:,k) = [[min_hor/12,5,10,15,20,25,30,100,M/12,inf]', net_mom_inf([min_hor, 60, 120, 180, 240, 300, 360, 1200, M, M+1], :)];

end

log_1m_mom_selected     = log_mom_selected(:,:,1);
grs_1m_mom_selected     = grs_mom_selected(:,:,1);
net_1m_mom_selected     = net_mom_selected(:,:,1);
log_1m_mom_inf_selected = log_mom_inf_selected(:,:,1);
grs_1m_mom_inf_selected = grs_mom_inf_selected(:,:,1);
net_1m_mom_inf_selected = net_mom_inf_selected(:,:,1);

log_3m_mom_selected     = log_mom_selected(:,:,2);
grs_3m_mom_selected     = grs_mom_selected(:,:,2);
net_3m_mom_selected     = net_mom_selected(:,:,2);
log_3m_mom_inf_selected = log_mom_inf_selected(:,:,2);
grs_3m_mom_inf_selected = grs_mom_inf_selected(:,:,2);
net_3m_mom_inf_selected = net_mom_inf_selected(:,:,2);

log_12m_mom_selected     = log_mom_selected(:,:,3);
grs_12m_mom_selected     = grs_mom_selected(:,:,3);
net_12m_mom_selected     = net_mom_selected(:,:,3);
log_12m_mom_inf_selected = log_mom_inf_selected(:,:,3);
grs_12m_mom_inf_selected = grs_mom_inf_selected(:,:,3);
net_12m_mom_inf_selected = net_mom_inf_selected(:,:,3);

  