% ============================================= %
% Description
% ============================================= %

% Produces Tables 1-3 in the main text. Furthermore, it produces Tables A1-A13 and A18-A20, Panel A of Tables
% A14-A17, and Figures A1-A8 in the Online Appendix.

% ============================================= %
% Clear
% ============================================= %
clc;
close all;
% clear all;

% ============================================= %
% Options
% ============================================= %

%  date_begin  = datenum('1/31/1951');
date_begin  = datenum('1/31/1975');
%  date_begin  = datenum('10/31/1983');

 date_end    = datenum('12/31/2015'); 
%  date_end    = datenum('12/31/2007'); 


sample      = 3;
            % 1 = all countries
            % 2 = developed countries
            % 3 = G10 countries
            
writing  = 2; %saving portfolio composition (slow)
           
switch sample
    case 1
        Number_portfolios   = 5;
    case 2
        Number_portfolios   = 4;
    case 3
        Number_portfolios   = 3;
end

switch writing 
    case 1
        writing_results   =   1;
    case 2
        writing_results   =   2;
end

horizon       = 1;       % 1 : 1 month, 3 : 3 months, 12 : 12 months
nboot         = 10000;

sorting_variable    = 4; % 1 : Using the portfolio sorting variable at each date
                         % 2 : Using unconditional averages of the portfolio sorting variable
                         % 3 : Using deviations from the unconditional averages of the portfolio sorting variable
                         % 4 : Using deviations from the benchmark of the portfolio sorting variable
                                                  
NW_lag_choice       = 5; % 1 : lags = max(6,horizon+1) 
                         % 2 : lags = ceil(4*(size(BondsHPER,1)/100)^(2/9))-1
                         % 3 : lags = ceil(0.75*(size(BondsHPER,1))^(1/3))-1
                         % 4 : lags = max(ceil(4*(size(BondsHPER,1)/100)^(2/9))-1,ceil(0.75*(size(BondsHPER,1))^(1/3))-1)
                         % 5 : lags = ceil(1.3*(size(BondsHPER,1))^(1/2))-1
                         
                         
                         
ScalingFactor    = 12/horizon;

rng(1234);

% ============================================= %
% Import Bond Return Indices from GFD
% ============================================= %
str = computer;
if strcmp(str,'PCWIN')==1;
    newpath   =  strcat(pwd,'\');
elseif strcmp(str,'PCWIN64')==1;
    newpath   =  strcat(pwd,'\');
else
    newpath   =  strcat(pwd,'/');
end;

load(strcat(newpath,'Bonds_local_M.mat'),'GFD_Bonds_local_M');    % Bond indices in local currency
load(strcat(newpath,'Bonds_dollar_M.mat'),'GFD_Bonds_dollar_M');  % Bond indices in USD
load(strcat(newpath,'TB_M.mat'),'GFD_TB_M');                      % TB indices in local currency
load(strcat(newpath,'Yields_M.mat'),'GFD_Yields_M');              % Yields
load(strcat(newpath,'CPI_M.mat'),'CPI_M_GFD');                    % CPI 
load(strcat(newpath,'Ratings_Backfilled_M.mat'),'Ratings_Backfilled_M');                     % Backfilled ratings data (start in January 1974)   
load(strcat(newpath,'Ratings_wOutlook_Backfilled_M.mat'),'Ratings_wOutlook_Backfilled_M');   % Backfilled ratings (with outlook) data (start in January 1974)  
Ratings_M          = Ratings_Backfilled_M;
Ratings_wOutlook_M = Ratings_wOutlook_Backfilled_M;

Yields_M_GFD       = GFD_Yields_M(:,[1:45,47:54,56:end]);    % eliminate duplicate (46,55) 
TBILL_M_GFD        = GFD_TB_M(:,[1:13,15:52,54:56,58:end]);  % eliminate duplicate (14,53,57)
Bonds_M_GFD_local  = GFD_Bonds_local_M(:,[1:7,9:end]);       % eliminate duplicate (8)
Bonds_M_GFD_dollar = GFD_Bonds_dollar_M(:,[1:7,9:end]);      % eliminate duplicate (8)

Ratings_M(1,105)          = 528;  % add country code for Taiwan
Ratings_wOutlook_M(1,105) = 528;  % add country code for Taiwan

CPI_M_GFD           = [CPI_M_GFD(1,:); NaN(600,size(CPI_M_GFD,2)); CPI_M_GFD(2:end,:)];  % Change starting date to January 1900
CPI_M_GFD(2:601,1)  = GFD_Bonds_local_M(2:601,1);

Ratings_M           = [Ratings_M(1,:); NaN(900,size(Ratings_M,2)); Ratings_M(2:end,:)];  % Change starting date to January 1900
Ratings_M(2:901,1)  = GFD_Bonds_local_M(2:901,1);

Ratings_wOutlook_M           = [Ratings_wOutlook_M(1,:); NaN(900,size(Ratings_wOutlook_M,2)); Ratings_wOutlook_M(2:end,:)];  % Change starting date to January 1900
Ratings_wOutlook_M(2:901,1)  = GFD_Bonds_local_M(2:901,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% ============================================= %
% Select Sample
% ============================================= %

Countries_Wright = {'Australia';'Canada';'Germany';'Japan';'New Zealand';'Norway';'Sweden';'Switzerland';'United Kingdom'; 'United States'}; 
% Countries_Wright = {'Australia';'Canada';'Germany';'Japan';New Zealand';'Norway';''Sweden';'Switzerland';'United States';'United Kingdom'}; % U.K. as the home country

Countries_Dev = {'Australia';'Austria';'Belgium';'Canada';'Denmark';'Finland';'France';'Germany';...
        'Greece';'Ireland';'Italy';'Japan';'Netherlands';'New Zealand';'Norway';'Portugal';'Spain';'Sweden';'Switzerland';'United Kingdom'; 'United States'};    
    
Countries_All = {'Australia';'Austria';'Belgium';'Canada';'Denmark';...
        'Finland';'France';'Germany';...
        'Greece';'India';'Ireland';'Italy';'Japan';...
        'Mexico';'Malaysia';'Netherlands';'New Zealand';'Norway';'Pakistan';'Philippines';'Poland';...
        'Portugal';'South Africa';'Singapore'; ...
        'Spain';'Sweden';'Switzerland';'Taiwan';'Thailand';'United Kingdom'; 'United States'};

switch sample
    case 1
        Countries_Sample = Countries_All;
    case 2
        Countries_Sample = Countries_Dev;
    case 3
        Countries_Sample = Countries_Wright;
end;

[IMF_codes, IMF_names]         = xlsread('IMF_codes.xls');
ListIMFCodes                   = NaN*zeros(size(Countries_Sample,1),1);
ListIMFNames                   = cell(size(Countries_Sample,1),1);

for i=1:size(Countries_Sample,1);
    for j=1:size(IMF_names,1)
        if strcmpi(Countries_Sample(i,1),IMF_names(j,1))
            ListIMFCodes(i,1)=IMF_codes(j,1);
             ListIMFNames(i)=IMF_names(j);
        end;
    end;
end;


TBILL_M_GFD_Subsample                    = NaN*zeros(size(TBILL_M_GFD,1),1+size(Countries_Sample,1));
Yields_M_GFD_Subsample                   = NaN*zeros(size(Yields_M_GFD,1),1+size(Countries_Sample,1));
Bonds_M_GFD_dollar_Subsample             = NaN*zeros(size(Bonds_M_GFD_dollar,1),1+size(Countries_Sample,1));
Bonds_M_GFD_local_Subsample              = NaN*zeros(size(Bonds_M_GFD_local,1),1+size(Countries_Sample,1));
CPI_M_GFD_Subsample                      = NaN*zeros(size(CPI_M_GFD,1),1+size(Countries_Sample,1));
Ratings_M_Subsample                      = NaN*zeros(size(Ratings_M,1),1+size(Countries_Sample,1));
Ratings_wOutlook_M_Subsample             = NaN*zeros(size(Ratings_wOutlook_M,1),1+size(Countries_Sample,1));

TBILL_M_GFD_Subsample(:,1)                   = TBILL_M_GFD(:,1);                   % Dates
Yields_M_GFD_Subsample(:,1)                  = Yields_M_GFD(:,1);                  % Dates
Bonds_M_GFD_dollar_Subsample(:,1)            = Bonds_M_GFD_dollar(:,1);            % Dates
Bonds_M_GFD_local_Subsample(:,1)             = Bonds_M_GFD_local(:,1);             % Dates
CPI_M_GFD_Subsample(:,1)                     = CPI_M_GFD(:,1);                     % Dates
Ratings_M_Subsample(:,1)                     = Ratings_M(:,1);                     % Dates
Ratings_wOutlook_M_Subsample(:,1)            = Ratings_wOutlook_M(:,1);            % Dates

for i=1:size(ListIMFCodes,1)
   
   col_TB=find(TBILL_M_GFD(1,:)==ListIMFCodes(i,1));
    if isempty(col_TB)==0 
        TBILL_M_GFD_Subsample(:,1+i) = TBILL_M_GFD(:,col_TB);
    end;

    col_Ylds=find(Yields_M_GFD(1,:)==ListIMFCodes(i,1));
    if isempty(col_Ylds)==0 
        Yields_M_GFD_Subsample(:,1+i) = Yields_M_GFD(:,col_Ylds);
    end;
    
    col_Bonds_dollar=find(Bonds_M_GFD_dollar(1,:)==ListIMFCodes(i,1));
    if isempty(col_Bonds_dollar)==0 
        Bonds_M_GFD_dollar_Subsample(:,1+i) = Bonds_M_GFD_dollar(:,col_Bonds_dollar);
    end;
    
    col_Bonds =find(Bonds_M_GFD_local(1,:)==ListIMFCodes(i,1));
    if isempty(col_Bonds)==0 
        Bonds_M_GFD_local_Subsample(:,1+i) = Bonds_M_GFD_local(:,col_Bonds);
    end;
    
    col_CPI =find(CPI_M_GFD(1,:)==ListIMFCodes(i,1));
    if isempty(col_CPI)==0 
        CPI_M_GFD_Subsample(:,1+i) = CPI_M_GFD(:,col_CPI(1));
    end;
    
    col_Ratings =find(Ratings_M(1,:)==ListIMFCodes(i,1));
    if isempty(col_Ratings)==0 
        Ratings_M_Subsample(:,1+i) = Ratings_M(:,col_Ratings(1));
    end;
    
    col_RatingswO =find(Ratings_wOutlook_M(1,:)==ListIMFCodes(i,1));
    if isempty(col_RatingswO)==0 
        Ratings_wOutlook_M_Subsample(:,1+i) = Ratings_wOutlook_M(:,col_RatingswO(1));
    end;
    
end;

% Keep only subsample

TBILL_M_GFD                    = TBILL_M_GFD_Subsample;
Yields_M_GFD                   = Yields_M_GFD_Subsample;
Bonds_M_GFD_dollar             = Bonds_M_GFD_dollar_Subsample;
Bonds_M_GFD_local              = Bonds_M_GFD_local_Subsample;
CPI_M_GFD                      = CPI_M_GFD_Subsample;
Ratings_M                      = Ratings_M_Subsample;
Ratings_wOutlook_M             = Ratings_wOutlook_M_Subsample;


% ============================================= %
% Time-window
% ============================================= %

col_US           = find(TBILL_M_GFD(1,:)==111)-1;
col_base         = find(TBILL_M_GFD(1,:)==ListIMFCodes(end,1))-1;
  
line_begin_dol         = find(Bonds_M_GFD_dollar(:,1)==date_begin);
line_end_dol           = find(Bonds_M_GFD_dollar(:,1)==date_end);
BondsHPRDollar         = NaN(line_end_dol-line_begin_dol+1,size(Bonds_M_GFD_dollar,2)-1);
BondsHPRDollar_month   = NaN(line_end_dol-line_begin_dol+1,size(Bonds_M_GFD_dollar,2)-1);

BondsHPRDollar(1:end-(horizon-1),1:end)  = Bonds_M_GFD_dollar(line_begin_dol+(horizon-1):line_end_dol,2:end)./Bonds_M_GFD_dollar(line_begin_dol-1:line_end_dol-(horizon-1)-1,2:end)-1; % One-period bond USD HPR (first obs. is t0+horizon-1)
BondsHPRDollar_month(1:end,1:end)        = Bonds_M_GFD_dollar(line_begin_dol:line_end_dol,2:end)./Bonds_M_GFD_dollar(line_begin_dol-1:line_end_dol-1,2:end)-1; % One-month bond USD HPR (first obs. is t0)

line_begin       = find(Bonds_M_GFD_local(:,1)==date_begin);
line_end         = find(Bonds_M_GFD_local(:,1)==date_end);
BondsHPR         = NaN(line_end-line_begin+1,size(Bonds_M_GFD_local,2)-1);
BondsHPR_month   = NaN(line_end-line_begin+1,size(Bonds_M_GFD_local,2)-1);

BondsHPR(1:end-(horizon-1),1:end)   = Bonds_M_GFD_local(line_begin+(horizon-1):line_end,2:end)./Bonds_M_GFD_local(line_begin-1:line_end-(horizon-1)-1,2:end)-1; % One-period bond HPR (first obs. is t0+horizon-1)
BondsHPR_month(1:end,1:end)         = Bonds_M_GFD_local(line_begin:line_end,2:end)./Bonds_M_GFD_local(line_begin-1:line_end-1,2:end)-1; % One-month bond HPR (first obs. is t0)

dates_sample     = Bonds_M_GFD_local(line_begin:line_end,1);
dates            = dates_sample- datenum('30-Dec-1899');

ImpliedFX_US        = (1+BondsHPR)./(1+BondsHPRDollar);             % One-period FX changes, defined in foreign currency per USD (first obs. is t0+horizon-1)
ImpliedFX_US_month  = (1+BondsHPR_month)./(1+BondsHPRDollar_month); % One-month FX changes, defined in foreign currency per USD (first obs. is t0)

ImpliedFX        = ImpliedFX_US./repmat(ImpliedFX_US(:,col_base),1,size(ImpliedFX_US,2));                          % Adjusting for non-USD base currency (foreign currency per unit of base currency)
ImpliedFX_month  = ImpliedFX_US_month./repmat(ImpliedFX_US_month(:,col_base),1,size(ImpliedFX_US_month,2));        % Adjusting for non-USD base currency (foreign currency per unit of base currency)

BondsHPRDollar         = (1+BondsHPRDollar).*ImpliedFX_US./ImpliedFX -1;                      % Adjusting for non-USD base currency
BondsHPRDollar_month   = (1+BondsHPRDollar_month).*ImpliedFX_US_month./ImpliedFX_month -1 ;   % Adjusting for non-USD base currency


line_begin       = find(TBILL_M_GFD(:,1)==date_begin);
line_end         = find(TBILL_M_GFD(:,1)==date_end);
TB               = NaN(line_end-line_begin+1,size(TBILL_M_GFD,2)-1);
TB_month         = NaN(line_end-line_begin+1,size(TBILL_M_GFD,2)-1);

TB(1:end-(horizon-1),1:end)               = TBILL_M_GFD(line_begin+(horizon-1):line_end,2:end)./TBILL_M_GFD(line_begin-1:line_end-(horizon-1)-1,2:end)-1; % period TB rate (first obs. is t0+horizon-1; theoretically known at t0-1)
TB_month(1:end,1:end)                     = TBILL_M_GFD(line_begin:line_end,2:end)./TBILL_M_GFD(line_begin-1:line_end-1,2:end)-1; % 1-month TB rate (first obs. is t0; theoretically known at t0-1)

Yield            = (1+Yields_M_GFD(line_begin-1:line_end-1,2:end)/100).^(1/12)-1; % 10-year yield rates per month lagged one month (first obs. is t0-1)
Slope            = log(1+Yield)-log(1+TB_month);  % First obs. is t0-1 

TB_month_Long    = TBILL_M_GFD(line_begin-120:line_end,2:end)./TBILL_M_GFD(line_begin-121:line_end-1,2:end)-1;  % TB_month starting 120 obs. early
Yield_Long       = (1+Yields_M_GFD(line_begin-121:line_end-1,2:end)/100).^(1/12)-1;                             % Yield starting 120 obs. early
Slope_Long       = log(1+Yield_Long)-log(1+TB_month_Long);                                                      % Slope starting 120 obs. early

TB_month_Rolling_Mean = NaN(size(TB_month));
Yield_Rolling_Mean    = NaN(size(TB_month));
Slope_Rolling_Mean    = NaN(size(TB_month));

for t = 1:size(TB_month,1)
    
TB_month_Rolling_Mean(t,:) = nanmean(TB_month_Long(t:t+119,:)); 
Yield_Rolling_Mean(t,:)    = nanmean(Yield_Long(t:t+119,:)); 
Slope_Rolling_Mean(t,:)    = nanmean(Slope_Long(t:t+119,:)); 

end

line_begin       = find(CPI_M_GFD(:,1)==date_begin);
line_end         = find(CPI_M_GFD(:,1)==date_end);
CPIHPR           = NaN(line_end-line_begin+1,size(CPI_M_GFD,2)-1);
CPIHPR_month     = NaN(line_end-line_begin+1,size(CPI_M_GFD,2)-1);

CPIHPR(1:end-(horizon-1),1:end)   = CPI_M_GFD(line_begin+(horizon-1):line_end,2:end)./CPI_M_GFD(line_begin-1:line_end-(horizon-1)-1,2:end)-1; % One-period CPI change (first obs. is t0+horizon-1)
CPIHPR_month(1:end,1:end)         = CPI_M_GFD(line_begin:line_end,2:end)./CPI_M_GFD(line_begin-1:line_end-1,2:end)-1; % One-month CPI change (first obs. is t0)


Ratings_M_clean  = Ratings_M(2:end,:);
line_begin       = find(Ratings_M_clean(:,1)==date_begin);
line_end         = find(Ratings_M_clean(:,1)==date_end);
Ratings          = NaN(line_end-line_begin+1,size(Ratings_M_clean,2)-1);

Ratings(1:end,1:end)              = Ratings_M_clean(line_begin-1:line_end-1,2:end);          % period credit ratings (first obs. is t0-1)

Ratings_wOutlook_M_clean = Ratings_wOutlook_M(2:end,:);
line_begin       = find(Ratings_wOutlook_M_clean(:,1)==date_begin);
line_end         = find(Ratings_wOutlook_M_clean(:,1)==date_end);
RatingsOut       = NaN(line_end-line_begin+1,size(Ratings_wOutlook_M_clean,2)-1);

RatingsOut(1:end,1:end)           = Ratings_wOutlook_M_clean(line_begin-1:line_end-1,2:end); % period credit ratings, adjusted for outlook (first obs. is t0-1)

line_begin       = find(Yields_M_GFD(:,1)==date_begin);
line_end         = find(Yields_M_GFD(:,1)==date_end);


% Log FX Excess Returns

FXR              = NaN*zeros(size(ImpliedFX));
FXEXR            = NaN*zeros(size(ImpliedFX));
FX               = NaN*zeros(size(ImpliedFX));
FWD              = NaN*zeros(size(ImpliedFX));

for i=1:size(ImpliedFX,2)
     FXR(:,i)      = log((ImpliedFX(:,i)).^(-1).*(1+TB(:,i)));
     FXEXR(:,i)    = log((ImpliedFX(:,i)).^(-1).*(1+TB(:,i))./(1+TB(:,col_base)));
     FWD(:,i)      = log((1+TB(:,i))./(1+TB(:,col_base)));
     FX(:,i)       = log((ImpliedFX(:,i)).^(-1));
end


% Log Bond HPRs and Excess HPRs

BondsLogHPRDollar      = NaN*zeros(size(BondsHPRDollar));
BondsHPERDollar        = NaN*zeros(size(BondsHPRDollar));
BondsLogHPR            = NaN*zeros(size(BondsHPR));
BondsHPER              = NaN*zeros(size(BondsHPR));
CPIHPER                = NaN*zeros(size(CPIHPR));
BondsHPRDiff           = NaN*zeros(size(BondsHPR));
BondsHPRDiff_local     = NaN*zeros(size(BondsHPR));
CPIHPRDiff_local       = NaN*zeros(size(CPIHPR));

for i=1:size(BondsHPRDollar,2)
     BondsLogHPRDollar(:,i) = log(1+BondsHPRDollar(:,i));
     BondsHPERDollar(:,i)   = log((1+BondsHPRDollar(:,i))./(1+TB(:,col_base)));
end;

for i=1:size(BondsHPR,2)
     BondsLogHPR(:,i)     = log(1+BondsHPR(:,i));
     BondsHPER(:,i)       = log((1+BondsHPR(:,i))./(1+TB(:,i)));
end;

for i=1:size(CPIHPER,2)
     CPIHPER(:,i)         = log((1+CPIHPR(:,i)));
end;

for i=1:size(BondsHPRDiff,2)
      BondsHPRDiff(:,i)             = log((1+BondsHPRDollar(:,i))./(1+ BondsHPR(:,col_base)));
      BondsHPRDiff_local(:,i)       = log((1+BondsHPR(:,i))./(1+ BondsHPR(:,col_base)));
      CPIHPRDiff_local(:,i)         = log((1+CPIHPR(:,i))./(1+ CPIHPR(:,col_base)));
end

% Long-Term UIP deviations

LT_UIP              = NaN*zeros(size(ImpliedFX));

for i=1:size(ImpliedFX,2)
    for t=1:size(ImpliedFX,1)-120
        LT_UIP(t,i)   = log(1+Yield(t,i)) - log(1+Yield(t,col_base)) - (1/120)*sum(log(ImpliedFX_month(t:t+119,i)));  % Yields: first obs. is t0-1; implied FX one-month changes: first obs. is t0
    end;
end


% ============================================= %
% Build Portfolios -- Sorted By Short-Term Interest Rates
% ============================================= %

% Get rid of interest rates if no data on currency or bond returns and vice-versa

for t=1:size(TB,1)                                               % For each date
    for c=1:size(TB,2)                                           % For each country
       if  isnan(FX(t,c)*FWD(t,c)*TB(t,c)*TB_month(t,c)*BondsHPERDollar(t,c)*BondsHPER(t,c)*FXEXR(t,c)*BondsHPRDiff(t,c))==1;
           TB(t,c)                    = NaN;
           TB_month(t,c)              = NaN;
           BondsHPERDollar(t,c)       = NaN;
           BondsHPER(t,c)             = NaN;
           FXEXR(t,c)                 = NaN;
           BondsHPRDiff(t,c)          = NaN;
           BondsHPRDiff_local(t,c)    = NaN;
           FX(t,c)                    = NaN;
           FWD(t,c)                   = NaN;
           Slope(t,c)                 = NaN;
       end         
    end;
end;
                         
% Sort on TB_month: the one-month interest rate (first obs. is interest rate from t0-1 to t0, theoretically known at t0-1); first obs. of portfolio HPRs are period returns from t0-1 to t0+horizon-1
  
[Portfolios_FXEXR,Portfolios_BondsHPER,Portfolios_BondsHPERDollar,Portfolios_BondsHPRDiff,Portfolios_VarSort,Track_portfolios] = ...
    Build_portfolios_for4variables(FXEXR(:,1:end-1),BondsHPER(:,1:end-1),BondsHPERDollar(:,1:end-1),BondsHPRDiff(:,1:end-1),TB_month(:,1:end-1),sorting_variable,Number_portfolios,TB_month_Rolling_Mean(:,1:end-1));

[Portfolios_FXEXR,Portfolios_BondsHPER,Portfolios_BondsHPERDollar,Portfolios_BondsHPRDiff_local,Portfolios_VarSort,Track_portfolios_IR2] = ...
    Build_portfolios_for4variables(FXEXR(:,1:end-1),BondsHPER(:,1:end-1),BondsHPERDollar(:,1:end-1),BondsHPRDiff_local(:,1:end-1),TB_month(:,1:end-1),sorting_variable,Number_portfolios,TB_month_Rolling_Mean(:,1:end-1));

[Portfolios_FWD,Portfolios_FX,Portfolios_BondsHPR,Portfolios_BondsHPRDollar,Portfolios_VarSort,Track_portfolios_IR3] = ...
    Build_portfolios_for4variables(FWD(:,1:end-1),FX,BondsHPR(:,1:end-1),BondsHPRDollar(:,1:end-1),TB_month(:,1:end-1),sorting_variable,Number_portfolios,TB_month_Rolling_Mean(:,1:end-1));  
  
[~,~,~,Portfolios_SLOPE,Portfolios_VarSort,Track_portfolios_IR4] = ...
    Build_portfolios_for4variables(FWD(:,1:end-1),BondsHPR(:,1:end-1),BondsHPRDollar(:,1:end-1),Slope(:,1:end-1),TB_month(:,1:end-1),sorting_variable,Number_portfolios,TB_month_Rolling_Mean(:,1:end-1));

[~,~,~,Portfolios_LT_UIP, Portfolios_VarSort, Track_portfolios_IR5] = ...
    Build_portfolios_for4variables(FWD(:,1:end-1),BondsHPR(:,1:end-1),BondsHPRDollar(:,1:end-1),LT_UIP(:,1:end-1),TB_month(:,1:end-1),sorting_variable,Number_portfolios,TB_month_Rolling_Mean(:,1:end-1));

[~,Portfolios_Ratings,Portfolios_RatingsOut,Portfolios_CPIHPER, Portfolios_VarSort, Track_portfolios_IR6] = ...
    Build_portfolios_for4variables(FWD(:,1:end-1),Ratings(:,1:end-1),RatingsOut(:,1:end-1),CPIHPER(:,1:end-1),TB_month(:,1:end-1),sorting_variable,Number_portfolios,TB_month_Rolling_Mean(:,1:end-1));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for m=1:Number_portfolios
for t=1:size(Track_portfolios,1)
    A=[];
    B=[];
    k=1;
for i=1:size(Track_portfolios,2)
    if Track_portfolios(t,i)==m
        A= [A ListIMFCodes(i)];
        B{k}= ListIMFNames{i};
        k=k+1;
    end 
end    
Portfolios_Comp{t,m}=A;
Portfolios_CompNames{t,m}=B;
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%bootstrapping
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Portfolios_BondsHPER_logdiff         =(Portfolios_BondsHPER(:,Number_portfolios))-(Portfolios_BondsHPER(:,1));
Portfolios_BondsHPERDollar_logdiff   =(Portfolios_BondsHPERDollar(:,Number_portfolios))-(Portfolios_BondsHPERDollar(:,1));
Portfolios_FXEXR_logdiff             =(Portfolios_FXEXR(:,Number_portfolios))-(Portfolios_FXEXR(:,1));
Portfolios_BondsHPRDiff_logdiff      =(Portfolios_BondsHPRDiff(:,Number_portfolios))-(Portfolios_BondsHPRDiff(:,1));
Portfolios_FX_logdiff                =(Portfolios_FX(:,Number_portfolios))-(Portfolios_FX(:,1));
Portfolios_FWD_logdiff               =(Portfolios_FWD(:,Number_portfolios))-(Portfolios_FWD(:,1));
Portfolios_BondsHPRDiff_local_logdiff=(Portfolios_BondsHPRDiff_local(:,Number_portfolios)) - (Portfolios_BondsHPRDiff_local(:,1));
Portfolios_SLOPE_logdiff             =(Portfolios_SLOPE(:,Number_portfolios))-(Portfolios_SLOPE(:,1));
Portfolios_CPIHPER_logdiff           =Portfolios_CPIHPER(:,Number_portfolios) - Portfolios_CPIHPER(:,1);
Portfolios_Ratings_logdiff           =Portfolios_Ratings(:,Number_portfolios) - Portfolios_Ratings(:,1);
Portfolios_RatingsOut_logdiff        =Portfolios_RatingsOut(:,Number_portfolios) - Portfolios_RatingsOut(:,1);

portfolio_data                                    = [(Portfolios_FXEXR) Portfolios_FXEXR_logdiff (Portfolios_BondsHPER) Portfolios_BondsHPER_logdiff (Portfolios_BondsHPERDollar) Portfolios_BondsHPERDollar_logdiff  (Portfolios_BondsHPRDiff) Portfolios_BondsHPRDiff_logdiff Portfolios_FX Portfolios_FX_logdiff Portfolios_BondsHPRDiff_local Portfolios_BondsHPRDiff_local_logdiff Portfolios_FWD Portfolios_FWD_logdiff  Portfolios_SLOPE Portfolios_SLOPE_logdiff Portfolios_CPIHPER Portfolios_CPIHPER_logdiff Portfolios_Ratings Portfolios_Ratings_logdiff Portfolios_RatingsOut Portfolios_RatingsOut_logdiff];

[standard_error_mean,standard_error_SR]           = bootstrap_portfolio_returns_growth(portfolio_data,nboot,horizon);

Total_portfolios=Number_portfolios+1;

Portfolios_FXEXR_se(1:Total_portfolios)                   = standard_error_mean(1:Total_portfolios);
Portfolios_BondsHPER_se(1:Total_portfolios)               = standard_error_mean(1+Total_portfolios:2*Total_portfolios);
Portfolios_BondsHPERDollar_se(1:Total_portfolios)         = standard_error_mean(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_BondsHPRDiff_se(1:Total_portfolios)            = standard_error_mean(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_se(1:Total_portfolios)                      = standard_error_mean(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_BondsHPRDiff_local_se(1:Total_portfolios)      = standard_error_mean(1+5*Total_portfolios:6*Total_portfolios);
Portfolios_FWD_se(1:Total_portfolios)                     = standard_error_mean(1+6*Total_portfolios:7*Total_portfolios);
Portfolios_SLOPE_se(1:Total_portfolios)                   = standard_error_mean(1+7*Total_portfolios:8*Total_portfolios);
Portfolios_CPIHPER_se(1:Total_portfolios)                 = standard_error_mean(1+8*Total_portfolios:9*Total_portfolios);
Portfolios_Ratings_se(1:Total_portfolios)                 = standard_error_mean(1+9*Total_portfolios:10*Total_portfolios);
Portfolios_RatingsOut_se(1:Total_portfolios)              = standard_error_mean(1+10*Total_portfolios:11*Total_portfolios);

Portfolios_FXEXR_SR_se(1:Total_portfolios)                = standard_error_SR(1:Total_portfolios);
Portfolios_BondsHPER_SR_se(1:Total_portfolios)            = standard_error_SR(1+Total_portfolios:2*Total_portfolios);
Portfolios_BondsHPERDollar_SR_se(1:Total_portfolios)      = standard_error_SR(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_BondsHPRDiff_SR_se(1:Total_portfolios)         = standard_error_SR(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_SR_se(1:Total_portfolios)                   = standard_error_SR(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_BondsHPRDiff_local_SR_se(1:Total_portfolios)   = standard_error_SR(1+5*Total_portfolios:6*Total_portfolios);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


Table = [100*nanmean((Portfolios_FXEXR))*ScalingFactor           100*nanmean((Portfolios_FXEXR_logdiff))*ScalingFactor;                 
         100*Portfolios_FXEXR_se*ScalingFactor; 
         100*nanstd((Portfolios_FXEXR))*sqrt(ScalingFactor)      100*nanstd((Portfolios_FXEXR_logdiff))*sqrt(ScalingFactor); 
         nanmean((Portfolios_FXEXR))*sqrt(ScalingFactor)./nanstd((Portfolios_FXEXR))  nanmean((Portfolios_FXEXR_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_FXEXR_logdiff)); 
         Portfolios_FXEXR_SR_se*sqrt(ScalingFactor);
         100*nanmean((Portfolios_BondsHPER))*ScalingFactor       100*nanmean((Portfolios_BondsHPER_logdiff))*ScalingFactor;   
         100*Portfolios_BondsHPER_se*ScalingFactor; 
         100*nanstd((Portfolios_BondsHPER))*sqrt(ScalingFactor)  100*nanstd((Portfolios_BondsHPER_logdiff))*sqrt(ScalingFactor); 
         nanmean((Portfolios_BondsHPER))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPER))      nanmean((Portfolios_BondsHPER_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPER_logdiff));
         Portfolios_BondsHPER_SR_se*sqrt(ScalingFactor);
         100*nanmean((Portfolios_BondsHPERDollar))*ScalingFactor 100*nanmean((Portfolios_BondsHPERDollar_logdiff))*ScalingFactor;       
         100*Portfolios_BondsHPERDollar_se*ScalingFactor;
         100*nanstd((Portfolios_BondsHPERDollar))*sqrt(ScalingFactor)  100*nanstd((Portfolios_BondsHPERDollar_logdiff))*sqrt(ScalingFactor); 
         nanmean((Portfolios_BondsHPERDollar))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPERDollar)) nanmean((Portfolios_BondsHPERDollar_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPERDollar_logdiff));
         Portfolios_BondsHPERDollar_SR_se*sqrt(ScalingFactor);
         100*nanmean((Portfolios_BondsHPRDiff))*ScalingFactor    100*nanmean((Portfolios_BondsHPRDiff_logdiff))*ScalingFactor; 
         100*Portfolios_BondsHPRDiff_se*ScalingFactor; 
         100*nanstd((Portfolios_BondsHPRDiff))*sqrt(ScalingFactor)     100*nanstd((Portfolios_BondsHPRDiff_logdiff))*sqrt(ScalingFactor); 
         nanmean((Portfolios_BondsHPRDiff))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPRDiff))       nanmean((Portfolios_BondsHPRDiff_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPRDiff_logdiff));
         Portfolios_BondsHPRDiff_SR_se*sqrt(ScalingFactor);
         100*nanmean((Portfolios_FX))*ScalingFactor              100*nanmean((Portfolios_FX_logdiff))*ScalingFactor; 
         100*nanmean((Portfolios_FWD))*ScalingFactor             100*nanmean((Portfolios_FWD_logdiff))*ScalingFactor;
         100*nanmean((Portfolios_SLOPE))*ScalingFactor           100*nanmean((Portfolios_SLOPE_logdiff))*ScalingFactor;   
         100*Portfolios_SLOPE_se*ScalingFactor;
         100*nanmean(Portfolios_LT_UIP)*ScalingFactor           100*nanmean(Portfolios_LT_UIP(:,end) - Portfolios_LT_UIP(:,1))*ScalingFactor
         100*nanmean(Portfolios_CPIHPER)*ScalingFactor          100*nanmean(Portfolios_CPIHPER_logdiff)*ScalingFactor
         100*Portfolios_CPIHPER_se*ScalingFactor; 
         100*nanstd(Portfolios_CPIHPER)*sqrt(ScalingFactor)     100*nanstd(Portfolios_CPIHPER_logdiff)*sqrt(ScalingFactor);
         nanmean(Portfolios_Ratings)                            nanmean(Portfolios_Ratings_logdiff);
         Portfolios_Ratings_se;
         nanstd(Portfolios_Ratings)                             nanstd(Portfolios_Ratings_logdiff);
         nanmean(Portfolios_RatingsOut)                         nanmean(Portfolios_RatingsOut_logdiff)
         Portfolios_RatingsOut_se;
         nanstd(Portfolios_RatingsOut)                          nanstd(Portfolios_RatingsOut_logdiff)];

rows = {'Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','FX Mean','FWD Mean','Slope Mean','Slope s.e.','LT UIP','CPI Mean','s.e.','CPI Std', 'Ratings Mean','s.e.','Ratings Std','Ratings (Outlook) Mean', 's.e.', 'Ratings (Outlook) Std'}; 
Opts.Style = 'jf';
disp(' ');
disp('----------------------------------------------');
disp('Countries Sorted By Interest Rates');
disp('----------------------------------------------'); 
disp('Currency Risk Premium / Local Term Premium / Dollar Term Premium / Local (in USD) minus US HPR');
mat2tex(Table,'%6.2f',rows,[], [], Opts)


% Figures

figure
plot(dates_sample,cumsum((Portfolios_FXEXR(:,end)))-cumsum((Portfolios_FXEXR(:,1))),'r--','LineWidth',3)
hold on
plot(dates_sample,cumsum((Portfolios_BondsHPER(:,end)))-cumsum((Portfolios_BondsHPER(:,1))),'b','LineWidth',2)
grid
title('The Interest Rate Carry Trade')
legend('FX Premium','Local Currency Bond Premium','Location','NorthWest')
ylabel('cumulative log returns')
dateaxis
axis('tight')
print('-depsc2',strcat(newpath,'Cum_Excess_Returns_Sorted_By_Short_Term_Interest_Rates_Local_GFD_Sample_',int2str(sample),...
    '_Start_',int2str(year(date_begin)),'_End_',int2str(year(date_end))));

figure
plot(dates_sample,cumsum((Portfolios_FXEXR(:,end)))-cumsum((Portfolios_FXEXR(:,1))),'r--','LineWidth',3)
hold on
plot(dates_sample,cumsum((Portfolios_BondsHPERDollar(:,end)))-cumsum((Portfolios_BondsHPERDollar(:,1))),'b','LineWidth',2)
grid
title('The Interest Rate Carry Trade')
legend('FX Premium','Dollar Bond Premium','Location','NorthWest')
ylabel('cumulative log returns')
dateaxis
axis('tight')
print('-depsc2',strcat(newpath,'Cum_Excess_Returns_Sorted_By_Short_Term_Interest_Rates_USD_GFD_Sample_',int2str(sample),...
    '_Start_',int2str(year(date_begin)),'_End_',int2str(year(date_end))));

figure
if sample==3
for i=1:9
subplot(4,3,i)
plot(dates_sample(1:end-horizon),Track_portfolios(1:end-horizon,i))
title(strcat(Countries_Sample(i)),'FontSize',20)
axis('tight')
set(gca,'YTick',[1;2;3])
set(gca,'FontSize',20)
axis([dates_sample(1) dates_sample(end-horizon) 0.5 3.5])
dateaxis
set(gcf, 'Position', get(0, 'Screensize'));hold on
end
elseif sample==2
for i=1:20
subplot(5,4,i)
plot(dates_sample(1:end-horizon),Track_portfolios(1:end-horizon,i))
title(strcat(Countries_Sample(i)),'FontSize',10)
axis('tight')
set(gca,'YTick',[1;2;3;4])
set(gca,'FontSize',6)
axis([dates_sample(1) dates_sample(end-horizon) 0.5 4.5])
dateaxis
set(gcf, 'Position', get(0, 'Screensize'));hold on
hold on
end
elseif sample==1
for i=1:30
subplot(7,5,i)
plot(dates_sample(1:end-horizon),Track_portfolios(1:end-horizon,i))
title(strcat(Countries_Sample(i)),'FontSize',10)
axis('tight')
set(gca,'YTick',[1;2;3;4;5])
set(gca,'FontSize',6)
axis([dates_sample(1) dates_sample(end-horizon) 0.5 5.5])
dateaxis
set(gcf, 'Position', get(0, 'Screensize'));hold on
hold on
end
end
print('-depsc2',strcat(newpath,'PortComp_Sorted_By_Short_Term_Interest_Rates_GFD_Sample_',int2str(sample),...
    '_Start_',int2str(year(date_begin)),'_End_',int2str(year(date_end))));


% ============================================= %
% Build Portfolios -- Sorted By Yield Curve Slopes
% ============================================= %
  
% Get rid of interest rates if no data on currency or bond returns and vice-versa

for t=1:size(TB,1)                                               % For each date
    for c=1:size(TB,2)                                           % For each country
       if  isnan(FX(t,c)*FWD(t,c)*TB(t,c)*TB_month(t,c)*Yield(t,c)*BondsHPERDollar(t,c)*BondsHPER(t,c)*FXEXR(t,c)*BondsHPRDiff(t,c))==1;
           TB(t,c)              = NaN;
           Slope(t,c)           = NaN;
           BondsHPERDollar(t,c) = NaN;
           BondsHPER(t,c)       = NaN;
           FXEXR(t,c)           = NaN;
           BondsHPRDiff(t,c)    = NaN;
           BondsHPRDiff_local(t,c)    = NaN;
           FX(t,c)                    = NaN;
           FWD(t,c)           = NaN;
       end         
    end;
end;
                         
% Sort on Slope: first obs. is t0-1; first obs. of portfolio HPRs are period returns from t0-1 to t0+horizon-1
                            
[Portfolios_FXEXR,Portfolios_BondsHPER,Portfolios_BondsHPERDollar,Portfolios_BondsHPRDiff,Portfolios_VarSort,Track_portfolios] = ...
    Build_portfolios_for4variables(FXEXR(:,1:end-1),BondsHPER(:,1:end-1),BondsHPERDollar(:,1:end-1),BondsHPRDiff(:,1:end-1),Slope(:,1:end-1),sorting_variable,Number_portfolios,Slope_Rolling_Mean(:,1:end-1));

[Portfolios_FXEXR,Portfolios_BondsHPER,Portfolios_BondsHPERDollar,Portfolios_BondsHPRDiff_local,Portfolios_VarSort,Track_portfolios_Slope2] = ...
    Build_portfolios_for4variables(FXEXR(:,1:end-1),BondsHPER(:,1:end-1),BondsHPERDollar(:,1:end-1),BondsHPRDiff_local(:,1:end-1),Slope(:,1:end-1),sorting_variable,Number_portfolios,Slope_Rolling_Mean(:,1:end-1));

[Portfolios_FWD,Portfolios_FX,Portfolios_BondsHPR,Portfolios_BondsHPRDollar,Portfolios_VarSort,Track_portfolios_Slope3] = ...
    Build_portfolios_for4variables(FWD(:,1:end-1),FX(:,1:end-1),BondsHPR(:,1:end-1),BondsHPRDollar(:,1:end-1),Slope(:,1:end-1),sorting_variable,Number_portfolios,Slope_Rolling_Mean(:,1:end-1));
   
[~,~,~,Portfolios_SLOPE,Portfolios_VarSort,Track_portfolios_Slope4] = ...
    Build_portfolios_for4variables(FWD(:,1:end-1),BondsHPR(:,1:end-1),BondsHPRDollar(:,1:end-1),Slope(:,1:end-1),Slope(:,1:end-1),sorting_variable,Number_portfolios,Slope_Rolling_Mean(:,1:end-1));

[~,~,~,Portfolios_LT_UIP, Portfolios_VarSort, Track_portfolios_Slope5] = ...
    Build_portfolios_for4variables(FWD(:,1:end-1),BondsHPR(:,1:end-1),BondsHPRDollar(:,1:end-1),LT_UIP(:,1:end-1),Slope(:,1:end-1),sorting_variable,Number_portfolios,Slope_Rolling_Mean(:,1:end-1));

[~,Portfolios_Ratings,Portfolios_RatingsOut,Portfolios_CPIHPER, Portfolios_VarSort, Track_portfolios_Slope6] = ...
    Build_portfolios_for4variables(FWD(:,1:end-1),Ratings(:,1:end-1),RatingsOut(:,1:end-1),CPIHPER(:,1:end-1),Slope(:,1:end-1),sorting_variable,Number_portfolios,Slope_Rolling_Mean(:,1:end-1));
      

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for m=1:Number_portfolios
for t=1:size(Track_portfolios,1)
    A=[];
    B=[];
    k=1;
for i=1:size(Track_portfolios,2)
    if Track_portfolios(t,i)==m
        A= [A ListIMFCodes(i)];
        B{k}= ListIMFNames{i};
        k=k+1;
    end 
end    
Portfolios_Comp{t,m}=A;
Portfolios_CompNames{t,m}=B;
end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%bootstrapping
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Portfolios_BondsHPER_logdiff         =(Portfolios_BondsHPER(:,Number_portfolios))-(Portfolios_BondsHPER(:,1));
Portfolios_BondsHPERDollar_logdiff   =(Portfolios_BondsHPERDollar(:,Number_portfolios))-(Portfolios_BondsHPERDollar(:,1));
Portfolios_FXEXR_logdiff             =(Portfolios_FXEXR(:,Number_portfolios))-(Portfolios_FXEXR(:,1));
Portfolios_BondsHPRDiff_logdiff      =(Portfolios_BondsHPRDiff(:,Number_portfolios))-(Portfolios_BondsHPRDiff(:,1));
Portfolios_FX_logdiff                =(Portfolios_FX(:,Number_portfolios))-(Portfolios_FX(:,1));
Portfolios_FWD_logdiff               =(Portfolios_FWD(:,Number_portfolios))-(Portfolios_FWD(:,1));
Portfolios_SLOPE_logdiff             =(Portfolios_SLOPE(:,Number_portfolios))-(Portfolios_SLOPE(:,1));
Portfolios_CPIHPER_logdiff           =Portfolios_CPIHPER(:,Number_portfolios) - Portfolios_CPIHPER(:,1);
Portfolios_Ratings_logdiff           =Portfolios_Ratings(:,Number_portfolios) - Portfolios_Ratings(:,1);
Portfolios_RatingsOut_logdiff        =Portfolios_RatingsOut(:,Number_portfolios) - Portfolios_RatingsOut(:,1);

Portfolios_BondsHPRDiff_local_logdiff=(Portfolios_BondsHPRDiff_local(:,Number_portfolios)) - (Portfolios_BondsHPRDiff_local(:,1));

portfolio_data                                    = [(Portfolios_FXEXR) Portfolios_FXEXR_logdiff (Portfolios_BondsHPER) Portfolios_BondsHPER_logdiff (Portfolios_BondsHPERDollar) Portfolios_BondsHPERDollar_logdiff  (Portfolios_BondsHPRDiff) Portfolios_BondsHPRDiff_logdiff Portfolios_FX Portfolios_FX_logdiff Portfolios_BondsHPRDiff_local Portfolios_BondsHPRDiff_local_logdiff Portfolios_FWD Portfolios_FWD_logdiff  Portfolios_SLOPE Portfolios_SLOPE_logdiff Portfolios_CPIHPER Portfolios_CPIHPER_logdiff Portfolios_Ratings Portfolios_Ratings_logdiff Portfolios_RatingsOut Portfolios_RatingsOut_logdiff];

[standard_error_mean,standard_error_SR]                               = bootstrap_portfolio_returns_growth(portfolio_data,nboot,horizon);

Total_portfolios=Number_portfolios+1;

Portfolios_FXEXR_se(1:Total_portfolios)                   = standard_error_mean(1:Total_portfolios);
Portfolios_BondsHPER_se(1:Total_portfolios)               = standard_error_mean(1+Total_portfolios:2*Total_portfolios);
Portfolios_BondsHPERDollar_se(1:Total_portfolios)         = standard_error_mean(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_BondsHPRDiff_se(1:Total_portfolios)            = standard_error_mean(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_se(1:Total_portfolios)                      = standard_error_mean(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_BondsHPRDiff_local_se(1:Total_portfolios)      = standard_error_mean(1+5*Total_portfolios:6*Total_portfolios);
Portfolios_FWD_se(1:Total_portfolios)                     = standard_error_mean(1+6*Total_portfolios:7*Total_portfolios);
Portfolios_SLOPE_se(1:Total_portfolios)                   = standard_error_mean(1+7*Total_portfolios:8*Total_portfolios);
Portfolios_CPIHPER_se(1:Total_portfolios)                 = standard_error_mean(1+8*Total_portfolios:9*Total_portfolios);
Portfolios_Ratings_se(1:Total_portfolios)                 = standard_error_mean(1+9*Total_portfolios:10*Total_portfolios);
Portfolios_RatingsOut_se(1:Total_portfolios)              = standard_error_mean(1+10*Total_portfolios:11*Total_portfolios);

Portfolios_FXEXR_SR_se(1:Total_portfolios)                = standard_error_SR(1:Total_portfolios);
Portfolios_BondsHPER_SR_se(1:Total_portfolios)            = standard_error_SR(1+Total_portfolios:2*Total_portfolios);
Portfolios_BondsHPERDollar_SR_se(1:Total_portfolios)      = standard_error_SR(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_BondsHPRDiff_SR_se(1:Total_portfolios)         = standard_error_SR(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_SR_se(1:Total_portfolios)                   = standard_error_SR(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_BondsHPRDiff_local_SR_se(1:Total_portfolios)   = standard_error_SR(1+5*Total_portfolios:6*Total_portfolios);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


Table = [100*nanmean((Portfolios_FXEXR))*ScalingFactor           100*nanmean((Portfolios_FXEXR_logdiff))*ScalingFactor;                 
         100*Portfolios_FXEXR_se*ScalingFactor; 
         100*nanstd((Portfolios_FXEXR))*sqrt(ScalingFactor)      100*nanstd((Portfolios_FXEXR_logdiff))*sqrt(ScalingFactor); 
         nanmean((Portfolios_FXEXR))*sqrt(ScalingFactor)./nanstd((Portfolios_FXEXR))  nanmean((Portfolios_FXEXR_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_FXEXR_logdiff)); 
         Portfolios_FXEXR_SR_se*sqrt(ScalingFactor);
         100*nanmean((Portfolios_BondsHPER))*ScalingFactor       100*nanmean((Portfolios_BondsHPER_logdiff))*ScalingFactor;   
         100*Portfolios_BondsHPER_se*ScalingFactor; 
         100*nanstd((Portfolios_BondsHPER))*sqrt(ScalingFactor)  100*nanstd((Portfolios_BondsHPER_logdiff))*sqrt(ScalingFactor); 
         nanmean((Portfolios_BondsHPER))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPER))      nanmean((Portfolios_BondsHPER_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPER_logdiff));
         Portfolios_BondsHPER_SR_se*sqrt(ScalingFactor);
         100*nanmean((Portfolios_BondsHPERDollar))*ScalingFactor 100*nanmean((Portfolios_BondsHPERDollar_logdiff))*ScalingFactor;       
         100*Portfolios_BondsHPERDollar_se*ScalingFactor;
         100*nanstd((Portfolios_BondsHPERDollar))*sqrt(ScalingFactor)  100*nanstd((Portfolios_BondsHPERDollar_logdiff))*sqrt(ScalingFactor); 
         nanmean((Portfolios_BondsHPERDollar))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPERDollar)) nanmean((Portfolios_BondsHPERDollar_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPERDollar_logdiff));
         Portfolios_BondsHPERDollar_SR_se*sqrt(ScalingFactor);
         100*nanmean((Portfolios_BondsHPRDiff))*ScalingFactor    100*nanmean((Portfolios_BondsHPRDiff_logdiff))*ScalingFactor; 
         100*Portfolios_BondsHPRDiff_se*ScalingFactor; 
         100*nanstd((Portfolios_BondsHPRDiff))*sqrt(ScalingFactor)     100*nanstd((Portfolios_BondsHPRDiff_logdiff))*sqrt(ScalingFactor); 
         nanmean((Portfolios_BondsHPRDiff))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPRDiff))       nanmean((Portfolios_BondsHPRDiff_logdiff))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPRDiff_logdiff));
         Portfolios_BondsHPRDiff_SR_se*sqrt(ScalingFactor);
         100*nanmean((Portfolios_FX))*ScalingFactor              100*nanmean((Portfolios_FX_logdiff))*ScalingFactor; 
         100*nanmean((Portfolios_FWD))*ScalingFactor             100*nanmean((Portfolios_FWD_logdiff))*ScalingFactor;
         100*nanmean((Portfolios_SLOPE))*ScalingFactor           100*nanmean((Portfolios_SLOPE_logdiff))*ScalingFactor;   
         100*Portfolios_SLOPE_se*ScalingFactor;
         100*nanmean(Portfolios_LT_UIP)*ScalingFactor           100*nanmean(Portfolios_LT_UIP(:,end) - Portfolios_LT_UIP(:,1))*ScalingFactor
         100*nanmean(Portfolios_CPIHPER)*ScalingFactor          100*nanmean(Portfolios_CPIHPER_logdiff)*ScalingFactor
         100*Portfolios_CPIHPER_se*ScalingFactor;
         100*nanstd(Portfolios_CPIHPER)*sqrt(ScalingFactor)     100*nanstd(Portfolios_CPIHPER_logdiff)*sqrt(ScalingFactor);
         nanmean(Portfolios_Ratings)                            nanmean(Portfolios_Ratings_logdiff);
         Portfolios_Ratings_se; 
         nanstd(Portfolios_Ratings)                             nanstd(Portfolios_Ratings_logdiff);
         nanmean(Portfolios_RatingsOut)                         nanmean(Portfolios_RatingsOut_logdiff);
         Portfolios_RatingsOut_se;
         nanstd(Portfolios_RatingsOut)                          nanstd(Portfolios_RatingsOut_logdiff)];


rows = {'Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','Mean','s.e.','Std','SR','s.e.','FX Mean','FWD Mean','Slope Mean','Slope s.e.','LT UIP','CPI Mean','s.e.','CPI Std', 'Ratings Mean','s.e.','Ratings Std','Ratings (Outlook) Mean', 's.e.', 'Ratings (Outlook) Std'}; 
Opts.Style = 'jf'; 
disp(' ');
disp('----------------------------------------------');
disp('Countries Sorted By Slopes');
disp('----------------------------------------------'); 
disp('Currency Risk Premium / Local Term Premium / Dollar Term Premium / Local (in USD) minus US HPR');
mat2tex(Table,'%6.2f',rows,[], [], Opts)


% Figures

figure
plot(dates_sample,cumsum((Portfolios_FXEXR(:,1)))-cumsum((Portfolios_FXEXR(:,end))),'r--','LineWidth',3)
hold on
plot(dates_sample,cumsum((Portfolios_BondsHPER(:,1)))-cumsum((Portfolios_BondsHPER(:,end))),'b','LineWidth',2)
grid
title('The Slope Carry Trade')
legend('FX Premium','Local Currency Bond Premium','Location','NorthWest')
ylabel('cumulative log returns')
dateaxis
axis('tight')
print('-depsc2',strcat(newpath,'Cum_Excess_Returns_Sorted_By_Interest_Rate_Slopes_Local_GFD_Sample_',int2str(sample),...
    '_Start_',int2str(year(date_begin)),'_End_',int2str(year(date_end))));

figure
plot(dates_sample,cumsum((Portfolios_FXEXR(:,1)))-cumsum((Portfolios_FXEXR(:,end))),'r--','LineWidth',3)
hold on
plot(dates_sample,cumsum((Portfolios_BondsHPERDollar(:,1)))-cumsum((Portfolios_BondsHPERDollar(:,end))),'b','LineWidth',2)
grid
title('The Slope Carry Trade')
legend('FX Premium','Dollar Bond Premium','Location','NorthWest')
ylabel('cumulative log returns')
dateaxis
axis('tight')
print('-depsc2',strcat(newpath,'Cum_Excess_Returns_Sorted_By_Interest_Rate_Slopes_GFD_Sample_',int2str(sample),...
    '_Start_',int2str(year(date_begin)),'_End_',int2str(year(date_end))));


figure
if sample==3
for i=1:9
    subplot(4,3,i)
plot(dates_sample(1:end-horizon),Track_portfolios(1:end-horizon,i))
title(strcat(Countries_Sample(i)),'FontSize',20)
axis('tight')
set(gca,'YTick',[1;2;3])
axis([dates_sample(1) dates_sample(end-horizon) 0.5 3.5])
dateaxis
set(gcf, 'Position', get(0, 'Screensize'));hold on
hold on
end

elseif sample==2
for i=1:20
subplot(5,4,i)
plot(dates_sample(1:end-horizon),Track_portfolios(1:end-horizon,i))
title(strcat(Countries_Sample(i)))
axis('tight')
set(gca,'YTick',[1;2;3;4])
set(gca,'FontSize',6)
axis([dates_sample(1) dates_sample(end-horizon) 0.5 4.5])
dateaxis
set(gcf, 'Position', get(0, 'Screensize'));hold on
hold on
end


elseif sample==1
for i=1:30
subplot(7,5,i)
plot(dates_sample(1:end-horizon),Track_portfolios(1:end-horizon,i))
title(strcat(Countries_Sample(i)))
axis('tight')
set(gca,'YTick',[1;2;3;4;5])
set(gca,'FontSize',4)
axis([dates_sample(1) dates_sample(end-horizon) 0.5 5.5])
dateaxis
set(gcf, 'Position', get(0, 'Screensize'));hold on
hold on
end
end
print('-depsc2',strcat(newpath,'PortComp_Sorted_By_Interest_Rate_Slopes_GFD_Sample_',int2str(sample),...
    '_Start_',int2str(year(date_begin)),'_End_',int2str(year(date_end))));



% ============================================= %
% Time-Series Regressions: Individual Currencies
% ============================================= %

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Time-Series Predictability Regressions: Foreign Bond HPERs (and Components) on Interest Rates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nc = size(Countries_Sample,1);

if NW_lag_choice == 1
    
    lags_vec = max(6,horizon+1);  
    
    elseif NW_lag_choice == 2
        
    lags_vec = ceil(4*(size(BondsHPER,1)/100)^(2/9))-1;
    
    elseif NW_lag_choice == 3
        
    lags_vec = ceil(0.75*(size(BondsHPER,1))^(1/3))-1;
    
    elseif NW_lag_choice == 4
        
    lags_vec = max(ceil(4*(size(BondsHPER,1)/100)^(2/9))-1,ceil(0.75*(size(BondsHPER,1))^(1/3))-1);
    
    elseif NW_lag_choice == 5
        
    lags_vec = ceil(1.3*(size(BondsHPER,1))^(1/2))-1;
    
end
   
maxlags   = size(lags_vec,2);

TSPred_BetasIRDiff_con          = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiff_con_se       = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiff_con_boot     = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiff_con_se_boot  = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiff              = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiff_se           = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiff_boot         = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiff_se_boot      = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiff_R2adj        = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiff_Nobs         = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiff_con_pval_b   = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiff_pval_b       = NaN*zeros(Nc,maxlags);

TSPred_BetasFXIRDiff_con          = NaN*zeros(Nc,maxlags);
TSPred_BetasFXIRDiff_con_se       = NaN*zeros(Nc,maxlags);
TSPred_BetasFXIRDiff_con_boot     = NaN*zeros(Nc,maxlags);
TSPred_BetasFXIRDiff_con_se_boot  = NaN*zeros(Nc,maxlags);
TSPred_BetasFXIRDiff              = NaN*zeros(Nc,maxlags);
TSPred_BetasFXIRDiff_se           = NaN*zeros(Nc,maxlags);
TSPred_BetasFXIRDiff_boot         = NaN*zeros(Nc,maxlags);
TSPred_BetasFXIRDiff_se_boot      = NaN*zeros(Nc,maxlags);
TSPred_BetasFXIRDiff_R2adj        = NaN*zeros(Nc,maxlags);
TSPred_BetasFXIRDiff_Nobs         = NaN*zeros(Nc,maxlags);
TSPred_BetasFXIRDiff_con_pval_b   = NaN*zeros(Nc,maxlags);
TSPred_BetasFXIRDiff_pval_b       = NaN*zeros(Nc,maxlags);

TSPred_BetasIRDiffLocal_con          = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiffLocal_con_se       = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiffLocal_con_boot     = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiffLocal_con_se_boot  = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiffLocal              = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiffLocal_se           = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiffLocal_boot         = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiffLocal_se_boot      = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiffLocal_R2adj        = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiffLocal_Nobs         = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiffLocal_con_pval_b   = NaN*zeros(Nc,maxlags);
TSPred_BetasIRDiffLocal_pval_b       = NaN*zeros(Nc,maxlags);


block_length = horizon;
    
    
for n=1:Nc
        
for lags = lags_vec
    
    % Regress USD bond HPER differentials (foreign minus US) on interest rate differential (foreign minus US)

    lhv = (BondsHPERDollar(:,n)-BondsHPERDollar(:,end))*ScalingFactor;
    rhv = [ones(size(lhv,1),1) 12*log(1+TB_month(:,n))-12*log(1+TB_month(:,end))];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc 
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                     = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasIRDiff_con(n,1)         = bv(1);
        TSPred_BetasIRDiff_con_se(n,1)      = sebv(1);
        TSPred_BetasIRDiff_con_boot(n,1)    = bootstrap_stats(1,1);
        TSPred_BetasIRDiff_con_se_boot(n,1) = bootstrap_stats(1,2);
        TSPred_BetasIRDiff(n,1)        = bv(2);
        TSPred_BetasIRDiff_se(n,1)     = sebv(2);
        TSPred_BetasIRDiff_boot(n,1)   = bootstrap_stats(2,1);
        TSPred_BetasIRDiff_se_boot(n,1)= bootstrap_stats(2,2);
        TSPred_BetasIRDiff_R2adj(n,1)  = R2vadj;
        TSPred_BetasIRDiff_Nobs(n,1)   = size(lhv_s,1);
        TSPred_BetasIRDiff_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasIRDiff_con(n,1)/TSPred_BetasIRDiff_con_se(n,1),(lags+1)/TSPred_BetasIRDiff_Nobs(n,1));                 
        TSPred_BetasIRDiff_pval_b(n,1)     = tstat_pval_b(TSPred_BetasIRDiff(n,1)/TSPred_BetasIRDiff_se(n,1),(lags+1)/TSPred_BetasIRDiff_Nobs(n,1));   
        
        
    end
  
    
    % Regress FX excess returns on interest rate differential (foreign minus US); this is a Fama (1984) UIP regression

    lhv = FXEXR(:,n)*ScalingFactor;
    % lhv = FX(:,n)*ScalingFactor; % Alternative decomposition
    rhv = [ones(size(lhv,1),1) 12*log(1+TB_month(:,n))-12*log(1+TB_month(:,end))];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc 
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasFXIRDiff_con(n,1)    = bv(1);
        TSPred_BetasFXIRDiff_con_se(n,1) = sebv(1);
        TSPred_BetasFXIRDiff_con_boot(n,1)    = bootstrap_stats(1,1);
        TSPred_BetasFXIRDiff_con_se_boot(n,1) = bootstrap_stats(1,2);
        TSPred_BetasFXIRDiff(n,1)        = bv(2);
        TSPred_BetasFXIRDiff_se(n,1)     = sebv(2);
        TSPred_BetasFXIRDiff_boot(n,1)   = bootstrap_stats(2,1);
        TSPred_BetasFXIRDiff_se_boot(n,1)= bootstrap_stats(2,2);
        TSPred_BetasFXIRDiff_R2adj(n,1)  = R2vadj;
        TSPred_BetasFXIRDiff_Nobs(n,1)   = size(lhv_s,1);
        TSPred_BetasFXIRDiff_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasFXIRDiff_con(n,1)/TSPred_BetasFXIRDiff_con_se(n,1),(lags+1)/TSPred_BetasFXIRDiff_Nobs(n,1));                 
        TSPred_BetasFXIRDiff_pval_b(n,1)     = tstat_pval_b(TSPred_BetasFXIRDiff(n,1)/TSPred_BetasFXIRDiff_se(n,1),(lags+1)/TSPred_BetasFXIRDiff_Nobs(n,1));  
       
        
    end;
    
    % Regress bond local currency HPER differentials (foreign minus US) on interest rate differential (foreign minus US)

    lhv = (BondsHPER(:,n)-BondsHPER(:,end))*ScalingFactor;
    % lhv = (log(1+BondsHPRDollar(:,n))-log(1+BondsHPRDollar(:,end)))*ScalingFactor; % Alternative decomposition
    rhv = [ones(size(lhv,1),1) 12*log(1+TB_month(:,n))-12*log(1+TB_month(:,end))];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc 
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasIRDiffLocal_con(n,1)    = bv(1);
        TSPred_BetasIRDiffLocal_con_se(n,1) = sebv(1);
        TSPred_BetasIRDiffLocal_con_boot(n,1)    = bootstrap_stats(1,1);
        TSPred_BetasIRDiffLocal_con_se_boot(n,1) = bootstrap_stats(1,2);
        TSPred_BetasIRDiffLocal(n,1)        = bv(2);
        TSPred_BetasIRDiffLocal_se(n,1)     = sebv(2);
        TSPred_BetasIRDiffLocal_boot(n,1)   = bootstrap_stats(2,1);
        TSPred_BetasIRDiffLocal_se_boot(n,1)= bootstrap_stats(2,2);
        TSPred_BetasIRDiffLocal_R2adj(n,1)  = R2vadj;
        TSPred_BetasIRDiffLocal_Nobs(n,1)   = size(lhv_s,1);
        TSPred_BetasIRDiffLocal_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasIRDiffLocal_con(n,1)/TSPred_BetasIRDiffLocal_con_se(n,1),(lags+1)/TSPred_BetasIRDiffLocal_Nobs(n,1));                 
        TSPred_BetasIRDiffLocal_pval_b(n,1)     = tstat_pval_b(TSPred_BetasIRDiffLocal(n,1)/TSPred_BetasIRDiffLocal_se(n,1),(lags+1)/TSPred_BetasIRDiffLocal_Nobs(n,1));  
            
    end
    
end

end

for n = 1:Nc-1
    
    BondHPER_Diff(:,n)       = BondsHPER(:,n)-BondsHPER(:,end);
    BondHPERDollar_Diff(:,n) = BondsHPERDollar(:,n)-BondsHPERDollar(:,end);
    FXEXR_Panel(:,n)         = FXEXR(:,n);
    CPIHPER_Diff(:,n)        = CPIHPER(:,n)-CPIHPER(:,end); 
    TB_month_Diff(:,n)       = log(1+TB_month(:,n))-log(1+TB_month(:,end));
    Slope_Diff(:,n)          = Slope(:,n)-Slope(:,end);
    Panel_Unit(:,n)          = n*ones(size(BondsHPER,1),1);
    Panel_Time(:,n)          = [1:size(BondsHPER,1)]';
    Ratings_Diff(:,n)        = Ratings(:,n)-Ratings(:,end);
    Ratings_Fgn(:,n)         = Ratings(:,n);            
    Ratings_US(:,n)          = Ratings(:,end);
    RatingsOut_Diff(:,n)     = RatingsOut(:,n)-RatingsOut(:,end);
    RatingsOut_Fgn(:,n)      = RatingsOut(:,n);            
    RatingsOut_US(:,n)       = RatingsOut(:,end);
    
end


for lags = lags_vec

% Panel regressions with Driscoll-Kraay s.e.

Panel_Var = [ScalingFactor*BondHPERDollar_Diff, 12*TB_month_Diff];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasIRDiff_con(Nc+2,1)           = NaN;
TSPred_BetasIRDiff_con_se(Nc+2,1)        = NaN; 
TSPred_BetasIRDiff(Nc+2,1)               = theta(1,1);
TSPred_BetasIRDiff_se(Nc+2,1)            = theta_se(1,1);
TSPred_BetasIRDiff_Nobs(Nc+2,1)          = sum(sum(~isnan(yhat)));
TSPred_BetasIRDiff_R2adj(Nc+2,1)         = 1-(1-Rsq)*(TSPred_BetasIRDiff_Nobs(Nc+2,1)-1)/(TSPred_BetasIRDiff_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasIRDiff_con_boot(Nc+2,1)      = NaN;
TSPred_BetasIRDiff_con_se_boot(Nc+2,1)   = NaN;
TSPred_BetasIRDiff_boot(Nc+2,1)          = bootstrap_stats(1,1);
TSPred_BetasIRDiff_se_boot(Nc+2,1)       = bootstrap_stats(1,2);
TSPred_BetasIRDiff_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasIRDiff_con(Nc+2,1)/TSPred_BetasIRDiff_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasIRDiff_pval_b(Nc+2,1)        = tstat_pval_b(TSPred_BetasIRDiff(Nc+2,1)/TSPred_BetasIRDiff_se(Nc+2,1), (lags+1)/size(y_DK,1));                 



Panel_Var = [ScalingFactor*FXEXR_Panel, 12*TB_month_Diff];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasFXIRDiff_con(Nc+2,1)         = NaN;
TSPred_BetasFXIRDiff_con_se(Nc+2,1)      = NaN; 
TSPred_BetasFXIRDiff(Nc+2,1)             = theta(1,1);
TSPred_BetasFXIRDiff_se(Nc+2,1)          = theta_se(1,1);
TSPred_BetasFXIRDiff_Nobs(Nc+2,1)        = sum(sum(~isnan(yhat)));
TSPred_BetasFXIRDiff_R2adj(Nc+2,1)       = 1-(1-Rsq)*(TSPred_BetasFXIRDiff_Nobs(Nc+2,1)-1)/(TSPred_BetasFXIRDiff_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasFXIRDiff_con_boot(Nc+2,1)    = NaN;
TSPred_BetasFXIRDiff_con_se_boot(Nc+2,1) = NaN;
TSPred_BetasFXIRDiff_boot(Nc+2,1)        = bootstrap_stats(1,1);
TSPred_BetasFXIRDiff_se_boot(Nc+2,1)     = bootstrap_stats(1,2);
TSPred_BetasFXIRDiff_con_pval_b(Nc+2,1)  = tstat_pval_b(TSPred_BetasFXIRDiff_con(Nc+2,1)/TSPred_BetasFXIRDiff_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasFXIRDiff_pval_b(Nc+2,1)      = tstat_pval_b(TSPred_BetasFXIRDiff(Nc+2,1)/TSPred_BetasFXIRDiff_se(Nc+2,1), (lags+1)/size(y_DK,1));                 



Panel_Var   = [ScalingFactor*BondHPER_Diff, 12*TB_month_Diff];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se]                  = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasIRDiffLocal_con(Nc+2,1)         = NaN;
TSPred_BetasIRDiffLocal_con_se(Nc+2,1)      = NaN;
TSPred_BetasIRDiffLocal(Nc+2,1)             = theta(1,1);
TSPred_BetasIRDiffLocal_se(Nc+2,1)          = theta_se(1,1);
TSPred_BetasIRDiffLocal_Nobs(Nc+2,1)        = sum(sum(~isnan(yhat)));
TSPred_BetasIRDiffLocal_R2adj(Nc+2,1)       = 1-(1-Rsq)*(TSPred_BetasIRDiffLocal_Nobs(Nc+2,1)-1)/(TSPred_BetasIRDiffLocal_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasIRDiffLocal_con_boot(Nc+2,1)    = NaN;
TSPred_BetasIRDiffLocal_con_se_boot(Nc+2,1) = NaN;
TSPred_BetasIRDiffLocal_boot(Nc+2,1)        = bootstrap_stats(1,1);
TSPred_BetasIRDiffLocal_se_boot(Nc+2,1)     = bootstrap_stats(1,2);
TSPred_BetasIRDiffLocal_con_pval_b(Nc+2,1)  = tstat_pval_b(TSPred_BetasIRDiffLocal_con(Nc+2,1)/TSPred_BetasIRDiffLocal_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasIRDiffLocal_pval_b(Nc+2,1)      = tstat_pval_b(TSPred_BetasIRDiffLocal(Nc+2,1)/TSPred_BetasIRDiffLocal_se(Nc+2,1), (lags+1)/size(y_DK,1));                 


% Joint estimation

T          = size(BondsHPERDollar,1)-horizon+1;
R1_con     = zeros(Nc-1,6*(Nc-1));
R1_slope   = zeros(Nc-1,6*(Nc-1));
R1_all     = zeros(2*(Nc-1),6*(Nc-1));
R2_con     = zeros(Nc-1,6*(Nc-1));
R2_slope   = zeros(Nc-1,6*(Nc-1));
R2_all     = zeros(2*(Nc-1),6*(Nc-1));
R3_con     = zeros(Nc-1,6*(Nc-1));
R3_slope   = zeros(Nc-1,6*(Nc-1));
R3_all     = zeros(2*(Nc-1),6*(Nc-1));
R_diff     = zeros(Nc-1,6*(Nc-1));
r          = zeros(Nc-1,1);
r_all      = zeros(2*(Nc-1),1);
rhv        = zeros(3*T*(Nc-1),6*(Nc-1));
for n=1:Nc-1
lhv((n-1)*T+1:n*T,1)                                                     = (BondsHPERDollar(1:end-horizon+1,n)-BondsHPERDollar(1:end-horizon+1,end))*ScalingFactor;
rhv((n-1)*T+1:n*T,(n-1)*2+1:n*2)                                         = [ones(T,1) 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end))];
lhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,1)                                   = FXEXR(1:end-horizon+1,n)*ScalingFactor;
rhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,2*(Nc-1)+(n-1)*2+1:2*(Nc-1)+n*2)     = [ones(T,1) 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end))];
lhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,1)                               = (BondsHPER(1:end-horizon+1,n)-BondsHPER(1:end-horizon+1,end))*ScalingFactor;
rhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,4*(Nc-1)+(n-1)*2+1:4*(Nc-1)+n*2) = [ones(T,1) 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end))];
R1_con(n,2*n-1)          =  1;
R1_slope(n,2*n)          =  1;
R1_all(n,2*n-1)          =  1;
R1_all(Nc-1+n,2*n)       =  1;
R2_con(n,2*(Nc-1)+2*n-1) =  1;
R2_slope(n,2*(Nc-1)+2*n) =  1;
R2_all(n,2*(Nc-1)+2*n-1)    =  1;
R2_all(Nc-1+n,2*(Nc-1)+2*n) =  1;
R3_con(n,4*(Nc-1)+2*n-1) =  1;
R3_slope(n,4*(Nc-1)+2*n) =  1;
R3_all(n,4*(Nc-1)+2*n-1)      =  1;
R3_all(Nc-1+n,4*(Nc-1)+2*n)   =  1;
R_diff(n,2*n)            =  1;
R_diff(n,2*(Nc-1)+2*n)   = -1;
end
[lhv_s,rhv_s]                        = FindCommonSample(lhv,rhv);
[bv,sebv,R2v,R2vadj,v]               = olsgmm(lhv_s,rhv_s,lags,1);
W_IRDiff_con(1,1)         = (R1_con*bv - r)'*inv(R1_con*v*R1_con')*(R1_con*bv - r);          % test if all constants are zero in the HPERDollar regressions
Wp_IRDiff_con(1,1)        = 1-chi2cdf(W_IRDiff_con(1,1),Nc-1);
W_IRDiff_slope(1,1)       = (R1_slope*bv - r)'*inv(R1_slope*v*R1_slope')*(R1_slope*bv - r);  % test if all slopes are zero in the HPERDollar regressions
Wp_IRDiff_slope(1,1)      = 1-chi2cdf(W_IRDiff_slope(1,1),Nc-1);
W_IRDiff_all(1,1)         = (R1_all*bv - r_all)'*inv(R1_all*v*R1_all')*(R1_all*bv - r_all);  % test if all constants and slopes are zero in the HPERDollar regressions
Wp_IRDiff_all(1,1)        = 1-chi2cdf(W_IRDiff_all(1,1),2*(Nc-1));
W_FXIRDiff_con(1,1)       = (R2_con*bv - r)'*inv(R2_con*v*R2_con')*(R2_con*bv - r);          % test if all constants are zero in the FXEXR regressions
Wp_FXIRDiff_con(1,1)      = 1-chi2cdf(W_FXIRDiff_con(1,1),(Nc-1));
W_FXIRDiff_slope(1,1)     = (R2_slope*bv - r)'*inv(R2_slope*v*R2_slope')*(R2_slope*bv - r);  % test if all slopes are zero in the FXEXR regressions
Wp_FXIRDiff_slope(1,1)    = 1-chi2cdf(W_FXIRDiff_slope(1,1),Nc-1);
W_FXIRDiff_all(1,1)       = (R2_all*bv - r_all)'*inv(R2_all*v*R2_all')*(R2_all*bv - r_all);  % test if all constants and slopes are zero in the FXEXR regressions
Wp_FXIRDiff_all(1,1)      = 1-chi2cdf(W_FXIRDiff_all(1,1),2*(Nc-1));
W_IRDiffLocal_con(1,1)    = (R3_con*bv - r)'*inv(R3_con*v*R3_con')*(R3_con*bv - r);          % test if all constants are zero in the HPER regressions
Wp_IRDiffLocal_con(1,1)   = 1-chi2cdf(W_IRDiffLocal_con(1,1),Nc-1);
W_IRDiffLocal_slope(1,1)  = (R3_slope*bv - r)'*inv(R3_slope*v*R3_slope')*(R3_slope*bv - r);  % test if all slopes are zero in the HPER regressions
Wp_IRDiffLocal_slope(1,1) = 1-chi2cdf(W_IRDiffLocal_slope(1,1),Nc-1);
W_IRDiffLocal_all(1,1)    = (R3_all*bv - r_all)'*inv(R3_all*v*R3_all')*(R3_all*bv - r_all);  % test if all constants and slopes are zero in the HPER regressions
Wp_IRDiffLocal_all(1,1)   = 1-chi2cdf(W_IRDiffLocal_all(1,1),2*(Nc-1));
W_IRDiff_TermStr(Nc+2,1)  = (R_diff*bv - r)'*inv(R_diff*v*R_diff')*(R_diff*bv - r);          % test if all HPERDollar/FXEXR slope differentials are zero
Wp_IRDiff_TermStr(Nc+2,1) = 1-chi2cdf(W_IRDiff_TermStr(Nc+2,1),Nc-1);

[Wp_IRDiff_con_b(1,1),Wp_IRDiff_con_b_ratio(1,1)]               = chisq_pval_b(W_IRDiff_con(1,1),Nc-1,(lags+1)/T);
[Wp_IRDiff_slope_b(1,1),Wp_IRDiff_slope_b_ratio(1,1)]           = chisq_pval_b(W_IRDiff_slope(1,1),Nc-1,(lags+1)/T);
[Wp_IRDiff_all_b(1,1),Wp_IRDiff_all_b_ratio(1,1)]               = chisq_pval_b(W_IRDiff_all(1,1),2*(Nc-1),(lags+1)/T);
[Wp_FXIRDiff_con_b(1,1),Wp_FXIRDiff_con_b_ratio(1,1)]           = chisq_pval_b(W_FXIRDiff_con(1,1),Nc-1,(lags+1)/T);
[Wp_FXIRDiff_slope_b(1,1),Wp_FXIRDiff_slope_b_ratio(1,1)]       = chisq_pval_b(W_FXIRDiff_slope(1,1),Nc-1,(lags+1)/T);
[Wp_FXIRDiff_all_b(1,1),Wp_FXIRDiff_all_b_ratio(1,1)]           = chisq_pval_b(W_FXIRDiff_all(1,1),2*(Nc-1),(lags+1)/T);
[Wp_IRDiffLocal_con_b(1,1),Wp_IRDiffLocal_con_b_ratio(1,1)]     = chisq_pval_b(W_IRDiffLocal_con(1,1),Nc-1,(lags+1)/T);
[Wp_IRDiffLocal_slope_b(1,1),Wp_IRDiffLocal_slope_b_ratio(1,1)] = chisq_pval_b(W_IRDiffLocal_slope(1,1),Nc-1,(lags+1)/T);
[Wp_IRDiffLocal_all_b(1,1),Wp_IRDiffLocal_all_b_ratio(1,1)]     = chisq_pval_b(W_IRDiffLocal_all(1,1),2*(Nc-1),(lags+1)/T); 
[Wp_IRDiff_TermStr_b(Nc+2,1),Wp_IRDiff_TermStr_b_ratio(Nc+2,1)] = chisq_pval_b(W_IRDiff_TermStr(Nc+2,1),Nc-1,(lags+1)/T);

end

% Panel regression

lhv_panel = [];
rhv_panel = zeros(2*T*(Nc-1),2*Nc);
R_slope = [zeros(1,Nc-1) 1 zeros(1,Nc-1) -1];
R_con_1 = zeros(Nc-1,2*Nc);
R_con_2 = zeros(Nc-1,2*Nc);
r = 0;
r_con = zeros(Nc-1,1);
for n = 1:Nc-1
T = size(BondsHPERDollar,1)-horizon+1;
R = [0 1 0 -1];
r = 0;
lhv  = [(BondsHPERDollar(1:end-horizon+1,n)-BondsHPERDollar(1:end-horizon+1,end))*ScalingFactor;  FXEXR(1:end-horizon+1,n)*ScalingFactor];
rhv  = zeros(2*T,4);
rhv(1:T,1:2)     = [ones(T,1) 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end))];
rhv(T+1:2*T,3:4) = [ones(T,1) 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end))];
[lhv_s,rhv_s]                     = FindCommonSample(lhv,rhv);
[bv,sebv,R2v,R2vadj,v]            = olsgmm(lhv_s,rhv_s,lags,1);
W_IRDiff_TermStr(n,1)  = (R*bv - r)'*inv(R*v*R')*(R*bv - r);      % test if long-term and short-term bond slopes are equal
Wp_IRDiff_TermStr(n,1) = 1-chi2cdf(W_IRDiff_TermStr(n,1),1);
Wp_IRDiff_TermStr_b(n,1) = chisq_pval_b(W_IRDiff_TermStr(n,1),1,(lags+1)/T);


lhv_panel = [lhv_panel; lhv];
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,n) = ones(T,1);
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,Nc)= 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end));
rhv_panel((n-1)*2*T+T+1:n*2*T,Nc+n)  = ones(T,1);
rhv_panel((n-1)*2*T+T+1:n*2*T,2*Nc)  = 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end));

lhv_panel_DK(:,n)        = lhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:);
lhv_panel_DK(:,Nc-1+n)   = lhv_panel((n-1)*2*T+T+1:n*2*T,:);
rhv_panel_DK(:,n,:)      = reshape(rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:),T,[]);
rhv_panel_DK(:,Nc-1+n,:) = reshape(rhv_panel((n-1)*2*T+T+1:n*2*T,:),T,[]);

R_con_1(n,n)    = 1;
R_con_2(n,Nc+n) = 1;
end
[bv,~,~,~,~,~,~,v,sebv]              = HszDk5dwPs(lhv_panel_DK,ones(size(lhv_panel_DK,1),1),rhv_panel_DK,1,lags);
W_IRDiff_TermStr(Nc+1,1)  = (R_slope*bv - r)'*inv(R_slope*v*R_slope')*(R_slope*bv - r);            % test if long-term and short-term bond slopes are equal in the panel regression
Wp_IRDiff_TermStr(Nc+1,1) = 1-chi2cdf(W_IRDiff_TermStr(Nc+1,1),1);
W_IRDiff_con(2,1)         = (R_con_1*bv - r_con)'*inv(R_con_1*v*R_con_1')*(R_con_1*bv - r_con);    % test if all USD bond return constants are zero in the panel regression
Wp_IRDiff_con(2,1)        = 1-chi2cdf(W_IRDiff_con(2,1),Nc-1);
W_FXIRDiff_con(2,1)       = (R_con_2*bv - r_con)'*inv(R_con_2*v*R_con_2')*(R_con_2*bv - r_con);    % test if all FXEXR constants are zero in the panel regression
Wp_FXIRDiff_con(2,1)      = 1-chi2cdf(W_FXIRDiff_con(2,1),Nc-1);

Wp_IRDiff_TermStr_b(Nc+1,1) = chisq_pval_b(W_IRDiff_TermStr(Nc+1,1),1,(lags+1)/T); 
Wp_IRDiff_con_b(2,1)        = chisq_pval_b(W_IRDiff_con(2,1),Nc-1,(lags+1)/T); 
Wp_FXIRDiff_con_b(2,1)      = chisq_pval_b(W_FXIRDiff_con(2,1),Nc-1,(lags+1)/T); 

% Tables

rows = Countries_Sample';
rows{1,Nc+1} = '--';  
rows{1,Nc+2} = 'Panel (DK)';
rows{1,Nc+3} = 'Joint zero (p-value)';

Table_TS_IR_Collected_Simple = [TSPred_BetasIRDiff_con TSPred_BetasIRDiff_con_se ...
                         TSPred_BetasIRDiff TSPred_BetasIRDiff_se 100*TSPred_BetasIRDiff_R2adj...
                         TSPred_BetasFXIRDiff_con TSPred_BetasFXIRDiff_con_se ...
                         TSPred_BetasFXIRDiff TSPred_BetasFXIRDiff_se 100*TSPred_BetasFXIRDiff_R2adj...
                         TSPred_BetasIRDiffLocal_con TSPred_BetasIRDiffLocal_con_se...
                         TSPred_BetasIRDiffLocal TSPred_BetasIRDiffLocal_se 100*TSPred_BetasIRDiffLocal_R2adj...
                         [Wp_IRDiff_TermStr(1:Nc+1,1); Wp_IRDiff_TermStr(Nc+1,1)]...
                         TSPred_BetasIRDiff_Nobs];       
Table_TS_IR_Collected_Simple = [Table_TS_IR_Collected_Simple; Wp_IRDiff_con(1,1) NaN Wp_IRDiff_slope(1,1) NaN NaN Wp_FXIRDiff_con(1,1) NaN Wp_FXIRDiff_slope(1,1) NaN NaN Wp_IRDiffLocal_con(1,1) NaN Wp_IRDiffLocal_slope(1,1) NaN NaN Wp_IRDiff_TermStr(Nc+2,1) NaN];
cols  = {'coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','p-value','Obs.'}; 
disp(' ');
disp('-----------------------------------------');
disp('Collected Summary Results of Time-Series Regressions on Interest Rate Differentials');
disp('-----------------------------------------');
mat2tex(Table_TS_IR_Collected_Simple,'%6.2f',rows,cols, [], Opts)

                                  
Table_TS_IR_Collected_Simple_b = [TSPred_BetasIRDiff_con TSPred_BetasIRDiff_con_se TSPred_BetasIRDiff_con_pval_b ...
                         TSPred_BetasIRDiff TSPred_BetasIRDiff_se TSPred_BetasIRDiff_pval_b 100*TSPred_BetasIRDiff_R2adj...
                         TSPred_BetasFXIRDiff_con TSPred_BetasFXIRDiff_con_se TSPred_BetasFXIRDiff_con_pval_b ...
                         TSPred_BetasFXIRDiff TSPred_BetasFXIRDiff_se TSPred_BetasFXIRDiff_pval_b 100*TSPred_BetasFXIRDiff_R2adj...
                         TSPred_BetasIRDiffLocal_con TSPred_BetasIRDiffLocal_con_se TSPred_BetasIRDiffLocal_con_pval_b ...
                         TSPred_BetasIRDiffLocal TSPred_BetasIRDiffLocal_se TSPred_BetasIRDiffLocal_pval_b 100*TSPred_BetasIRDiffLocal_R2adj...
                         [Wp_IRDiff_TermStr_b(1:Nc+1,1); Wp_IRDiff_TermStr_b(Nc+1,1)]...
                         TSPred_BetasIRDiff_Nobs];       
Table_TS_IR_Collected_Simple_b = [Table_TS_IR_Collected_Simple_b; Wp_IRDiff_con_b(1,1) NaN NaN Wp_IRDiff_slope_b(1,1) NaN NaN NaN Wp_FXIRDiff_con_b(1,1) NaN NaN Wp_FXIRDiff_slope_b(1,1) NaN NaN NaN Wp_IRDiffLocal_con_b(1,1) NaN NaN Wp_IRDiffLocal_slope_b(1,1) NaN NaN NaN Wp_IRDiff_TermStr_b(Nc+2,1) NaN];
cols  = {'coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','p-value','Obs.'}; 
disp(' ');
disp('-----------------------------------------');
disp('Collected Summary Results of Time-Series Regressions on Interest Rate Differentials (fixed-b)');
disp('-----------------------------------------');
mat2tex(Table_TS_IR_Collected_Simple_b,'%6.2f',rows,cols, [], Opts)                                  

% Build Portfolios -- Sorted By Short-Term Interest Rates

% Get rid of interest rates if no data on currency or bond returns and vice-versa

for t=1:size(TB,1)                                               % For each date
    for c=1:size(TB,2)                                           % For each country
       if  isnan(FX(t,c)*FWD(t,c)*TB(t,c)*TB_month(t,c)*BondsHPERDollar(t,c)*BondsHPER(t,c)*FXEXR(t,c)*BondsHPRDiff(t,c))==1;
           TB(t,c)                    = NaN;
           TB_month(t,c)              = NaN;
           BondsHPERDollar(t,c)       = NaN;
           BondsHPER(t,c)             = NaN;
           FXEXR(t,c)                 = NaN;
           BondsHPRDiff(t,c)          = NaN;
           BondsHPRDiff_local(t,c)    = NaN;
           FX(t,c)                    = NaN;
           FWD(t,c)                   = NaN;
           Slope(t,c)                 = NaN;
       end         
    end;
end;

% Sign

sign_mat_IR = sign(TB_month-repmat(TB_month(:,end),1,Nc));

FXEXR_signed              = (FXEXR-repmat(FXEXR(:,end),1,Nc)).*sign_mat_IR; 
BondsHPR_signed           = (BondsHPR-repmat(BondsHPR(:,end),1,Nc)).*sign_mat_IR;  
BondsHPRDollar_signed     = (BondsHPRDollar-repmat(BondsHPRDollar(:,end),1,Nc)).*sign_mat_IR; 
BondsHPER_signed          = (BondsHPER-repmat(BondsHPER(:,end),1,Nc)).*sign_mat_IR;
BondsHPERDollar_signed    = (BondsHPERDollar-repmat(BondsHPERDollar(:,end),1,Nc)).*sign_mat_IR;
BondsHPRDiff_signed       = (BondsHPRDiff-repmat(BondsHPRDiff(:,end),1,Nc)).*sign_mat_IR; 
BondsHPRDiff_local_signed = (BondsHPRDiff_local-repmat(BondsHPRDiff_local(:,end),1,Nc)).*sign_mat_IR;
FWD_signed                = (FWD-repmat(FWD(:,end),1,Nc)).*sign_mat_IR;
FX_signed                 = (FX-repmat(FX(:,end),1,Nc)).*sign_mat_IR;
Slope_signed              = (Slope-repmat(Slope(:,end),1,Nc)).*sign_mat_IR;  
LT_UIP_signed             = (LT_UIP-repmat(LT_UIP(:,end),1,Nc)).*sign_mat_IR;
CPIHPER_signed            = (CPIHPER-repmat(CPIHPER(:,end),1,Nc)).*sign_mat_IR;
Ratings_signed            = (Ratings-repmat(Ratings(:,end),1,Nc)).*sign_mat_IR;  
RatingsOut_signed         = (RatingsOut-repmat(RatingsOut(:,end),1,Nc)).*sign_mat_IR;


% Sort on TB_month: the one-month interest rate (first obs. is interest rate from t0-1 to t0, theoretically known at t0-1); first obs. of portfolio HPRs are period returns from t0-1 to t0+horizon-1
    
for n=1:Nc
    
    [Portfolios_FXEXR(:,n),Portfolios_BondsHPER(:,n),Portfolios_BondsHPERDollar(:,n),Portfolios_BondsHPRDiff(:,n),Portfolios_VarSort(:,n),Track_portfolios(:,n)] = ...
    Build_portfolios_for4variables(FXEXR_signed(:,n),BondsHPER_signed(:,n),BondsHPERDollar_signed(:,n),BondsHPRDiff_signed(:,n),TB_month(:,n),1,1,TB_month_Rolling_Mean(:,n));

    [Portfolios_FXEXR(:,n),Portfolios_BondsHPER(:,n),Portfolios_BondsHPERDollar(:,n),Portfolios_BondsHPRDiff_local(:,n),Portfolios_VarSort(:,n),Track_portfolios_IR2_signed(:,n)] = ...
    Build_portfolios_for4variables(FXEXR_signed(:,n),BondsHPER_signed(:,n),BondsHPERDollar_signed(:,n),BondsHPRDiff_local_signed(:,n),TB_month(:,n),1,1,TB_month_Rolling_Mean(:,n));

    [Portfolios_FWD(:,n),Portfolios_FX(:,n),Portfolios_BondsHPR(:,n),Portfolios_BondsHPRDollar(:,n),Portfolios_VarSort(:,n),Track_portfolios_IR3_signed(:,n)] = ...
    Build_portfolios_for4variables(FWD_signed(:,n),FX_signed,BondsHPR_signed(:,n),BondsHPRDollar_signed(:,n),TB_month(:,n),1,1,TB_month_Rolling_Mean(:,n));  
  
    [~,~,~,Portfolios_SLOPE(:,n),Portfolios_VarSort(:,n),Track_portfolios_IR4_signed(:,n)] = ...
    Build_portfolios_for4variables(FWD_signed(:,n),BondsHPR_signed(:,n),BondsHPRDollar_signed(:,n),Slope_signed(:,n),TB_month(:,n),1,1,TB_month_Rolling_Mean(:,n));

    [~,~,~,Portfolios_LT_UIP(:,n), Portfolios_VarSort(:,n), Track_portfolios_IR5_signed(:,n)] = ...
    Build_portfolios_for4variables(FWD_signed(:,n),BondsHPR_signed(:,n),BondsHPRDollar_signed(:,n),LT_UIP_signed(:,n),TB_month(:,n),1,1,TB_month_Rolling_Mean(:,n));

    [~,Portfolios_Ratings(:,n),Portfolios_RatingsOut(:,n),Portfolios_CPIHPER(:,n), Portfolios_VarSort(:,n), Track_portfolios_IR6_signed(:,n)] = ...
    Build_portfolios_for4variables(FWD_signed(:,n),Ratings_signed(:,n),RatingsOut_signed(:,n),CPIHPER_signed(:,n),TB_month(:,n),1,1,TB_month_Rolling_Mean(:,n));

end

   
    
[Portfolios_FXEXR(:,Nc+1),Portfolios_BondsHPER(:,Nc+1),Portfolios_BondsHPERDollar(:,Nc+1),Portfolios_BondsHPRDiff(:,Nc+1),Portfolios_VarSort(:,Nc+1),Track_portfolios] = ...
    Build_portfolios_for4variables(FXEXR_signed(:,1:end-1),BondsHPER_signed(:,1:end-1),BondsHPERDollar_signed(:,1:end-1),BondsHPRDiff_signed(:,1:end-1),TB_month(:,1:end-1),1,1,TB_month_Rolling_Mean(:,1:end-1));

[Portfolios_FXEXR(:,Nc+1),Portfolios_BondsHPER(:,Nc+1),Portfolios_BondsHPERDollar(:,Nc+1),Portfolios_BondsHPRDiff_local(:,Nc+1),Portfolios_VarSort(:,Nc+1),Track_portfolios_IR2_signed] = ...
    Build_portfolios_for4variables(FXEXR_signed(:,1:end-1),BondsHPER_signed(:,1:end-1),BondsHPERDollar_signed(:,1:end-1),BondsHPRDiff_local_signed(:,1:end-1),TB_month(:,1:end-1),1,1,TB_month_Rolling_Mean(:,1:end-1));

[Portfolios_FWD(:,Nc+1),Portfolios_FX(:,Nc+1),Portfolios_BondsHPR(:,Nc+1),Portfolios_BondsHPRDollar(:,Nc+1),Portfolios_VarSort(:,Nc+1),Track_portfolios_IR3_signed] = ...
    Build_portfolios_for4variables(FWD_signed(:,1:end-1),FX_signed,BondsHPR_signed(:,1:end-1),BondsHPRDollar_signed(:,1:end-1),TB_month(:,1:end-1),1,1,TB_month_Rolling_Mean(:,1:end-1));  
  
[~,~,~,Portfolios_SLOPE(:,Nc+1),Portfolios_VarSort(:,Nc+1),Track_portfolios_IR4_signed] = ...
    Build_portfolios_for4variables(FWD_signed(:,1:end-1),BondsHPR_signed(:,1:end-1),BondsHPRDollar_signed(:,1:end-1),Slope_signed(:,1:end-1),TB_month(:,1:end-1),1,1,TB_month_Rolling_Mean(:,1:end-1));

[~,~,~,Portfolios_LT_UIP(:,Nc+1), Portfolios_VarSort(:,Nc+1), Track_portfolios_IR5_signed] = ...
    Build_portfolios_for4variables(FWD_signed(:,1:end-1),BondsHPR_signed(:,1:end-1),BondsHPRDollar_signed(:,1:end-1),LT_UIP_signed(:,1:end-1),TB_month(:,1:end-1),1,1,TB_month_Rolling_Mean(:,1:end-1));

[~,Portfolios_Ratings(:,Nc+1),Portfolios_RatingsOut(:,Nc+1),Portfolios_CPIHPER(:,Nc+1), Portfolios_VarSort(:,Nc+1), Track_portfolios_IR6_signed] = ...
    Build_portfolios_for4variables(FWD_signed(:,1:end-1),Ratings_signed(:,1:end-1),RatingsOut_signed(:,1:end-1),CPIHPER_signed(:,1:end-1),TB_month(:,1:end-1),1,1,TB_month_Rolling_Mean(:,1:end-1));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%bootstrapping
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

portfolio_data                                    = [(Portfolios_FXEXR) (Portfolios_BondsHPER) (Portfolios_BondsHPERDollar) (Portfolios_BondsHPRDiff) Portfolios_FX Portfolios_BondsHPRDiff_local Portfolios_FWD Portfolios_SLOPE Portfolios_CPIHPER Portfolios_Ratings Portfolios_RatingsOut];

[standard_error_mean,standard_error_SR]           = bootstrap_portfolio_returns_growth(portfolio_data,nboot,horizon);

Total_portfolios=Nc+1;

Portfolios_FXEXR_se                  = standard_error_mean(1:Total_portfolios);
Portfolios_BondsHPER_se              = standard_error_mean(1+Total_portfolios:2*Total_portfolios);
Portfolios_BondsHPERDollar_se        = standard_error_mean(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_BondsHPRDiff_se           = standard_error_mean(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_se                     = standard_error_mean(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_BondsHPRDiff_local_se     = standard_error_mean(1+5*Total_portfolios:6*Total_portfolios);
Portfolios_FWD_se                    = standard_error_mean(1+6*Total_portfolios:7*Total_portfolios);
Portfolios_SLOPE_se                  = standard_error_mean(1+7*Total_portfolios:8*Total_portfolios);
Portfolios_CPIHPER_se                = standard_error_mean(1+8*Total_portfolios:9*Total_portfolios);
Portfolios_Ratings_se                = standard_error_mean(1+9*Total_portfolios:10*Total_portfolios);
Portfolios_RatingsOut_se             = standard_error_mean(1+10*Total_portfolios:11*Total_portfolios);

Portfolios_FXEXR_SR_se               = standard_error_SR(1:Total_portfolios);
Portfolios_BondsHPER_SR_se           = standard_error_SR(1+Total_portfolios:2*Total_portfolios);
Portfolios_BondsHPERDollar_SR_se     = standard_error_SR(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_BondsHPRDiff_SR_se        = standard_error_SR(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_SR_se                  = standard_error_SR(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_BondsHPRDiff_local_SR_se  = standard_error_SR(1+5*Total_portfolios:6*Total_portfolios);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Table_Ptf_IR_signed_Simple = [100*nanmean((Portfolios_BondsHPERDollar))*ScalingFactor;       
         100*Portfolios_BondsHPERDollar_se*ScalingFactor;
         100*nanstd((Portfolios_BondsHPERDollar))*sqrt(ScalingFactor); 
         nanmean((Portfolios_BondsHPERDollar))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPERDollar));
         Portfolios_BondsHPERDollar_SR_se*sqrt(ScalingFactor);
         NaN(1,Nc+1);
         100*nanmean((Portfolios_FXEXR))*ScalingFactor;                 
         100*Portfolios_FXEXR_se*ScalingFactor; 
         100*nanstd((Portfolios_FXEXR))*sqrt(ScalingFactor); 
         nanmean((Portfolios_FXEXR))*sqrt(ScalingFactor)./nanstd((Portfolios_FXEXR)); 
         Portfolios_FXEXR_SR_se*sqrt(ScalingFactor);
         NaN(1,Nc+1);
         100*nanmean((Portfolios_BondsHPER))*ScalingFactor;   
         100*Portfolios_BondsHPER_se*ScalingFactor; 
         100*nanstd((Portfolios_BondsHPER))*sqrt(ScalingFactor); 
         nanmean((Portfolios_BondsHPER))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPER));
         Portfolios_BondsHPER_SR_se*sqrt(ScalingFactor)]';
rows = Countries_Sample';      
rows{1,Nc+1} = 'Equally-weighted';   
Opts.Style = 'jf';
disp(' ');
disp('----------------------------------------------');
disp('Long-Short Portfolio Sorted By Interest Rates');
disp('----------------------------------------------'); 
mat2tex(Table_Ptf_IR_signed_Simple,'%6.2f',rows,[], [], Opts)


    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Time-Series Predictability Regressions: Foreign Bond HPERs (and Components) on Yield Curve Slopes
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nc   = size(Countries_Sample,1);

if NW_lag_choice == 1
    
    lags_vec = max(6,horizon+1);  
    
    elseif NW_lag_choice == 2
        
    lags_vec = ceil(4*(size(BondsHPER,1)/100)^(2/9))-1;
    
    elseif NW_lag_choice == 3
        
    lags_vec = ceil(0.75*(size(BondsHPER,1))^(1/3))-1;
    
    elseif NW_lag_choice == 4
        
    lags_vec = max(ceil(4*(size(BondsHPER,1)/100)^(2/9))-1,ceil(0.75*(size(BondsHPER,1))^(1/3))-1);
    
    elseif NW_lag_choice == 5
        
    lags_vec = ceil(1.3*(size(BondsHPER,1))^(1/2))-1;
        
end

maxlags   = size(lags_vec,2);

TSPred_BetasSlopeDiff_con          = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiff_con_se       = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiff_con_boot     = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiff_con_se_boot  = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiff              = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiff_se           = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiff_boot         = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiff_se_boot      = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiff_R2adj        = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiff_Nobs         = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiff_con_pval_b   = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiff_pval_b       = NaN*zeros(Nc,maxlags);

TSPred_BetasFXSlopeDiff_con         = NaN*zeros(Nc,maxlags);
TSPred_BetasFXSlopeDiff_con_se      = NaN*zeros(Nc,maxlags);
TSPred_BetasFXSlopeDiff_con_boot    = NaN*zeros(Nc,maxlags);
TSPred_BetasFXSlopeDiff_con_se_boot = NaN*zeros(Nc,maxlags);
TSPred_BetasFXSlopeDiff             = NaN*zeros(Nc,maxlags);
TSPred_BetasFXSlopeDiff_se          = NaN*zeros(Nc,maxlags);
TSPred_BetasFXSlopeDiff_boot        = NaN*zeros(Nc,maxlags);
TSPred_BetasFXSlopeDiff_se_boot     = NaN*zeros(Nc,maxlags);
TSPred_BetasFXSlopeDiff_R2adj       = NaN*zeros(Nc,maxlags);
TSPred_BetasFXSlopeDiff_Nobs        = NaN*zeros(Nc,maxlags);
TSPred_BetasFXSlopeDiff_con_pval_b  = NaN*zeros(Nc,maxlags);
TSPred_BetasFXSlopeDiff_con_pval    = NaN*zeros(Nc,maxlags);

TSPred_BetasSlopeDiffLocal_con          = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiffLocal_con_se       = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiffLocal_con_boot     = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiffLocal_con_se_boot  = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiffLocal              = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiffLocal_se           = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiffLocal_boot         = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiffLocal_se_boot      = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiffLocal_R2adj        = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiffLocal_Nobs         = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiffLocal_con_pval_b   = NaN*zeros(Nc,maxlags);
TSPred_BetasSlopeDiffLocal_con_pval     = NaN*zeros(Nc,maxlags);


block_length = horizon;

for n=1:Nc
    
for lags = lags_vec    
    
    % Regress USD bond HPR differentials (foreign minus US) on slope differentials (foreign minus US)

    lhv = (BondsHPERDollar(:,n)-BondsHPERDollar(:,end))*ScalingFactor;
    rhv = [ones(size(lhv,1),1) 12*Slope(:,n)-12*Slope(:,end)];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasSlopeDiff_con(n,1)        = bv(1);
        TSPred_BetasSlopeDiff_con_se(n,1)     = sebv(1);
        TSPred_BetasSlopeDiff_con_boot(n,1)   = bootstrap_stats(1,1);
        TSPred_BetasSlopeDiff_con_se_boot(n,1)= bootstrap_stats(1,2);
        TSPred_BetasSlopeDiff(n,1)        = bv(2);
        TSPred_BetasSlopeDiff_se(n,1)     = sebv(2);
        TSPred_BetasSlopeDiff_boot(n,1)   = bootstrap_stats(2,1);
        TSPred_BetasSlopeDiff_se_boot(n,1)= bootstrap_stats(2,2);
        TSPred_BetasSlopeDiff_R2adj(n,1)  = R2vadj;
        TSPred_BetasSlopeDiff_Nobs(n,1)   = size(lhv_s,1);
        TSPred_BetasSlopeDiff_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasSlopeDiff_con(n,1)/TSPred_BetasSlopeDiff_con_se(n,1),(lags+1)/TSPred_BetasSlopeDiff_Nobs(n,1));                 
        TSPred_BetasSlopeDiff_pval_b(n,1)     = tstat_pval_b(TSPred_BetasSlopeDiff(n,1)/TSPred_BetasSlopeDiff_se(n,1),(lags+1)/TSPred_BetasSlopeDiff_Nobs(n,1));   
    end;
    
    
    % Regress excess FX returns on slope differentials (foreign minus US)

    lhv = FXEXR(:,n)*ScalingFactor;
    % lhv = FX(:,n)*ScalingFactor; % Alternative decomposition
    rhv = [ones(size(lhv,1),1) 12*Slope(:,n)-12*Slope(:,end)];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc 
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasFXSlopeDiff_con(n,1)        = bv(1);
        TSPred_BetasFXSlopeDiff_con_se(n,1)     = sebv(1);
        TSPred_BetasFXSlopeDiff_con_boot(n,1)   = bootstrap_stats(1,1);
        TSPred_BetasFXSlopeDiff_con_se_boot(n,1)= bootstrap_stats(1,2);
        TSPred_BetasFXSlopeDiff(n,1)        = bv(2);
        TSPred_BetasFXSlopeDiff_se(n,1)     = sebv(2);
        TSPred_BetasFXSlopeDiff_boot(n,1)   = bootstrap_stats(2,1);
        TSPred_BetasFXSlopeDiff_se_boot(n,1)= bootstrap_stats(2,2);
        TSPred_BetasFXSlopeDiff_R2adj(n,1)  = R2vadj;
        TSPred_BetasFXSlopeDiff_Nobs(n,1)   = size(lhv_s,1);
        TSPred_BetasFXSlopeDiff_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasFXSlopeDiff_con(n,1)/TSPred_BetasFXSlopeDiff_con_se(n,1),(lags+1)/TSPred_BetasFXSlopeDiff_Nobs(n,1));                 
        TSPred_BetasFXSlopeDiff_pval_b(n,1)     = tstat_pval_b(TSPred_BetasFXSlopeDiff(n,1)/TSPred_BetasFXSlopeDiff_se(n,1),(lags+1)/TSPred_BetasFXSlopeDiff_Nobs(n,1));   
    end;
    
    
    % Regress bond local currency HPER differentials (foreign minus US) on slope differentials (foreign minus US)

    lhv = (BondsHPER(:,n)-BondsHPER(:,end))*ScalingFactor;
    % lhv = (log(1+BondsHPR(:,n))-log(1+BondsHPR(:,end)))*ScalingFactor; % Alternative decomposition
    rhv = [ones(size(lhv,1),1) 12*Slope(:,n)-12*Slope(:,end)];
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc 
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasSlopeDiffLocal_con(n,1)        = bv(1);
        TSPred_BetasSlopeDiffLocal_con_se(n,1)     = sebv(1);
        TSPred_BetasSlopeDiffLocal_con_boot(n,1)   = bootstrap_stats(1,1);
        TSPred_BetasSlopeDiffLocal_con_se_boot(n,1)= bootstrap_stats(1,2);
        TSPred_BetasSlopeDiffLocal(n,1)        = bv(2);
        TSPred_BetasSlopeDiffLocal_se(n,1)     = sebv(2);
        TSPred_BetasSlopeDiffLocal_boot(n,1)   = bootstrap_stats(2,1);
        TSPred_BetasSlopeDiffLocal_se_boot(n,1)= bootstrap_stats(2,2);
        TSPred_BetasSlopeDiffLocal_R2adj(n,1)  = R2vadj;
        TSPred_BetasSlopeDiffLocal_Nobs(n,1)   = size(lhv_s,1);
        TSPred_BetasSlopeDiffLocal_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasSlopeDiffLocal_con(n,1)/TSPred_BetasSlopeDiffLocal_con_se(n,1),(lags+1)/TSPred_BetasSlopeDiffLocal_Nobs(n,1));                 
        TSPred_BetasSlopeDiffLocal_pval_b(n,1)     = tstat_pval_b(TSPred_BetasSlopeDiffLocal(n,1)/TSPred_BetasSlopeDiffLocal_se(n,1),(lags+1)/TSPred_BetasSlopeDiffLocal_Nobs(n,1));   
    end;

        
end;

end

for lags = lags_vec

% Panel regressions with Driscoll-Kraay s.e.

Panel_Var = [ScalingFactor*BondHPERDollar_Diff, 12*Slope_Diff];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasSlopeDiff_con(Nc+2,1)     = NaN;
TSPred_BetasSlopeDiff_con_se(Nc+2,1)  = NaN;
TSPred_BetasSlopeDiff(Nc+2,1)         = theta(1,1);
TSPred_BetasSlopeDiff_se(Nc+2,1)      = theta_se(1,1);
TSPred_BetasSlopeDiff_Nobs(Nc+2,1)    = sum(sum(~isnan(yhat)));
TSPred_BetasSlopeDiff_R2adj(Nc+2,1)   = 1-(1-Rsq)*(TSPred_BetasSlopeDiff_Nobs(Nc+2,1)-1)/(TSPred_BetasSlopeDiff_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasSlopeDiff_con_boot(Nc+2,1)    = NaN;
TSPred_BetasSlopeDiff_con_se_boot(Nc+2,1) = NaN;
TSPred_BetasSlopeDiff_boot(Nc+2,1)        = bootstrap_stats(1,1);
TSPred_BetasSlopeDiff_se_boot(Nc+2,1)     = bootstrap_stats(1,2);
TSPred_BetasSlopeDiff_con_pval_b(Nc+2,1)  = tstat_pval_b(TSPred_BetasSlopeDiff_con(Nc+2,1)/TSPred_BetasSlopeDiff_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasSlopeDiff_pval_b(Nc+2,1)      = tstat_pval_b(TSPred_BetasSlopeDiff(Nc+2,1)/TSPred_BetasSlopeDiff_se(Nc+2,1), (lags+1)/size(y_DK,1));                 



Panel_Var = [ScalingFactor*FXEXR_Panel, 12*Slope_Diff];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasFXSlopeDiff_con(Nc+2,1)    = NaN;
TSPred_BetasFXSlopeDiff_con_se(Nc+2,1) = NaN;
TSPred_BetasFXSlopeDiff(Nc+2,1)        = theta(1,1);
TSPred_BetasFXSlopeDiff_se(Nc+2,1)     = theta_se(1,1);
TSPred_BetasFXSlopeDiff_Nobs(Nc+2,1)   = sum(sum(~isnan(yhat)));
TSPred_BetasFXSlopeDiff_R2adj(Nc+2,1)  = 1-(1-Rsq)*(TSPred_BetasFXSlopeDiff_Nobs(Nc+2,1)-1)/(TSPred_BetasFXSlopeDiff_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasFXSlopeDiff_con_boot(Nc+2,1)    = NaN;
TSPred_BetasFXSlopeDiff_con_se_boot(Nc+2,1) = NaN;
TSPred_BetasFXSlopeDiff_boot(Nc+2,1)        = bootstrap_stats(1,1);
TSPred_BetasFXSlopeDiff_se_boot(Nc+2,1)     = bootstrap_stats(1,2);
TSPred_BetasFXSlopeDiff_con_pval_b(Nc+2,1)  = tstat_pval_b(TSPred_BetasFXSlopeDiff_con(Nc+2,1)/TSPred_BetasFXSlopeDiff_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasFXSlopeDiff_pval_b(Nc+2,1)      = tstat_pval_b(TSPred_BetasFXSlopeDiff(Nc+2,1)/TSPred_BetasFXSlopeDiff_se(Nc+2,1), (lags+1)/size(y_DK,1));                 

 

Panel_Var = [ScalingFactor*BondHPER_Diff, 12*Slope_Diff];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK        = Panel_Var(:,1:Nc-1);
X_DK(:,:,1) = Panel_Var(:,Nc:2*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+1) = 0*y_DK;
    X_DK(:,j,j+1) = ones(size(y_DK,1),1);
end

[theta, ~, ~,~,yhat,Rsq,~,~,theta_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasSlopeDiffLocal_con(Nc+2,1)    = NaN;
TSPred_BetasSlopeDiffLocal_con_se(Nc+2,1) = NaN;
TSPred_BetasSlopeDiffLocal(Nc+2,1)        = theta(1,1);
TSPred_BetasSlopeDiffLocal_se(Nc+2,1)     = theta_se(1,1);
TSPred_BetasSlopeDiffLocal_Nobs(Nc+2,1)   = sum(sum(~isnan(yhat)));
TSPred_BetasSlopeDiffLocal_R2adj(Nc+2,1)  = 1-(1-Rsq)*(TSPred_BetasSlopeDiffLocal_Nobs(Nc+2,1)-1)/(TSPred_BetasSlopeDiffLocal_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasSlopeDiffLocal_con_boot(Nc+2,1)    = NaN;
TSPred_BetasSlopeDiffLocal_con_se_boot(Nc+2,1) = NaN;
TSPred_BetasSlopeDiffLocal_boot(Nc+2,1)        = bootstrap_stats(1,1);
TSPred_BetasSlopeDiffLocal_se_boot(Nc+2,1)     = bootstrap_stats(1,2);
TSPred_BetasSlopeDiffLocal_con_pval_b(Nc+2,1)  = tstat_pval_b(TSPred_BetasSlopeDiffLocal_con(Nc+2,1)/TSPred_BetasSlopeDiffLocal_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasSlopeDiffLocal_pval_b(Nc+2,1)      = tstat_pval_b(TSPred_BetasSlopeDiffLocal(Nc+2,1)/TSPred_BetasSlopeDiffLocal_se(Nc+2,1), (lags+1)/size(y_DK,1));                 



% Joint estimation

T          = size(BondsHPERDollar,1)-horizon+1;
R1_con     = zeros(Nc-1,6*(Nc-1));
R1_slope   = zeros(Nc-1,6*(Nc-1));
R1_all     = zeros(2*(Nc-1),6*(Nc-1));
R2_con     = zeros(Nc-1,6*(Nc-1));
R2_slope   = zeros(Nc-1,6*(Nc-1));
R2_all     = zeros(2*(Nc-1),6*(Nc-1));
R3_con     = zeros(Nc-1,6*(Nc-1));
R3_slope   = zeros(Nc-1,6*(Nc-1));
R3_all     = zeros(2*(Nc-1),6*(Nc-1));
R_diff     = zeros(Nc-1,6*(Nc-1));
r          = zeros(Nc-1,1);
r_all      = zeros(2*(Nc-1),1);
rhv        = zeros(3*T*(Nc-1),6*(Nc-1));
for n=1:Nc-1
lhv((n-1)*T+1:n*T,1)                                                     = (BondsHPERDollar(1:end-horizon+1,n)-BondsHPERDollar(1:end-horizon+1,end))*ScalingFactor;
rhv((n-1)*T+1:n*T,(n-1)*2+1:n*2)                                         = [ones(T,1) 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end)];
lhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,1)                                   = FXEXR(1:end-horizon+1,n)*ScalingFactor;
rhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,2*(Nc-1)+(n-1)*2+1:2*(Nc-1)+n*2)     = [ones(T,1) 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end)];
lhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,1)                               = (BondsHPER(1:end-horizon+1,n)-BondsHPER(1:end-horizon+1,end))*ScalingFactor;
rhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,4*(Nc-1)+(n-1)*2+1:4*(Nc-1)+n*2) = [ones(T,1) 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end)];


R1_con(n,2*n-1)          =  1;
R1_slope(n,2*n)          =  1;
R1_all(n,2*n-1)          =  1;
R1_all(Nc-1+n,2*n)       =  1;
R2_con(n,2*(Nc-1)+2*n-1) =  1;
R2_slope(n,2*(Nc-1)+2*n) =  1;
R2_all(n,2*(Nc-1)+2*n-1) =  1;
R2_all(Nc-1+n,2*(Nc-1)+2*n) =  1;
R3_con(n,4*(Nc-1)+2*n-1) =  1;
R3_slope(n,4*(Nc-1)+2*n) =  1;
R3_all(n,4*(Nc-1)+2*n-1) =  1;
R3_all(Nc-1+n,4*(Nc-1)+2*n) =  1;
R_diff(n,2*n)            =  1;
R_diff(n,2*(Nc-1)+2*n)   = -1;
end
[lhv_s,rhv_s]                           = FindCommonSample(lhv,rhv);
[bv,sebv,R2v,R2vadj,v]                  = olsgmm(lhv_s,rhv_s,lags,1);
W_SlopeDiff_con(1,1)         = (R1_con*bv - r)'*inv(R1_con*v*R1_con')*(R1_con*bv - r);          % test if all constants are zero in the HPERDollar regressions
Wp_SlopeDiff_con(1,1)        = 1-chi2cdf(W_SlopeDiff_con(1,1),Nc-1);
W_SlopeDiff_slope(1,1)       = (R1_slope*bv - r)'*inv(R1_slope*v*R1_slope')*(R1_slope*bv - r);  % test if all slopes are zero in the HPERDollar regressions
Wp_SlopeDiff_slope(1,1)      = 1-chi2cdf(W_SlopeDiff_slope(1,1),Nc-1);
W_SlopeDiff_all(1,1)         = (R1_all*bv - r_all)'*inv(R1_all*v*R1_all')*(R1_all*bv - r_all);  % test if all constants and slopes are zero in the HPERDollar regressions
Wp_SlopeDiff_all(1,1)        = 1-chi2cdf(W_SlopeDiff_all(1,1),2*(Nc-1));
W_FXSlopeDiff_con(1,1)       = (R2_con*bv - r)'*inv(R2_con*v*R2_con')*(R2_con*bv - r);          % test if all constants are zero in the FXEXR regressions
Wp_FXSlopeDiff_con(1,1)      = 1-chi2cdf(W_FXSlopeDiff_con(1,1),Nc-1);
W_FXSlopeDiff_slope(1,1)     = (R2_slope*bv - r)'*inv(R2_slope*v*R2_slope')*(R2_slope*bv - r);  % test if all slopes are zero in the FXEXR regressions
Wp_FXSlopeDiff_slope(1,1)    = 1-chi2cdf(W_FXSlopeDiff_slope(1,1),Nc-1);
W_FXSlopeDiff_all(1,1)       = (R2_all*bv - r_all)'*inv(R2_all*v*R2_all')*(R2_all*bv - r_all);  % test if all constants and slopes are zero in the FXEXR regressions
Wp_FXSlopeDiff_all(1,1)      = 1-chi2cdf(W_FXSlopeDiff_all(1,1),2*(Nc-1));
W_SlopeDiffLocal_con(1,1)    = (R3_con*bv - r)'*inv(R3_con*v*R3_con')*(R3_con*bv - r);          % test if all constants are zero in the HPER regressions
Wp_SlopeDiffLocal_con(1,1)   = 1-chi2cdf(W_SlopeDiffLocal_con(1,1),Nc-1);
W_SlopeDiffLocal_slope(1,1)  = (R3_slope*bv - r)'*inv(R3_slope*v*R3_slope')*(R3_slope*bv - r);  % test if all slopes are zero in the HPER regressions
Wp_SlopeDiffLocal_slope(1,1) = 1-chi2cdf(W_SlopeDiffLocal_slope(1,1),Nc-1);
W_SlopeDiffLocal_all(1,1)    = (R3_all*bv - r_all)'*inv(R3_all*v*R3_all')*(R3_all*bv - r_all);  % test if all constants and slopes are zero in the HPER regressions
Wp_SlopeDiffLocal_all(1,1)   = 1-chi2cdf(W_SlopeDiffLocal_all(1,1),2*(Nc-1));
W_SlopeDiff_TermStr(Nc+2,1)  = (R_diff*bv - r)'*inv(R_diff*v*R_diff')*(R_diff*bv - r);          % test if all HPERDollar/FXEXR slope differentials are zero
Wp_SlopeDiff_TermStr(Nc+2,1) = 1-chi2cdf(W_SlopeDiff_TermStr(Nc+2,1),Nc-1);

Wp_SlopeDiff_con_b(1,1)        = chisq_pval_b(W_SlopeDiff_con(1,1),Nc-1,(lags+1)/T);
Wp_SlopeDiff_slope_b(1,1)      = chisq_pval_b(W_SlopeDiff_slope(1,1),Nc-1,(lags+1)/T);
Wp_SlopeDiff_all_b(1,1)        = chisq_pval_b(W_SlopeDiff_all(1,1),2*(Nc-1),(lags+1)/T);
Wp_FXSlopeDiff_con_b(1,1)      = chisq_pval_b(W_FXSlopeDiff_con(1,1),Nc-1,(lags+1)/T);
Wp_FXSlopeDiff_slope_b(1,1)    = chisq_pval_b(W_FXSlopeDiff_slope(1,1),Nc-1,(lags+1)/T);
Wp_FXSlopeDiff_all_b(1,1)      = chisq_pval_b(W_FXSlopeDiff_all(1,1),2*(Nc-1),(lags+1)/T);
Wp_SlopeDiffLocal_con_b(1,1)   = chisq_pval_b(W_SlopeDiffLocal_con(1,1),Nc-1,(lags+1)/T);
Wp_SlopeDiffLocal_slope_b(1,1) = chisq_pval_b(W_SlopeDiffLocal_slope(1,1),Nc-1,(lags+1)/T);
Wp_SlopeDiffLocal_all_b(1,1)   = chisq_pval_b(W_SlopeDiffLocal_all(1,1),2*(Nc-1),(lags+1)/T); 
Wp_SlopeDiff_TermStr_b(Nc+2,1) = chisq_pval_b(W_SlopeDiff_TermStr(Nc+2,1),Nc-1,(lags+1)/T);

end


% Panel regression

clear lhv_panel_DK rhv_panel_DK;
lhv_panel = [];
rhv_panel = zeros(2*T*(Nc-1),2*Nc);
R_slope = [zeros(1,Nc-1) 1 zeros(1,Nc-1) -1];
R_con_1 = zeros(Nc-1,2*Nc);
R_con_2 = zeros(Nc-1,2*Nc);
r = 0;
r_con = zeros(Nc-1,1);
for n = 1:Nc-1
T = size(BondsHPERDollar,1)-horizon+1;
R = [0 1 0 -1];
r = 0;
lhv  = [(BondsHPERDollar(1:end-horizon+1,n)-BondsHPERDollar(1:end-horizon+1,end))*ScalingFactor;  FXEXR(1:end-horizon+1,n)*ScalingFactor];
rhv  = zeros(2*T,4);
rhv(1:T,1:2)     = [ones(T,1) 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end)];
rhv(T+1:2*T,3:4) = [ones(T,1) 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end)];
[lhv_s,rhv_s]                        = FindCommonSample(lhv,rhv);
[bv,sebv,R2v,R2vadj,v]               = olsgmm(lhv_s,rhv_s,lags,1);
W_SlopeDiff_TermStr(n,1)  = (R*bv - r)'*inv(R*v*R')*(R*bv - r);
Wp_SlopeDiff_TermStr(n,1) = 1-chi2cdf(W_SlopeDiff_TermStr(n,1),1);
Wp_SlopeDiff_TermStr_b(n,1) = chisq_pval_b(W_SlopeDiff_TermStr(n,1),1,(lags+1)/T);


lhv_panel = [lhv_panel; lhv];
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,n) = ones(T,1);
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,Nc)= 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end);
rhv_panel((n-1)*2*T+T+1:n*2*T,Nc+n)  = ones(T,1);
rhv_panel((n-1)*2*T+T+1:n*2*T,2*Nc)  = 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end);

lhv_panel_DK(:,n)        = lhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:);
lhv_panel_DK(:,Nc-1+n)   = lhv_panel((n-1)*2*T+T+1:n*2*T,:);
rhv_panel_DK(:,n,:)      = reshape(rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:),T,[]);
rhv_panel_DK(:,Nc-1+n,:) = reshape(rhv_panel((n-1)*2*T+T+1:n*2*T,:),T,[]);

R_con_1(n,n)    = 1;
R_con_2(n,Nc+n) = 1;
end

[bv, ~, ~,~,~,~,~,v,~]                  = HszDk5dwPs(lhv_panel_DK,ones(T,1),rhv_panel_DK,1,lags);
W_SlopeDiff_TermStr(Nc+1,1)  = (R_slope*bv - r)'*inv(R_slope*v*R_slope')*(R_slope*bv - r);            % test if long-term and short-term bond slopes are equal in the panel regression
Wp_SlopeDiff_TermStr(Nc+1,1) = 1-chi2cdf(W_SlopeDiff_TermStr(Nc+1,1),1);
W_SlopeDiff_con(2,1)         = (R_con_1*bv - r_con)'*inv(R_con_1*v*R_con_1')*(R_con_1*bv - r_con);    % test if all USD bond return constants are zero in the panel regression
Wp_SlopeDiff_con(2,1)        = 1-chi2cdf(W_SlopeDiff_con(2,1),Nc-1);
W_FXSlopeDiff_con(2,1)       = (R_con_2*bv - r_con)'*inv(R_con_2*v*R_con_2')*(R_con_2*bv - r_con);    % test if all FXEXR constants are zero in the panel regression
Wp_FXSlopeDiff_con(2,1)      = 1-chi2cdf(W_FXSlopeDiff_con(2, 1),Nc-1);

Wp_SlopeDiff_TermStr_b(Nc+1,1) = chisq_pval_b(W_SlopeDiff_TermStr(Nc+1,1),1,(lags+1)/T); 
Wp_SlopeDiff_con_b(2,1)        = chisq_pval_b(W_SlopeDiff_con(2,1),Nc-1,(lags+1)/T); 
Wp_FXSlopeDiff_con_b(2,1)      = chisq_pval_b(W_FXSlopeDiff_con(2,1),Nc-1,(lags+1)/T); 


% Tables

rows = Countries_Sample';                   
rows{1,Nc+1} = '--';   
rows{1,Nc+2} = 'Panel (DK)';
rows{1,Nc+3} = 'Joint zero (p-value)';

Table_TS_Slope_Collected_Simple = [TSPred_BetasSlopeDiff_con TSPred_BetasSlopeDiff_con_se ...
                         TSPred_BetasSlopeDiff TSPred_BetasSlopeDiff_se 100*TSPred_BetasSlopeDiff_R2adj...
                         TSPred_BetasFXSlopeDiff_con TSPred_BetasFXSlopeDiff_con_se ...
                         TSPred_BetasFXSlopeDiff TSPred_BetasFXSlopeDiff_se 100*TSPred_BetasFXSlopeDiff_R2adj...
                         TSPred_BetasSlopeDiffLocal_con TSPred_BetasSlopeDiffLocal_con_se...
                         TSPred_BetasSlopeDiffLocal TSPred_BetasSlopeDiffLocal_se 100*TSPred_BetasSlopeDiffLocal_R2adj...
                         [Wp_SlopeDiff_TermStr(1:Nc+1,1); Wp_SlopeDiff_TermStr(Nc+1,1)]...
                         TSPred_BetasSlopeDiff_Nobs];    
Table_TS_Slope_Collected_Simple = [Table_TS_Slope_Collected_Simple; Wp_SlopeDiff_con(1,1) NaN Wp_SlopeDiff_slope(1,1) NaN NaN Wp_FXSlopeDiff_con(1,1) NaN Wp_FXSlopeDiff_slope(1,1) NaN NaN Wp_SlopeDiffLocal_con(1,1) NaN Wp_SlopeDiffLocal_slope(1,1) NaN NaN Wp_SlopeDiff_TermStr(Nc+2,1) NaN];                     
cols  = {'coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','p-value','Obs.'}; 
disp(' ');
disp('-----------------------------------------');
disp('Collected Summary Results of Time-Series Regressions on Slope Differentials');
disp('-----------------------------------------');
mat2tex(Table_TS_Slope_Collected_Simple,'%6.2f',rows,cols, [], Opts)

Table_TS_Slope_Collected_Simple_b = [TSPred_BetasSlopeDiff_con TSPred_BetasSlopeDiff_con_se TSPred_BetasSlopeDiff_con_pval_b ...
                         TSPred_BetasSlopeDiff TSPred_BetasSlopeDiff_se TSPred_BetasSlopeDiff_pval_b 100*TSPred_BetasSlopeDiff_R2adj...
                         TSPred_BetasFXSlopeDiff_con TSPred_BetasFXSlopeDiff_con_se TSPred_BetasFXSlopeDiff_con_pval_b ...
                         TSPred_BetasFXSlopeDiff TSPred_BetasFXSlopeDiff_se TSPred_BetasFXSlopeDiff_pval_b 100*TSPred_BetasFXSlopeDiff_R2adj...
                         TSPred_BetasSlopeDiffLocal_con TSPred_BetasSlopeDiffLocal_con_se TSPred_BetasSlopeDiffLocal_con_pval_b ...
                         TSPred_BetasSlopeDiffLocal TSPred_BetasSlopeDiffLocal_se TSPred_BetasSlopeDiffLocal_pval_b 100*TSPred_BetasSlopeDiffLocal_R2adj...
                         [Wp_SlopeDiff_TermStr_b(1:Nc+1,1); Wp_SlopeDiff_TermStr_b(Nc+1,1)]...
                         TSPred_BetasSlopeDiff_Nobs];       
Table_TS_Slope_Collected_Simple_b = [Table_TS_Slope_Collected_Simple_b; Wp_SlopeDiff_con_b(1,1) NaN NaN Wp_SlopeDiff_slope_b(1,1) NaN NaN NaN Wp_FXSlopeDiff_con_b(1,1) NaN NaN Wp_FXSlopeDiff_slope_b(1,1) NaN NaN NaN Wp_SlopeDiffLocal_con_b(1,1) NaN NaN Wp_SlopeDiffLocal_slope_b(1,1) NaN NaN NaN Wp_SlopeDiff_TermStr_b(Nc+2,1) NaN];
cols  = {'coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','p-value','Obs.'}; 
disp(' ');
disp('-----------------------------------------');
disp('Collected Summary Results of Time-Series Regressions on Slope Differentials (fixed-b)');
disp('-----------------------------------------');
mat2tex(Table_TS_Slope_Collected_Simple_b,'%6.2f',rows,cols, [], Opts)      

% Build Portfolios -- Sorted By Yield Curve Slopes
  
% Get rid of interest rates if no data on currency or bond returns and vice-versa

for t=1:size(TB,1)                                               % For each date
    for c=1:size(TB,2)                                           % For each country
       if  isnan(FX(t,c)*FWD(t,c)*TB(t,c)*TB_month(t,c)*Yield(t,c)*BondsHPERDollar(t,c)*BondsHPER(t,c)*FXEXR(t,c)*BondsHPRDiff(t,c))==1;
           TB(t,c)              = NaN;
           Slope(t,c)           = NaN;
           BondsHPERDollar(t,c) = NaN;
           BondsHPER(t,c)       = NaN;
           FXEXR(t,c)           = NaN;
           BondsHPRDiff(t,c)    = NaN;
           BondsHPRDiff_local(t,c)    = NaN;
           FX(t,c)                    = NaN;
           FWD(t,c)           = NaN;
       end         
    end;
end;

% Sign

sign_mat_Slope = sign(Slope-repmat(Slope(:,end),1,Nc));

FXEXR_signed              = (FXEXR-repmat(FXEXR(:,end),1,Nc)).*sign_mat_Slope; 
BondsHPR_signed           = (BondsHPR-repmat(BondsHPR(:,end),1,Nc)).*sign_mat_Slope;  
BondsHPRDollar_signed     = (BondsHPRDollar-repmat(BondsHPRDollar(:,end),1,Nc)).*sign_mat_Slope; 
BondsHPER_signed          = (BondsHPER-repmat(BondsHPER(:,end),1,Nc)).*sign_mat_Slope;
BondsHPERDollar_signed    = (BondsHPERDollar-repmat(BondsHPERDollar(:,end),1,Nc)).*sign_mat_Slope;
BondsHPRDiff_signed       = (BondsHPRDiff-repmat(BondsHPRDiff(:,end),1,Nc)).*sign_mat_Slope; 
BondsHPRDiff_local_signed = (BondsHPRDiff_local-repmat(BondsHPRDiff_local(:,end),1,Nc)).*sign_mat_Slope;
FWD_signed                = (FWD-repmat(FWD(:,end),1,Nc)).*sign_mat_Slope;
FX_signed                 = (FX-repmat(FX(:,end),1,Nc)).*sign_mat_Slope;
Slope_signed              = (Slope-repmat(Slope(:,end),1,Nc)).*sign_mat_Slope;  
LT_UIP_signed             = (LT_UIP-repmat(LT_UIP(:,end),1,Nc)).*sign_mat_Slope;
CPIHPER_signed            = (CPIHPER-repmat(CPIHPER(:,end),1,Nc)).*sign_mat_Slope;
Ratings_signed            = (Ratings-repmat(Ratings(:,end),1,Nc)).*sign_mat_Slope;  
RatingsOut_signed         = (RatingsOut-repmat(RatingsOut(:,end),1,Nc)).*sign_mat_Slope;

% Sort on Slope: first obs. is t0-1; first obs. of portfolio HPRs are period returns from t0-1 to t0+horizon-1
    
for n=1:Nc
    
    [Portfolios_FXEXR(:,n),Portfolios_BondsHPER(:,n),Portfolios_BondsHPERDollar(:,n),Portfolios_BondsHPRDiff(:,n),Portfolios_VarSort(:,n),Track_portfolios(:,n)] = ...
    Build_portfolios_for4variables(FXEXR_signed(:,n),BondsHPER_signed(:,n),BondsHPERDollar_signed(:,n),BondsHPRDiff_signed(:,n),Slope(:,n),1,1,Slope_Rolling_Mean(:,n));

    [Portfolios_FXEXR(:,n),Portfolios_BondsHPER(:,n),Portfolios_BondsHPERDollar(:,n),Portfolios_BondsHPRDiff_local(:,n),Portfolios_VarSort(:,n),Track_portfolios_Slope2_signed(:,n)] = ...
    Build_portfolios_for4variables(FXEXR_signed(:,n),BondsHPER_signed(:,n),BondsHPERDollar_signed(:,n),BondsHPRDiff_local_signed(:,n),Slope(:,n),1,1,Slope_Rolling_Mean(:,n));

    [Portfolios_FWD(:,n),Portfolios_FX(:,n),Portfolios_BondsHPR(:,n),Portfolios_BondsHPRDollar(:,n),Portfolios_VarSort(:,n),Track_portfolios_Slope3_signed(:,n)] = ...
    Build_portfolios_for4variables(FWD_signed(:,n),FX_signed,BondsHPR_signed(:,n),BondsHPRDollar_signed(:,n),Slope(:,n),1,1,Slope_Rolling_Mean(:,n));  
  
    [~,~,~,Portfolios_SLOPE(:,n),Portfolios_VarSort(:,n),Track_portfolios_Slope4_signed(:,n)] = ...
    Build_portfolios_for4variables(FWD_signed(:,n),BondsHPR_signed(:,n),BondsHPRDollar_signed(:,n),Slope_signed(:,n),Slope(:,n),1,1,Slope_Rolling_Mean(:,n));

    [~,~,~,Portfolios_LT_UIP(:,n), Portfolios_VarSort(:,n), Track_portfolios_Slope5_signed(:,n)] = ...
    Build_portfolios_for4variables(FWD_signed(:,n),BondsHPR_signed(:,n),BondsHPRDollar_signed(:,n),LT_UIP_signed(:,n),Slope(:,n),1,1,Slope_Rolling_Mean(:,n));

    [~,Portfolios_Ratings(:,n),Portfolios_RatingsOut(:,n),Portfolios_CPIHPER(:,n), Portfolios_VarSort(:,n), Track_portfolios_Slope6_signed(:,n)] = ...
    Build_portfolios_for4variables(FWD_signed(:,n),Ratings_signed(:,n),RatingsOut_signed(:,n),CPIHPER_signed(:,n),Slope(:,n),1,1,Slope_Rolling_Mean(:,n));

end

    
[Portfolios_FXEXR(:,Nc+1),Portfolios_BondsHPER(:,Nc+1),Portfolios_BondsHPERDollar(:,Nc+1),Portfolios_BondsHPRDiff(:,Nc+1),Portfolios_VarSort(:,Nc+1),Track_portfolios] = ...
    Build_portfolios_for4variables(FXEXR_signed(:,1:end-1),BondsHPER_signed(:,1:end-1),BondsHPERDollar_signed(:,1:end-1),BondsHPRDiff_signed(:,1:end-1),Slope(:,1:end-1),1,1,Slope_Rolling_Mean(:,1:end-1));

[Portfolios_FXEXR(:,Nc+1),Portfolios_BondsHPER(:,Nc+1),Portfolios_BondsHPERDollar(:,Nc+1),Portfolios_BondsHPRDiff_local(:,Nc+1),Portfolios_VarSort(:,Nc+1),Track_portfolios_IR2_signed] = ...
    Build_portfolios_for4variables(FXEXR_signed(:,1:end-1),BondsHPER_signed(:,1:end-1),BondsHPERDollar_signed(:,1:end-1),BondsHPRDiff_local_signed(:,1:end-1),Slope(:,1:end-1),1,1,Slope_Rolling_Mean(:,1:end-1));

[Portfolios_FWD(:,Nc+1),Portfolios_FX(:,Nc+1),Portfolios_BondsHPR(:,Nc+1),Portfolios_BondsHPRDollar(:,Nc+1),Portfolios_VarSort(:,Nc+1),Track_portfolios_IR3_signed] = ...
    Build_portfolios_for4variables(FWD_signed(:,1:end-1),FX_signed,BondsHPR_signed(:,1:end-1),BondsHPRDollar_signed(:,1:end-1),Slope(:,1:end-1),1,1,Slope_Rolling_Mean(:,1:end-1));  
  
[~,~,~,Portfolios_SLOPE(:,Nc+1),Portfolios_VarSort(:,Nc+1),Track_portfolios_IR4_signed] = ...
    Build_portfolios_for4variables(FWD_signed(:,1:end-1),BondsHPR_signed(:,1:end-1),BondsHPRDollar_signed(:,1:end-1),Slope_signed(:,1:end-1),Slope(:,1:end-1),1,1,Slope_Rolling_Mean(:,1:end-1));

[~,~,~,Portfolios_LT_UIP(:,Nc+1), Portfolios_VarSort(:,Nc+1), Track_portfolios_IR5_signed] = ...
    Build_portfolios_for4variables(FWD_signed(:,1:end-1),BondsHPR_signed(:,1:end-1),BondsHPRDollar_signed(:,1:end-1),LT_UIP_signed(:,1:end-1),Slope(:,1:end-1),1,1,Slope_Rolling_Mean(:,1:end-1));

[~,Portfolios_Ratings(:,Nc+1),Portfolios_RatingsOut(:,Nc+1),Portfolios_CPIHPER(:,Nc+1), Portfolios_VarSort(:,Nc+1), Track_portfolios_IR6_signed] = ...
    Build_portfolios_for4variables(FWD_signed(:,1:end-1),Ratings_signed(:,1:end-1),RatingsOut_signed(:,1:end-1),CPIHPER_signed(:,1:end-1),Slope(:,1:end-1),1,1,Slope_Rolling_Mean(:,1:end-1));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%bootstrapping
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

portfolio_data                                    = [(Portfolios_FXEXR) (Portfolios_BondsHPER) (Portfolios_BondsHPERDollar) (Portfolios_BondsHPRDiff) Portfolios_FX Portfolios_BondsHPRDiff_local Portfolios_FWD Portfolios_SLOPE Portfolios_CPIHPER Portfolios_Ratings Portfolios_RatingsOut];

[standard_error_mean,standard_error_SR]                               = bootstrap_portfolio_returns_growth(portfolio_data,nboot,horizon);

Total_portfolios=Nc+1;

Portfolios_FXEXR_se(1:Total_portfolios)                   = standard_error_mean(1:Total_portfolios);
Portfolios_BondsHPER_se(1:Total_portfolios)               = standard_error_mean(1+Total_portfolios:2*Total_portfolios);
Portfolios_BondsHPERDollar_se(1:Total_portfolios)         = standard_error_mean(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_BondsHPRDiff_se(1:Total_portfolios)            = standard_error_mean(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_se(1:Total_portfolios)                      = standard_error_mean(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_BondsHPRDiff_local_se(1:Total_portfolios)      = standard_error_mean(1+5*Total_portfolios:6*Total_portfolios);
Portfolios_FWD_se(1:Total_portfolios)                     = standard_error_mean(1+6*Total_portfolios:7*Total_portfolios);
Portfolios_SLOPE_se(1:Total_portfolios)                   = standard_error_mean(1+7*Total_portfolios:8*Total_portfolios);
Portfolios_CPIHPER_se(1:Total_portfolios)                 = standard_error_mean(1+8*Total_portfolios:9*Total_portfolios);
Portfolios_Ratings_se(1:Total_portfolios)                 = standard_error_mean(1+9*Total_portfolios:10*Total_portfolios);
Portfolios_RatingsOut_se(1:Total_portfolios)              = standard_error_mean(1+10*Total_portfolios:11*Total_portfolios);

Portfolios_FXEXR_SR_se(1:Total_portfolios)                = standard_error_SR(1:Total_portfolios);
Portfolios_BondsHPER_SR_se(1:Total_portfolios)            = standard_error_SR(1+Total_portfolios:2*Total_portfolios);
Portfolios_BondsHPERDollar_SR_se(1:Total_portfolios)      = standard_error_SR(1+2*Total_portfolios:3*Total_portfolios);
Portfolios_BondsHPRDiff_SR_se(1:Total_portfolios)         = standard_error_SR(1+3*Total_portfolios:4*Total_portfolios);
Portfolios_FX_SR_se(1:Total_portfolios)                   = standard_error_SR(1+4*Total_portfolios:5*Total_portfolios);
Portfolios_BondsHPRDiff_local_SR_se(1:Total_portfolios)   = standard_error_SR(1+5*Total_portfolios:6*Total_portfolios);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


Table_Ptf_Slope_signed_Simple = [-100*nanmean((Portfolios_BondsHPERDollar))*ScalingFactor;       
         100*Portfolios_BondsHPERDollar_se*ScalingFactor;
         100*nanstd((Portfolios_BondsHPERDollar))*sqrt(ScalingFactor); 
         -nanmean((Portfolios_BondsHPERDollar))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPERDollar));
         Portfolios_BondsHPERDollar_SR_se*sqrt(ScalingFactor);
         NaN(1,11);
         -100*nanmean((Portfolios_FXEXR))*ScalingFactor;                 
         100*Portfolios_FXEXR_se*ScalingFactor; 
         100*nanstd((Portfolios_FXEXR))*sqrt(ScalingFactor); 
         -nanmean((Portfolios_FXEXR))*sqrt(ScalingFactor)./nanstd((Portfolios_FXEXR)); 
         Portfolios_FXEXR_SR_se*sqrt(ScalingFactor);
         NaN(1,11);
         -100*nanmean((Portfolios_BondsHPER))*ScalingFactor;   
         100*Portfolios_BondsHPER_se*ScalingFactor; 
         100*nanstd((Portfolios_BondsHPER))*sqrt(ScalingFactor); 
         -nanmean((Portfolios_BondsHPER))*sqrt(ScalingFactor)./nanstd((Portfolios_BondsHPER));
         Portfolios_BondsHPER_SR_se*sqrt(ScalingFactor)]';
rows = Countries_Sample';      
rows{1,Nc+1} = 'Equally-weighted';   
Opts.Style = 'jf';
disp(' ');
disp('----------------------------------------------');
disp('Long-Short Portfolio Sorted By Slopes');
disp('----------------------------------------------'); 
mat2tex(Table_Ptf_Slope_signed_Simple,'%6.2f',rows,[], [], Opts)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Time-Series Predictability Regressions: Foreign Bond HPERs (and Components) on Interest Rates, Inflation Rates and Credit Ratings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nc = size(Countries_Sample,1);

if NW_lag_choice == 1
    
    lags = max(6,horizon+1);  
    
    elseif NW_lag_choice == 2
        
    lags = ceil(4*(size(BondsHPER,1)/100)^(2/9))-1;
    
    elseif NW_lag_choice == 3
        
    lags = ceil(0.75*(size(BondsHPER,1))^(1/3))-1;
    
    elseif NW_lag_choice == 4
        
    lags = max(ceil(4*(size(BondsHPER,1)/100)^(2/9))-1,ceil(0.75*(size(BondsHPER,1))^(1/3))-1);
    
    elseif NW_lag_choice == 5
        
    lags = ceil(1.3*(size(BondsHPER,1))^(1/2))-1;
        
end


TSPred_BetasIRInflDiff_con         = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_con_se      = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_con_boot    = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_con_se_boot = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C1          = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C1_se       = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C1_boot     = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C1_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C2          = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C2_se       = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C2_boot     = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C2_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C3          = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C3_se       = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C3_boot     = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_C3_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_R2adj       = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_Nobs        = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_con_pval_b  = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiff_pval_b      = NaN*zeros(Nc,1);

TSPred_BetasFXIRInflDiff_con         = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_con_se      = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_con_boot    = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_con_se_boot = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C1          = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C1_se       = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C1_boot     = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C1_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C2          = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C2_se       = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C2_boot     = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C2_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C3          = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C3_se       = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C3_boot     = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_C3_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_R2adj       = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_Nobs        = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_con_pval_b  = NaN*zeros(Nc,1);
TSPred_BetasFXIRInflDiff_pval_b      = NaN*zeros(Nc,1);

TSPred_BetasIRInflDiffLocal_con         = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_con_se      = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_con_boot    = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_con_se_boot = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C1          = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C1_se       = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C1_boot     = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C1_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C2          = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C2_se       = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C2_boot     = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C2_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C3          = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C3_se       = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C3_boot     = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_C3_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_R2adj       = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_Nobs        = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_con_pval_b  = NaN*zeros(Nc,1);
TSPred_BetasIRInflDiffLocal_pval_b      = NaN*zeros(Nc,1);

block_length = horizon;


for n=1:Nc
    
    % Regress USD bond HPR differentials (foreign minus US) on interest rate, inflation and credit rating differentials (foreign minus US) 

    lhv = (BondsHPERDollar(:,n)-BondsHPERDollar(:,end))*ScalingFactor;
    rhv = [ones(size(lhv,1),1) 12*log(1+TB_month(:,n))-12*log(1+TB_month(:,end)) 12*CPIHPER(:,n)-12*CPIHPER(:,end) Ratings(:,n)-Ratings(:,end)];
    if range(Ratings(:,n)-Ratings(:,end)) == 0
            rhv = [ones(size(lhv,1),1) 12*log(1+TB_month(:,n))-12*log(1+TB_month(:,end)) 12*CPIHPER(:,n)-12*CPIHPER(:,end)];
    end
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasIRInflDiff_con(n,1)        = bv(1);
        TSPred_BetasIRInflDiff_con_se(n,1)     = sebv(1);
        TSPred_BetasIRInflDiff_con_boot(n,1)   = bootstrap_stats(1,1);
        TSPred_BetasIRInflDiff_con_se_boot(n,1)= bootstrap_stats(1,2);
        TSPred_BetasIRInflDiff_C1(n,1)         = bv(2);
        TSPred_BetasIRInflDiff_C1_se(n,1)      = sebv(2);
        TSPred_BetasIRInflDiff_C1_boot(n,1)    = bootstrap_stats(2,1);
        TSPred_BetasIRInflDiff_C1_se_boot(n,1) = bootstrap_stats(2,2);
        TSPred_BetasIRInflDiff_C2(n,1)         = bv(3);
        TSPred_BetasIRInflDiff_C2_se(n,1)      = sebv(3);
        TSPred_BetasIRInflDiff_C2_boot(n,1)    = bootstrap_stats(3,1);
        TSPred_BetasIRInflDiff_C2_se_boot(n,1) = bootstrap_stats(3,2);
        if range(Ratings(:,n)-Ratings(:,end)) > 0
            TSPred_BetasIRInflDiff_C3(n,1)         = bv(4);
            TSPred_BetasIRInflDiff_C3_se(n,1)      = sebv(4);
            TSPred_BetasIRInflDiff_C3_boot(n,1)    = bootstrap_stats(4,1);
            TSPred_BetasIRInflDiff_C3_se_boot(n,1) = bootstrap_stats(4,2);
        end
        TSPred_BetasIRInflDiff_R2adj(n,1)      = R2vadj;
        TSPred_BetasIRInflDiff_Nobs(n,1)       = size(lhv_s,1);
        TSPred_BetasIRInflDiff_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasIRInflDiff_con(n,1)/TSPred_BetasIRInflDiff_con_se(n,1),(lags+1)/TSPred_BetasIRInflDiff_Nobs(n,1));                 
        TSPred_BetasIRInflDiff_C1_pval_b(n,1)  = tstat_pval_b(TSPred_BetasIRInflDiff_C1(n,1)/TSPred_BetasIRInflDiff_C1_se(n,1),(lags+1)/TSPred_BetasIRInflDiff_Nobs(n,1));   
    end;
    
    
    % Regress excess FX returns on interest rate, inflation and credit rating differentials (foreign minus US) 

    lhv = FXEXR(:,n)*ScalingFactor;
    rhv = [ones(size(lhv,1),1) 12*log(1+TB_month(:,n))-12*log(1+TB_month(:,end)) 12*CPIHPER(:,n)-12*CPIHPER(:,end) Ratings(:,n)-Ratings(:,end)];
    if range(Ratings(:,n)-Ratings(:,end)) == 0
            rhv = [ones(size(lhv,1),1) 12*log(1+TB_month(:,n))-12*log(1+TB_month(:,end)) 12*CPIHPER(:,n)-12*CPIHPER(:,end)];
    end
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasFXIRInflDiff_con(n,1)        = bv(1);
        TSPred_BetasFXIRInflDiff_con_se(n,1)     = sebv(1);
        TSPred_BetasFXIRInflDiff_con_boot(n,1)   = bootstrap_stats(1,1);
        TSPred_BetasFXIRInflDiff_con_se_boot(n,1)= bootstrap_stats(1,2);
        TSPred_BetasFXIRInflDiff_C1(n,1)         = bv(2);
        TSPred_BetasFXIRInflDiff_C1_se(n,1)      = sebv(2);
        TSPred_BetasFXIRInflDiff_C1_boot(n,1)    = bootstrap_stats(2,1);
        TSPred_BetasFXIRInflDiff_C1_se_boot(n,1) = bootstrap_stats(2,2);
        TSPred_BetasFXIRInflDiff_C2(n,1)         = bv(3);
        TSPred_BetasFXIRInflDiff_C2_se(n,1)      = sebv(3);
        TSPred_BetasFXIRInflDiff_C2_boot(n,1)    = bootstrap_stats(3,1);
        TSPred_BetasFXIRInflDiff_C2_se_boot(n,1) = bootstrap_stats(3,2);
        if range(Ratings(:,n)-Ratings(:,end)) > 0
            TSPred_BetasFXIRInflDiff_C3(n,1)         = bv(4);
            TSPred_BetasFXIRInflDiff_C3_se(n,1)      = sebv(4);
            TSPred_BetasFXIRInflDiff_C3_boot(n,1)    = bootstrap_stats(4,1);
            TSPred_BetasFXIRInflDiff_C3_se_boot(n,1) = bootstrap_stats(4,2);
        end
        TSPred_BetasFXIRInflDiff_R2adj(n,1)      = R2vadj;
        TSPred_BetasFXIRInflDiff_Nobs(n,1)       = size(lhv_s,1);
        TSPred_BetasFXIRInflDiff_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasFXIRInflDiff_con(n,1)/TSPred_BetasFXIRInflDiff_con_se(n,1),(lags+1)/TSPred_BetasFXIRInflDiff_Nobs(n,1));                 
        TSPred_BetasFXIRInflDiff_C1_pval_b(n,1)  = tstat_pval_b(TSPred_BetasFXIRInflDiff_C1(n,1)/TSPred_BetasFXIRInflDiff_C1_se(n,1),(lags+1)/TSPred_BetasFXIRInflDiff_Nobs(n,1));   
    end;
    
    
    % Regress bond local currency HPER differentials (foreign minus US) on interest rate, inflation and credit rating differentials (foreign minus US) 

    lhv = (BondsHPER(:,n)-BondsHPER(:,end))*ScalingFactor;
    rhv = [ones(size(lhv,1),1) 12*log(1+TB_month(:,n))-12*log(1+TB_month(:,end)) 12*CPIHPER(:,n)-12*CPIHPER(:,end) Ratings(:,n)-Ratings(:,end)];
    if range(Ratings(:,n)-Ratings(:,end)) == 0
            rhv = [ones(size(lhv,1),1) 12*log(1+TB_month(:,n))-12*log(1+TB_month(:,end)) 12*CPIHPER(:,n)-12*CPIHPER(:,end)];
    end
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasIRInflDiffLocal_con(n,1)        = bv(1);
        TSPred_BetasIRInflDiffLocal_con_se(n,1)     = sebv(1);
        TSPred_BetasIRInflDiffLocal_con_boot(n,1)   = bootstrap_stats(1,1);
        TSPred_BetasIRInflDiffLocal_con_se_boot(n,1)= bootstrap_stats(1,2);
        TSPred_BetasIRInflDiffLocal_C1(n,1)         = bv(2);
        TSPred_BetasIRInflDiffLocal_C1_se(n,1)      = sebv(2);
        TSPred_BetasIRInflDiffLocal_C1_boot(n,1)    = bootstrap_stats(2,1);
        TSPred_BetasIRInflDiffLocal_C1_se_boot(n,1) = bootstrap_stats(2,2);
        TSPred_BetasIRInflDiffLocal_C2(n,1)         = bv(3);
        TSPred_BetasIRInflDiffLocal_C2_se(n,1)      = sebv(3);
        TSPred_BetasIRInflDiffLocal_C2_boot(n,1)    = bootstrap_stats(3,1);
        TSPred_BetasIRInflDiffLocal_C2_se_boot(n,1) = bootstrap_stats(3,2);
        if range(Ratings(:,n)-Ratings(:,end)) > 0
            TSPred_BetasIRInflDiffLocal_C3(n,1)         = bv(4);
            TSPred_BetasIRInflDiffLocal_C3_se(n,1)      = sebv(4);
            TSPred_BetasIRInflDiffLocal_C3_boot(n,1)    = bootstrap_stats(4,1);
            TSPred_BetasIRInflDiffLocal_C3_se_boot(n,1) = bootstrap_stats(4,2);
        end
        TSPred_BetasIRInflDiffLocal_R2adj(n,1)      = R2vadj;
        TSPred_BetasIRInflDiffLocal_Nobs(n,1)       = size(lhv_s,1);
        TSPred_BetasIRInflDiffLocal_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasIRInflDiffLocal_con(n,1)/TSPred_BetasIRInflDiffLocal_con_se(n,1),(lags+1)/TSPred_BetasIRInflDiffLocal_Nobs(n,1));                 
        TSPred_BetasIRInflDiffLocal_C1_pval_b(n,1)  = tstat_pval_b(TSPred_BetasIRInflDiffLocal_C1(n,1)/TSPred_BetasIRInflDiffLocal_C1_se(n,1),(lags+1)/TSPred_BetasIRInflDiffLocal_Nobs(n,1));   
    end;
    
        
end;


% Panel regressions with Driscoll-Kraay s.e.

Panel_Var = [ScalingFactor*BondHPERDollar_Diff, 12*TB_month_Diff, 12*CPIHPER_Diff, Ratings_Fgn-Ratings_US];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK         = Panel_Var(:,1:Nc-1);
X_DK(:,:,1)  = Panel_Var(:,Nc:2*(Nc-1));
X_DK(:,:,2)  = Panel_Var(:,2*Nc-1:3*(Nc-1));
X_DK(:,:,3)  = Panel_Var(:,3*Nc-2:4*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+3) = 0*y_DK;
    X_DK(:,j,j+3) = ones(size(y_DK,1),1);
end

[TSPred_BetasIRInflDiff, ~, ~,~,yhat,Rsq,~,~,TSPred_BetasIRInflDiff_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasIRInflDiff_con(Nc+2,1)     = NaN;
TSPred_BetasIRInflDiff_con_se(Nc+2,1)  = NaN;
TSPred_BetasIRInflDiff_C1(Nc+2,1)      = TSPred_BetasIRInflDiff(1,1);
TSPred_BetasIRInflDiff_C1_se(Nc+2,1)   = TSPred_BetasIRInflDiff_se(1,1);
TSPred_BetasIRInflDiff_C2(Nc+2,1)      = TSPred_BetasIRInflDiff(2,1);
TSPred_BetasIRInflDiff_C2_se(Nc+2,1)   = TSPred_BetasIRInflDiff_se(2,1);
TSPred_BetasIRInflDiff_C3(Nc+2,1)      = TSPred_BetasIRInflDiff(3,1);
TSPred_BetasIRInflDiff_C3_se(Nc+2,1)   = TSPred_BetasIRInflDiff_se(3,1);
TSPred_BetasIRInflDiff_Nobs(Nc+2,1)    = sum(sum(~isnan(yhat)));
TSPred_BetasIRInflDiff_R2adj(Nc+2,1)   = 1-(1-Rsq)*(TSPred_BetasIRInflDiff_Nobs(Nc+2,1)-1)/(TSPred_BetasIRInflDiff_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasIRInflDiff_con_boot(Nc+2,1)    = NaN;
TSPred_BetasIRInflDiff_con_se_boot(Nc+2,1) = NaN;
TSPred_BetasIRInflDiff_C1_boot(Nc+2,1)     = bootstrap_stats(1,1);
TSPred_BetasIRInflDiff_C1_se_boot(Nc+2,1)  = bootstrap_stats(1,2);
TSPred_BetasIRInflDiff_C2_boot(Nc+2,1)     = bootstrap_stats(2,1);
TSPred_BetasIRInflDiff_C2_se_boot(Nc+2,1)  = bootstrap_stats(2,2);
TSPred_BetasIRInflDiff_C3_boot(Nc+2,1)     = bootstrap_stats(3,1);
TSPred_BetasIRInflDiff_C3_se_boot(Nc+2,1)  = bootstrap_stats(3,2);
TSPred_BetasIRInflDiff_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasIRInflDiff_con(Nc+2,1)/TSPred_BetasIRInflDiff_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasIRInflDiff_C1_pval_b(Nc+2,1)     = tstat_pval_b(TSPred_BetasIRInflDiff_C1(Nc+2,1)/TSPred_BetasIRInflDiff_C1_se(Nc+2,1), (lags+1)/size(y_DK,1));                 


Panel_Var = [ScalingFactor*FXEXR_Panel, 12*TB_month_Diff, 12*CPIHPER_Diff, Ratings_Fgn-Ratings_US];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK         = Panel_Var(:,1:Nc-1);
X_DK(:,:,1)  = Panel_Var(:,Nc:2*(Nc-1));
X_DK(:,:,2)  = Panel_Var(:,2*Nc-1:3*(Nc-1));
X_DK(:,:,3)  = Panel_Var(:,3*Nc-2:4*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+3) = 0*y_DK;
    X_DK(:,j,j+3) = ones(size(y_DK,1),1);
end

[TSPred_BetasFXIRInflDiff, ~, ~,~,yhat,Rsq,~,~,TSPred_BetasFXIRInflDiff_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasFXIRInflDiff_con(Nc+2,1)     = NaN;
TSPred_BetasFXIRInflDiff_con_se(Nc+2,1)  = NaN;
TSPred_BetasFXIRInflDiff_C1(Nc+2,1)      = TSPred_BetasFXIRInflDiff(1,1);
TSPred_BetasFXIRInflDiff_C1_se(Nc+2,1)   = TSPred_BetasFXIRInflDiff_se(1,1);
TSPred_BetasFXIRInflDiff_C2(Nc+2,1)      = TSPred_BetasFXIRInflDiff(2,1);
TSPred_BetasFXIRInflDiff_C2_se(Nc+2,1)   = TSPred_BetasFXIRInflDiff_se(2,1);
TSPred_BetasFXIRInflDiff_C3(Nc+2,1)      = TSPred_BetasFXIRInflDiff(3,1);
TSPred_BetasFXIRInflDiff_C3_se(Nc+2,1)   = TSPred_BetasFXIRInflDiff_se(3,1);
TSPred_BetasFXIRInflDiff_Nobs(Nc+2,1)    = sum(sum(~isnan(yhat)));
TSPred_BetasFXIRInflDiff_R2adj(Nc+2,1)   = 1-(1-Rsq)*(TSPred_BetasFXIRInflDiff_Nobs(Nc+2,1)-1)/(TSPred_BetasFXIRInflDiff_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasFXIRInflDiff_con_boot(Nc+2,1)       = NaN;
TSPred_BetasFXIRInflDiff_con_se_boot(Nc+2,1)    = NaN;
TSPred_BetasFXIRInflDiff_C1_boot(Nc+2,1)        = bootstrap_stats(1,1);
TSPred_BetasFXIRInflDiff_C1_se_boot(Nc+2,1)     = bootstrap_stats(1,2);
TSPred_BetasFXIRInflDiff_C2_boot(Nc+2,1)        = bootstrap_stats(2,1);
TSPred_BetasFXIRInflDiff_C2_se_boot(Nc+2,1)     = bootstrap_stats(2,2);
TSPred_BetasFXIRInflDiff_C3_boot(Nc+2,1)        = bootstrap_stats(3,1);
TSPred_BetasFXIRInflDiff_C3_se_boot(Nc+2,1)     = bootstrap_stats(3,2);
TSPred_BetasFXIRInflDiff_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasFXIRInflDiff_con(Nc+2,1)/TSPred_BetasFXIRInflDiff_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasFXIRInflDiff_C1_pval_b(Nc+2,1)     = tstat_pval_b(TSPred_BetasFXIRInflDiff_C1(Nc+2,1)/TSPred_BetasFXIRInflDiff_C1_se(Nc+2,1), (lags+1)/size(y_DK,1));                 



Panel_Var = [ScalingFactor*BondHPER_Diff, 12*TB_month_Diff, 12*CPIHPER_Diff, Ratings_Fgn-Ratings_US];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK         = Panel_Var(:,1:Nc-1);
X_DK(:,:,1)  = Panel_Var(:,Nc:2*(Nc-1));
X_DK(:,:,2)  = Panel_Var(:,2*Nc-1:3*(Nc-1));
X_DK(:,:,3)  = Panel_Var(:,3*Nc-2:4*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+3) = 0*y_DK;
    X_DK(:,j,j+3) = ones(size(y_DK,1),1);
end

[TSPred_BetasIRInflDiffLocal, ~, ~,~,yhat,Rsq,~,~,TSPred_BetasIRInflDiffLocal_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasIRInflDiffLocal_con(Nc+2,1)     = NaN;
TSPred_BetasIRInflDiffLocal_con_se(Nc+2,1)  = NaN;
TSPred_BetasIRInflDiffLocal_C1(Nc+2,1)      = TSPred_BetasIRInflDiffLocal(1,1);
TSPred_BetasIRInflDiffLocal_C1_se(Nc+2,1)   = TSPred_BetasIRInflDiffLocal_se(1,1);
TSPred_BetasIRInflDiffLocal_C2(Nc+2,1)      = TSPred_BetasIRInflDiffLocal(2,1);
TSPred_BetasIRInflDiffLocal_C2_se(Nc+2,1)   = TSPred_BetasIRInflDiffLocal_se(2,1);
TSPred_BetasIRInflDiffLocal_C3(Nc+2,1)      = TSPred_BetasIRInflDiffLocal(3,1);
TSPred_BetasIRInflDiffLocal_C3_se(Nc+2,1)   = TSPred_BetasIRInflDiffLocal_se(3,1);
TSPred_BetasIRInflDiffLocal_Nobs(Nc+2,1)    = sum(sum(~isnan(yhat)));
TSPred_BetasIRInflDiffLocal_R2adj(Nc+2,1)   = 1-(1-Rsq)*(TSPred_BetasIRInflDiffLocal_Nobs(Nc+2,1)-1)/(TSPred_BetasIRInflDiffLocal_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasIRInflDiffLocal_con_boot(Nc+2,1)    = NaN;
TSPred_BetasIRInflDiffLocal_con_se_boot(Nc+2,1) = NaN;
TSPred_BetasIRInflDiffLocal_C1_boot(Nc+2,1)     = bootstrap_stats(1,1);
TSPred_BetasIRInflDiffLocal_C1_se_boot(Nc+2,1)  = bootstrap_stats(1,2);
TSPred_BetasIRInflDiffLocal_C2_boot(Nc+2,1)     = bootstrap_stats(2,1);
TSPred_BetasIRInflDiffLocal_C2_se_boot(Nc+2,1)  = bootstrap_stats(2,2);
TSPred_BetasIRInflDiffLocal_C3_boot(Nc+2,1)     = bootstrap_stats(3,1);
TSPred_BetasIRInflDiffLocal_C3_se_boot(Nc+2,1)  = bootstrap_stats(3,2);
TSPred_BetasIRInflDiffLocal_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasIRInflDiffLocal_con(Nc+2,1)/TSPred_BetasIRInflDiffLocal_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasIRInflDiffLocal_C1_pval_b(Nc+2,1)     = tstat_pval_b(TSPred_BetasIRInflDiffLocal_C1(Nc+2,1)/TSPred_BetasIRInflDiffLocal_C1_se(Nc+2,1), (lags+1)/size(y_DK,1));                 


% Joint estimation

T = size(BondsHPERDollar,1)-horizon+1;

R1_con     = zeros(Nc-1,9*(Nc-1));
R1_slope   = zeros(Nc-1,9*(Nc-1));
R1_all     = zeros(2*(Nc-1),9*(Nc-1));
R2_con     = zeros(Nc-1,9*(Nc-1));
R2_slope   = zeros(Nc-1,9*(Nc-1));
R2_all     = zeros(2*(Nc-1),9*(Nc-1));
R3_con     = zeros(Nc-1,9*(Nc-1));
R3_slope   = zeros(Nc-1,9*(Nc-1));
R3_all     = zeros(2*(Nc-1),9*(Nc-1));
R_diff     = zeros(Nc-1,9*(Nc-1));
r          = zeros(Nc-1,1);
r_all      = zeros(2*(Nc-1),1);
rhv        = zeros(3*T*(Nc-1),12*(Nc-1));
for n=1:Nc-1
lhv((n-1)*T+1:n*T,1)                                                     = (BondsHPERDollar(1:end-horizon+1,n)-BondsHPERDollar(1:end-horizon+1,end))*ScalingFactor;
lhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,1)                                   = FXEXR(1:end-horizon+1,n)*ScalingFactor;
lhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,1)                               = (BondsHPER(1:end-horizon+1,n)-BondsHPER(1:end-horizon+1,end))*ScalingFactor;
rhv((n-1)*T+1:n*T,(n-1)*4+1:n*4)                                         = [ones(T,1) 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end)) 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end) Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end)];
rhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,4*(Nc-1)+(n-1)*4+1:4*(Nc-1)+n*4)     = [ones(T,1) 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end)) 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end) Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end)];
rhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,8*(Nc-1)+(n-1)*4+1:8*(Nc-1)+n*4) = [ones(T,1) 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end)) 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end) Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end)];

R1_con(n,3*(n-1)+1)            =  1;
R1_slope(n,3*(n-1)+2)          =  1;
R1_all(n,3*(n-1)+1)            =  1;
R1_all(Nc-1+n,3*(n-1)+2)            =  1;
R2_con(n,3*(Nc-1)+3*(n-1)+1)   =  1;
R2_slope(n,3*(Nc-1)+3*(n-1)+2) =  1;
R2_all(n,3*(Nc-1)+3*(n-1)+1)   =  1;
R2_all(Nc-1+n,3*(Nc-1)+3*(n-1)+2)   =  1;
R3_con(n,6*(Nc-1)+3*(n-1)+1)   =  1;
R3_slope(n,6*(Nc-1)+3*(n-1)+2) =  1;
R3_all(n,6*(Nc-1)+3*(n-1)+1)   =  1;
R3_all(Nc-1+n,6*(Nc-1)+3*(n-1)+2)   =  1;
R_diff(n,3*(n-1)+2)            =  1;
R_diff(n,3*(Nc-1)+3*(n-1)+2)   = -1;
end
[lhv_s,rhv_s]                 = FindCommonSample(lhv,rhv);
zero_col =  find(all(rhv_s==0));                                                                 % find columns with all zeros
rhv_s(:,all(~rhv_s,1)) = [];                                                                     % remove columns with all zeros
[bv,sebv,R2v,R2vadj,v]        = olsgmm(lhv_s,rhv_s,lags,1);
for k = zero_col;
    bv = [bv(1:k-1); NaN; bv(k:end)];
    v  = [v(1:k-1,:); NaN(1,size(v,2)); v(k:end,:)];
    v  = [v(:,1:k-1), NaN(size(v,1),1), v(:,k:end)];
end
bv(4:4:108) = [];
v(4:4:108,:) = [];
v(:,4:4:108) = [];
W_IRInflDiff_con(1,1)         = (R1_con*bv - r)'*inv(R1_con*v*R1_con')*(R1_con*bv - r);          % test if all constants are zero in the HPERDollar regressions
Wp_IRInflDiff_con(1,1)        = 1-chi2cdf(W_IRInflDiff_con(1,1),Nc-1);
W_IRInflDiff_slope(1,1)       = (R1_slope*bv - r)'*inv(R1_slope*v*R1_slope')*(R1_slope*bv - r);  % test if all slopes are zero in the HPERDollar regressions
Wp_IRInflDiff_slope(1,1)      = 1-chi2cdf(W_IRInflDiff_slope(1,1),Nc-1);
W_IRInflDiff_all(1,1)         = (R1_all*bv - r_all)'*inv(R1_all*v*R1_all')*(R1_all*bv - r_all);  % test if all constants and slopes are zero in the HPERDollar regressions
Wp_IRInflDiff_all(1,1)        = 1-chi2cdf(W_IRInflDiff_all(1,1),2*(Nc-1));
W_FXIRInflDiff_con(1,1)       = (R2_con*bv - r)'*inv(R2_con*v*R2_con')*(R2_con*bv - r);          % test if all constants are zero in the FXEXR regressions
Wp_FXIRInflDiff_con(1,1)      = 1-chi2cdf(W_FXIRInflDiff_con(1,1),Nc-1);
W_FXIRInflDiff_slope(1,1)     = (R2_slope*bv - r)'*inv(R2_slope*v*R2_slope')*(R2_slope*bv - r);  % test if all slopes are zero in the FXEXR regressions
Wp_FXIRInflDiff_slope(1,1)    = 1-chi2cdf(W_FXIRInflDiff_slope(1,1),Nc-1);
W_FXIRInflDiff_all(1,1)       = (R2_all*bv - r_all)'*inv(R2_all*v*R2_all')*(R2_all*bv - r_all);  % test if all constants and slopes are zero in the FXEXR regressions
Wp_FXIRInflDiff_all(1,1)      = 1-chi2cdf(W_FXIRInflDiff_all(1,1),2*(Nc-1));
W_IRInflDiffLocal_con(1,1)    = (R3_con*bv - r)'*inv(R3_con*v*R3_con')*(R3_con*bv - r);          % test if all constants are zero in the HPER regressions
Wp_IRInflDiffLocal_con(1,1)   = 1-chi2cdf(W_IRInflDiffLocal_con(1,1),Nc-1);
W_IRInflDiffLocal_slope(1,1)  = (R3_slope*bv - r)'*inv(R3_slope*v*R3_slope')*(R3_slope*bv - r);  % test if all slopes are zero in the HPER regressions
Wp_IRInflDiffLocal_slope(1,1) = 1-chi2cdf(W_IRInflDiffLocal_slope(1,1),Nc-1);
W_IRInflDiffLocal_all(1,1)    = (R3_all*bv - r_all)'*inv(R3_all*v*R3_all')*(R3_all*bv - r_all);  % test if all constants and slopes are zero in the HPER regressions
Wp_IRInflDiffLocal_all(1,1)   = 1-chi2cdf(W_IRInflDiffLocal_all(1,1),2*(Nc-1));
W_IRInflDiff_TermStr(Nc+2,1)  = (R_diff*bv - r)'*inv(R_diff*v*R_diff')*(R_diff*bv - r);          % test if all HPERDollar/FXEXR slope differentials are zero
Wp_IRInflDiff_TermStr(Nc+2,1) = 1-chi2cdf(W_IRInflDiff_TermStr(Nc+2,1),Nc-1);

Wp_IRInflDiff_con_b(1,1)        = chisq_pval_b(W_IRInflDiff_con(1,1),Nc-1,(lags+1)/T);
Wp_IRInflDiff_slope_b(1,1)      = chisq_pval_b(W_IRInflDiff_slope(1,1),Nc-1,(lags+1)/T);
Wp_IRInflDiff_all_b(1,1)        = chisq_pval_b(W_IRInflDiff_all(1,1),2*(Nc-1),(lags+1)/T);
Wp_FXIRInflDiff_con_b(1,1)      = chisq_pval_b(W_FXIRInflDiff_con(1,1),Nc-1,(lags+1)/T);
Wp_FXIRInflDiff_slope_b(1,1)    = chisq_pval_b(W_FXIRInflDiff_slope(1,1),Nc-1,(lags+1)/T);
Wp_FXIRInflDiff_all_b(1,1)      = chisq_pval_b(W_FXIRInflDiff_all(1,1),2*(Nc-1),(lags+1)/T);
Wp_IRInflDiffLocal_con_b(1,1)   = chisq_pval_b(W_IRInflDiffLocal_con(1,1),Nc-1,(lags+1)/T);
Wp_IRInflDiffLocal_slope_b(1,1) = chisq_pval_b(W_IRInflDiffLocal_slope(1,1),Nc-1,(lags+1)/T);
Wp_IRInflDiffLocal_all_b(1,1)   = chisq_pval_b(W_IRInflDiffLocal_all(1,1),2*(Nc-1),(lags+1)/T); 
Wp_IRInflDiff_TermStr_b(Nc+2,1) = chisq_pval_b(W_IRInflDiff_TermStr(Nc+2,1),Nc-1,(lags+1)/T);

% Panel regression

lhv_panel    = [];
rhv_panel    = zeros(2*T*(Nc-1),2*(Nc+2));
lhv_panel_DK = NaN(T,2*(Nc-1));
rhv_panel_DK = NaN(T,2*(Nc-1),2*(Nc+2));
R = [0 1 0 0 -1 0];
R_slope      = [zeros(1,Nc-1) 1 zeros(1,2) zeros(1,Nc-1) -1 zeros(1,2)];
R_con_1      = zeros(Nc-1,2*(Nc+2));
R_con_2      = zeros(Nc-1,2*(Nc+2));
r            = 0;
r_con        = zeros(Nc-1,1);
for n = 1:Nc-1
T = size(BondsHPERDollar,1)-horizon+1;
lhv  = [(BondsHPERDollar(1:end-horizon+1,n)-BondsHPERDollar(1:end-horizon+1,end))*ScalingFactor;  FXEXR(1:end-horizon+1,n)*ScalingFactor];
rhv  = zeros(2*T,8);
rhv(1:T,1:4)     = [ones(T,1) 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end)) 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end) Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end)];
rhv(T+1:2*T,5:8) = [ones(T,1) 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end)) 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end) Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end)];
[lhv_s,rhv_s]                     = FindCommonSample(lhv,rhv);
zero_col =  find(all(rhv_s==0));                                                                 % find columns with all zeros
rhv_s(:,all(~rhv_s,1)) = [];                                                                     % remove columns with all zeros
[bv,sebv,R2v,R2vadj,v]            = olsgmm(lhv_s,rhv_s,lags,1);
for k = zero_col;
    bv = [bv(1:k-1); NaN; bv(k:end)];
    v  = [v(1:k-1,:); NaN(1,size(v,2)); v(k:end,:)];
    v  = [v(:,1:k-1), NaN(size(v,1),1), v(:,k:end)];
end
bv([4 8]) = [];
v(:,[4 8]) = [];
v([4 8],:) = [];
W_IRInflDiff_TermStr(n,1)  = (R*bv - r)'*inv(R*v*R')*(R*bv - r);      % test if long-term and short-term bond slopes are equal
Wp_IRInflDiff_TermStr(n,1) = 1-chi2cdf(W_IRInflDiff_TermStr(n,1),1);
Wp_IRInflDiff_TermStr_b(n,1) = chisq_pval_b(W_IRInflDiff_TermStr(n,1),1,(lags+1)/T);


lhv_panel = [lhv_panel; lhv];
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,n)    = ones(T,1);
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,Nc)   = 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end));
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,Nc+1) = 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end);
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,Nc+2) = Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end);
rhv_panel((n-1)*2*T+T+1:n*2*T,Nc+2+n)   = ones(T,1);
rhv_panel((n-1)*2*T+T+1:n*2*T,2*Nc+2)   = 12*log(1+TB_month(1:end-horizon+1,n))-12*log(1+TB_month(1:end-horizon+1,end));
rhv_panel((n-1)*2*T+T+1:n*2*T,2*Nc+3)   = 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end);
rhv_panel((n-1)*2*T+T+1:n*2*T,2*Nc+4)   = Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end);

lhv_panel_DK(:,n)        = lhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:);
lhv_panel_DK(:,Nc-1+n)   = lhv_panel((n-1)*2*T+T+1:n*2*T,:);
rhv_panel_DK(:,n,:)      = reshape(rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:),T,[]);
rhv_panel_DK(:,Nc-1+n,:) = reshape(rhv_panel((n-1)*2*T+T+1:n*2*T,:),T,[]);

R_con_1(n,n)      = 1;
R_con_2(n,Nc+2+n) = 1;
end
[bv,~,~,~,~,~,~,v,sebv]       = HszDk5dwPs(lhv_panel_DK,ones(size(lhv_panel_DK,1),1),rhv_panel_DK,1,lags);
W_IRInflDiff_TermStr(Nc+1,1)  = (R_slope*bv - r)'*inv(R_slope*v*R_slope')*(R_slope*bv - r);            % test if long-term and short-term bond slopes are equal in the panel regression
Wp_IRInflDiff_TermStr(Nc+1,1) = 1-chi2cdf(W_IRInflDiff_TermStr(Nc+1,1),1);
W_IRInflDiff_con(2,1)         = (R_con_1*bv - r_con)'*inv(R_con_1*v*R_con_1')*(R_con_1*bv - r_con);    % test if all USD bond return constants are zero in the panel regression
Wp_IRInflDiff_con(2,1)        = 1-chi2cdf(W_IRInflDiff_con(2,1),Nc-1);
W_FXIRInflDiff_con(2,1)       = (R_con_2*bv - r_con)'*inv(R_con_2*v*R_con_2')*(R_con_2*bv - r_con);    % test if all FXEXR constants are zero in the panel regression
Wp_FXIRInflDiff_con(2,1)      = 1-chi2cdf(W_FXIRInflDiff_con(2,1),Nc-1);

Wp_IRInflDiff_TermStr_b(Nc+1,1) = chisq_pval_b(W_IRInflDiff_TermStr(Nc+1,1),1,(lags+1)/T); 
Wp_IRInflDiff_con_b(2,1)        = chisq_pval_b(W_IRInflDiff_con(2,1),Nc-1,(lags+1)/T); 
Wp_FXIRInflDiff_con_b(2,1)      = chisq_pval_b(W_FXIRInflDiff_con(2,1),Nc-1,(lags+1)/T); 


% Tables

rows = Countries_Sample';                   
rows{1,Nc+1} = '--';   
rows{1,Nc+2} = 'Panel (DK)';
rows{1,Nc+3} = 'Joint zero (p-value)';

Table_TS_IRInfl_Collected_Simple = [TSPred_BetasIRInflDiff_con TSPred_BetasIRInflDiff_con_se ...
                         TSPred_BetasIRInflDiff_C1 TSPred_BetasIRInflDiff_C1_se 100*TSPred_BetasIRInflDiff_R2adj...
                         TSPred_BetasFXIRInflDiff_con TSPred_BetasFXIRInflDiff_con_se...
                         TSPred_BetasFXIRInflDiff_C1 TSPred_BetasFXIRInflDiff_C1_se 100*TSPred_BetasFXIRInflDiff_R2adj...
                         TSPred_BetasIRInflDiffLocal_con TSPred_BetasIRInflDiffLocal_con_se ...
                         TSPred_BetasIRInflDiffLocal_C1 TSPred_BetasIRInflDiffLocal_C1_se 100*TSPred_BetasIRInflDiffLocal_R2adj...
                         [Wp_IRInflDiff_TermStr(1:Nc+1,1); Wp_IRInflDiff_TermStr(Nc+1,1)]...
                         TSPred_BetasIRInflDiff_Nobs];   
Table_TS_IRInfl_Collected_Simple = [Table_TS_IRInfl_Collected_Simple; Wp_IRInflDiff_con(1,1) NaN Wp_IRInflDiff_slope(1,1) NaN NaN Wp_FXIRInflDiff_con(1,1) NaN Wp_FXIRInflDiff_slope(1,1) NaN NaN Wp_IRInflDiffLocal_con(1,1) NaN Wp_IRInflDiffLocal_slope(1,1) NaN NaN Wp_IRInflDiff_TermStr(Nc+2,1) NaN];                                          
cols  = {'coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','p-value','Obs.'}; 
disp(' ');
disp('-----------------------------------------');
disp('Collected Results of Time-Series Regressions on Interest Rate, Inflation and Credit Rating Differentials');
disp('-----------------------------------------');
mat2tex(Table_TS_IRInfl_Collected_Simple,'%6.2f',rows,cols, [], Opts)
                                  
Table_TS_IRInfl_Collected_Simple_b = [TSPred_BetasIRInflDiff_con TSPred_BetasIRInflDiff_con_se TSPred_BetasIRInflDiff_con_pval_b ...
                         TSPred_BetasIRInflDiff_C1 TSPred_BetasIRInflDiff_C1_se TSPred_BetasIRInflDiff_C1_pval_b 100*TSPred_BetasIRInflDiff_R2adj...
                         TSPred_BetasFXIRInflDiff_con TSPred_BetasFXIRInflDiff_con_se TSPred_BetasFXIRInflDiff_con_pval_b...
                         TSPred_BetasFXIRInflDiff_C1 TSPred_BetasFXIRInflDiff_C1_se TSPred_BetasFXIRInflDiff_C1_pval_b 100*TSPred_BetasFXIRInflDiff_R2adj...
                         TSPred_BetasIRInflDiffLocal_con TSPred_BetasIRInflDiffLocal_con_se TSPred_BetasIRInflDiffLocal_con_pval_b...
                         TSPred_BetasIRInflDiffLocal_C1 TSPred_BetasIRInflDiffLocal_C1_se TSPred_BetasIRInflDiffLocal_C1_pval_b 100*TSPred_BetasIRInflDiffLocal_R2adj...
                         [Wp_IRInflDiff_TermStr_b(1:Nc+1,1); Wp_IRInflDiff_TermStr_b(Nc+1,1)]...
                         TSPred_BetasIRInflDiff_Nobs];   
Table_TS_IRInfl_Collected_Simple_b = [Table_TS_IRInfl_Collected_Simple_b; Wp_IRInflDiff_con_b(1,1) NaN NaN Wp_IRInflDiff_slope_b(1,1) NaN NaN NaN Wp_FXIRInflDiff_con_b(1,1) NaN NaN Wp_FXIRInflDiff_slope_b(1,1) NaN NaN NaN Wp_IRInflDiffLocal_con_b(1,1) NaN NaN Wp_IRInflDiffLocal_slope_b(1,1) NaN NaN NaN Wp_IRInflDiff_TermStr_b(Nc+2,1) NaN];                                          
cols  = {'coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','p-value','Obs.'}; 
disp(' ');
disp('-----------------------------------------');
disp('Collected Results of Time-Series Regressions on Interest Rate, Inflation and Credit Rating Differentials (fixed-b)');
disp('-----------------------------------------');
mat2tex(Table_TS_IRInfl_Collected_Simple_b,'%6.2f',rows,cols, [], Opts)
                                  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Time-Series Predictability Regressions: Foreign Bond HPERs (and Components) on Yield Curve Slopes, Inflation Rates and Credit Ratings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nc   = size(Countries_Sample,1);

if NW_lag_choice == 1
    
    lags = max(6,horizon+1);  
    
    elseif NW_lag_choice == 2
        
    lags = ceil(4*(size(BondsHPER,1)/100)^(2/9))-1;
    
    elseif NW_lag_choice == 3
        
    lags = ceil(0.75*(size(BondsHPER,1))^(1/3))-1;
    
    elseif NW_lag_choice == 4
        
    lags = max(ceil(4*(size(BondsHPER,1)/100)^(2/9))-1,ceil(0.75*(size(BondsHPER,1))^(1/3))-1);
    
    elseif NW_lag_choice == 5
        
    lags = ceil(1.3*(size(BondsHPER,1))^(1/2))-1;
    
end


TSPred_BetasSlopeInflDiff_con         = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_con_se      = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_con_boot    = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_con_se_boot = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C1          = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C1_se       = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C1_boot     = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C1_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C2          = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C2_se       = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C2_boot     = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C2_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C3          = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C3_se       = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C3_boot     = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_C3_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_R2adj       = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiff_Nobs        = NaN*zeros(Nc,1);

TSPred_BetasFXSlopeInflDiff_con         = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_con_se      = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_con_boot    = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_con_se_boot = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C1          = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C1_se       = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C1_boot     = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C1_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C2          = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C2_se       = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C2_boot     = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C2_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C3          = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C3_se       = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C3_boot     = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_C3_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_R2adj       = NaN*zeros(Nc,1);
TSPred_BetasFXSlopeInflDiff_Nobs        = NaN*zeros(Nc,1);

TSPred_BetasSlopeInflDiffLocal_con         = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_con_se      = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_con_boot    = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_con_se_boot = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C1          = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C1_se       = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C1_boot     = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C1_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C2          = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C2_se       = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C2_boot     = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C2_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C3          = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C3_se       = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C3_boot     = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_C3_se_boot  = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_R2adj       = NaN*zeros(Nc,1);
TSPred_BetasSlopeInflDiffLocal_Nobs        = NaN*zeros(Nc,1);

block_length = horizon;


for n=1:Nc
    
    % Regress USD bond HPR differentials (foreign minus US) on slope, inflation and credit rating differentials (foreign minus US)
    
    lhv = (BondsHPERDollar(:,n)-BondsHPERDollar(:,end))*ScalingFactor;
    rhv = [ones(size(lhv,1),1) 12*Slope(:,n)-12*Slope(:,end) 12*CPIHPER(:,n)-12*CPIHPER(:,end) Ratings(:,n)-Ratings(:,end)];
    if range(Ratings(:,n)-Ratings(:,end)) == 0
            rhv = [ones(size(lhv,1),1) 12*Slope(:,n)-12*Slope(:,end) 12*CPIHPER(:,n)-12*CPIHPER(:,end)];
    end
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasSlopeInflDiff_con(n,1)        = bv(1);
        TSPred_BetasSlopeInflDiff_con_se(n,1)     = sebv(1);
        TSPred_BetasSlopeInflDiff_con_boot(n,1)   = bootstrap_stats(1,1);
        TSPred_BetasSlopeInflDiff_con_se_boot(n,1)= bootstrap_stats(1,2);
        TSPred_BetasSlopeInflDiff_C1(n,1)         = bv(2);
        TSPred_BetasSlopeInflDiff_C1_se(n,1)      = sebv(2);
        TSPred_BetasSlopeInflDiff_C1_boot(n,1)    = bootstrap_stats(2,1);
        TSPred_BetasSlopeInflDiff_C1_se_boot(n,1) = bootstrap_stats(2,2);
        TSPred_BetasSlopeInflDiff_C2(n,1)         = bv(3);
        TSPred_BetasSlopeInflDiff_C2_se(n,1)      = sebv(3);
        TSPred_BetasSlopeInflDiff_C2_boot(n,1)    = bootstrap_stats(3,1);
        TSPred_BetasSlopeInflDiff_C2_se_boot(n,1) = bootstrap_stats(3,2);
        if range(Ratings(:,n)-Ratings(:,end)) > 0
            TSPred_BetasSlopeInflDiff_C3(n,1)         = bv(4);
            TSPred_BetasSlopeInflDiff_C3_se(n,1)      = sebv(4);
            TSPred_BetasSlopeInflDiff_C3_boot(n,1)    = bootstrap_stats(4,1);
            TSPred_BetasSlopeInflDiff_C3_se_boot(n,1) = bootstrap_stats(4,2);
        end
        TSPred_BetasSlopeInflDiff_R2adj(n,1)      = R2vadj;
        TSPred_BetasSlopeInflDiff_Nobs(n,1)       = size(lhv_s,1);
        TSPred_BetasSlopeInflDiff_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasSlopeInflDiff_con(n,1)/TSPred_BetasSlopeInflDiff_con_se(n,1),(lags+1)/TSPred_BetasSlopeInflDiff_Nobs(n,1));                 
        TSPred_BetasSlopeInflDiff_C1_pval_b(n,1)  = tstat_pval_b(TSPred_BetasSlopeInflDiff_C1(n,1)/TSPred_BetasSlopeInflDiff_C1_se(n,1),(lags+1)/TSPred_BetasSlopeInflDiff_Nobs(n,1));   
    end
    
    % Regress excess FX returns on slope, inflation and credit rating differentials (foreign minus US) 

    lhv = FXEXR(:,n)*ScalingFactor;
    rhv = [ones(size(lhv,1),1) 12*Slope(:,n)-12*Slope(:,end) 12*CPIHPER(:,n)-12*CPIHPER(:,end) Ratings(:,n)-Ratings(:,end)];
    if range(Ratings(:,n)-Ratings(:,end)) == 0
            rhv = [ones(size(lhv,1),1) 12*Slope(:,n)-12*Slope(:,end) 12*CPIHPER(:,n)-12*CPIHPER(:,end)];
    end
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasFXSlopeInflDiff_con(n,1)        = bv(1);
        TSPred_BetasFXSlopeInflDiff_con_se(n,1)     = sebv(1);
        TSPred_BetasFXSlopeInflDiff_con_boot(n,1)   = bootstrap_stats(1,1);
        TSPred_BetasFXSlopeInflDiff_con_se_boot(n,1)= bootstrap_stats(1,2);
        TSPred_BetasFXSlopeInflDiff_C1(n,1)         = bv(2);
        TSPred_BetasFXSlopeInflDiff_C1_se(n,1)      = sebv(2);
        TSPred_BetasFXSlopeInflDiff_C1_boot(n,1)    = bootstrap_stats(2,1);
        TSPred_BetasFXSlopeInflDiff_C1_se_boot(n,1) = bootstrap_stats(2,2);
        TSPred_BetasFXSlopeInflDiff_C2(n,1)         = bv(3);
        TSPred_BetasFXSlopeInflDiff_C2_se(n,1)      = sebv(3);
        TSPred_BetasFXSlopeInflDiff_C2_boot(n,1)    = bootstrap_stats(3,1);
        TSPred_BetasFXSlopeInflDiff_C2_se_boot(n,1) = bootstrap_stats(3,2);
        if range(Ratings(:,n)-Ratings(:,end)) > 0
            TSPred_BetasFXSlopeInflDiff_C3(n,1)         = bv(4);
            TSPred_BetasFXSlopeInflDiff_C3_se(n,1)      = sebv(4);
            TSPred_BetasFXSlopeInflDiff_C3_boot(n,1)    = bootstrap_stats(4,1);
            TSPred_BetasFXSlopeInflDiff_C3_se_boot(n,1) = bootstrap_stats(4,2);
        end
        TSPred_BetasFXSlopeInflDiff_R2adj(n,1)      = R2vadj;
        TSPred_BetasFXSlopeInflDiff_Nobs(n,1)       = size(lhv_s,1);
        TSPred_BetasFXSlopeInflDiff_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasFXSlopeInflDiff_con(n,1)/TSPred_BetasFXSlopeInflDiff_con_se(n,1),(lags+1)/TSPred_BetasFXSlopeInflDiff_Nobs(n,1));                 
        TSPred_BetasFXSlopeInflDiff_C1_pval_b(n,1)  = tstat_pval_b(TSPred_BetasFXSlopeInflDiff_C1(n,1)/TSPred_BetasFXSlopeInflDiff_C1_se(n,1),(lags+1)/TSPred_BetasFXSlopeInflDiff_Nobs(n,1));   
    end;
 
    % Regress bond local currency HPER differentials (foreign minus US) on slope, inflation and credit rating differentials (foreign minus US)

    lhv = (BondsHPER(:,n)-BondsHPER(:,end))*ScalingFactor;
    rhv = [ones(size(lhv,1),1) 12*Slope(:,n)-12*Slope(:,end) 12*CPIHPER(:,n)-12*CPIHPER(:,end) Ratings(:,n)-Ratings(:,end)];
    if range(Ratings(:,n)-Ratings(:,end)) == 0
            rhv = [ones(size(lhv,1),1) 12*Slope(:,n)-12*Slope(:,end) 12*CPIHPER(:,n)-12*CPIHPER(:,end)];
    end
    [lhv_s,rhv_s]              = FindCommonSample(lhv,rhv);
    weight = 1;
    if size(rhv_s,1)>0 && n<Nc %(excludes US)
        [lhv_s,rhv_s]                  = FindCommonSample(lhv,rhv);
        [bv,sebv,R2v,R2vadj]           = olsgmm(lhv_s,rhv_s,lags,weight);
        bootstrap_stats                = regression_bootstrap_short([lhv_s,rhv_s], block_length, 1000);
        TSPred_BetasSlopeInflDiffLocal_con(n,1)        = bv(1);
        TSPred_BetasSlopeInflDiffLocal_con_se(n,1)     = sebv(1);
        TSPred_BetasSlopeInflDiffLocal_con_boot(n,1)   = bootstrap_stats(1,1);
        TSPred_BetasSlopeInflDiffLocal_con_se_boot(n,1)= bootstrap_stats(1,2);
        TSPred_BetasSlopeInflDiffLocal_C1(n,1)         = bv(2);
        TSPred_BetasSlopeInflDiffLocal_C1_se(n,1)      = sebv(2);
        TSPred_BetasSlopeInflDiffLocal_C1_boot(n,1)    = bootstrap_stats(2,1);
        TSPred_BetasSlopeInflDiffLocal_C1_se_boot(n,1) = bootstrap_stats(2,2);
        TSPred_BetasSlopeInflDiffLocal_C2(n,1)         = bv(3);
        TSPred_BetasSlopeInflDiffLocal_C2_se(n,1)      = sebv(3);
        TSPred_BetasSlopeInflDiffLocal_C2_boot(n,1)    = bootstrap_stats(3,1);
        TSPred_BetasSlopeInflDiffLocal_C2_se_boot(n,1) = bootstrap_stats(3,2);
        if range(Ratings(:,n)-Ratings(:,end)) > 0
            TSPred_BetasSlopeInflDiffLocal_C3(n,1)         = bv(4);
            TSPred_BetasSlopeInflDiffLocal_C3_se(n,1)      = sebv(4);
            TSPred_BetasSlopeInflDiffLocal_C3_boot(n,1)    = bootstrap_stats(4,1);
            TSPred_BetasSlopeInflDiffLocal_C3_se_boot(n,1) = bootstrap_stats(4,2);
        end
        TSPred_BetasSlopeInflDiffLocal_R2adj(n,1)      = R2vadj;
        TSPred_BetasSlopeInflDiffLocal_Nobs(n,1)       = size(lhv_s,1);
        TSPred_BetasSlopeInflDiffLocal_con_pval_b(n,1) = tstat_pval_b(TSPred_BetasSlopeInflDiffLocal_con(n,1)/TSPred_BetasSlopeInflDiffLocal_con_se(n,1),(lags+1)/TSPred_BetasSlopeInflDiffLocal_Nobs(n,1));                 
        TSPred_BetasSlopeInflDiffLocal_C1_pval_b(n,1)  = tstat_pval_b(TSPred_BetasSlopeInflDiffLocal_C1(n,1)/TSPred_BetasSlopeInflDiffLocal_C1_se(n,1),(lags+1)/TSPred_BetasSlopeInflDiffLocal_Nobs(n,1));   
    end;
    
      
end;

% Panel regressions with Driscoll-Kraay s.e.

Panel_Var = [ScalingFactor*BondHPERDollar_Diff, 12*Slope_Diff, 12*CPIHPER_Diff, Ratings_Fgn-Ratings_US];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK         = Panel_Var(:,1:Nc-1);
X_DK(:,:,1)  = Panel_Var(:,Nc:2*(Nc-1));
X_DK(:,:,2)  = Panel_Var(:,2*Nc-1:3*(Nc-1));
X_DK(:,:,3)  = Panel_Var(:,3*Nc-2:4*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+3) = 0*y_DK;
    X_DK(:,j,j+3) = ones(size(y_DK,1),1);
end

[TSPred_BetasSlopeInflDiff, ~, ~,~,yhat,Rsq,~,~,TSPred_BetasSlopeInflDiff_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasSlopeInflDiff_con(Nc+2,1)     = NaN;
TSPred_BetasSlopeInflDiff_con_se(Nc+2,1)  = NaN;
TSPred_BetasSlopeInflDiff_C1(Nc+2,1)      = TSPred_BetasSlopeInflDiff(1,1);
TSPred_BetasSlopeInflDiff_C1_se(Nc+2,1)   = TSPred_BetasSlopeInflDiff_se(1,1);
TSPred_BetasSlopeInflDiff_C2(Nc+2,1)      = TSPred_BetasSlopeInflDiff(2,1);
TSPred_BetasSlopeInflDiff_C2_se(Nc+2,1)   = TSPred_BetasSlopeInflDiff_se(2,1);
TSPred_BetasSlopeInflDiff_C3(Nc+2,1)      = TSPred_BetasSlopeInflDiff(3,1);
TSPred_BetasSlopeInflDiff_C3_se(Nc+2,1)   = TSPred_BetasSlopeInflDiff_se(3,1);
TSPred_BetasSlopeInflDiff_Nobs(Nc+2,1)    = sum(sum(~isnan(yhat)));
TSPred_BetasSlopeInflDiff_R2adj(Nc+2,1)   = 1-(1-Rsq)*(TSPred_BetasSlopeInflDiff_Nobs(Nc+2,1)-1)/(TSPred_BetasSlopeInflDiff_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasSlopeInflDiff_con_boot(Nc+2,1)    = NaN;
TSPred_BetasSlopeInflDiff_con_se_boot(Nc+2,1) = NaN;
TSPred_BetasSlopeInflDiff_C1_boot(Nc+2,1)     = bootstrap_stats(1,1);
TSPred_BetasSlopeInflDiff_C1_se_boot(Nc+2,1)  = bootstrap_stats(1,2);
TSPred_BetasSlopeInflDiff_C2_boot(Nc+2,1)     = bootstrap_stats(2,1);
TSPred_BetasSlopeInflDiff_C2_se_boot(Nc+2,1)  = bootstrap_stats(2,2);
TSPred_BetasSlopeInflDiff_C3_boot(Nc+2,1)     = bootstrap_stats(3,1);
TSPred_BetasSlopeInflDiff_C3_se_boot(Nc+2,1)  = bootstrap_stats(3,2);
TSPred_BetasSlopeInflDiff_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasSlopeInflDiff_con(Nc+2,1)/TSPred_BetasSlopeInflDiff_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasSlopeInflDiff_C1_pval_b(Nc+2,1)     = tstat_pval_b(TSPred_BetasSlopeInflDiff_C1(Nc+2,1)/TSPred_BetasSlopeInflDiff_C1_se(Nc+2,1), (lags+1)/size(y_DK,1));                 


Panel_Var = [ScalingFactor*FXEXR_Panel, 12*Slope_Diff, 12*CPIHPER_Diff, Ratings_Fgn-Ratings_US];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK         = Panel_Var(:,1:Nc-1);
X_DK(:,:,1)  = Panel_Var(:,Nc:2*(Nc-1));
X_DK(:,:,2)  = Panel_Var(:,2*Nc-1:3*(Nc-1));
X_DK(:,:,3)  = Panel_Var(:,3*Nc-2:4*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+3) = 0*y_DK;
    X_DK(:,j,j+3) = ones(size(y_DK,1),1);
end

[TSPred_BetasFXSlopeInflDiff, ~, ~,~,yhat,Rsq,~,~,TSPred_BetasFXSlopeInflDiff_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasFXSlopeInflDiff_con(Nc+2,1)     = NaN;
TSPred_BetasFXSlopeInflDiff_con_se(Nc+2,1)  = NaN;
TSPred_BetasFXSlopeInflDiff_C1(Nc+2,1)      = TSPred_BetasFXSlopeInflDiff(1,1);
TSPred_BetasFXSlopeInflDiff_C1_se(Nc+2,1)   = TSPred_BetasFXSlopeInflDiff_se(1,1);
TSPred_BetasFXSlopeInflDiff_C2(Nc+2,1)      = TSPred_BetasFXSlopeInflDiff(2,1);
TSPred_BetasFXSlopeInflDiff_C2_se(Nc+2,1)   = TSPred_BetasFXSlopeInflDiff_se(2,1);
TSPred_BetasFXSlopeInflDiff_C3(Nc+2,1)      = TSPred_BetasFXSlopeInflDiff(3,1);
TSPred_BetasFXSlopeInflDiff_C3_se(Nc+2,1)   = TSPred_BetasFXSlopeInflDiff_se(3,1);
TSPred_BetasFXSlopeInflDiff_Nobs(Nc+2,1)    = sum(sum(~isnan(yhat)));
TSPred_BetasFXSlopeInflDiff_R2adj(Nc+2,1)   = 1-(1-Rsq)*(TSPred_BetasFXSlopeInflDiff_Nobs(Nc+2,1)-1)/(TSPred_BetasFXSlopeInflDiff_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasFXSlopeInflDiff_con_boot(Nc+2,1)       = NaN;
TSPred_BetasFXSlopeInflDiff_con_se_boot(Nc+2,1)    = NaN;
TSPred_BetasFXSlopeInflDiff_C1_boot(Nc+2,1)        = bootstrap_stats(1,1);
TSPred_BetasFXSlopeInflDiff_C1_se_boot(Nc+2,1)     = bootstrap_stats(1,2);
TSPred_BetasFXSlopeInflDiff_C2_boot(Nc+2,1)        = bootstrap_stats(2,1);
TSPred_BetasFXSlopeInflDiff_C2_se_boot(Nc+2,1)     = bootstrap_stats(2,2);
TSPred_BetasFXSlopeInflDiff_C3_boot(Nc+2,1)        = bootstrap_stats(3,1);
TSPred_BetasFXSlopeInflDiff_C3_se_boot(Nc+2,1)     = bootstrap_stats(3,2);
TSPred_BetasFXSlopeInflDiff_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasFXSlopeInflDiff_con(Nc+2,1)/TSPred_BetasFXSlopeInflDiff_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasFXSlopeInflDiff_C1_pval_b(Nc+2,1)     = tstat_pval_b(TSPred_BetasFXSlopeInflDiff_C1(Nc+2,1)/TSPred_BetasFXSlopeInflDiff_C1_se(Nc+2,1), (lags+1)/size(y_DK,1));                 



Panel_Var = [ScalingFactor*BondHPER_Diff, 12*Slope_Diff  , 12*CPIHPER_Diff, Ratings_Fgn-Ratings_US];
Panel_Var = Panel_Var(1:end-horizon+1,:);
y_DK         = Panel_Var(:,1:Nc-1);
X_DK(:,:,1)  = Panel_Var(:,Nc:2*(Nc-1));
X_DK(:,:,2)  = Panel_Var(:,2*Nc-1:3*(Nc-1));
X_DK(:,:,3)  = Panel_Var(:,3*Nc-2:4*(Nc-1));
for j=1:Nc-1
    X_DK(:,:,j+3) = 0*y_DK;
    X_DK(:,j,j+3) = ones(size(y_DK,1),1);
end

[TSPred_BetasSlopeInflDiffLocal, ~, ~,~,yhat,Rsq,~,~,TSPred_BetasSlopeInflDiffLocal_se] = HszDk5dwPs(y_DK,ones(size(Panel_Var,1),1),X_DK,1,lags);
TSPred_BetasSlopeInflDiffLocal_con(Nc+2,1)     = NaN;
TSPred_BetasSlopeInflDiffLocal_con_se(Nc+2,1)  = NaN;
TSPred_BetasSlopeInflDiffLocal_C1(Nc+2,1)      = TSPred_BetasSlopeInflDiffLocal(1,1);
TSPred_BetasSlopeInflDiffLocal_C1_se(Nc+2,1)   = TSPred_BetasSlopeInflDiffLocal_se(1,1);
TSPred_BetasSlopeInflDiffLocal_C2(Nc+2,1)      = TSPred_BetasSlopeInflDiffLocal(2,1);
TSPred_BetasSlopeInflDiffLocal_C2_se(Nc+2,1)   = TSPred_BetasSlopeInflDiffLocal_se(2,1);
TSPred_BetasSlopeInflDiffLocal_C3(Nc+2,1)      = TSPred_BetasSlopeInflDiffLocal(3,1);
TSPred_BetasSlopeInflDiffLocal_C3_se(Nc+2,1)   = TSPred_BetasSlopeInflDiffLocal_se(3,1);
TSPred_BetasSlopeInflDiffLocal_Nobs(Nc+2,1)    = sum(sum(~isnan(yhat)));
TSPred_BetasSlopeInflDiffLocal_R2adj(Nc+2,1)   = 1-(1-Rsq)*(TSPred_BetasSlopeInflDiffLocal_Nobs(Nc+2,1)-1)/(TSPred_BetasSlopeInflDiffLocal_Nobs(Nc+2,1)-size(X_DK,3));
bootstrap_stats  = regression_bootstrap_short_panel_updated([y_DK,reshape(X_DK,size(y_DK,1),[])], block_length, 1000, Nc-1);
TSPred_BetasSlopeInflDiffLocal_con_boot(Nc+2,1)    = NaN;
TSPred_BetasSlopeInflDiffLocal_con_se_boot(Nc+2,1) = NaN;
TSPred_BetasSlopeInflDiffLocal_C1_boot(Nc+2,1)     = bootstrap_stats(1,1);
TSPred_BetasSlopeInflDiffLocal_C1_se_boot(Nc+2,1)  = bootstrap_stats(1,2);
TSPred_BetasSlopeInflDiffLocal_C2_boot(Nc+2,1)     = bootstrap_stats(2,1);
TSPred_BetasSlopeInflDiffLocal_C2_se_boot(Nc+2,1)  = bootstrap_stats(2,2);
TSPred_BetasSlopeInflDiffLocal_C3_boot(Nc+2,1)     = bootstrap_stats(3,1);
TSPred_BetasSlopeInflDiffLocal_C3_se_boot(Nc+2,1)  = bootstrap_stats(3,2);
TSPred_BetasSlopeInflDiffLocal_con_pval_b(Nc+2,1)    = tstat_pval_b(TSPred_BetasSlopeInflDiffLocal_con(Nc+2,1)/TSPred_BetasSlopeInflDiffLocal_con_se(Nc+2,1), (lags+1)/size(y_DK,1));                  
TSPred_BetasSlopeInflDiffLocal_C1_pval_b(Nc+2,1)     = tstat_pval_b(TSPred_BetasSlopeInflDiffLocal_C1(Nc+2,1)/TSPred_BetasSlopeInflDiffLocal_C1_se(Nc+2,1), (lags+1)/size(y_DK,1));                 


% Joint estimation

T = size(BondsHPERDollar,1)-horizon+1;

R1_con     = zeros(Nc-1,9*(Nc-1));
R1_slope   = zeros(Nc-1,9*(Nc-1));
R1_all     = zeros(2*(Nc-1),9*(Nc-1));
R2_con     = zeros(Nc-1,9*(Nc-1));
R2_slope   = zeros(Nc-1,9*(Nc-1));
R2_all     = zeros(2*(Nc-1),9*(Nc-1));
R3_con     = zeros(Nc-1,9*(Nc-1));
R3_slope   = zeros(Nc-1,9*(Nc-1));
R3_all     = zeros(2*(Nc-1),9*(Nc-1));
R_diff     = zeros(Nc-1,9*(Nc-1));
r          = zeros(Nc-1,1);
r_all      = zeros(2*(Nc-1),1);
rhv        = zeros(3*T*(Nc-1),12*(Nc-1));
for n=1:Nc-1
lhv((n-1)*T+1:n*T,1)                                                     = (BondsHPERDollar(1:end-horizon+1,n)-BondsHPERDollar(1:end-horizon+1,end))*ScalingFactor;
lhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,1)                                   = FXEXR(1:end-horizon+1,n)*ScalingFactor;
lhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,1)                               = (BondsHPER(1:end-horizon+1,n)-BondsHPER(1:end-horizon+1,end))*ScalingFactor;
rhv((n-1)*T+1:n*T,(n-1)*4+1:n*4)                                         = [ones(T,1) 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end) 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end) Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end)];
rhv(T*(Nc-1)+(n-1)*T+1:T*(Nc-1)+n*T,4*(Nc-1)+(n-1)*4+1:4*(Nc-1)+n*4)     = [ones(T,1) 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end) 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end) Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end)];
rhv(2*T*(Nc-1)+(n-1)*T+1:2*T*(Nc-1)+n*T,8*(Nc-1)+(n-1)*4+1:8*(Nc-1)+n*4) = [ones(T,1) 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end) 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end) Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end)];

R1_con(n,3*(n-1)+1)            =  1;
R1_slope(n,3*(n-1)+2)          =  1;
R1_all(n,3*(n-1)+1)            =  1;
R1_all(Nc-1+n,3*(n-1)+2)       =  1;
R2_con(n,3*(Nc-1)+3*(n-1)+1)   =  1;
R2_slope(n,3*(Nc-1)+3*(n-1)+2) =  1;
R2_all(n,3*(Nc-1)+3*(n-1)+1)   =  1;
R2_all(Nc-1+n,3*(Nc-1)+3*(n-1)+2) =  1;
R3_con(n,6*(Nc-1)+3*(n-1)+1)   =  1;
R3_slope(n,6*(Nc-1)+3*(n-1)+2) =  1;
R3_all(n,6*(Nc-1)+3*(n-1)+1)   =  1;
R3_all(Nc-1+n,6*(Nc-1)+3*(n-1)+2)   =  1;
R_diff(n,3*(n-1)+2)            =  1;
R_diff(n,3*(Nc-1)+3*(n-1)+2)   = -1;
end
[lhv_s,rhv_s]                 = FindCommonSample(lhv,rhv);
zero_col =  find(all(rhv_s==0));                                                                 % find columns with all zeros
rhv_s(:,all(~rhv_s,1)) = [];                                                                     % remove columns with all zeros
[bv,sebv,R2v,R2vadj,v]        = olsgmm(lhv_s,rhv_s,lags,1);
for k = zero_col;
    bv = [bv(1:k-1); NaN; bv(k:end)];
    v  = [v(1:k-1,:); NaN(1,size(v,2)); v(k:end,:)];
    v  = [v(:,1:k-1), NaN(size(v,1),1), v(:,k:end)];
end
bv(4:4:108) = [];
v(4:4:108,:) = [];
v(:,4:4:108) = [];
W_SlopeInflDiff_con(1,1)         = (R1_con*bv - r)'*inv(R1_con*v*R1_con')*(R1_con*bv - r);          % test if all constants are zero in the HPERDollar regressions
Wp_SlopeInflDiff_con(1,1)        = 1-chi2cdf(W_SlopeInflDiff_con(1,1),Nc-1);
W_SlopeInflDiff_slope(1,1)       = (R1_slope*bv - r)'*inv(R1_slope*v*R1_slope')*(R1_slope*bv - r);  % test if all slopes are zero in the HPERDollar regressions
Wp_SlopeInflDiff_slope(1,1)      = 1-chi2cdf(W_SlopeInflDiff_slope(1,1),Nc-1);
W_SlopeInflDiff_all(1,1)         = (R1_all*bv - r_all)'*inv(R1_all*v*R1_all')*(R1_all*bv - r_all);  % test if all constants and slopes are zero in the HPERDollar regressions
Wp_SlopeInflDiff_all(1,1)        = 1-chi2cdf(W_SlopeInflDiff_all(1,1),2*(Nc-1));
W_FXSlopeInflDiff_con(1,1)       = (R2_con*bv - r)'*inv(R2_con*v*R2_con')*(R2_con*bv - r);          % test if all constants are zero in the FXEXR regressions
Wp_FXSlopeInflDiff_con(1,1)      = 1-chi2cdf(W_FXSlopeInflDiff_con(1,1),Nc-1);
W_FXSlopeInflDiff_slope(1,1)     = (R2_slope*bv - r)'*inv(R2_slope*v*R2_slope')*(R2_slope*bv - r);  % test if all slopes are zero in the FXEXR regressions
Wp_FXSlopeInflDiff_slope(1,1)    = 1-chi2cdf(W_FXSlopeInflDiff_slope(1,1),Nc-1);
W_FXSlopeInflDiff_all(1,1)       = (R2_all*bv - r_all)'*inv(R2_all*v*R2_all')*(R2_all*bv - r_all);  % test if all constants and slopes are zero in the FXEXR regressions
Wp_FXSlopeInflDiff_all(1,1)      = 1-chi2cdf(W_FXSlopeInflDiff_all(1,1),2*(Nc-1));
W_SlopeInflDiffLocal_con(1,1)    = (R3_con*bv - r)'*inv(R3_con*v*R3_con')*(R3_con*bv - r);          % test if all constants are zero in the HPER regressions
Wp_SlopeInflDiffLocal_con(1,1)   = 1-chi2cdf(W_SlopeInflDiffLocal_con(1,1),Nc-1);
W_SlopeInflDiffLocal_slope(1,1)  = (R3_slope*bv - r)'*inv(R3_slope*v*R3_slope')*(R3_slope*bv - r);  % test if all slopes are zero in the HPER regressions
Wp_SlopeInflDiffLocal_slope(1,1) = 1-chi2cdf(W_SlopeInflDiffLocal_slope(1,1),Nc-1);
W_SlopeInflDiffLocal_all(1,1)    = (R3_all*bv - r_all)'*inv(R3_all*v*R3_all')*(R3_all*bv - r_all);  % test if all constants and slopes are zero in the HPER regressions
Wp_SlopeInflDiffLocal_all(1,1)   = 1-chi2cdf(W_SlopeInflDiffLocal_all(1,1),2*(Nc-1));
W_SlopeInflDiff_TermStr(Nc+2,1)  = (R_diff*bv - r)'*inv(R_diff*v*R_diff')*(R_diff*bv - r);          % test if all HPERDollar/FXEXR slope differentials are zero
Wp_SlopeInflDiff_TermStr(Nc+2,1) = 1-chi2cdf(W_SlopeInflDiff_TermStr(Nc+2,1),Nc-1);

Wp_SlopeInflDiff_con_b(1,1)        = chisq_pval_b(W_SlopeInflDiff_con(1,1),Nc-1,(lags+1)/T);
Wp_SlopeInflDiff_slope_b(1,1)      = chisq_pval_b(W_SlopeInflDiff_slope(1,1),Nc-1,(lags+1)/T);
Wp_SlopeInflDiff_all_b(1,1)        = chisq_pval_b(W_SlopeInflDiff_all(1,1),2*(Nc-1),(lags+1)/T);
Wp_FXSlopeInflDiff_con_b(1,1)      = chisq_pval_b(W_FXSlopeInflDiff_con(1,1),Nc-1,(lags+1)/T);
Wp_FXSlopeInflDiff_slope_b(1,1)    = chisq_pval_b(W_FXSlopeInflDiff_slope(1,1),Nc-1,(lags+1)/T);
Wp_FXSlopeInflDiff_all_b(1,1)      = chisq_pval_b(W_FXSlopeInflDiff_all(1,1),2*(Nc-1),(lags+1)/T);
Wp_SlopeInflDiffLocal_con_b(1,1)   = chisq_pval_b(W_SlopeInflDiffLocal_con(1,1),Nc-1,(lags+1)/T);
Wp_SlopeInflDiffLocal_slope_b(1,1) = chisq_pval_b(W_SlopeInflDiffLocal_slope(1,1),Nc-1,(lags+1)/T);
Wp_SlopeInflDiffLocal_all_b(1,1)   = chisq_pval_b(W_SlopeInflDiffLocal_all(1,1),2*(Nc-1),(lags+1)/T); 
Wp_SlopeInflDiff_TermStr_b(Nc+2,1) = chisq_pval_b(W_SlopeInflDiff_TermStr(Nc+2,1),Nc-1,(lags+1)/T);

% Panel regression

lhv_panel    = [];
rhv_panel    = zeros(2*T*(Nc-1),2*(Nc+2));
lhv_panel_DK = NaN(T,2*(Nc-1));
rhv_panel_DK = NaN(T,2*(Nc-1),2*(Nc+2));
R = [0 1 0 0 -1 0];
R_slope      = [zeros(1,Nc-1) 1 zeros(1,2) zeros(1,Nc-1) -1 zeros(1,2)];
R_con_1      = zeros(Nc-1,2*(Nc+2));
R_con_2      = zeros(Nc-1,2*(Nc+2));
r            = 0;
r_con        = zeros(Nc-1,1);
for n = 1:Nc-1
T = size(BondsHPERDollar,1)-horizon+1;
lhv  = [(BondsHPERDollar(1:end-horizon+1,n)-BondsHPERDollar(1:end-horizon+1,end))*ScalingFactor;  FXEXR(1:end-horizon+1,n)*ScalingFactor];
rhv  = zeros(2*T,8);
rhv(1:T,1:4)     = [ones(T,1) 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end) 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end) Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end)];
rhv(T+1:2*T,5:8) = [ones(T,1) 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end) 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end) Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end)];
[lhv_s,rhv_s]                     = FindCommonSample(lhv,rhv);
zero_col =  find(all(rhv_s==0));                                                                 % find columns with all zeros
rhv_s(:,all(~rhv_s,1)) = [];                                                                     % remove columns with all zeros
[bv,sebv,R2v,R2vadj,v]            = olsgmm(lhv_s,rhv_s,lags,1);
for k = zero_col;
    bv = [bv(1:k-1); NaN; bv(k:end)];
    v  = [v(1:k-1,:); NaN(1,size(v,2)); v(k:end,:)];
    v  = [v(:,1:k-1), NaN(size(v,1),1), v(:,k:end)];
end
bv([4 8]) = [];
v(:,[4 8]) = [];
v([4 8],:) = [];
W_SlopeInflDiff_TermStr(n,1)  = (R*bv - r)'*inv(R*v*R')*(R*bv - r);      % test if long-term and short-term bond slopes are equal
Wp_SlopeInflDiff_TermStr(n,1) = 1-chi2cdf(W_SlopeInflDiff_TermStr(n,1),1);
Wp_SlopeInflDiff_TermStr_b(n,1) = chisq_pval_b(W_SlopeInflDiff_TermStr(n,1),1,(lags+1)/T);


lhv_panel = [lhv_panel; lhv];
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,n)    = ones(T,1);
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,Nc)   = 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end);
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,Nc+1) = 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end);
rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,Nc+2) = Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end);
rhv_panel((n-1)*2*T+T+1:n*2*T,Nc+2+n)   = ones(T,1);
rhv_panel((n-1)*2*T+T+1:n*2*T,2*Nc+2)   = 12*Slope(1:end-horizon+1,n)-12*Slope(1:end-horizon+1,end);
rhv_panel((n-1)*2*T+T+1:n*2*T,2*Nc+3)   = 12*CPIHPER(1:end-horizon+1,n)-12*CPIHPER(1:end-horizon+1,end);
rhv_panel((n-1)*2*T+T+1:n*2*T,2*Nc+4)   = Ratings(1:end-horizon+1,n)-Ratings(1:end-horizon+1,end);

lhv_panel_DK(:,n)        = lhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:);
lhv_panel_DK(:,Nc-1+n)   = lhv_panel((n-1)*2*T+T+1:n*2*T,:);
rhv_panel_DK(:,n,:)      = reshape(rhv_panel((n-1)*2*T+1:(n-1)*2*T+T,:),T,[]);
rhv_panel_DK(:,Nc-1+n,:) = reshape(rhv_panel((n-1)*2*T+T+1:n*2*T,:),T,[]);

R_con_1(n,n)      = 1;
R_con_2(n,Nc+2+n) = 1;
end
[bv,~,~,~,~,~,~,v,sebv]       = HszDk5dwPs(lhv_panel_DK,ones(size(lhv_panel_DK,1),1),rhv_panel_DK,1,lags);
W_SlopeInflDiff_TermStr(Nc+1,1)  = (R_slope*bv - r)'*inv(R_slope*v*R_slope')*(R_slope*bv - r);            % test if long-term and short-term bond slopes are equal in the panel regression
Wp_SlopeInflDiff_TermStr(Nc+1,1) = 1-chi2cdf(W_SlopeInflDiff_TermStr(Nc+1,1),1);
W_SlopeInflDiff_con(2,1)         = (R_con_1*bv - r_con)'*inv(R_con_1*v*R_con_1')*(R_con_1*bv - r_con);    % test if all USD bond return constants are zero in the panel regression
Wp_SlopeInflDiff_con(2,1)        = 1-chi2cdf(W_SlopeInflDiff_con(2,1),Nc-1);
W_FXSlopeInflDiff_con(2,1)       = (R_con_2*bv - r_con)'*inv(R_con_2*v*R_con_2')*(R_con_2*bv - r_con);    % test if all FXEXR constants are zero in the panel regression
Wp_FXSlopeInflDiff_con(2,1)      = 1-chi2cdf(W_FXSlopeInflDiff_con(2,1),Nc-1);

Wp_SlopeInflDiff_TermStr_b(Nc+1,1) = chisq_pval_b(W_SlopeInflDiff_TermStr(Nc+1,1),1,(lags+1)/T); 
Wp_SlopeInflDiff_con_b(2,1)        = chisq_pval_b(W_SlopeInflDiff_con(2,1),Nc-1,(lags+1)/T); 
Wp_FXSlopeInflDiff_con_b(2,1)      = chisq_pval_b(W_FXSlopeInflDiff_con(2,1),Nc-1,(lags+1)/T); 


% Tables

rows = Countries_Sample';                   
rows{1,Nc+1} = '--';   
rows{1,Nc+2} = 'Panel (DK)';
rows{1,Nc+3} = 'Joint zero (p-value)';

Table_TS_SlopeInfl_Collected_Simple = [TSPred_BetasSlopeInflDiff_con TSPred_BetasSlopeInflDiff_con_se ...
                         TSPred_BetasSlopeInflDiff_C1 TSPred_BetasSlopeInflDiff_C1_se 100*TSPred_BetasSlopeInflDiff_R2adj...
                         TSPred_BetasFXSlopeInflDiff_con TSPred_BetasFXSlopeInflDiff_con_se...
                         TSPred_BetasFXSlopeInflDiff_C1 TSPred_BetasFXSlopeInflDiff_C1_se 100*TSPred_BetasFXSlopeInflDiff_R2adj...
                         TSPred_BetasSlopeInflDiffLocal_con TSPred_BetasSlopeInflDiffLocal_con_se ...
                         TSPred_BetasSlopeInflDiffLocal_C1 TSPred_BetasSlopeInflDiffLocal_C1_se 100*TSPred_BetasSlopeInflDiffLocal_R2adj...
                         [Wp_SlopeInflDiff_TermStr(1:Nc+1,1); Wp_SlopeInflDiff_TermStr(Nc+1,1)]...
                         TSPred_BetasSlopeInflDiff_Nobs];   
Table_TS_SlopeInfl_Collected_Simple = [Table_TS_SlopeInfl_Collected_Simple; Wp_SlopeInflDiff_con(1,1) NaN Wp_SlopeInflDiff_slope(1,1) NaN NaN Wp_FXSlopeInflDiff_con(1,1) NaN Wp_FXSlopeInflDiff_slope(1,1) NaN NaN Wp_SlopeInflDiffLocal_con(1,1) NaN Wp_SlopeInflDiffLocal_slope(1,1) NaN NaN Wp_SlopeInflDiff_TermStr(Nc+2,1) NaN];                                          
cols  = {'coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','coeff','s.e.','coeff','s.e.','R2adj(%)','p-value','Obs.'}; 
disp(' ');
disp('-----------------------------------------');
disp('Collected Results of Time-Series Regressions on Slope, Inflation and Credit Rating Differentials');
disp('-----------------------------------------');
mat2tex(Table_TS_SlopeInfl_Collected_Simple,'%6.2f',rows,cols, [], Opts)

Table_TS_SlopeInfl_Collected_Simple_b = [TSPred_BetasSlopeInflDiff_con TSPred_BetasSlopeInflDiff_con_se TSPred_BetasSlopeInflDiff_con_pval_b...
                         TSPred_BetasSlopeInflDiff_C1 TSPred_BetasSlopeInflDiff_C1_se TSPred_BetasSlopeInflDiff_C1_pval_b 100*TSPred_BetasSlopeInflDiff_R2adj...
                         TSPred_BetasFXSlopeInflDiff_con TSPred_BetasFXSlopeInflDiff_con_se TSPred_BetasFXSlopeInflDiff_con_pval_b ...
                         TSPred_BetasFXSlopeInflDiff_C1 TSPred_BetasFXSlopeInflDiff_C1_se TSPred_BetasFXSlopeInflDiff_C1_pval_b 100*TSPred_BetasFXSlopeInflDiff_R2adj...
                         TSPred_BetasSlopeInflDiffLocal_con TSPred_BetasSlopeInflDiffLocal_con_se TSPred_BetasSlopeInflDiffLocal_con_pval_b...
                         TSPred_BetasSlopeInflDiffLocal_C1 TSPred_BetasSlopeInflDiffLocal_C1_se TSPred_BetasSlopeInflDiffLocal_C1_pval_b 100*TSPred_BetasSlopeInflDiffLocal_R2adj...
                         [Wp_SlopeInflDiff_TermStr_b(1:Nc+1,1); Wp_SlopeInflDiff_TermStr_b(Nc+1,1)]...
                         TSPred_BetasSlopeInflDiff_Nobs];   
Table_TS_SlopeInfl_Collected_Simple_b = [Table_TS_SlopeInfl_Collected_Simple_b; Wp_SlopeInflDiff_con_b(1,1) NaN NaN Wp_SlopeInflDiff_slope_b(1,1) NaN NaN NaN Wp_FXSlopeInflDiff_con_b(1,1) NaN NaN Wp_FXSlopeInflDiff_slope_b(1,1) NaN NaN NaN Wp_SlopeInflDiffLocal_con_b(1,1) NaN NaN Wp_SlopeInflDiffLocal_slope_b(1,1) NaN NaN NaN Wp_SlopeInflDiff_TermStr_b(Nc+2,1) NaN];                                          
cols  = {'coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','coeff','s.e.','p-value','coeff','s.e.','p-value','R2adj(%)','p-value','Obs.'}; 
disp(' ');
disp('-----------------------------------------');
disp('Collected Results of Time-Series Regressions on Slope, Inflation and Credit Rating Differentials (fixed-b)');
disp('-----------------------------------------');
mat2tex(Table_TS_SlopeInfl_Collected_Simple_b,'%6.2f',rows,cols, [], Opts)
 
